@@ -3,10 +3,10 @@
  *  Module    : extern.h
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 1998-07-13
+ *  Updated   : 2001-11-10
  *  Notes     :
  *
- * Copyright (c) 1997-2001 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1997-2002 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -140,12 +140,20 @@
 #ifdef DECL_INET_ADDR
 	extern unsigned long inet_addr (const char *);
 #endif /* DECL_INET_ADDR */
+#if 0 /* breaks gcc 3.0 -std=c89 on SuSE 7.1 */
+#	ifdef DECL_INET_ATON
+		extern int inet_aton(const char *, struct in_addr *);
+#	endif /* DECL_INET_ATON */
+#endif /* 0 */
 #ifdef DECL_IOCTL
 	extern int ioctl (int, int, void *);
 #endif /* DECL_IOCTL */
 #if defined(DECL_ISASCII) && !defined(isascii)
 	extern int isascii (int);
 #endif /* DECL_ISASCII && !isascii */
+#ifdef DECL_KILL
+	extern int kill (pid_t, int);
+#endif /* DECL_KILL */
 #ifdef DECL_MALLOC
 	extern void *malloc (size_t);
 #endif /* DECL_MALLOC */
@@ -380,12 +388,16 @@ extern constext *help_select[];
 extern constext *help_thread[];
 extern constext *txt_colors[];
 extern constext *txt_kill_level_type[];
+#ifdef CHARSET_CONVERSION
+	extern constext *txt_mime_charsets[];
+#endif /* CHARSET_CONVERSION */
 extern constext *txt_marks[];
 extern constext *txt_mime_encodings[NUM_MIME_ENCODINGS];
 extern constext *txt_onoff[];
 extern constext *txt_post_process_type[];
 extern constext *txt_show_from[];
-extern constext *txt_sort_type[];
+extern constext *txt_sort_a_type[];	/* a=articles */
+extern constext *txt_sort_t_type[];	/* t=threads */
 extern constext *txt_strip_bogus_type[];
 extern constext *txt_thread[];
 extern constext *txt_wildcard_type[];
@@ -471,6 +483,7 @@ extern constext txt_command_failed[];
 extern constext txt_connecting[];
 extern constext txt_connecting_port[];
 extern constext txt_copyright_notice[];
+extern constext txt_cr[];
 extern constext txt_creating_active[];
 extern constext txt_creating_newsrc[];
 extern constext txt_default[];
@@ -525,12 +538,12 @@ extern constext txt_error_header_line_bad_encoding[];
 extern constext txt_error_header_line_blank[];
 extern constext txt_error_header_line_colon[];
 extern constext txt_error_header_line_comma[];
-extern constext txt_error_header_line_empty_subject[];
-extern constext txt_error_header_line_empty_newsgroups[];
-extern constext txt_error_header_line_groups_contd[];
-extern constext txt_error_header_line_missing_newsgroups[];
-extern constext txt_error_header_line_missing_subject[];
-extern constext txt_error_header_line_missing_target[];
+extern constext txt_error_header_line_empty[];
+#ifndef FOLLOW_USEFOR_DRAFT
+	extern constext txt_error_header_line_comma[];
+	extern constext txt_error_header_line_groups_contd[];
+#endif /* !FOLLOW_USEFOR_DRAFT */
+extern constext txt_error_header_line_missing[];
 extern constext txt_error_header_line_space[];
 extern constext txt_error_no_domain_name[];
 extern constext txt_error_no_enter_permission[];
@@ -538,9 +551,12 @@ extern constext txt_error_no_from[];
 extern constext txt_error_no_read_permission[];
 extern constext txt_error_no_such_file[];
 extern constext txt_error_no_write_permission[];
+extern constext txt_error_passwd_missing[];
 extern constext txt_error_plural[];
 extern constext txt_error_sender_in_header_not_allowed[];
 extern constext txt_error_singular[];
+extern constext txt_error_unknown_dlevel[];
+extern constext txt_esc[];
 extern constext txt_exiting[];
 extern constext txt_external_mail_done[];
 #ifndef HAVE_LIBUU
@@ -568,47 +584,51 @@ extern constext txt_group_plural[];
 extern constext txt_group_select_com[];
 extern constext txt_group_selection[];
 extern constext txt_group_singular[];
-extern constext txt_help_disp[];
-extern constext txt_help_disp_[];
-extern constext txt_help_navi[];
-extern constext txt_help_navi_[];
-extern constext txt_help_misc[];
-extern constext txt_help_misc_[];
-extern constext txt_help_ops[];
-extern constext txt_help_ops_[];
-extern constext txt_help_B[];
-extern constext txt_help_B_[];
-extern constext txt_help_D[];
-extern constext txt_help_I[];
-extern constext txt_help_K[];
-extern constext txt_help_L[];
-extern constext txt_help_M[];
-extern constext txt_help_Q[];
-extern constext txt_help_S[];
-extern constext txt_help_T[];
-extern constext txt_help_U[];
-extern constext txt_help_W[];
-extern constext txt_help_X[];
-extern constext txt_help_a[];
-extern constext txt_help_b[];
+extern constext txt_help_article_autokill[];
+extern constext txt_help_article_autoselect[];
+extern constext txt_help_article_browse_urls[];
+extern constext txt_help_article_by_num[];
+extern constext txt_help_article_cancel[];
+extern constext txt_help_article_edit[];
+extern constext txt_help_article_first_in_thread[];
+extern constext txt_help_article_first_page[];
+extern constext txt_help_article_followup[];
+extern constext txt_help_article_followup_no_quote[];
+extern constext txt_help_article_followup_with_header[];
+extern constext txt_help_article_last_in_thread[];
+extern constext txt_help_article_last_page[];
+extern constext txt_help_article_mark_thread_read[];
+extern constext txt_help_article_next[];
+extern constext txt_help_article_next_thread[];
+extern constext txt_help_article_next_unread[];
+extern constext txt_help_article_parent[];
+#ifdef HAVE_PGP_GPG
+	extern constext txt_help_article_pgp[];
+#endif /* HAVE_PGP_GPG */
+extern constext txt_help_article_prev[];
+extern constext txt_help_article_prev_unread[];
+extern constext txt_help_article_quick_kill[];
+extern constext txt_help_article_quick_select[];
+extern constext txt_help_article_quit_to_select_level[];
+extern constext txt_help_article_read_next_unread[];
+extern constext txt_help_article_reply[];
+extern constext txt_help_article_reply_no_quote[];
+extern constext txt_help_article_reply_with_header[];
+extern constext txt_help_article_repost[];
+extern constext txt_help_article_search_backwards[];
+extern constext txt_help_article_search_forwards[];
+extern constext txt_help_article_show_raw[];
+extern constext txt_help_article_skip_quote[];
+extern constext txt_help_article_toggle_formfeed[];
+#ifdef HAVE_COLOR
+	extern constext txt_help_article_toggle_highlight[];
+#endif /* HAVE_COLOR */
+extern constext txt_help_article_toggle_rot13[];
+extern constext txt_help_article_toggle_tabwidth[];
+extern constext txt_help_article_toggle_tex2iso[];
+extern constext txt_help_article_toggle_uue[];
+extern constext txt_help_article_view_attachments[];
 extern constext txt_help_bug[];
-extern constext txt_help_cC[];
-extern constext txt_help_c[];
-extern constext txt_help_colon[];
-extern constext txt_help_ctrl_a[];
-extern constext txt_help_ctrl_d[];
-extern constext txt_help_ctrl_f[];
-extern constext txt_help_ctrl_h[];
-extern constext txt_help_ctrl_k[];
-extern constext txt_help_ctrl_l[];
-extern constext txt_help_ctrl_n[];
-extern constext txt_help_ctrl_o[];
-extern constext txt_help_ctrl_t[];
-extern constext txt_help_d[];
-extern constext txt_help_dash[];
-extern constext txt_help_e[];
-extern constext txt_help_equal[];
-extern constext txt_help_esc[];
 extern constext txt_help_filter_from[];
 extern constext txt_help_filter_lines[];
 extern constext txt_help_filter_msgid[];
@@ -616,92 +636,119 @@ extern constext txt_help_filter_subj[];
 extern constext txt_help_filter_text[];
 extern constext txt_help_filter_text_type[];
 extern constext txt_help_filter_time[];
-extern constext txt_help_g[];
-extern constext txt_help_g_caret_dollar[];
-extern constext txt_help_g_cr[];
-extern constext txt_help_g_G[];
-extern constext txt_help_g_ctrl_r[];
-extern constext txt_help_g_d[];
-extern constext txt_help_g_hash[];
-extern constext txt_help_g_num[];
-extern constext txt_help_g_q[];
-extern constext txt_help_g_r[];
-extern constext txt_help_g_search[];
-extern constext txt_help_g_search_[];
-extern constext txt_help_g_T[];
-extern constext txt_help_g_tab[];
-extern constext txt_help_g_x[];
-extern constext txt_help_g_y[];
-extern constext txt_help_g_z[];
-extern constext txt_help_h[];
-extern constext txt_help_hash[];
-extern constext txt_help_i[];
-extern constext txt_help_i_caret_dollar[];
-extern constext txt_help_i_at[];
-extern constext txt_help_i_cr[];
-extern constext txt_help_i_dot[];
-extern constext txt_help_i_n[];
-extern constext txt_help_i_num[];
-extern constext txt_help_i_p[];
-extern constext txt_help_i_search[];
-extern constext txt_help_i_star[];
-extern constext txt_help_i_tab[];
-extern constext txt_help_i_tilda[];
-extern constext txt_help_j[];
+extern constext txt_help_global_article_range[];
+extern constext txt_help_global_esc[];
+extern constext txt_help_global_help[];
+extern constext txt_help_global_last_art[];
+extern constext txt_help_global_line_down[];
+extern constext txt_help_global_line_up[];
+extern constext txt_help_global_lookup_art[];
+extern constext txt_help_global_mail[];
+extern constext txt_help_global_option_menu[];
+extern constext txt_help_global_page_down[];
+extern constext txt_help_global_page_up[];
+#ifndef DONT_HAVE_PIPING
+	extern constext txt_help_global_pipe[];
+#endif /* !DONT_HAVE_PIPING */
+extern constext txt_help_global_post[];
+extern constext txt_help_global_post_postponed[];
+extern constext txt_help_global_posting_history[];
+extern constext txt_help_global_previous_menu[];
+#ifndef DISABLE_PRINTING
+	extern constext txt_help_global_print[];
+#endif /* !DISABLE_PRINTING */
+extern constext txt_help_global_quit_tin[];
+extern constext txt_help_global_redraw_screen[];
+extern constext txt_help_global_save[];
+extern constext txt_help_global_save_tagged[];
+extern constext txt_help_global_search_auth_backwards[];
+extern constext txt_help_global_search_auth_forwards[];
+extern constext txt_help_global_search_body[];
+extern constext txt_help_global_search_body_comment[];
+extern constext txt_help_global_search_subj_backwards[];
+extern constext txt_help_global_search_subj_forwards[];
+#ifndef NO_SHELL_ESCAPE
+	extern constext txt_help_global_shell_escape[];
+#endif /* !NO_SHELL_ESCAPE */
+extern constext txt_help_global_tag[];
+#ifdef HAVE_COLOR
+	extern constext txt_help_global_toggle_color[];
+#endif /* HAVE_COLOR */
+extern constext txt_help_global_toggle_info_line[];
+extern constext txt_help_global_toggle_inverse_video[];
+extern constext txt_help_global_toggle_mini_help[];
+extern constext txt_help_global_version[];
+extern constext txt_help_group_catchup[];
+extern constext txt_help_group_catchup_next[];
+extern constext txt_help_group_first_thread[];
+extern constext txt_help_group_goto_group[];
+extern constext txt_help_group_last_thread[];
+extern constext txt_help_group_list_thread[];
+extern constext txt_help_group_mark_article_unread[];
+extern constext txt_help_group_mark_thread_read[];
+extern constext txt_help_group_mark_thread_unread[];
+extern constext txt_help_group_mark_unsel_art_read[];
+extern constext txt_help_group_next[];
+extern constext txt_help_group_next_unread_art[];
+extern constext txt_help_group_next_unread_article[];
+extern constext txt_help_group_prev[];
+extern constext txt_help_group_prev_unread_art[];
+extern constext txt_help_group_read_article[];
+extern constext txt_help_group_repost[];
+extern constext txt_help_group_reverse_thread_selection[];
+extern constext txt_help_group_select_all[];
+extern constext txt_help_group_select_thread[];
+extern constext txt_help_group_select_thread_if_unread_selected[];
+extern constext txt_help_group_select_thread_pattern[];
+extern constext txt_help_group_tag_parts[];
+extern constext txt_help_group_thread_by_num[];
+extern constext txt_help_group_toggle_getart_limit[];
+extern constext txt_help_group_toggle_read_articles[];
+extern constext txt_help_group_toggle_subj_display[];
+extern constext txt_help_group_toggle_thread_selection[];
+extern constext txt_help_group_toggle_threading[];
+extern constext txt_help_group_undo_thread_selection[];
+extern constext txt_help_group_untag_thread[];
 extern constext txt_help_kill_scope[];
-extern constext txt_help_l[];
-extern constext txt_help_m[];
-extern constext txt_help_n[];
-extern constext txt_help_p_S[];
-extern constext txt_help_p_U[];
-extern constext txt_help_p_V[];
-extern constext txt_help_p_caret_dollar[];
-extern constext txt_help_p_at[];
-extern constext txt_help_p_cr[];
-extern constext txt_help_p_ctrl_e[];
-extern constext txt_help_p_ctrl_w[];
-extern constext txt_help_p_d[];
-extern constext txt_help_p_dot[];
-extern constext txt_help_p_f[];
-extern constext txt_help_p_g[];
-extern constext txt_help_p_k[];
-extern constext txt_help_p_lb[];
-extern constext txt_help_p_rb[];
-extern constext txt_help_p_m[];
-extern constext txt_help_p_n[];
-extern constext txt_help_p_num[];
-extern constext txt_help_p_p[];
-extern constext txt_help_p_r[];
-extern constext txt_help_p_s[];
-extern constext txt_help_p_search[];
-extern constext txt_help_p_star[];
-extern constext txt_help_p_tab[];
-extern constext txt_help_p_tilda[];
-extern constext txt_help_p_u[];
-extern constext txt_help_p_z[];
-extern constext txt_help_plus[];
-extern constext txt_help_q[];
-extern constext txt_help_quick_kill[];
-extern constext txt_help_quick_select[];
-extern constext txt_help_r[];
-extern constext txt_help_s[];
-extern constext txt_help_sel_c[];
+extern constext txt_help_select_catchup[];
+extern constext txt_help_select_catchup_next_unread[];
+extern constext txt_help_select_first_group[];
+extern constext txt_help_select_goto_group[];
+extern constext txt_help_select_group_by_num[];
+extern constext txt_help_select_group_range[];
+extern constext txt_help_select_last_group[];
+extern constext txt_help_select_mark_group_unread[];
+extern constext txt_help_select_move_group[];
+extern constext txt_help_select_next_unread_group[];
+extern constext txt_help_select_next_unread_group[];
+extern constext txt_help_select_quit[];
+extern constext txt_help_select_quit_no_write[];
+extern constext txt_help_select_read_group[];
+extern constext txt_help_select_reset_newsrc[];
 extern constext txt_help_select_scope[];
-extern constext txt_help_semicolon[];
-extern constext txt_help_t[];
-extern constext txt_help_t_caret_dollar[];
-extern constext txt_help_t_cr[];
-extern constext txt_help_t_num[];
-extern constext txt_help_thd_c[];
-extern constext txt_help_thd_C[];
-extern constext txt_help_thd_K[];
-extern constext txt_help_tex[];
-extern constext txt_help_thread[];
-extern constext txt_help_u[];
-extern constext txt_help_v[];
-extern constext txt_help_w[];
-extern constext txt_help_x[];
-extern constext txt_help_y[];
+extern constext txt_help_select_search_group_backwards[];
+extern constext txt_help_select_search_group_comment[];
+extern constext txt_help_select_search_group_forwards[];
+extern constext txt_help_select_subscribe[];
+extern constext txt_help_select_subscribe_pattern[];
+extern constext txt_help_select_sync_with_active[];
+extern constext txt_help_select_toggle_descriptions[];
+extern constext txt_help_select_toggle_read_groups[];
+extern constext txt_help_select_unsubscribe[];
+extern constext txt_help_select_unsubscribe_pattern[];
+extern constext txt_help_select_yank_active[];
+extern constext txt_help_thread_article_by_num[];
+extern constext txt_help_thread_catchup[];
+extern constext txt_help_thread_catchup_next_unread[];
+extern constext txt_help_thread_first_article[];
+extern constext txt_help_thread_last_article[];
+extern constext txt_help_thread_mark_article_read[];
+extern constext txt_help_thread_read_article[];
+extern constext txt_help_thread_toggle_subj_display[];
+extern constext txt_help_title_disp[];
+extern constext txt_help_title_misc[];
+extern constext txt_help_title_navi[];
+extern constext txt_help_title_ops[];
 extern constext txt_hit_space_for_more[];
 extern constext txt_index_page_com[];
 extern constext txt_info_add_kill[];
@@ -751,6 +798,8 @@ extern constext txt_matching_cmd_line_groups[];
 extern constext txt_mini_group_1[];
 extern constext txt_mini_group_2[];
 extern constext txt_mini_group_3[];
+extern constext txt_mini_info_1[];
+extern constext txt_mini_info_2[];
 extern constext txt_mini_page_1[];
 extern constext txt_mini_page_2[];
 extern constext txt_mini_page_3[];
@@ -826,7 +875,6 @@ extern constext txt_posted_info_file[];
 extern constext txt_posting[];
 extern constext txt_postpone_repost[];
 extern constext txt_prompt_fup_ignore[];
-extern constext txt_prompt_unchanged_art[];
 extern constext txt_prompt_unchanged_mail[];
 extern constext txt_prompt_see_postponed[];
 extern constext txt_quick_filter_kill[];
@@ -945,20 +993,20 @@ extern constext txt_untagged_art[];
 extern constext txt_untagged_thread[];
 extern constext txt_unthreading_arts[];
 extern constext txt_updated[];
-#ifndef HAVE_LIBUU
-#if 0
-	extern constext txt_uudecoding[];
-#endif /* 0 */
-#endif /* !HAVE_LIBUU */
 #ifndef HAVE_UUDECODE
 	extern constext txt_uuencode_not_supported[];
 #endif /* !HAVE_UUDECODE */
 extern constext txt_value_out_of_range[];
 extern constext txt_view_attachment[];
 extern constext txt_warn_art_line_too_long[];
+extern constext txt_warn_article_unchanged[];
 extern constext txt_warn_blank_subject[];
 extern constext txt_warn_cancel[];
 extern constext txt_warn_encoding_and_external_inews[];
+#ifdef FOLLOW_USEFOR_DRAFT
+	extern constext txt_warn_header_line_comma[];
+	extern constext txt_warn_header_line_groups_contd[];
+#endif /* FOLLOW_USEFOR_DRAFT */
 extern constext txt_warn_newsrc[];
 extern constext txt_warn_multiple_sigs[];
 extern constext txt_warn_re_but_no_references[];
@@ -1081,7 +1129,6 @@ extern struct t_filters glob_filter;
 extern struct t_group *active;
 extern struct t_newnews *newnews;
 extern struct t_option option_table[];
-extern struct t_posted *posted;
 extern struct t_save *save;
 
 extern t_bool can_post;
@@ -1260,10 +1307,6 @@ extern constext txt_processing_saved_arts[];
 	extern t_bool tin_bbs_mode;
 #endif /* M_AMIGA */
 
-#ifdef M_OS2
-	extern char TMPDIR[PATH_LEN];
-#endif /* M_OS2 */
-
 #ifndef NO_ETIQUETTE
 	extern constext txt_warn_posting_etiquette[];
 #endif /* NO_ETIQUETTE */
@@ -1282,103 +1325,114 @@ extern constext txt_processing_saved_arts[];
 	extern t_bool force_auth_on_conn_open;
 #endif /* NNTP_ABLE */
 
-extern struct opttxt txt_beginner_level;
-extern struct opttxt txt_show_description;
-extern struct opttxt txt_show_author;
-extern struct opttxt txt_draw_arrow;
-extern struct opttxt txt_inverse_okay;
-extern struct opttxt txt_thread_articles;
-extern struct opttxt txt_sort_article_type;
-extern struct opttxt txt_pos_first_unread;
-extern struct opttxt txt_show_only_unread_arts;
-extern struct opttxt txt_show_only_unread_groups;
-extern struct opttxt txt_kill_level;
-extern struct opttxt txt_tab_goto_next_unread;
-extern struct opttxt txt_space_goto_next_unread;
-extern struct opttxt txt_pgdn_goto_next;
-extern struct opttxt txt_auto_list_thread;
-extern struct opttxt txt_tab_after_X_selection;
+extern struct opttxt txt_add_posted_to_filter;
+extern struct opttxt txt_advertising;
+extern struct opttxt txt_alternative_handling;
 extern struct opttxt txt_art_marked_deleted;
 extern struct opttxt txt_art_marked_inrange;
+extern struct opttxt txt_art_marked_killed;
+extern struct opttxt txt_art_marked_read;
+extern struct opttxt txt_art_marked_read_selected;
+extern struct opttxt txt_art_marked_recent;
 extern struct opttxt txt_art_marked_return;
 extern struct opttxt txt_art_marked_selected;
-extern struct opttxt txt_art_marked_recent;
 extern struct opttxt txt_art_marked_unread;
-extern struct opttxt txt_art_marked_read;
-extern struct opttxt txt_art_marked_killed;
-extern struct opttxt txt_art_marked_read_selected;
-extern struct opttxt txt_groupname_max_length;
-extern struct opttxt txt_show_lines;
-extern struct opttxt txt_show_score;
-extern struct opttxt txt_full_page_scroll;
-extern struct opttxt txt_show_last_line_prev_page;
-extern struct opttxt txt_show_signatures;
-extern struct opttxt txt_hide_uue;
-extern struct opttxt txt_news_headers_to_display;
-extern struct opttxt txt_news_headers_to_not_display;
-extern struct opttxt txt_show_xcommentto;
-extern struct opttxt txt_alternative_handling;
-extern struct opttxt txt_strip_re_regex;
-extern struct opttxt txt_strip_was_regex;
+extern struct opttxt txt_auto_bcc;
+extern struct opttxt txt_auto_cc;
+extern struct opttxt txt_auto_list_thread;
+extern struct opttxt txt_auto_reconnect;
+extern struct opttxt txt_auto_save;
+extern struct opttxt txt_batch_save;
+extern struct opttxt txt_beginner_level;
+extern struct opttxt txt_cache_overview_files;
 extern struct opttxt txt_catchup_read_groups;
-extern struct opttxt txt_group_catchup_on_exit;
-extern struct opttxt txt_thread_catchup_on_exit;
+extern struct opttxt txt_color_options;
 extern struct opttxt txt_confirm_action;
 extern struct opttxt txt_confirm_to_quit;
-extern struct opttxt txt_use_mouse;
-extern struct opttxt txt_use_getart_limit;
+extern struct opttxt txt_display_options;
+extern struct opttxt txt_draw_arrow;
+extern struct opttxt txt_editor_format;
+extern struct opttxt txt_expert_options;
+extern struct opttxt txt_filter_days;
+extern struct opttxt txt_force_screen_redraw;
+extern struct opttxt txt_full_page_scroll;
 extern struct opttxt txt_getart_limit;
-extern struct opttxt txt_recent_time;
+extern struct opttxt txt_getart_limit_options;
+extern struct opttxt txt_group_catchup_on_exit;
+extern struct opttxt txt_groupname_max_length;
+extern struct opttxt txt_hide_uue;
+extern struct opttxt txt_inews_prog;
+extern struct opttxt txt_inverse_okay;
+extern struct opttxt txt_keep_dead_articles;
+extern struct opttxt txt_keep_posted_articles;
+extern struct opttxt txt_keep_posted_articles_file;
+extern struct opttxt txt_kill_level;
+extern struct opttxt txt_mail_8bit_header;
 extern struct opttxt txt_mail_address;
+extern struct opttxt txt_mail_mime_encoding;
+extern struct opttxt txt_mail_quote_format;
+extern struct opttxt txt_maildir;
+extern struct opttxt txt_mailer_format;
+extern struct opttxt txt_mark_saved_read;
+extern struct opttxt txt_mm_charset;
+extern struct opttxt txt_news_headers_to_display;
+extern struct opttxt txt_news_headers_to_not_display;
+extern struct opttxt txt_news_quote_format;
+extern struct opttxt txt_pgdn_goto_next;
+extern struct opttxt txt_pos_first_unread;
+extern struct opttxt txt_post_8bit_header;
+extern struct opttxt txt_post_mime_encoding;
+extern struct opttxt txt_post_process;
+extern struct opttxt txt_post_process_view;
+extern struct opttxt txt_posting_options;
+extern struct opttxt txt_print_header;
+extern struct opttxt txt_printer;
+extern struct opttxt txt_process_only_unread;
 extern struct opttxt txt_prompt_followupto;
-extern struct opttxt txt_sigfile;
-extern struct opttxt txt_sigdashes;
-extern struct opttxt txt_signature_repost;
 extern struct opttxt txt_quote_chars;
 extern struct opttxt txt_quote_empty_lines;
 extern struct opttxt txt_quote_signatures;
-extern struct opttxt txt_news_quote_format;
-extern struct opttxt txt_xpost_quote_format;
-extern struct opttxt txt_mail_quote_format;
-extern struct opttxt txt_advertising;
-extern struct opttxt txt_mm_charset;
-extern struct opttxt txt_post_mime_encoding;
-extern struct opttxt txt_post_8bit_header;
-extern struct opttxt txt_post_process_view;
-extern struct opttxt txt_mail_mime_encoding;
-extern struct opttxt txt_mail_8bit_header;
-extern struct opttxt txt_strip_blanks;
-extern struct opttxt txt_auto_cc;
-extern struct opttxt txt_auto_bcc;
-extern struct opttxt txt_spamtrap_warning_addresses;
-extern struct opttxt txt_filter_days;
-extern struct opttxt txt_add_posted_to_filter;
-extern struct opttxt txt_maildir;
-extern struct opttxt txt_save_to_mmdf_mailbox;
-extern struct opttxt txt_batch_save;
-extern struct opttxt txt_savedir;
-extern struct opttxt txt_auto_save;
-extern struct opttxt txt_mark_saved_read;
-extern struct opttxt txt_post_process;
-extern struct opttxt txt_process_only_unread;
-extern struct opttxt txt_print_header;
-extern struct opttxt txt_printer;
-extern struct opttxt txt_wildcard;
-extern struct opttxt txt_force_screen_redraw;
-extern struct opttxt txt_start_editor_offset;
-extern struct opttxt txt_editor_format;
-extern struct opttxt txt_use_builtin_inews;
-extern struct opttxt txt_mailer_format;
-extern struct opttxt txt_use_mailreader_i;
-extern struct opttxt txt_unlink_article;
-extern struct opttxt txt_keep_posted_articles;
-extern struct opttxt txt_keep_posted_articles_file;
-extern struct opttxt txt_keep_dead_articles;
-extern struct opttxt txt_strip_newsrc;
-extern struct opttxt txt_strip_bogus;
+extern struct opttxt txt_recent_time;
 extern struct opttxt txt_reread_active_file_secs;
-extern struct opttxt txt_auto_reconnect;
-extern struct opttxt txt_cache_overview_files;
+extern struct opttxt txt_save_to_mmdf_mailbox;
+extern struct opttxt txt_savedir;
+extern struct opttxt txt_saving_options;
+extern struct opttxt txt_show_author;
+extern struct opttxt txt_show_description;
+extern struct opttxt txt_show_last_line_prev_page;
+extern struct opttxt txt_show_lines;
+extern struct opttxt txt_show_only_unread_arts;
+extern struct opttxt txt_show_only_unread_groups;
+extern struct opttxt txt_show_score;
+extern struct opttxt txt_show_signatures;
+extern struct opttxt txt_sigdashes;
+extern struct opttxt txt_sigfile;
+extern struct opttxt txt_signature_repost;
+extern struct opttxt txt_sort_article_type;
+extern struct opttxt txt_sort_threads_type;
+extern struct opttxt txt_space_goto_next_unread;
+extern struct opttxt txt_spamtrap_warning_addresses;
+extern struct opttxt txt_start_editor_offset;
+extern struct opttxt txt_strip_blanks;
+extern struct opttxt txt_strip_bogus;
+extern struct opttxt txt_strip_newsrc;
+extern struct opttxt txt_strip_re_regex;
+extern struct opttxt txt_strip_was_regex;
+extern struct opttxt txt_tab_after_X_selection;
+extern struct opttxt txt_tab_goto_next_unread;
+extern struct opttxt txt_tex2iso_conv;
+extern struct opttxt txt_thread_articles;
+extern struct opttxt txt_thread_catchup_on_exit;
+extern struct opttxt txt_unlink_article;
+extern struct opttxt txt_use_getart_limit;
+extern struct opttxt txt_use_mailreader_i;
+extern struct opttxt txt_use_mouse;
+extern struct opttxt txt_wildcard;
+extern struct opttxt txt_xpost_quote_format;
+#ifdef CHARSET_CONVERSION
+	extern struct opttxt txt_mm_local_charset;
+	extern struct opttxt txt_mm_network_charset;
+#endif /* CHARSET_CONVERSION */
 #ifdef HAVE_COLOR
 	extern struct opttxt txt_quote_regex;
 	extern struct opttxt txt_quote_regex2;
