@@ -6,7 +6,7 @@
  *  Updated   : 1995-12-21; 2000-06-10, Dirk Nimmich
  *  Notes     :
  *
- * Copyright (c) 1994-2001 Steve Robbins <stever@cs.mcgill.ca>
+ * Copyright (c) 1994-2002 Steve Robbins <stever@cs.mcgill.ca>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -161,6 +161,7 @@
 #define iKeyToggleInfoLastLine 'i'
 #define iKeyDown2 'j'
 #define iKeyUp2 'k'
+#define iKeyPrint 'o'
 #define iKeyQuit 'q'
 #define iKeyVersion 'v'
 #define iKeyPost 'w'
@@ -232,7 +233,7 @@
 #define iKeyGroupMail 'm'
 #define iKeyGroupNextGroup 'n'
 #ifndef DISABLE_PRINTING
-#	define iKeyGroupPrint 'o'
+#	define iKeyPrint 'o'
 #endif /* !DISABLE_PRINTING */
 #define iKeyGroupPrevGroup 'p'
 #define iKeyGroupToggleReadUnread 'r'
@@ -304,9 +305,6 @@
 #define iKeyPageListThd 'l'
 #define iKeyPageMail 'm'
 #define iKeyPageNextArt 'n'
-#ifndef DISABLE_PRINTING
-#	define iKeyPagePrint 'o'
-#endif /* !DISABLE_PRINTING */
 #define iKeyPagePrevArt 'p'
 #define iKeyPageReplyQuote 'r'
 #define iKeyPageSave 's'
@@ -429,6 +427,7 @@ typedef struct mkm {
 	t_menukeys feed_supersede_article;
 	t_menukeys filter_quit_edit_save;
 	t_menukeys group_nav;
+	t_menukeys info_nav;
 	t_menukeys nrctbl_create;
 	t_menukeys page_nav;
 	t_menukeys pgp_mail;
