@@ -3,10 +3,10 @@
  *  Module    : proto.h
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   :
- *  Updated   : 2000-01-03
+ *  Updated   : 2002-01-29
  *  Notes     :
  *
- * Copyright (c) 1997-2001 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2002 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,7 +49,7 @@
 #endif /* CASE_PROBLEM */
 
 /* active.c */
-extern char group_flag (int ch);
+extern char group_flag (char ch);
 extern int get_active_num (void);
 extern t_bool match_group_list (const char *group, const char *group_list);
 extern t_bool parse_active_line (char *line, long *max, long *min, char *moderated);
@@ -62,13 +62,14 @@ extern void read_news_active_file (void);
 
 /* art.c */
 extern char *find_nov_file (struct t_group *group, int mode);
+extern int global_get_multipart_info (int aindex, MultiPartInfo *setme);
 extern t_bool index_group (struct t_group *group);
 extern void do_update (t_bool catchup);
 extern void find_base (struct t_group *group);
 extern void make_threads (struct t_group *group, t_bool rethread);
 extern void set_article (struct t_article *art);
 extern void show_art_msg (char *group);
-extern void sort_arts (unsigned int sort_art_type);
+extern void sort_arts (unsigned /* int */ sort_art_type);
 extern void write_nov_file (struct t_group *group);
 
 /* attrib.c */
@@ -82,12 +83,10 @@ extern void write_attributes_file (const char *file);
 
 /* charset.c */
 extern t_bool is_art_tex_encoded (FILE *fp);
-extern void convert_to_printable (char* buf);
+extern void convert_body2printable (char* buf);
 extern void convert_iso2asc (char *iso, char *asc, int t);
 extern void convert_tex2iso (char *from, char *to);
-#if 1
-	extern void convert_body2printable (char* buf);
-#endif /* 1 */
+extern void convert_to_printable (char* buf);
 
 /* color.c */
 #ifdef HAVE_COLOR
@@ -98,14 +97,15 @@ extern void draw_pager_line (const char *str, int flags);
 
 /* config.c */
 extern char **ulBuildArgv (char *cmd, int *new_argc);
+extern char *fmt_option_prompt (char *dst, int len, t_bool editing, int option);
 extern char *quote_space_to_dash (char *str);
 extern const char *print_boolean (t_bool value);
 extern int change_config_file (struct t_group *group);
 extern int option_row (int option);
 extern t_bool match_boolean (char *line, const char *pat, t_bool *dst);
 extern t_bool match_integer (char *line, const char *pat, int *dst, int maxval);
-extern t_bool match_long (char *line, const char *pat, long *dst);
 extern t_bool match_list (char *line, constext *pat, constext *const *table, size_t tablelen, int *dst);
+extern t_bool match_long (char *line, const char *pat, long *dst);
 extern t_bool match_string (char *line, const char *pat, char *dst, size_t dstlen);
 extern t_bool read_config_file (char *file, t_bool global_file);
 extern void quote_dash_to_space (char *str);
@@ -116,6 +116,7 @@ extern void write_config_file (char *file);
 /* cook.c */
 extern const char *get_filename (t_param *ptr);
 extern t_bool cook_article (t_bool wrap_lines, t_openartinfo *artinfo, int tabs, t_bool uue);
+extern t_bool expand_ctrl_chars (char *to, const char *from, int length, size_t cook_width);
 
 /* curses.c */
 extern OUTC_RETTYPE outchar (OUTC_ARGS);
@@ -168,17 +169,17 @@ extern void setup_screen (void);
 extern void envargs (int *Pargc, char ***Pargv, const char *envstr);
 
 /* feed.c */
-/* extern char get_post_proc_type (int proc_type); */
 extern void feed_articles (int function, int level, struct t_group *group, int respnum);
 
 /* filter.c */
+extern int unfilter_articles (void);
 extern t_bool filter_articles (struct t_group *group);
 extern t_bool filter_menu (int type, struct t_group *group, struct t_article *art);
 extern t_bool quick_filter (int type, struct t_group *group, struct t_article *art);
-extern t_bool quick_filter_select_posted_art (struct t_group *group, const char *subj);
+extern t_bool quick_filter_select_posted_art (struct t_group *group, const char *subj, const char *a_message_id);
 extern t_bool read_filter_file (const char *file);
 extern void free_all_filter_arrays (void);
-extern int unfilter_articles (void);
+extern void refresh_filter_menu (void);
 
 /* getline.c */
 extern char *tin_getline (const char *prompt, int number_only, const char *str, int max_chars, t_bool passwd, int which_hist);
@@ -211,13 +212,13 @@ extern void hash_init (void);
 extern void hash_reclaim (void);
 
 /* help.c */
-extern void show_info_page (int type, const char *help[], const char *title);
+extern void show_help_page (const int level, const char *title);
 extern void show_mini_help (int level);
 extern void toggle_mini_help (int level);
 #ifdef USE_CURSES
-	extern void display_info_page (t_bool first);
+	extern void display_help_page (t_bool first);
 #else
-	extern void display_info_page (void);
+	extern void display_help_page (void);
 #endif /* USE_CURSES */
 
 /* header.c */
@@ -252,8 +253,8 @@ extern void joinpath (char *result, const char *dir, const char *file);
 /* keymap.c */
 extern char *printascii(char *buf, int ch);
 extern t_bool read_keymap_file (void);
-extern void free_keymaps (void);
 extern void build_keymaps (void);
+extern void free_keymaps (void);
 
 /* list.c */
 extern char *random_organization (char *in_org);
@@ -262,12 +263,13 @@ extern struct t_group *group_add (const char *group);
 extern struct t_group *group_find (const char *group_name);
 extern unsigned long hash_groupname (const char *group);
 extern void init_group_hash (void);
-#if 0
-	extern struct t_group *psGrpFirst (void);
-	extern struct t_group *psGrpLast (void);
-	extern struct t_group *psGrpNext (void);
-	extern struct t_group *psGrpPrev (void);
-#endif /* 0 */
+
+/* lock.c */
+extern int fd_lock(int fd, t_bool block);
+extern int fd_unlock(int fd);
+extern int test_fd_lock(int fd);
+extern t_bool dot_lock(const char *filename);
+extern t_bool dot_unlock(const char *filename);
 
 /* mail.c */
 extern t_bool art_edit (struct t_group *psGrp, struct t_article *psArt);
@@ -317,6 +319,7 @@ extern char *quote_wild_whitespace (char *str);
 extern const char *get_val (const char *env, const char *def);
 extern const char *gnksa_strerror (int errcode);
 extern int gnksa_check_from (char *from);
+extern int gnksa_split_from (const char *from, char *address, char *realname, int *addrtype);
 extern int get_initials (int respnum, char *s, int maxsize);
 extern int gnksa_do_check_from(const char *from, char *address, char *realname);
 extern int my_chdir (char *path);
@@ -345,22 +348,26 @@ extern void make_group_path (char *name, char *path);
 extern void read_input_history_file (void);
 extern void rename_file (const char *old_filename, const char *new_filename);
 extern void show_inverse_video_status (void);
-extern void strip_address (char *the_address, char *stripped_address);
 extern void strip_double_ngs (char *ngs_list);
 extern void strip_line (char *line);
 extern void strip_name (char *the_address, char *stripped_address);
 extern void tin_done (int ret);
 extern void toggle_inverse_video (void);
-extern void vPrintBugAddress (void);
 #if 0
 	extern void parse_from (char *from_line, char *eaddr, char *fname);
 #else
 	extern int parse_from (const char *from, char *address, char *realname);
 #endif /* 0 */
-#ifdef LOCAL_CHARSET
-	extern void buffer_to_local (char *b);
-	extern void buffer_to_network (char *b);
-#endif /* LOCAL_CHARSET */
+#if defined(LOCAL_CHARSET) || defined(MAC_OS_X)
+	extern void buffer_to_local (char *line);
+#else
+#	ifdef CHARSET_CONVERSION
+	 extern void buffer_to_local (char *line, const char* network_charset, const char *local_charset);
+#	endif /* CHARSET_CONVERSION */
+#endif /* LOCAL_CHARSET || MAC_OS_X */
+#if defined(LOCAL_CHARSET) || defined(MAC_OS_X) || defined(CHARSET_CONVERSION)
+	extern void buffer_to_network (char *line);
+#endif /* LOCAL_CHARSET || MAC_OS_X || CHARSET_CONVERSION */
 #ifdef HAVE_COLOR
 	extern t_bool toggle_color (void);
 	extern void show_color_status (void);
@@ -445,7 +452,9 @@ extern void vGet1GrpArtInfo (struct t_group *grp);
 
 /* page.c */
 extern int show_page (struct t_group *group, int respnum, int *threadnum);
+extern void display_info_page (int part);
 extern void draw_page (const char *group, int part);
+extern void info_pager (FILE *info_fh, const char *title, t_bool wrap_at_ends);
 extern void resize_article (t_bool wrap_lines, t_openartinfo *artinfo);
 extern void toggle_raw (struct t_group *group);
 
@@ -497,7 +506,7 @@ extern int prompt_num (int ch, const char *prompt);
 extern int prompt_yn (int line, const char *prompt, t_bool default_answer);
 extern int prompt_msgid (void);
 extern t_bool prompt_default_string (const char *prompt, char *buf, int buf_len, char *default_prompt, int which_hist);
-extern t_bool prompt_menu_string (int line, int col, char *var);
+extern t_bool prompt_menu_string (int line, const char *prompt, char *var);
 extern t_bool prompt_option_char (int option);
 extern t_bool prompt_option_num (int option);
 extern t_bool prompt_option_string (int option);
@@ -537,12 +546,6 @@ extern void rfc1557_encode (char *line, FILE *f, int e);
 	extern void rfc1922_encode (char *line, FILE *f, int e);
 #endif /* 0 */
 
-/* rfc2047.c */
-extern char *rfc1522_decode (const char *s);
-extern char *rfc1522_encode (char *s,t_bool ismail);
-extern int mmdecode (const char *what, int encoding, int delimiter, char *where, const char *charset);
-extern void rfc15211522_encode (const char *filename, constext *mime_encoding, t_bool allow_8bit_header,t_bool ismail);
-
 /* rfc2046.c */
 extern const char *get_param (t_param *list, const char *name);
 extern char *parse_header (char *buf, const char *pat, t_bool decode);
@@ -553,6 +556,13 @@ extern t_part *new_part (t_part *part);
 extern void art_close (t_openartinfo *artinfo);
 extern void free_and_init_header (struct t_header *hdr);
 extern void free_parts (t_part *ptr);
+extern void unfold_header (char *line);
+
+/* rfc2047.c */
+extern char *rfc1522_decode (const char *s);
+extern char *rfc1522_encode (char *s,t_bool ismail);
+extern int mmdecode (const char *what, int encoding, int delimiter, char *where, const char *charset);
+extern void rfc15211522_encode (const char *filename, constext *mime_encoding, t_bool allow_8bit_header,t_bool ismail);
 
 /* save.c */
 extern int check_start_save_any_news (int function, t_bool catchup);
@@ -578,7 +588,7 @@ extern void show_progress (const char *txt, long count, long total);
 extern void show_title (char *title);
 extern void spin_cursor (void);
 extern void stow_cursor (void);
-extern void wait_message (int delay, const char *fmt, ...);
+extern void wait_message (int sdelay, const char *fmt, ...);
 
 /* search.c */
 extern int get_search_vectors (int *start, int *end);
@@ -586,7 +596,6 @@ extern int search (int key, int current_art, t_bool forward);
 extern int search_active (t_bool forward);
 extern int search_article (t_bool forward, int start_line, int lines, t_lineinfo *line, t_bool show_ctrl_l, FILE *fp);
 extern int search_config (t_bool forward, int current, int last);
-extern int search_help (t_bool forward, int current, int last);
 extern int search_body (int current_art);
 
 /* select.c */
@@ -610,15 +619,6 @@ extern void handle_resize (t_bool repaint);
 extern void set_noteslines (int num_lines);
 extern void set_signal_catcher (int flag);
 extern void set_signal_handlers (void);
-#if 0
-	extern void set_signals_art (void);
-	extern void set_signals_config (void);
-	extern void set_signals_group (void);
-	extern void set_signals_help (void);
-	extern void set_signals_page (void);
-	extern void set_signals_select (void);
-	extern void set_signals_thread (void);
-#endif /* 0 */
 
 /* strftime.c */
 extern size_t my_strftime (char *s, size_t maxsize, const char *format, struct tm *timeptr);
@@ -627,8 +627,8 @@ extern size_t my_strftime (char *s, size_t maxsize, const char *format, struct t
 extern char *eat_tab (char *s);
 extern char *my_strdup (const char *str);
 extern char *str_trim (char *string);
-extern const char *strcasestr (const char *haystack, const char *needle);
 extern char *tin_ltoa (long value, int digits);
+extern const char *strcasestr (const char *haystack, const char *needle);
 extern int sh_format (char *dst, size_t len, const char *fmt, ...);
 extern size_t mystrcat (char **t, const char *s);
 extern void my_strncpy (char *p, const char *q, size_t n);
@@ -658,6 +658,10 @@ extern void str_lwr (char *str);
 	extern char *my_strerror (int n);
 #	define strerror(n) my_strerror(n)
 #endif /* !HAVE_STRERROR */
+#ifndef HAVE_STRRSTR
+	extern char * my_strrstr(const char *str, const char *pat);
+#	define strrstr(s,p)	my_strrstr(s,p)
+#endif /* !HAVE_STRRSTR */
 
 /* tags.c */
 extern int line_is_tagged (int n);
@@ -670,8 +674,17 @@ extern void remove_tag (long art);
 extern void undo_auto_select_arts (void);
 extern void undo_selections (void);
 
+/* tmpfile.c */
+#ifndef HAVE_TMPFILE
+	extern FILE *tmpfile (void);
+#endif /* !HAVE_TMPFILE */
+
+/* my_tmpfile.c */
+extern int my_tmpfile(char *filename, size_t name_size, t_bool need_name, const char *base_dir);
+
 /* thread.c */
 extern int find_response (int i, int n);
+extern int get_score_of_thread (int n);
 extern int new_responses (int thread);
 extern int next_response (int n);
 extern int next_thread (int n);
@@ -695,5 +708,4 @@ extern void NSETRNG0 (t_bitmap *bitmap, long low, long high);
 extern void NSETRNG1 (t_bitmap *bitmap, long low, long high);
 extern void art_mark_xref_read (struct t_article *art);
 
-
 #endif /* !PROTO_H */
