@@ -3,10 +3,10 @@
  *  Module    : tin.h
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 1997-12-31
+ *  Updated   : 2001-07-22
  *  Notes     : #include files, #defines & struct's
  *
- * Copyright (c) 1997-2001 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1997-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -109,7 +109,7 @@
 
 #include	<signal.h>
 
-enum context { cMain, cArt, cConfig, cGroup, cHelp, cPage, cSelect, cThread };
+enum context { cMain, cArt, cConfig, cFilter, cGroup, cHelp, cInfopager, cPage, cSelect, cThread };
 enum resizer { cNo, cYes, cRedraw };
 
 #ifdef VMS
@@ -234,14 +234,25 @@ enum resizer { cNo, cYes, cRedraw };
 #	include <getopt.h>
 #endif /* HAVE_GETOPT_H */
 
-/* prefer string.h because it's Posix */
-#ifdef HAVE_STRING_H
-#	include <string.h>
+/*
+ * FIXME: make this autoconf
+ */
+#ifndef __QNX__
+#	ifdef HAVE_STRING_H
+#		include <string.h>
+#	else
+#		ifdef HAVE_STRINGS_H
+#			include <strings.h>
+#		endif /* HAVE_STRINGS_H */
+#	endif /* HAVE_STRING_H */
 #else
+#	ifdef HAVE_STRING_H
+#		include <string.h>
+#	endif /* HAVE_STRING_H */
 #	ifdef HAVE_STRINGS_H
 #		include <strings.h>
 #	endif /* HAVE_STRINGS_H */
-#endif /* HAVE_STRING_H */
+#endif /* !__QNX__ */
 
 #ifdef HAVE_FCNTL_H
 #	include <fcntl.h>
@@ -294,6 +305,10 @@ enum resizer { cNo, cYes, cRedraw };
 #	include <sys/wait.h>
 #endif /* HAVE_SYS_WAIT_H */
 
+#ifndef WEXITSTATUS
+#	define WEXITSTATUS(status)		((int) (((status) >> 8) & 0xFF))
+#endif /* !WEXITSTATUS */
+
 /*
  * Needed for timeout in user abort of indexing a group (BSD & SYSV variaties)
  */
@@ -336,14 +351,6 @@ enum resizer { cNo, cYes, cRedraw };
 #		include "amiga.h"
 #		define DIR_BUF	struct dirent
 #	endif /* M_AMIGA */
-#	ifdef M_OS2
-#		include "os_2.h"
-#		define DIR_BUF	struct dirent
-#	endif /* M_OS2 */
-#	ifdef WIN32
-#		include "win32.h"
-#		define DIR_BUF	struct direct
-#	endif /* WIN32 */
 #	ifdef M_XENIX
 #		include <sys/ndir.h>
 #		define DIR_BUF	struct direct
@@ -370,6 +377,16 @@ enum resizer { cNo, cYes, cRedraw };
 #	define STDIN_FILENO	0
 #endif /* !STDIN_FILENO */
 
+/*
+ * include <paths.h> if available to define _PATH_TMP
+ */
+#ifdef HAVE_PATHS_H
+#	include <paths.h>
+#endif /* HAVE_PATHS_H */
+#ifndef _PATH_TMP
+#	define _PATH_TMP	"/tmp/"
+#endif /* _PATH_TMP */
+
 /*
  * If OS misses the isascii() function
  */
@@ -478,7 +495,6 @@ enum resizer { cNo, cYes, cRedraw };
 #		define strchr(str, ch)	index(str, ch)
 #		define strrchr(str, ch)	rindex(str, ch)
 #	endif /* !HAVE_STRCHR */
-#	define DEFAULT_SHELL	"/bin/csh"
 #	if defined(__386BSD__) || defined(__bsdi__) || defined(__NetBSD__) || defined(__FreeBSD__) || defined(__OpenBSD__)
 #		define DEFAULT_PRINTER	"/usr/bin/lpr"
 #		define DEFAULT_SUM	"/usr/bin/cksum -o 1 <" /* use tailing <, otherwise get filename output too */
@@ -517,28 +533,6 @@ enum resizer { cNo, cYes, cRedraw };
 #		define DEFAULT_PRINTER	"PRINT/DELETE"
 #		define DEFAULT_UNSHAR	"unshar %s."
 #	endif /* VMS */
-#	ifdef M_OS2
-#		ifndef DEFAULT_EDITOR
-#			define DEFAULT_EDITOR	"epm /m"
-#		endif /* !DEFAULT_EDITOR */
-#		define DEFAULT_MAILBOX	"/mail"
-#		define DEFAULT_MAILER	"sendmail -af %s -f %s %s"
-#		define DEFAULT_POSTER	"postnews %s"
-#		define DEFAULT_PRINTER	"lpt1"
-#		define DEFAULT_SHELL	"cmd.exe"
-#		define DEFAULT_UNSHAR	"unshar %s"
-#	endif /* M_OS2 */
-#	ifdef WIN32
-#		ifndef DEFAULT_EDITOR
-#			define DEFAULT_EDITOR	"vi"
-#		endif /* !DEFAULT_EDITOR */
-#		define DEFAULT_MAILBOX	"/mail"
-#		define DEFAULT_MAILER	"sendmail"
-#		define DEFAULT_POSTER	"postnews %s"
-#		define DEFAULT_PRINTER	"lpt1"
-#		define DEFAULT_SHELL	"cmd.exe"
-#		define DEFAULT_UNSHAR	"unshar %s"
-#	endif /* WIN32 */
 #	ifdef QNX42
 #		ifndef DEFAULT_EDITOR
 #			define DEFAULT_EDITOR	"/bin/vedit"
@@ -561,16 +555,6 @@ enum resizer { cNo, cYes, cRedraw };
 #		define DEFAULT_MAILER	"/bin/rmail"
 #	endif /* UNIXPC */
 
-/* HP-UX >= 10 defines __STDC_EXT__ (ANSI) || __CLASSIC_C__ (K&R) */
-#	if defined (__hpux)
-#		if defined (__STDC_EXT__) || defined (__CLASSIC_C__)
-#			define DEFAULT_SHELL "/usr/bin/sh"
-#		endif /* __STDC_EXT__ || __CLASSIC_C__ */
-#	endif /* __hpux */
-
-#	ifndef DEFAULT_SHELL
-#		define DEFAULT_SHELL	"/bin/sh"
-#	endif /* !DEFAULT_SHELL */
 #	ifndef DEFAULT_PRINTER
 #		define DEFAULT_PRINTER	"/usr/bin/lp"
 #	endif /* !DEFAULT_PRINTER */
@@ -667,7 +651,7 @@ enum resizer { cNo, cYes, cRedraw };
 #define CONFIG_FILE	"tinrc"
 #define DEFAULT_MAILDIR	"Mail"
 #define DEFAULT_SAVEDIR	"News"
-
+#define DEFAULT_INEWS_PROG "--internal"
 
 /* MMDF-mailbox seperator */
 #ifndef MMDFHDRTXT
@@ -707,18 +691,28 @@ enum resizer { cNo, cYes, cRedraw };
  */
 /*
  * case insensitive
- * split out ftp (only ftp allows username:passwd@, RFC 1738)
+ * split out ftp (only ftp allows username:passwd@, RFC 1738)?
  */
-#define URL_REGEX "\\b(?:https?|ftp|gopher)://(?:[^:@/]*(?::[^:@/]*)?@)?(?:[^\\W_]+(?:(?:[-.][^\\W_]+)+)?\\.[a-z]{2,5}|localhost|(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?))(?::\\d+)?(?:/[^)\\>\"\\s]*|$|(?=[)\\>\"\\s]))"
+#if 0 /* this one is ok for IPv4 */
+#	define URL_REGEX	"\\b(?:https?|ftp|gopher)://(?:[^:@/]*(?::[^:@/]*)?@)?(?:(?:[^\\W_](?:(?:-(?!-)|[^\\W_]){0,61}[^\\W_])?\\.)+[a-z]{2,6}\\.?|localhost|(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?))(?::\\d+)?(?:/[^)\\>\"\\s]*|$|(?=[)\\>\"\\s]))"
+#else	/* this one should be IPv6 safe - test me! */
+#	define URL_REGEX	"\\b(?:https?|ftp|gopher)://(?:[^:@/]*(?::[^:@/]*)?@)?(?:(?:[^\\W_](?:(?:-(?!-)|[^\\W_]){0,61}[^\\W_])?\\.)+[a-z]{2,6}\\.?|localhost|(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)|\\[(?:(?:[0-9A-F]{0,4}:){1,7}[0-9A-F]{1,4}|(?:[0-9A-F]{0,4}:){1,3}(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?))\\])(?::\\d+)?(?:/[^)\\>\"\\s]*|$|(?=[)\\>\"\\s]))"
+#endif /* 0 */
 /*
  * case insensitive
  * check against RFC 2368
  */
-#define MAIL_REGEX	"\\b(?:mailto:(?:(?:[-\\w$.+!*'(),;/?:@&=]|(?:%[\\da-f]{2}))+))"
+/* #define MAIL_REGEX	"\\b(?:mailto:(?:(?:[-\\w$.+!*'(),;/?:@&=]|(?:%[\\da-f]{2}))+))" */
+#define MAIL_REGEX	"\\b(?:mailto:(?:[-\\w$.+!*'(),;/?:@&=]|%[\\da-f]{2})+)"
+
 /*
  * case insensitive
  */
-#define NEWS_REGEX	"\\b(?:s?news|nntp):[^\\s@]+[@.][^\\s@]+(?:$|(?=[\\s.>\"]))\\b"
+#define NEWS_REGEX	"\\b(?:s?news|nntp):[^\\s@]+[@.][^\\s@]+(?:$|(?=[\\s.><,\"/():]))\\b"
+/*
+ * case insensitive, no implemented
+ */
+#define TELNET_REGEX	"\\btelnet://(?:[^:@/]*(?::[^:@/]*)?@)?(?:(?:[^\\W_](?:(?:-(?!-)|[^\\W_]){0,61}[^\\W_])?\\.)+[a-z]{2,6}\\.?||localhost|(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?))(?::\\d+)?/?"
 
 
 #define FILTER_FILE	"filter"
@@ -752,6 +746,13 @@ enum resizer { cNo, cYes, cRedraw };
 /* Philip Hazel's Perl regular expressions library */
 #include	<pcre.h>
 
+#if defined(HAVE_ICONV) && !defined(LOCAL_CHARSET)
+#	define CHARSET_CONVERSION 1
+#	ifdef HAVE_ICONV_H
+#		include <iconv.h>
+#	endif /* HAVE_ICONV_H */
+#endif /* HAVE_ICONV && !LOCAL_CHARSET */
+
 #ifndef MAX
 #	define MAX(a,b)	(((a) > (b)) ? (a) : (b))
 #endif /* !MAX */
@@ -784,18 +785,13 @@ enum resizer { cNo, cYes, cRedraw };
 #	define PATH_LEN	256
 #endif /* VMS || M_AMIGA */
 
-#if defined(M_OS2) || defined(M_UNIX)
+#if defined(M_UNIX)
 #	ifndef MAXPATHLEN
 #		define MAXPATHLEN	256
 #	endif /* !MAXPATHLEN */
 #	define LEN	1024
 #	define PATH_LEN	MAXPATHLEN
-#endif /* M_OS2 || M_UNIX */
-
-#if defined(WIN32)
-#	define LEN	1024
-#	define PATH_LEN	MAX_PATH
-#endif /* WIN32 */
+#endif /* M_UNIX */
 
 #define NEWSRC_LINE	8192
 
@@ -875,6 +871,10 @@ enum resizer { cNo, cYes, cRedraw };
 #define MIME_ENCODING_QP	2
 #define MIME_ENCODING_7BIT	3
 
+#ifdef CHARSET_CONVERSION			/* can/should do charset conversion via iconv() */
+#	define NUM_MIME_CHARSETS 27	/* # known 'outgoing' charsets */
+#endif /* CHARSET_CONVERSION */
+
 /*
  * Number of charset-traslation tables (iso2asci)
  */
@@ -947,20 +947,6 @@ enum resizer { cNo, cYes, cRedraw };
 /*
  * used by get_arrow_key()
  */
-#ifdef WIN32
-#	define KEYMAP_UNKNOWN		0
-#	define KEYMAP_UP		0xA6
-#	define KEYMAP_DOWN		0xA8
-#	define KEYMAP_LEFT		0xA5
-#	define KEYMAP_RIGHT		0xA7
-#	define KEYMAP_PAGE_UP		0xA1
-#	define KEYMAP_PAGE_DOWN	0xA2
-#	define KEYMAP_HOME		0xA4
-#	define KEYMAP_END		0xA3
-#	define KEYMAP_DEL		0
-#	define KEYMAP_INS		0
-#	define KEYMAP_MOUSE		0
-#else
 #	define KEYMAP_UNKNOWN		0
 #	define KEYMAP_UP		1
 #	define KEYMAP_DOWN		2
@@ -973,7 +959,6 @@ enum resizer { cNo, cYes, cRedraw };
 #	define KEYMAP_DEL		9
 #	define KEYMAP_INS		10
 #	define KEYMAP_MOUSE		11
-#endif /* WIN32 */
 
 
 /*
@@ -992,6 +977,7 @@ enum resizer { cNo, cYes, cRedraw };
 #define GROUP_LEVEL	2
 #define THREAD_LEVEL	3
 #define PAGE_LEVEL	4
+#define INFO_PAGER	5
 
 #define MINI_HELP_LINES		5
 
@@ -1011,15 +997,14 @@ enum resizer { cNo, cYes, cRedraw };
 
 /*
  * Threading strategies available
- * NB: The ordering is important in that threading methods that don't use
- *     references should be < THREAD_REFS
  */
 #define THREAD_NONE		0
 #define THREAD_SUBJ		1
 #define THREAD_REFS		2
 #define THREAD_BOTH		3
+#define THREAD_MULTI		4
 
-#define THREAD_MAX		THREAD_BOTH
+#define THREAD_MAX		THREAD_MULTI
 
 /*
  * Values for show_author
@@ -1040,15 +1025,23 @@ enum resizer { cNo, cYes, cRedraw };
  * used in art.c
  * sort types on arts[] array
  */
-#define SORT_BY_NOTHING		0
-#define SORT_BY_SUBJ_DESCEND	1
-#define SORT_BY_SUBJ_ASCEND	2
-#define SORT_BY_FROM_DESCEND	3
-#define SORT_BY_FROM_ASCEND	4
-#define SORT_BY_DATE_DESCEND	5
-#define SORT_BY_DATE_ASCEND	6
-#define SORT_BY_SCORE_DESCEND	7
-#define SORT_BY_SCORE_ASCEND	8
+#define SORT_ARTICLES_BY_NOTHING		0
+#define SORT_ARTICLES_BY_SUBJ_DESCEND	1
+#define SORT_ARTICLES_BY_SUBJ_ASCEND	2
+#define SORT_ARTICLES_BY_FROM_DESCEND	3
+#define SORT_ARTICLES_BY_FROM_ASCEND	4
+#define SORT_ARTICLES_BY_DATE_DESCEND	5
+#define SORT_ARTICLES_BY_DATE_ASCEND	6
+#define SORT_ARTICLES_BY_SCORE_DESCEND	7
+#define SORT_ARTICLES_BY_SCORE_ASCEND	8
+
+/*
+ * used in art.c
+ * sort types on base[] array
+ */
+#define SORT_THREADS_BY_NOTHING			0
+#define SORT_THREADS_BY_SCORE_DESCEND	1
+#define SORT_THREADS_BY_SCORE_ASCEND	2
 
 /*
  * Search keys
@@ -1089,10 +1082,6 @@ enum resizer { cNo, cYes, cRedraw };
 /*
  * used in post.c
  */
-#define HEADER_TO		0
-#define HEADER_SUBJECT		1
-#define HEADER_NEWSGROUPS	2
-
 #define POSTED_NONE		0			/* Article wasn't posted */
 #define POSTED_REDRAW		1			/* redraw needed in any case */
 #define POSTED_OK		2			/* posted normally */
@@ -1129,14 +1118,12 @@ enum resizer { cNo, cYes, cRedraw };
 /*
  * Assertion verifier
  */
-#if !defined(M_OS2) && !defined(WIN32)
-#	undef assert
-#	ifdef CPP_DOES_EXPAND
-#		define assert(p)	if(! (p)) asfail(__FILE__, __LINE__, #p); else (void)0;
-#	else
-#		define assert(p)	if(! (p)) asfail(__FILE__, __LINE__, "p"); else (void)0;
-#	endif /* CPP_DOES_EXPAND */
-#endif /*!M_OS2 && !WIN32*/
+#undef assert
+#ifdef CPP_DOES_EXPAND
+#	define assert(p)	if(! (p)) asfail(__FILE__, __LINE__, #p); else (void)0;
+#else
+#	define assert(p)	if(! (p)) asfail(__FILE__, __LINE__, "p"); else (void)0;
+#endif /* CPP_DOES_EXPAND */
 
 #define ESC	27
 
@@ -1383,9 +1370,6 @@ struct t_attribute
 	char *sigfile;				/* sig file if other than ~/.Sig */
 	char *organization;			/* organization name */
 	char *followup_to;			/* where posts should be redirected */
-#ifndef DISABLE_PRINTING
-	char *printer;				/* printer command & parameters */
-#endif /* !DISABLE_PRINTING */
 	char *quick_kill_scope;			/* quick filter kill scope */
 	char *quick_select_scope;		/* quick filter select scope */
 	char *mailing_list;			/* mail list email address */
@@ -1409,14 +1393,16 @@ struct t_attribute
 	unsigned batch_save:1;			/* 0=none, 1=save -S/mail -M */
 	unsigned delete_tmp_files:1;		/* 0=leave, 1=delete */
 	unsigned show_only_unread:1;		/* 0=all, 1=only unread */
-	unsigned thread_arts:2;			/* 0=unthread, 1=subject, 2=refs, 3=both */
+	unsigned thread_arts:3;			/* 0=unthread, 1=subject, 2=refs, 3=both, 4=multipart*/
 	unsigned show_author:4;			/* 0=none, 1=name, 2=addr, 3=both */
 	unsigned sort_art_type:4;		/* 0=none, 1=subj descend, 2=subj ascend,
 						   3=from descend, 4=from ascend,
 						   5=date descend, 6=date ascend,
 						   7=score descend, 8=score ascend */
+	unsigned sort_threads_type:2;	/* 0=none, 1=score descend, 2=score ascend */
 	unsigned int post_proc_type:2;		/* 0=none, 1=shar, 2=uudecode */
 	unsigned int x_comment_to:1;		/* insert X-Comment-To: in Followup */
+	unsigned int tex2iso_conv:1;		/* Convert TeX2ISO */
 };
 
 /*
@@ -1588,6 +1574,9 @@ struct t_art_stat
 	char art_mark;		/* mark to use for this thread - not used for groups */
 	int score;		/* maximum score */
 	time_t time;		/* latest time */
+	int multipart_total; /* 0=not multipart, >0 = number of articles in the multipart */
+	int multipart_have; /* number of articles we actually have found */
+	int multipart_compare_len; /* length of subject which contains non-specific multipart info */
 };
 
 
@@ -1716,15 +1705,9 @@ typedef struct {
 	typedef char *t_comptype;
 #endif /* HAVE_COMPTYPE_CHAR */
 
-#ifdef M_OS2
-#	define _CDECL	_cdecl
-#	define _FAR_	_far16
-#	define SEPDIR	'\\'
-#else
 #	define _CDECL
 #	define _FAR_
 #	define SEPDIR	'/'
-#endif /* M_OS2 */
 
 /*
  * mouse buttons for use in xterm
@@ -1734,9 +1717,9 @@ typedef struct {
 #define MOUSE_BUTTON_3		2
 
 #define TIN_EDITOR_FMT_OFF		"%E %F"
+#define NEWSGROUPS_FILE		"newsgroups"
 
 #ifdef M_AMIGA
-#	define NEWSGROUPS_FILE	"newsgroups"
 #	define REDIRECT_OUTPUT	"> NIL:"
 #	define REDIRECT_PGP_OUTPUT	"> NIL:"
 #	define ENV_VAR_GROUPS		"TIN_GROUPS"
@@ -1751,7 +1734,6 @@ typedef struct {
 extern void joinpath (char *result, const char *dir, const char *file);
 #endif /* M_AMIGA */
 #ifdef VMS
-#	define NEWSGROUPS_FILE	"newsgroups"
 #	define REDIRECT_OUTPUT	""
 #	define REDIRECT_PGP_OUTPUT	""
 #	define ENV_VAR_MAILER		"TIN_MAILER"
@@ -1767,43 +1749,15 @@ extern void joinpath (char *result, const char *dir, const char *file);
 extern void joindir (char *result, const char *dir, const char *file);
 #endif /* VMS */
 
-#ifdef M_OS2
-#	define NEWSGROUPS_FILE	"newsgroups"
-#	define REDIRECT_OUTPUT	"> NUL"
-#	define REDIRECT_PGP_OUTPUT	"> NUL"
-#	define ENV_VAR_GROUPS		"TIN_GROUPS"
-#	define ENV_VAR_MAILER		"TIN_MAIL"
-#	define ENV_VAR_POSTER		"TIN_POST"
-#	define ENV_VAR_SHELL		"COMSPEC"
-#	define TIN_EDITOR_FMT_ON	"%E %F"
-#	define METAMAIL_CMD		"%s -e -p -m \"tin\""
-extern void joinpath (char *result, char *dir, char *file);
-#endif /* M_OS2 */
-
-#ifdef WIN32
-#	define NEWSGROUPS_FILE	"newsgroups"
-#	define REDIRECT_OUTPUT	"> NUL"
-#	define REDIRECT_PGP_OUTPUT "> NUL"
-#	define ENV_VAR_GROUPS		"TIN_GROUPS"
-#	define ENV_VAR_MAILER		"TIN_MAIL"
-#	define ENV_VAR_POSTER		"TIN_POST"
-#	define ENV_VAR_SHELL		"COMSPEC"
-#	define TIN_EDITOR_FMT_ON	"%E +%N %F"
-#	define MAILER_FORMAT		"%M -t %T -f %U -s \"%S\" -F %F"
-#	define METAMAIL_CMD		"%s -e -p -m \"tin\""
-extern void joinpath (char *result, char *dir, char *file);
-#endif /* WIN32 */
-
 #ifdef M_UNIX
-#	define NEWSGROUPS_FILE		"newsgroups"
 #	define REDIRECT_OUTPUT		"> /dev/null 2>&1"
 #	define REDIRECT_PGP_OUTPUT		"> /dev/null"
 #	define ENV_VAR_MAILER		"MAILER"
 #	define ENV_VAR_SHELL		"SHELL"
 #	define TIN_EDITOR_FMT_ON		"%E +%N %F"
-#	define MAILER_FORMAT		"%M -t < %F"
+#	define MAILER_FORMAT		"%M -oi -t < %F"
 #	define METAMAIL_CMD		"%s -e -p -m \"tin\""
-#	define TMPDIR		"/tmp/"
+#	define TMPDIR	"/tmp/"
 #	ifdef	HAVE_KEY_PREFIX
 #		define KEY_PREFIX		0x8f: case 0x9b
 #	endif /* HAVE_KEY_PREFIX */
@@ -1811,9 +1765,6 @@ extern void joinpath (char *result, char *dir, char *file);
 
 /* fallback values */
 /* FIXME! */
-#ifndef NEWSGROUPS_FILE
-#	define NEWSGROUPS_FILE		""
-#endif /* !NEWSGROUPS_FILE */
 #ifndef REDIRECT_OUTPUT
 #	define REDIRECT_OUTPUT		""
 #endif /* !REDIRECT_OUTPUT */
@@ -1840,13 +1791,6 @@ extern void joinpath (char *result, char *dir, char *file);
 #endif /* !TMPDIR */
 
 #if !defined(S_ISDIR)
-#	if defined(WIN32)
-#		define S_ISDIR(m)	((m) & _S_IFDIR)
-#	else
-#		if defined(M_OS2)
-#			define S_ISDIR(m)	((m) & S_IF_DIR)
-#		endif /* M_OS2 */
-#	endif /* WIN32 */
 #	if defined(M_UNIX) || defined(VMS) || defined(M_AMIGA)
 #		define S_ISDIR(m)	(((m) & S_IFMT) == S_IFDIR)
 #	endif /* M_UNIX || VMS || M_AMIGA */
@@ -1923,11 +1867,11 @@ extern void joinpath (char *result, char *dir, char *file);
 
 #define SIZEOF(array)	((int)(sizeof array / sizeof array[0]))
 
-#define FreeIfNeeded(p)	if (p != (char *)0) free((char *)p)
-
 #if 0
+#	define FreeIfNeeded(p)	if (p != (char *)0) free((char *)p)
 #	define FreeAndNull(p)	if (p != (char *)0) { free((char *)p); p = (char *)0; }
 #else
+#	define FreeIfNeeded(p)	if (p != NULL) free((void *)p)
 #	define FreeAndNull(p)	if (p != NULL) { free((void *)p); p = NULL; }
 #endif /* 0 */
 
@@ -1954,15 +1898,17 @@ extern void joinpath (char *result, char *dir, char *file);
 #	endif /* __STDC__ */
 #endif /* !SIG_ARGS */
 
+#ifndef __LCLINT__ /* lclint doesn't like it */
 /* stifle complaints about not-a-prototype from gcc */
-#ifdef DECL_SIG_CONST
-#	undef	SIG_DFL
-#	define SIG_DFL	(void (*)(SIG_ARGS))0
-#	undef	SIG_IGN
-#	define SIG_IGN	(void (*)(SIG_ARGS))1
-#	undef	SIG_ERR
-#	define SIG_ERR	(void (*)(SIG_ARGS))-1
-#endif /* DECL_SIG_CONST */
+#	ifdef DECL_SIG_CONST
+#		undef	SIG_DFL
+#		define SIG_DFL	(void (*)(SIG_ARGS))0
+#		undef	SIG_IGN
+#		define SIG_IGN	(void (*)(SIG_ARGS))1
+#		undef	SIG_ERR
+#		define SIG_ERR	(void (*)(SIG_ARGS))-1
+#	endif /* DECL_SIG_CONST */
+#endif /* !__LCLINT__ */
 
 /*
  * tputs() function-param
@@ -2008,10 +1954,6 @@ typedef	OUTC_RETTYPE (*OutcPtr) (OUTC_ARGS);
 #	endif /* !PROTO_H */
 #endif /* !__CPROTO__ */
 
-#if defined(WIN32)
-#	include	"msmail.h"
-#endif /* WIN32 */
-
 /*
  * rfc1521/rfc1522 interface
  */
@@ -2032,17 +1974,6 @@ typedef void (*BodyPtr) (char *, FILE *, int);
 #	define DMALLOC_FUNC_CHECK
 #endif /* USE_DMALLOC */
 
-#if defined(WIN32) && defined(DEBUG) && defined(CHECKHEAP)
-#	undef malloc
-#	undef realloc
-#	undef calloc
-#	undef free
-#	define malloc(cb)		mymalloc(cb, __LINE__, __FILE__)
-#	define realloc(pv,cb)		myrealloc(pv, cb, __LINE__, __FILE__)
-#	define calloc(cbs,cbe)		mycalloc(cbs, cbe, __LINE__, __FILE__)
-#	define free(pv)		myfree(pv, __LINE__, __FILE__)
-#endif /* WIN32 && DEBUG && CHECKHEAP */
-
 #ifdef DOALLOC
 	extern char *doalloc (char *, size_t);
 	extern char *docalloc (size_t, size_t);
@@ -2070,7 +2001,6 @@ typedef void (*BodyPtr) (char *, FILE *, int);
 #	endif /* !ferror */
 #endif /* __DECC */
 
-/* FIXME - check also for tmpfile() and provide fallback functions */
 #ifdef HAVE_TEMPNAM
 #	define my_tempnam(a,b)	tempnam(a,b)
 #else
@@ -2078,13 +2008,16 @@ typedef void (*BodyPtr) (char *, FILE *, int);
 #		define my_tempnam(a,b)	tmpnam((char *)0)
 #	endif /* HAVE_TMPNAM */
 #endif /* HAVE_TEMPNAM */
-#ifdef HAVE_MKSTEMP
-#	define my_mktemp(a)	mkstemp(a)
-#else
-#	ifdef HAVE_MKTEMP
-#		define my_mktemp(a) open(mktemp(a), O_RDWR, (mode_t)(S_IRUSR|S_IWUSR))
-#	endif /* HAVE_MKTEMP */
-#endif /* HAVE_MKSTEMP */
+
+#ifndef my_tmpfile_only
+/*
+ * shortcut if we arn't interrested in the tmpfiles filename/location
+ * argument can't be a pointer and if argument is changed on return
+ * we must unlik the tmp-file ourself
+ */
+#	define my_tmpfile_only(a)  my_tmpfile(a, sizeof(a)-1, FALSE, (char *) 0)
+#endif /* !my_tmpfile_only */
+
 
 /* define some standard places to look for a tin.defaults file */
 /*
@@ -2093,9 +2026,9 @@ typedef void (*BodyPtr) (char *, FILE *, int);
  * no system assigns.
  */
 #ifdef M_AMIGA
-#define TIN_DEFAULTS_BUILTIN "S:tin.defaults","ENV:tin.defaults",NULL
+#	define TIN_DEFAULTS_BUILTIN "S:tin.defaults","ENV:tin.defaults",NULL
 #else
-#define TIN_DEFAULTS_BUILTIN "/etc/opt/tin","/etc/tin","/etc","/usr/local/lib/tin","/usr/local/lib","/usr/local/etc/tin","/usr/local/etc","/usr/lib/tin","/usr/lib",NULL
+#	define TIN_DEFAULTS_BUILTIN "/etc/opt/tin","/etc/tin","/etc","/usr/local/lib/tin","/usr/local/lib","/usr/local/etc/tin","/usr/local/etc","/usr/lib/tin","/usr/lib",NULL
 #endif /* M_AMIGA */
 #ifdef TIN_DEFAULTS_DIR
 #	define TIN_DEFAULTS TIN_DEFAULTS_DIR,TIN_DEFAULTS_BUILTIN
@@ -2182,4 +2115,34 @@ extern struct tm *localtime(time_t *);
 #	define MM_CHARSET "US-ASCII"
 #endif /* !MM_CHARSET */
 
+/* TODO: move up to the 'right' place */
+#ifdef HAVE_SYS_FILE_H
+#	include <sys/file.h>
+#endif /* HAVE_SYS_FILE_H */
+
+#if !defined(SEEK_SET)
+#	define SEEK_SET 0L
+#endif /* !SEEK_SET */
+
+#if !defined(EOF)
+#	define EOF -1
+#endif /* !EOF */
+
+/* various filenames used by tin */
+#ifndef VMS	/* TODO: is the VMS case needed? */
+#	define TIN_ARTICLE_NAME	".article"
+#	define TIN_CANCEL_NAME	".cancel"
+#	define TIN_LETTER_NAME	".letter"
+#else
+#	define TIN_ARTICLE_NAME	"article."
+#	define TIN_CANCEL_NAME	"cancel."
+#	define TIN_LETTER_NAME	"letter."
+#endif /* !VMS */
+
+/* THREAD_WEIGHT without THREAD_SUM doesn't make sense */
+#if defined(THREAD_WEIGHT) && !defined(THREAD_SUM)
+#	define THREAD_SUM
+#endif /* THREAD_WEIGHT && !THREAD_SUM */
+
+
 #endif /* !TIN_H */
