@@ -6,7 +6,7 @@
  *  Updated   : 1999-11-19
  *  Notes     :
  *
- * Copyright (c) 1992-2001 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1992-2002 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -78,14 +78,14 @@ static void subscribe_new_group (char *group, char *autosubscribe, char *autouns
 
 
 /*
- *  Get default array size for active[] from environment (AmigaDOS)
- *  or just return the standard default.
+ * Get default array size for active[] from environment (AmigaDOS)
+ * or just return the standard default.
  */
 int
 get_active_num (
 	void)
 {
-#ifdef ENV_VAR_GROUPS /* M_AMIGA && M_OS2 && WIN32 only */
+#ifdef ENV_VAR_GROUPS /* M_AMIGA only */
 	char *ptr;
 	int num;
 
@@ -130,6 +130,10 @@ resync_active_file (
 	write_newsrc ();
 	read_news_active_file ();
 
+#ifdef HAVE_MH_MAIL_HANDLING
+	read_mail_active_file ();
+#endif /* HAVE_MH_MAIL_HANDLING */
+
 	if (read_cmd_line_groups ())
 		command_line = TRUE;
 
@@ -161,7 +165,7 @@ active_add (
 	const char *moderated)
 {
 	/* name - pre-initialised when group is made */
-	ptr->aliasedto = ((moderated[0] == '=') ? my_strdup(moderated+1) : (char *) 0);
+	ptr->aliasedto = ((moderated[0] == '=') ? my_strdup(moderated + 1) : (char *) 0);
 	ptr->description = (char *) 0;
 	/* spool - see below */
 	ptr->moderated = moderated[0];
@@ -181,12 +185,7 @@ active_add (
 	ptr->glob_filter = &glob_filter;
 	set_default_bitmap (ptr);
 
-#ifdef WIN32				/* Paths are in form - x:\a\b\c */
-	if (strchr(moderated, '\\'))
-#else
-	if (moderated[0] == '/')
-#endif /* WIN32 */
-	{
+	if (moderated[0] == '/') {
 		ptr->type = GROUP_TYPE_SAVE;
 		ptr->spooldir = my_strdup(moderated);
 	} else {
@@ -445,7 +444,7 @@ read_newsrc_active_file (
 	fclose(fp);
 
 	/*
-	 *  Exit if active file wasn't read correctly or is empty
+	 * Exit if active file wasn't read correctly or is empty
 	 */
 	if (tin_errno || !num_active) {
 		error_message (_(txt_active_file_is_empty), (read_news_via_nntp ? _(txt_servers_active) : news_active_file));
@@ -530,7 +529,7 @@ read_active_file (
 	TIN_FCLOSE (fp);
 
 	/*
-	 *  Exit if active file wasn't read correctly or is empty
+	 * Exit if active file wasn't read correctly or is empty
 	 */
 	if (tin_errno || !num_active) {
 		error_message (_(txt_active_file_is_empty), (read_news_via_nntp ? _(txt_servers_active) : news_active_file));
@@ -680,7 +679,7 @@ check_for_any_new_groups (
 		sprintf (buf, "%s %lu", new_newnews_host, (unsigned long int) new_newnews_time);
 		load_newnews_info (buf);
 	}
-	my_fputc('\n',stdout);
+	my_fputc('\n', stdout);
 }
 
 
@@ -797,7 +796,7 @@ match_group_list (
 		group_list += group_len;
 		list_len -= group_len;
 	}
-	return (accept);
+	return accept;
 }
 
 
@@ -867,7 +866,7 @@ find_newnews_index (
 
 	for (i = 0; i < num_newnews; i++) {
 		if (STRCMPEQ(cur_newnews_host, newnews[i].host))
-			return(i);
+			return i;
 	}
 
 	return -1;
@@ -880,7 +879,7 @@ find_newnews_index (
  */
 char
 group_flag (
-	int ch)
+	char ch)
 {
 	switch (ch) {
 		case 'm':
@@ -986,8 +985,7 @@ append_group_line (
 
 	file_tmp = get_tmpfilename(active_file);
 
-	if (!backup_file (active_file, file_tmp))
-	{
+	if (!backup_file (active_file, file_tmp)) {
 		free (file_tmp);
 		return;
 	}
