@@ -6,7 +6,7 @@
  *  Updated   : 1995-04-19
  *  Notes     :
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -74,6 +74,13 @@ static long find_first_unread (struct t_group *group);
 static t_bool parse_headers (FILE *fp, struct t_article *h);
 static void print_expired_arts (int num_expired);
 static void thread_by_subject (void);
+static void thread_by_multipart (void);
+static int global_look_for_multipart_info (int aindex, MultiPartInfo* setme, char start, char stop, int *offset);
+static int global_get_multiparts (int aindex, MultiPartInfo **malloc_and_setme_info);
+#ifdef THREAD_SUM
+	static void sort_base (unsigned int sort_threads_type);
+	static int score_comp_base (t_comptype p1, t_comptype p2);
+#endif /* THREAD_SUM */
 
 
 /*
@@ -139,6 +146,11 @@ find_base (
 			base[grpmenu.max++] = i;
 		}
 	}
+#ifdef THREAD_SUM
+	/* sort base[] */
+	if (group->attribute && group->attribute->sort_threads_type > SORT_THREADS_BY_NOTHING)
+		sort_base (group->attribute->sort_threads_type);
+#endif /* THREAD_SUM */
 }
 
 
@@ -200,7 +212,7 @@ index_group (
 #endif /* DEBUG_NEWSRC */
 
 	min = grpmenu.max ? base[0] : group->xmin;
-	max = grpmenu.max ? base[grpmenu.max-1] : min - 1;
+	max = grpmenu.max ? base[grpmenu.max - 1] : min - 1;
 
 	if (tinrc.use_getart_limit) {
 		if (tinrc.getart_limit > 0) {
@@ -514,22 +526,226 @@ thread_by_subject (
 
 
 /*
- *  Go through the articles in arts[] and create threads. There are
- *  4 strategies currently defined :
+ * This was brought over from tags.c, however this version doesn't not
+ * opperate on base_index
+ *
+ * Parses a subject header of the type "multipart message subject (01/42)"
+ * into a MultiPartInfo struct, or fails if the message subject isn't in the
+ * right form.
+ *
+ * @return nonzero on success
+ */
+int
+global_get_multipart_info (
+	int aindex,
+	MultiPartInfo *setme)
+{
+	int i, j, offi, offj;
+	MultiPartInfo setmei, setmej;
+
+	i = global_look_for_multipart_info(aindex, &setmei, '[', ']', &offi);
+	j = global_look_for_multipart_info(aindex, &setmej, '(', ')', &offj);
+
+	/* Ok i hits first */
+	if (offi > offj) {
+		*setme = setmei;
+		return i;
+	}
+
+	/* Its j or they are both the same (which must be zero!) so we don't care */
+	*setme = setmej;
+	return j;
+}
+
+
+/*
+ *	Again this was taken from tags.c, but works on global indicies into arts
+ *	rather then on base_index.
+ */
+static int
+global_look_for_multipart_info (
+	int aindex,
+	MultiPartInfo* setme,
+	char start,
+	char stop,
+	int *offset)
+{
+	char *subj = (char *) 0;
+	char *pch = (char *) 0;
+	MultiPartInfo tmp;
+
+	*offset = 0;
+
+	/* entry assertions */
+	assert (0 <= aindex && aindex < top_art && "invalid index");
+	assert (setme != NULL && "setme must not be NULL");
+
+	/* parse the message */
+	subj = arts[aindex].subject;
+	pch = strrchr (subj, start);
+	if (!pch || !isdigit((int) pch[1]))
+		return 0;
+	tmp.base_index = aindex; /* This could be confusing because we are actually storing the index into arts, not the base_index. */
+	tmp.subject_compare_len = pch - subj;
+	tmp.part_number = (int) strtol(pch + 1, &pch, 10);
+	if (*pch != '/' && *pch != '|')
+		return 0;
+	if (!isdigit((int) pch[1]))
+		return 0;
+	tmp.total = (int) strtol (pch + 1, &pch, 10);
+	if (*pch != stop)
+		return 0;
+	tmp.subject = subj;
+	*setme = tmp;
+	*offset = pch - subj;
+	return 1;
+}
+
+
+/*
+ * Taken from tags.c but changed to use indicies into arts[] instead of
+ * base_index. Changed so that even when we don't have all the parts we
+ * return a valid array.
+ *
+ * Tries to find all the parts to the multipart message pointed to by
+ * base_index.
+ *
+ * @return on success, the number of parts found. On failure, zero if not a
+ * multipart or the negative value of the first missing part.
+ * @param base_index index pointing to one of the messages in a multipart
+ * message.
+ * @param malloc_and_setme_info on success, set to a malloced array the
+ * parts found.
+ */
+static int
+global_get_multiparts (
+	int aindex,
+	MultiPartInfo **malloc_and_setme_info)
+{
+	int i = 0;
+	MultiPartInfo tmp, tmp2;
+	MultiPartInfo *info = 0;
+
+	/* entry assertions */
+	assert (0 <= aindex && aindex < top_art && "Invalid index");
+	assert (malloc_and_setme_info != NULL && "malloc_and_setme_info must not be NULL");
+
+	/* make sure this is a multipart message... */
+	if (!global_get_multipart_info(aindex, &tmp) || tmp.total < 1)
+		return 0;
+
+	/* make a temporary buffer to hold the multipart info... */
+	info = my_malloc (sizeof(MultiPartInfo) * tmp.total);
+
+	/* zero out part-number for the repost check below */
+	for (i = 0; i < tmp.total; ++i) {
+		info[i].total = tmp.total; /* Added this for thread_by_multipart */
+		info[i].part_number = -1;
+	}
+
+	/* try to find all the multiparts... */
+	for (i = 0; i < top_art; ++i) {
+		int part_index = 0;
+
+		if (strncmp (arts[i].subject, tmp.subject, tmp.subject_compare_len))
+			continue;
+		if (!global_get_multipart_info (i, &tmp2))
+			continue;
+
+		/* 'test (1/5)' is not the same as 'test (1/15)' */
+		if (tmp.total != tmp2.total)
+			continue;
+
+		part_index = tmp2.part_number - 1;
+
+		/* skip the "blah (00/102)" info messages... */
+		if (part_index < 0)
+			continue;
+
+		/* skip insane "blah (103/102) subjects... */
+		if (part_index >= tmp.total)
+			continue;
+
+		/* repost check: do we already have this part? */
+		if (info[part_index].part_number != -1) {
+			assert (info[part_index].part_number == tmp2.part_number && "bookkeeping error");
+			continue;
+		}
+
+		/* we have a match, hooray! */
+		info[part_index] = tmp2;
+	}
+
+	/* see if we got them all. */
+	for (i = 0; i < tmp.total; ++i) {
+		if (info[i].part_number != i + 1) {
+			*malloc_and_setme_info = info;
+			return -(i + 1); /* missing part #(i+1) */
+		}
+	}
+
+	/* looks like a success .. */
+	*malloc_and_setme_info = info;
+	return tmp.total;
+}
+
+
+/*
+ *	The algorithm uses the tag multipart searches to thread articles together.
+ */
+static void
+thread_by_multipart (
+	void)
+{
+	int i, j, threadNum, parent;
+	MultiPartInfo *minfo = NULL;
+
+	for (i = 0; i < top_art; i++) {
+
+		if (arts[i].thread != ART_NORMAL || IGNORE_ART(i) || arts[i].inthread || !global_get_multiparts(i, &minfo))
+			continue;
+
+		threadNum = -1;
+		parent = -1;
+		for (j = minfo[0].total-1; j >= 0; j--) {
+			if (minfo[j].part_number != -1) {
+				if (threadNum != -1)
+					arts[minfo[j].base_index].thread = threadNum;
+				arts[minfo[j].base_index].inthread = TRUE;
+				threadNum = minfo[j].base_index;
+				parent = j;
+			}
+		}
+
+		/* Thread parent doesn't register as being in a thread */
+		if ((parent != -1) && (minfo[parent].part_number != -1))
+			arts[minfo[parent].base_index].inthread = FALSE;
+	}
+	free(minfo);
+}
+
+
+/*
+ * Go through the articles in arts[] and create threads. There are
+ * 5 strategies currently defined :
  *
  *	THREAD_NONE		No threading
  *	THREAD_SUBJ		Threads are created using like Subject lines
  *	THREAD_REFS		Threads are created using the References headers
  *	THREAD_BOTH		Threads created using References and then Subject
+ *	THREAD_MULTI	Threads created using Subject to search for Multiparts
  *
- *  Apart from THREAD_NONE, .thread and .inthread are used, the
- *  first article in a thread should have .inthread set to FALSE, the
- *  rest TRUE.  Only do unexprired articles we haven't visited yet
- *  (arts[].thread == -1 ART_NORMAL).
- *
- *  The rethread parameter is a misnomer. Its only effect (if set) is
- *  to delete all threading information, not to rethread
+ * Apart from THREAD_NONE, .thread and .inthread are used, the
+ * first article in a thread should have .inthread set to FALSE, the
+ * rest TRUE.  Only do unexprired articles we haven't visited yet
+ * (arts[].thread == -1 ART_NORMAL).
  *
+ * The rethread parameter is a misnomer. Its only effect (if set) is
+ * to delete all threading information, not to rethread
+ */
+/* TODO: rewrite that user can easly combine different 'threading'
+ *       methods, i.e:
+ *       - thread_by_multipart() + collate_subjects()
  */
 void
 make_threads (
@@ -562,7 +778,7 @@ make_threads (
 	clear_art_ptrs();
 
 	/*
-	 *  The threading pointers need to be reset if re-threading
+	 * The threading pointers need to be reset if re-threading
 	 *	If using ref threading, revector the links back to the articles
 	 */
 	if (rethread || (group->attribute && group->attribute->thread_arts)) {
@@ -608,10 +824,13 @@ make_threads (
 			collate_subjects();
 			break;
 
+		case THREAD_MULTI:
+			thread_by_multipart();
+			break;
+
 		default: /* not reached */
 			break;
 	}
-
 	find_base (group);
 }
 
@@ -621,23 +840,23 @@ sort_arts (
 	unsigned int sort_art_type)
 {
 	switch (sort_art_type) {
-		case SORT_BY_NOTHING:		/* don't sort at all */
+		case SORT_ARTICLES_BY_NOTHING:		/* don't sort at all */
 			SortBy(artnum_comp);
 			break;
-		case SORT_BY_SUBJ_DESCEND:
-		case SORT_BY_SUBJ_ASCEND:
+		case SORT_ARTICLES_BY_SUBJ_DESCEND:
+		case SORT_ARTICLES_BY_SUBJ_ASCEND:
 			SortBy(subj_comp);
 			break;
-		case SORT_BY_FROM_DESCEND:
-		case SORT_BY_FROM_ASCEND:
+		case SORT_ARTICLES_BY_FROM_DESCEND:
+		case SORT_ARTICLES_BY_FROM_ASCEND:
 			SortBy(from_comp);
 			break;
-		case SORT_BY_DATE_DESCEND:
-		case SORT_BY_DATE_ASCEND:
+		case SORT_ARTICLES_BY_DATE_DESCEND:
+		case SORT_ARTICLES_BY_DATE_ASCEND:
 			SortBy(date_comp);
 			break;
-		case SORT_BY_SCORE_DESCEND:
-		case SORT_BY_SCORE_ASCEND:
+		case SORT_ARTICLES_BY_SCORE_DESCEND:
+		case SORT_ARTICLES_BY_SCORE_ASCEND:
 			SortBy(score_comp);
 			break;
 		default:
@@ -646,13 +865,28 @@ sort_arts (
 }
 
 
+#ifdef THREAD_SUM
+static void
+sort_base (
+	unsigned int sort_threads_type)
+{
+	switch (sort_threads_type) {
+		case SORT_THREADS_BY_SCORE_DESCEND:
+		case SORT_THREADS_BY_SCORE_ASCEND:
+			qsort (base, (size_t)grpmenu.max, sizeof(long), score_comp_base);
+			break;
+	}
+}
+#endif /* THREAD_SUM */
+
+
 /*
  * TODO
- * Seems to be only called when reading local spool and no overview files exist
- * Code reads (max_lineno) lines of article, presumably to catch headers like
- * Archive-name: which are not normally included in XOVER or even the normal
- * block of headers. How this is supposed to be useful when 99% of the time we'll
- * have overview data I don't know...
+ * Seems to be only called when reading local spool and no overview files
+ * exist. Code reads (max_lineno) lines of article, presumably to catch
+ * headers like Archive-name: which are not normally included in XOVER or
+ * even the normal block of headers. How this is supposed to be useful when
+ * 99% of the time we'll have overview data I don't know...
  */
 static t_bool
 parse_headers (
@@ -676,16 +910,17 @@ parse_headers (
 		if (lineno++ > max_lineno || h->archive)
 			break;
 
+		unfold_header (ptr);
 		switch (toupper((unsigned char)*ptr)) {
 			case 'A':	/* Archive-name:  optional */
-				if ((hdr = parse_header (ptr+1, "rchive-name", FALSE))) {
+				if ((hdr = parse_header (ptr + 1, "rchive-name", FALSE))) {
 					/* TODO - what if header of form news/group/name/part01 ? */
 					if ((s = strrchr (hdr, '/')) != NULL) {
-						if (STRNCASECMPEQ(s+1, "part", 4)) {
-							h->part = my_strdup (s+5);
+						if (STRNCASECMPEQ(s + 1, "part", 4)) {
+							h->part = my_strdup (s + 5);
 							strtok(h->part, "\n");
-						} else if (STRNCASECMPEQ(s+1, "patch", 5)) {
-							h->patch = my_strdup (s+6);
+						} else if (STRNCASECMPEQ(s + 1, "patch", 5)) {
+							h->patch = my_strdup (s + 6);
 							strtok(h->patch, "\n");
 						} else
 							continue;
@@ -697,15 +932,13 @@ parse_headers (
 				break;
 			case 'D':	/* Date:  mandatory */
 				if (!h->date) {
-					if ((hdr = parse_header (ptr+1, "ate", FALSE)))
+					if ((hdr = parse_header (ptr + 1, "ate", FALSE)))
 						h->date = parsedate (hdr, (struct _TIMEINFO *) 0);
 				}
 				break;
 			case 'F':	/* From:  mandatory */
-			case 'T':	/* To:    mandatory (mailbox) */
 				if (!got_from) {
-					if ((hdr = parse_header (ptr+1, "rom", FALSE)) ||
-									(hdr = parse_header (ptr+1, "o", FALSE))) {
+					if ((hdr = parse_header (ptr + 1, "rom", FALSE))) {
 						h->gnksa_code = parse_from (hdr, art_from_addr, art_full_name);
 						h->from = hash_str (art_from_addr);
 						if (*art_full_name)
@@ -716,7 +949,7 @@ parse_headers (
 				break;
 			case 'L':	/* Lines:  optional */
 				if (!got_lines) {
-					if ((hdr = parse_header (ptr+1, "ines", FALSE))) {
+					if ((hdr = parse_header (ptr + 1, "ines", FALSE))) {
 						h->line_count = atoi (hdr);
 						got_lines = TRUE;
 					}
@@ -724,19 +957,19 @@ parse_headers (
 				break;
 			case 'M':	/* Message-ID:  mandatory */
 				if (!h->msgid) {
-					if ((hdr = parse_header (ptr+1, "essage-ID", FALSE)))
+					if ((hdr = parse_header (ptr + 1, "essage-ID", FALSE)))
 						h->msgid = my_strdup (hdr);
 				}
 				break;
 			case 'R':	/* References:  optional */
 				if (!h->refs) {
-					if ((hdr = parse_header (ptr+1, "eferences", FALSE)))
+					if ((hdr = parse_header (ptr + 1, "eferences", FALSE)))
 						h->refs = my_strdup (hdr);
 				}
 
 				/* Received:  If found it's probably a mail article */
 				if (!got_received) {
-					if ((hdr = parse_header (ptr+1, "eceived", FALSE))) {
+					if ((hdr = parse_header (ptr + 1, "eceived", FALSE))) {
 						max_lineno = 50;		/* Get extra lines for some reason */
 						got_received = TRUE;
 					}
@@ -744,7 +977,7 @@ parse_headers (
 				break;
 			case 'S':	/* Subject:  mandatory */
 				if (!h->subject) {
-					if ((hdr = parse_header (ptr+1, "ubject", FALSE))) {
+					if ((hdr = parse_header (ptr + 1, "ubject", FALSE))) {
 						s = eat_re (eat_tab(rfc1522_decode(hdr)), FALSE);
 						h->subject = hash_str (s);
 					}
@@ -752,7 +985,7 @@ parse_headers (
 				break;
 			case 'X':	/* Xref:  optional */
 				if (!h->xref) {
-					if ((hdr = parse_header (ptr+1, "ref", FALSE)))
+					if ((hdr = parse_header (ptr + 1, "ref", FALSE)))
 						h->xref = my_strdup (hdr);
 				}
 				break;
@@ -866,7 +1099,7 @@ read_nov_file (
 		p = buf;
 
 		/*
-		 * read the article number
+		 * read the article number, guaranteed to be the first field
 		 */
 		artnum = atol (p);
 
@@ -900,7 +1133,12 @@ read_nov_file (
 			p = q + 1;
 
 		/*
-		 * READ subject
+		 * TODO: rewrite the code below as the field order is not fixed,
+		 *       but defined in "LIST OVERVIEW.FMT"
+		 */
+
+		/*
+		 * read Subject
 		 */
 		if ((q = strchr (p, '\t')) == (char *) 0) {
 #ifdef DEBUG
@@ -915,7 +1153,7 @@ read_nov_file (
 		p = q + 1;
 
 		/*
-		 * READ author
+		 * read From
 		 */
 		if ((q = strchr (p, '\t')) == (char *) 0) {
 #ifdef DEBUG
@@ -934,7 +1172,7 @@ read_nov_file (
 
 		p = q + 1;
 		/*
-		 * READ article date
+		 * read Date
 		 */
 		if ((q = strchr (p, '\t')) == (char *) 0) {
 #ifdef DEBUG
@@ -949,7 +1187,7 @@ read_nov_file (
 		p = q + 1;
 
 		/*
-		 * READ article message id
+		 * read Message-ID
 		 */
 		q = strchr (p, '\t');
 		if (q == (char *) 0 || p == q) {	/* Empty msgid's */
@@ -962,12 +1200,16 @@ read_nov_file (
 			*q = '\0';
 
 		/* TODO is no mesg-id allowed in rfc ? */
+		/*
+		 * draft-ietf-nntpext-base-13.txt, section 9.2.1.1
+		 * comes up with "<0>" - should we use it instead of '\0'?
+		 */
 		arts[top_art].msgid = ((*p) ? (my_strdup (p)) : ((char *) '\0'));
 
 		p = q + 1;
 
 		/*
-		 * READ article references
+		 * read References
 		 */
 		if ((q = strchr (p, '\t')) == (char *) 0) {
 #ifdef DEBUG
@@ -983,7 +1225,7 @@ read_nov_file (
 		p = q + 1;
 
 		/*
-		 * SKIP article bytes
+		 * skip Bytes
 		 */
 		if ((q = strchr (p, '\t')) == (char *) 0) {
 #ifdef DEBUG
@@ -997,7 +1239,7 @@ read_nov_file (
 		p = (q == (char *) 0 ? (char *) 0 : q + 1);
 
 		/*
-		 * READ article lines
+		 * read Lines
 		 */
 		if (p != (char *) 0) {
 			if ((q = strchr (p, '\t')) != (char *) 0)
@@ -1010,7 +1252,7 @@ read_nov_file (
 		}
 
 		/*
-		 * READ article xrefs
+		 * read Xref
 		 */
 		if (p != (char *) 0 && xref_supported) {
 			if ((q = strstr (p, "Xref: ")) == (char *) 0)
@@ -1052,7 +1294,7 @@ read_nov_file (
 	TIN_FCLOSE (fp);
 
 	if (tin_errno)
-		return(-1);
+		return -1;
 
 	return top_art;
 }
@@ -1117,7 +1359,7 @@ write_nov_file (
 	if (fp == (FILE *) 0)
 		error_message (_(txt_cannot_write_index), nov_file);
 	else {
-		if (group->attribute && group->attribute->sort_art_type != SORT_BY_NOTHING)
+		if (group->attribute && group->attribute->sort_art_type != SORT_ARTICLES_BY_NOTHING)
 			SortBy(artnum_comp);
 
 		if (!overview_index_filename)
@@ -1129,7 +1371,8 @@ write_nov_file (
 			if (article->thread != ART_EXPIRED && article->artnum >= group->xmin) {
 				fprintf (fp, "%ld\t%s\t%s\t%s\t%s\t%s\t%d\t%d",
 					article->artnum,
-					article->subject,
+					tinrc.post_8bit_header ? article->subject : rfc1522_encode(article->subject, FALSE),
+/*					rfc1522_encode(article->subject, FALSE), */
 					print_from (article),
 					print_date (article->date),
 					(article->msgid ? article->msgid : ""),
@@ -1354,7 +1597,7 @@ subj_comp (
 	/*
 	 * return result of strcmp (reversed for descending)
 	 */
-	return (CURR_GROUP.attribute->sort_art_type == SORT_BY_SUBJ_ASCEND
+	return (CURR_GROUP.attribute->sort_art_type == SORT_ARTICLES_BY_SUBJ_ASCEND
 			? (retval = strcasecmp (s1->subject, s2->subject))
 				? retval : ((s1->date - s2->date) > 0) ? 1 : -1
 			: (retval = strcasecmp (s2->subject, s1->subject))
@@ -1374,7 +1617,7 @@ from_comp (
 	/*
 	 * return result of strcmp (reversed for descending)
 	 */
-	return (CURR_GROUP.attribute->sort_art_type == SORT_BY_FROM_ASCEND
+	return (CURR_GROUP.attribute->sort_art_type == SORT_ARTICLES_BY_FROM_ASCEND
 			? (retval = strcasecmp (s1->from, s2->from))
 				? retval : ((s1->date - s2->date) > 0) ? 1 : -1
 			: (retval = strcasecmp (s2->from, s1->from))
@@ -1399,7 +1642,7 @@ date_comp (
 	const struct t_article *s1 = (const struct t_article *) p1;
 	const struct t_article *s2 = (const struct t_article *) p2;
 
-	if (CURR_GROUP.attribute->sort_art_type == SORT_BY_DATE_ASCEND) {
+	if (CURR_GROUP.attribute->sort_art_type == SORT_ARTICLES_BY_DATE_ASCEND) {
 		/*
 		 * s1->date less than s2->date
 		 */
@@ -1441,7 +1684,7 @@ score_comp (
 	const struct t_article *s1 = (const struct t_article *) p1;
 	const struct t_article *s2 = (const struct t_article *) p2;
 
-	if (CURR_GROUP.attribute->sort_art_type == SORT_BY_SCORE_ASCEND) {
+	if (CURR_GROUP.attribute->sort_art_type == SORT_ARTICLES_BY_SCORE_ASCEND) {
 		if (s1->score < s2->score)
 			return -1;
 
@@ -1459,31 +1702,60 @@ score_comp (
 }
 
 
+#ifdef THREAD_SUM
+/*
+ * Compares the total score of two threads. Used for sorting base[].
+ */
+static int
+score_comp_base (
+	t_comptype p1,
+	t_comptype p2)
+{
+	int a = get_score_of_thread (*(const long *)p1);
+	int b = get_score_of_thread (*(const long *)p2);
+
+	if (CURR_GROUP.attribute->sort_threads_type == SORT_THREADS_BY_SCORE_ASCEND) {
+		if (a > b)
+			return 1;
+		if (a < b)
+			return -1;
+	}
+	else {
+		if (a < b)
+			return 1;
+		if (a > b)
+			return -1;
+	}
+	return 0;
+}
+#endif /* THREAD_SUM */
+
+
 void
 set_article (
 	struct t_article *art)
 {
-	art->subject	= (char *) 0;
-	art->from	= (char *) 0;
-	art->name	= (char *) 0;
-	art->date	= (time_t) 0;
-	art->xref	= (char *) 0;
-	art->msgid	= (char *) 0;
-	art->refs	= (char *) 0;
-	art->refptr	= (struct t_msgid *) 0;
-	art->line_count	= -1;
-	art->archive	= (char *) 0;
-	art->part	= (char *) 0;
-	art->patch	= (char *) 0;
-	art->thread	= ART_EXPIRED;
-	art->status	= ART_UNREAD;
-	art->inthread	= FALSE;
-	art->killed	= FALSE;
-	art->tagged	= FALSE;
-	art->selected	= FALSE;
-	art->zombie	= FALSE;
-	art->delete_it	= FALSE;
-	art->inrange	= FALSE;
+	art->subject = (char *) 0;
+	art->from = (char *) 0;
+	art->name = (char *) 0;
+	art->date = (time_t) 0;
+	art->xref = (char *) 0;
+	art->msgid = (char *) 0;
+	art->refs = (char *) 0;
+	art->refptr = (struct t_msgid *) 0;
+	art->line_count = -1;
+	art->archive = (char *) 0;
+	art->part = (char *) 0;
+	art->patch = (char *) 0;
+	art->thread = ART_EXPIRED;
+	art->status = ART_UNREAD;
+	art->inthread = FALSE;
+	art->killed = FALSE;
+	art->tagged = FALSE;
+	art->selected = FALSE;
+	art->zombie = FALSE;
+	art->delete_it = FALSE;
+	art->inrange = FALSE;
 }
 
 
@@ -1500,7 +1772,7 @@ valid_artnum (
 	register int cur = 1;
 
 	while ((dctop /= 2))
-		cur *= 2;
+		cur <<= 1;
 
 	range = cur / 2;
 	cur--;
@@ -1575,7 +1847,7 @@ print_from (
 	*from = '\0';
 
 	if (article->name != (char *) 0)
-		snprintf (from, sizeof(from)-1, "%s <%s>", rfc1522_encode(article->name, FALSE), article->from);
+		snprintf (from, sizeof(from) - 1, "%s <%s>", tinrc.post_8bit_header ? article->name : rfc1522_encode(article->name, FALSE), article->from);
 	else
 		STRCPY (from, article->from);
 
