@@ -6,7 +6,7 @@
  *  Updated   : 1997-12-20
  *  Notes     : Group attribute routines
  *
- * Copyright (c) 1993-2001 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1993-2002 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -58,9 +58,7 @@
 #define ATTRIB_FROM			4
 #define ATTRIB_SIGFILE			5
 #define ATTRIB_FOLLOWUP_TO		6
-#ifndef DISABLE_PRINTING
-#	define ATTRIB_PRINTER		7
-#endif /* !DISABLE_PRINTING */
+/* 7 unused */
 #define ATTRIB_AUTO_SELECT		8
 #define ATTRIB_AUTO_SAVE		9
 #define ATTRIB_BATCH_SAVE		10
@@ -88,6 +86,8 @@
 #ifdef HAVE_ISPELL
 #	define ATTRIB_ISPELL		32
 #endif /* HAVE_ISPELL */
+#define ATTRIB_SORT_THREADS_TYPE	33
+#define ATTRIB_TEX2ISO_CONV		34
 
 /*
  * Local prototypes
@@ -122,9 +122,6 @@ set_default_attributes (
 	attributes->sigfile = tinrc.sigfile;
 	attributes->organization = (*default_organization ? default_organization : (char *) 0);
 	attributes->followup_to = (char *) 0;
-#ifndef DISABLE_PRINTING
-	attributes->printer = tinrc.printer;
-#endif /* !DISABLE_PRINTING */
 	attributes->mailing_list = (char *) 0;
 	attributes->x_headers = (char *) 0;
 	attributes->x_body = (char *) 0;
@@ -145,6 +142,7 @@ set_default_attributes (
 	attributes->show_only_unread = tinrc.show_only_unread_arts;
 	attributes->thread_arts = tinrc.thread_articles;
 	attributes->sort_art_type = tinrc.sort_article_type;
+	attributes->sort_threads_type = tinrc.sort_threads_type;
 	attributes->show_author = tinrc.show_author;
 	attributes->auto_save = tinrc.auto_save;
 	attributes->auto_select = FALSE;
@@ -152,11 +150,12 @@ set_default_attributes (
 	attributes->delete_tmp_files = FALSE;
 	attributes->post_proc_type = tinrc.post_process;
 	attributes->x_comment_to = FALSE;
+	attributes->tex2iso_conv = tinrc.tex2iso_conv;
 }
 
 
 /*
- *  Load global & local attributes into active[].attribute
+ * Load global & local attributes into active[].attribute
  */
 
 #define MATCH_BOOLEAN(pattern, type) \
@@ -245,9 +244,6 @@ read_attributes_file (
 				break;
 
 			case 'p':
-#ifndef DISABLE_PRINTING
-				MATCH_STRING ("printer=", ATTRIB_PRINTER);
-#endif /* !DISABLE_PRINTING */
 				MATCH_INTEGER ("post_proc_type=",
 					ATTRIB_POST_PROC_TYPE,
 					POST_PROC_UUDECODE);
@@ -282,13 +278,17 @@ read_attributes_file (
 				MATCH_BOOLEAN ("show_only_unread=", ATTRIB_SHOW_ONLY_UNREAD);
 				MATCH_INTEGER ("sort_art_type=",
 					ATTRIB_SORT_ART_TYPE,
-					SORT_BY_SCORE_ASCEND);
+					SORT_ARTICLES_BY_SCORE_ASCEND);
+				MATCH_INTEGER ("sort_threads_type=",
+					ATTRIB_SORT_THREADS_TYPE,
+					SORT_THREADS_BY_SCORE_DESCEND);
 				MATCH_INTEGER ("show_author=",
 					ATTRIB_SHOW_AUTHOR,
 					SHOW_FROM_BOTH);
 				break;
 
 			case 't':
+				MATCH_BOOLEAN ("tex2iso_conv=", ATTRIB_TEX2ISO_CONV);
 				MATCH_INTEGER ("thread_arts=", ATTRIB_THREAD_ARTS, THREAD_MAX);
 				break;
 
@@ -391,11 +391,6 @@ do_set_attrib (
 		case ATTRIB_FOLLOWUP_TO:
 			group->attribute->followup_to = my_strdup (data);
 			break;
-#ifndef DISABLE_PRINTING
-		case ATTRIB_PRINTER:
-			group->attribute->printer = my_strdup (data);
-			break;
-#endif /* !DISABLE_PRINTING */
 		case ATTRIB_AUTO_SELECT:
 			group->attribute->auto_select = *data;
 			break;
@@ -420,6 +415,9 @@ do_set_attrib (
 		case ATTRIB_SORT_ART_TYPE:
 			group->attribute->sort_art_type = *data;
 			break;
+		case ATTRIB_SORT_THREADS_TYPE:
+			group->attribute->sort_threads_type = *data;
+			break;
 		case ATTRIB_POST_PROC_TYPE:
 			group->attribute->post_proc_type = *data;
 			break;
@@ -470,6 +468,9 @@ do_set_attrib (
 			group->attribute->ispell = my_strdup (data);
 			break;
 #endif /* HAVE_ISPELL */
+		case ATTRIB_TEX2ISO_CONV:
+			group->attribute->tex2iso_conv = *data;
+			break;
 		default:
 			break;
 	}
@@ -478,7 +479,7 @@ do_set_attrib (
 
 
 /*
- *  Save the group attributes from active[].attribute to ~/.tin/attributes
+ * Save the group attributes from active[].attribute to ~/.tin/attributes
  */
 void
 write_attributes_file (
@@ -506,18 +507,18 @@ write_attributes_file (
 	if (!cmd_line && !batch_mode)
 		wait_message (0, _(txt_writing_attributes_file));
 
-	/* FIXME - move strings to lang.c */
+	/*
+	 * FIXME - move strings to lang.c
+	 * TODO: sort in a usefull order
+	 */
 	fprintf (fp, _("# Group attributes file for the TIN newsreader\n#\n"));
-	fprintf (fp, _("#  scope=STRING (ie. alt.sources, alt.*,!alt.bin* etc..) [mandatory]\n"));
+	fprintf (fp, _("#  scope=STRING (ie. alt.*,!alt.bin*) [mandatory]\n"));
 	fprintf (fp, _("#  maildir=STRING (ie. ~/Mail)\n"));
 	fprintf (fp, _("#  savedir=STRING (ie. ~user/News)\n"));
 	fprintf (fp, _("#  savefile=STRING (ie. =linux)\n"));
 	fprintf (fp, _("#  sigfile=STRING (ie. $var/sig)\n"));
 	fprintf (fp, _("#  organization=STRING (if beginning with '/' read from file)\n"));
 	fprintf (fp, _("#  followup_to=STRING\n"));
-#ifndef DISABLE_PRINTING
-	fprintf (fp, _("#  printer=STRING\n"));
-#endif /* !DISABLE_PRINTING */
 	fprintf (fp, _("#  mailing_list=STRING (ie. majordomo@list.org)\n"));
 	fprintf (fp, _("#  x_headers=STRING (ie. ~/.tin/extra-headers)\n"));
 	fprintf (fp, _("#  x_body=STRING (ie. ~/.tin/extra-body-text)\n"));
@@ -525,7 +526,7 @@ write_attributes_file (
 	fprintf (fp, _("#  news_quote_format=STRING\n"));
 	fprintf (fp, _("#  quote_chars=STRING (%%s, %%S for initials)\n"));
 #ifdef HAVE_ISPELL
-	fprintf (fp, _("#  ispell = STRING\n"));
+	fprintf (fp, _("#  ispell=STRING\n"));
 #endif /* HAVE_ISPELL */
 	fprintf (fp, _("#  auto_select=ON/OFF\n"));
 	fprintf (fp, _("#  auto_save=ON/OFF\n"));
@@ -533,7 +534,7 @@ write_attributes_file (
 	fprintf (fp, _("#  delete_tmp_files=ON/OFF\n"));
 	fprintf (fp, _("#  show_only_unread=ON/OFF\n"));
 	fprintf (fp, _("#  thread_arts=NUM\n"));
-	fprintf (fp, _("#    0=none, 1=subj, 2=refs, 3=both\n"));
+	fprintf (fp, _("#    0=none, 1=subj, 2=refs, 3=both, 4=multipart\n"));
 	fprintf (fp, _("#  show_author=NUM\n"));
 	fprintf (fp, _("#    0=none, 1=name, 2=addr, 3=both\n"));
 	fprintf (fp, _("#  sort_art_type=NUM\n"));
@@ -541,6 +542,8 @@ write_attributes_file (
 	fprintf (fp, _("#    3=from descend, 4=from ascend,\n"));
 	fprintf (fp, _("#    5=date descend, 6=date ascend\n"));
 	fprintf (fp, _("#    7=score descend, 8=score ascend\n"));
+	fprintf (fp, _("#  sort_threads_type=NUM\n"));
+	fprintf (fp, _("#    0=none, 1=score descend, 2=score ascend\n"));
 	fprintf (fp, _("#  post_proc_type=NUM\n"));
 	fprintf (fp, _("#    0=none, 1=unshar, 2=uudecode\n"));
 	fprintf (fp, _("#  quick_kill_scope=STRING (ie. talk.*)\n"));
@@ -558,6 +561,7 @@ write_attributes_file (
 	fprintf (fp, _("#    2=from (case sensitive) 3=from (ignore case)\n"));
 	fprintf (fp, _("#    4=msgid 5=lines\n"));
 	fprintf (fp, _("#  x_comment_to=ON/OFF\n"));
+	fprintf (fp, _("#  tex2iso_conv=ON/OFF\n"));
 	fprintf (fp, _("#\n# Note that it is best to put general (global scoping)\n"));
 	fprintf (fp, _("# entries first followed by group specific entries.\n#\n"));
 	fprintf (fp, _("############################################################################\n\n"));
@@ -594,9 +598,6 @@ write_attributes_file (
 		fprintf (fp, "sigfile=%s\n", group->attribute->sigfile);
 		fprintf (fp, "organization=%s\n", group->attribute->organization);
 		fprintf (fp, "followup_to=%s\n", group->attribute->followup_to);
-#	ifndef DISABLE_PRINTING
-		fprintf (fp, "printer=%s\n", group->attribute->printer);
-#	endif /* !DISABLE_PRINTING */
 		fprintf (fp, "mailing_list=%s\n", group->attribute->mailing_list);
 		fprintf (fp, "x_headers=%s\n", group->attribute->x_headers);
 		fprintf (fp, "x_body=%s\n", group->attribute->x_body);
@@ -619,6 +620,7 @@ write_attributes_file (
 		fprintf (fp, "delete_tmp_files=%s\n",
 			print_boolean (group->attribute->delete_tmp_files));
 		fprintf (fp, "sort_art_type=%d\n", group->attribute->sort_art_type);
+		fprintf (fp, "sort_threads_type=%d\n", group->attribute->sort_threads_type);
 		fprintf (fp, "show_author=%d\n", group->attribute->show_author);
 		fprintf (fp, "post_proc_type=%d\n", group->attribute->post_proc_type);
 		fprintf (fp, "quick_kill_scope=%s\n",
@@ -637,6 +639,8 @@ write_attributes_file (
 		fprintf (fp, "quick_select_header=%d\n\n", group->attribute->quick_select_header);
 		fprintf (fp, "x_comment_to=%s\n",
 			print_boolean (group->attribute->x_comment_to));
+		fprintf (fp, "tex2iso_conv=%s\n",
+			print_boolean (group->attribute->tex2iso_conv));
 	}
 #endif /* 0 */
 
@@ -702,9 +706,6 @@ dump_attributes (
 		fprintf (stderr, "sigfile=%s\n", group->attribute->sigfile);
 		fprintf (stderr, "organization=%s\n", group->attribute->organization);
 		fprintf (stderr, "followup_to=%s\n", group->attribute->followup_to);
-#		ifndef DISABLE_PRINTING
-		fprintf (stderr, "printer=%s\n", group->attribute->printer);
-#		endif /* !DISABLE_PRINTING */
 		fprintf (stderr, "mailing_list=%s\n", group->attribute->mailing_list);
 		fprintf (stderr, "x_headers=%s\n", group->attribute->x_headers);
 		fprintf (stderr, "x_body=%s\n", group->attribute->x_body);
@@ -727,6 +728,7 @@ dump_attributes (
 		fprintf (stderr, "delete_tmp_files=%s\n",
 			print_boolean (group->attribute->delete_tmp_files));
 		fprintf (stderr, "sort_art_type=%d\n", group->attribute->sort_art_type);
+		fprintf (stderr, "sort_threads_type=%d\n", group->attribute->sort_threads_type);
 		fprintf (stderr, "show_author=%d\n", group->attribute->show_author);
 		fprintf (stderr, "post_proc_type=%d\n", group->attribute->post_proc_type);
 		fprintf (stderr, "quick_kill_scope=%s\n",
@@ -745,6 +747,8 @@ dump_attributes (
 		fprintf (stderr, "quick_select_header=%d\n", group->attribute->quick_select_header);
 		fprintf (stderr, "x_comment_to=%s\n\n",
 			print_boolean (group->attribute->x_comment_to));
+		fprintf (stderr, "tex2iso_conv=%s\n\n",
+			print_boolean (group->attribute->tex2iso_conv));
 	}
 }
 #	endif /* DEBUG */
