@@ -7,7 +7,7 @@
  *  Notes     : Routines to authenticate to a news server via NNTP.
  *              DON'T USE get_respcode() THROUGHOUT THIS CODE.
  *
- * Copyright (c) 1997-2001 Dirk Nimmich <nimmich@uni-muenster.de>
+ * Copyright (c) 1997-2002 Dirk Nimmich <nimmich@uni-muenster.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -92,17 +92,17 @@ authinfo_generic (
 		(void) sscanf (authcmd, "%*d.%*d.%d", &cookiefd);
 
 	if (cookiefd == -1) {
-		char tempfile[BUFSIZ];
-
-		snprintf (tempfile, sizeof(tempfile) - 1, "%stin_AXXXXXX", TMPDIR);
-		if ((cookiefd = (my_mktemp (tempfile))) == -1) {
-			error_message (_(txt_cannot_create_uniq_name));
+		char tempfile[PATH_LEN];
+		if ((cookiefd = my_tmpfile_only(tempfile)) == -1) {
 #	ifdef DEBUG
 			debug_nntp ("authorization", txt_cannot_create_uniq_name);
 #	endif /* DEBUG */
 			return FALSE;
-		} else
-			(void) unlink (tempfile);
+		} else {
+			close(cookiefd);
+			if (tempfile[0] != '\0')
+				(void) unlink (tempfile);
+		}
 	}
 
 	strcpy (tmpbuf, "AUTHINFO GENERIC ");
