@@ -3,10 +3,10 @@
  *  Module    : config.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 1999-11-04
+ *  Updated   : 2001-11-10
  *  Notes     : Configuration file routines
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -63,11 +63,12 @@ static void show_config_page (void);
 	static t_bool match_color (char *line, const char *pat, int *dst, int max);
 #endif /* HAVE_COLOR */
 
-enum state { IGNORE, CHECK, UPGRADE };
+enum rc_state { IGNORE, CHECK, UPGRADE };
 
 #define DASH_TO_SPACE(mark)	(mark == '_' ? ' ' : mark)
 #define SPACE_TO_DASH(mark)	(mark == ' ' ? '_' : mark)
 
+
 /* FIXME: see doc/TODO and comments in the code */
 /*
  * If we don't find a matching version tag line at the top of the rc file,
@@ -82,24 +83,23 @@ check_upgrade (
 	char foo[60];
 	char bar[120]; /* should be enough */
 
-	my_strncpy(foo, txt_tinrc_header, sizeof(foo)-1);
-	snprintf(bar, sizeof(bar)-1, foo, PRODUCT, TINRC_VERSION);
+	my_strncpy(foo, txt_tinrc_header, sizeof(foo) - 1);
+	snprintf(bar, sizeof(bar) - 1, foo, PRODUCT, TINRC_VERSION);
 
 	if (strncmp(buf, bar, MIN(strlen(bar),strlen(buf))) == 0)
-		return(IGNORE);
+		return IGNORE;
 	else {
 		error_message (_(txt_warn_update), VERSION);
 		error_message (_(txt_return_key));
 		ReadCh();
-		return(UPGRADE);
+		return UPGRADE;
 	}
 }
 
+
 /*
  *  read local & global configuration defaults
  */
-
-
 t_bool
 read_config_file (
 	char *file,
@@ -131,7 +131,7 @@ read_config_file (
 			if (match_boolean (buf, "advertising=", &tinrc.advertising))
 				break;
 
-			if (match_boolean (buf, "auto_save=", &tinrc.auto_save))
+			if (match_boolean (buf, "alternative_handling=", &tinrc.alternative_handling))
 				break;
 
 			if (match_string (buf, "art_marked_deleted=", buf, sizeof (buf))) {
@@ -184,10 +184,10 @@ read_config_file (
 				break;
 #endif /* HAVE_METAMAIL */
 
-			if (match_boolean (buf, "auto_cc=", &tinrc.auto_cc))
+			if (match_boolean (buf, "auto_bcc=", &tinrc.auto_bcc))
 				break;
 
-			if (match_boolean (buf, "auto_bcc=", &tinrc.auto_bcc))
+			if (match_boolean (buf, "auto_cc=", &tinrc.auto_cc))
 				break;
 
 			if (match_boolean (buf, "auto_list_thread=", &tinrc.auto_list_thread))
@@ -196,7 +196,7 @@ read_config_file (
 			if (match_boolean (buf, "auto_reconnect=", &tinrc.auto_reconnect))
 				break;
 
-			if (match_boolean (buf, "alternative_handling=", &tinrc.alternative_handling))
+			if (match_boolean (buf, "auto_save=", &tinrc.auto_save))
 				break;
 
 			break;
@@ -449,6 +449,11 @@ read_config_file (
 			if (match_boolean (buf, "inverse_okay=", &tinrc.inverse_okay))
 				break;
 
+#if defined(NNTP_ABLE) || defined(NNTP_ONLY)
+			if (match_string (buf, "inews_prog=", tinrc.inews_prog, sizeof (tinrc.inews_prog)))
+				break;
+#endif /* NNTP_ABLE || NNTP_ONLY */
+
 			break;
 
 		case 'k':
@@ -488,6 +493,14 @@ read_config_file (
 			if (match_string (buf, "mm_charset=", tinrc.mm_charset, sizeof (tinrc.mm_charset)))
 				break;
 
+#ifdef CHARSET_CONVERSION
+			if (match_string (buf, "mm_local_charset=", tinrc.mm_local_charset, sizeof (tinrc.mm_local_charset)))
+				break;
+
+			if (match_list (buf, "mm_network_charset=", txt_mime_charsets, NUM_MIME_CHARSETS, &tinrc.mm_network_charset))
+				break;
+#endif /* CHARSET_CONVERSION */
+
 			if (match_boolean (buf, "mark_saved_read=", &tinrc.mark_saved_read))
 				break;
 
@@ -621,7 +634,10 @@ read_config_file (
 			if (match_boolean (buf, "start_editor_offset=", &tinrc.start_editor_offset))
 				break;
 
-			if (match_integer (buf, "sort_article_type=", &tinrc.sort_article_type, SORT_BY_SCORE_ASCEND))
+			if (match_integer (buf, "sort_article_type=", &tinrc.sort_article_type, SORT_ARTICLES_BY_SCORE_ASCEND))
+				break;
+
+			if (match_integer (buf, "sort_threads_type=", &tinrc.sort_threads_type, SORT_THREADS_BY_SCORE_ASCEND))
 				break;
 
 			if (match_boolean (buf, "show_last_line_prev_page=", &tinrc.show_last_line_prev_page))
@@ -648,9 +664,6 @@ read_config_file (
 			if (match_boolean (buf, "strip_newsrc=", &tinrc.strip_newsrc))
 				break;
 
-			if (match_boolean (buf, "show_xcommentto=", &tinrc.show_xcommentto))
-				break;
-
 			/* Regexp used to strip "Re: "s and similar */
 			if (match_string (buf, "strip_re_regex=", tinrc.strip_re_regex, sizeof (tinrc.strip_re_regex)))
 				break;
@@ -677,6 +690,9 @@ read_config_file (
 			if (match_boolean (buf, "tab_goto_next_unread=", &tinrc.tab_goto_next_unread))
 				break;
 
+			if (match_boolean (buf, "tex2iso_conv=", &tinrc.tex2iso_conv))
+				break;
+
 			if (match_boolean (buf, "thread_catchup_on_exit=", &tinrc.thread_catchup_on_exit))
 				break;
 
@@ -686,11 +702,6 @@ read_config_file (
 			if (match_boolean (buf, "unlink_article=", &tinrc.unlink_article))
 				break;
 
-#if defined(NNTP_ABLE) || defined(NNTP_ONLY)
-			if (match_boolean (buf, "use_builtin_inews=", &tinrc.use_builtin_inews))
-				break;
-#endif /* NNTP_ABLE || NNTP_ONLY */
-
 			if (match_boolean (buf, "use_getart_limit=", &tinrc.use_getart_limit))
 				break;
 
@@ -762,12 +773,17 @@ read_config_file (
 	if (!(tinrc.draw_arrow || tinrc.inverse_okay))
 		tinrc.draw_arrow = TRUE;
 
+#ifdef CHARSET_CONVERSION
+	if (!*tinrc.mm_local_charset)
+		strcpy(tinrc.mm_local_charset, tinrc.mm_charset);
+#endif /* CHARSET_CONVERSION */
+
 	return TRUE;
 }
 
 
 /*
- *  write config defaults to ~/.tin/tinrc
+ * write config defaults to file
  */
 void
 write_config_file (
@@ -785,7 +801,6 @@ write_config_file (
 
 	if ((fp = fopen (file_tmp, "w")) == (FILE *) 0) {
 		error_message (_(txt_filesystem_full_backup), CONFIG_FILE);
-		/* free memory for tmp-filename */
 		free (file_tmp);
 		return;
 	}
@@ -898,15 +913,15 @@ write_config_file (
 	fprintf (fp, _(txt_sort_article_type.tinrc));
 	fprintf (fp, "sort_article_type=%d\n\n", tinrc.sort_article_type);
 
+	fprintf (fp, _(txt_sort_threads_type.tinrc));
+	fprintf (fp, "sort_threads_type=%d\n\n", tinrc.sort_threads_type);
+
 	fprintf (fp, _(txt_maildir.tinrc));
 	fprintf (fp, "default_maildir=%s\n\n", tinrc.maildir);
 
 	fprintf (fp, _(txt_save_to_mmdf_mailbox.tinrc));
 	fprintf (fp, "save_to_mmdf_mailbox=%s\n\n", print_boolean (tinrc.save_to_mmdf_mailbox));
 
-	fprintf (fp, _(txt_show_xcommentto.tinrc));
-	fprintf (fp, "show_xcommentto=%s\n\n", print_boolean(tinrc.show_xcommentto));
-
 #ifndef DISABLE_PRINTING
 	fprintf (fp, _(txt_print_header.tinrc));
 	fprintf (fp, "print_header=%s\n\n", print_boolean (tinrc.print_header));
@@ -995,6 +1010,9 @@ write_config_file (
 	fprintf (fp, _(txt_show_signatures.tinrc));
 	fprintf (fp, "show_signatures=%s\n\n", print_boolean(tinrc.show_signatures));
 
+	fprintf (fp, _(txt_tex2iso_conv.tinrc));
+	fprintf (fp, "tex2iso_conv=%s\n\n", print_boolean (tinrc.tex2iso_conv));
+
 	fprintf (fp, _(txt_hide_uue.tinrc));
 	fprintf (fp, "hide_uue=%s\n\n", print_boolean(tinrc.hide_uue));
 
@@ -1040,8 +1058,8 @@ write_config_file (
 	fprintf (fp, "force_screen_redraw=%s\n\n", print_boolean (tinrc.force_screen_redraw));
 
 #if defined(NNTP_ABLE) || defined(NNTP_ONLY)
-	fprintf (fp, _(txt_use_builtin_inews.tinrc));
-	fprintf (fp, "use_builtin_inews=%s\n\n", print_boolean (tinrc.use_builtin_inews));
+	fprintf (fp, _(txt_inews_prog.tinrc));
+	fprintf (fp, "inews_prog=%s\n\n", tinrc.inews_prog);
 #endif /* NNTP_ABLE || NNTP_ONLY */
 
 	fprintf (fp, _(txt_auto_list_thread.tinrc));
@@ -1151,6 +1169,14 @@ write_config_file (
 	fprintf (fp, _(txt_mm_charset.tinrc));
 	fprintf (fp, "mm_charset=%s\n\n", tinrc.mm_charset);
 
+#ifdef CHARSET_CONVERSION
+	fprintf (fp, _(txt_mm_local_charset.tinrc));
+	fprintf (fp, "mm_local_charset=%s\n\n", tinrc.mm_local_charset);
+
+	fprintf (fp, _(txt_mm_network_charset.tinrc));
+	fprintf (fp, "mm_network_charset=%s\n\n", txt_mime_charsets[tinrc.mm_network_charset]);
+#endif /* CHARSET_CONVERSION */
+
 /* Not on Option Menu */
 #ifdef LOCAL_CHARSET
 	fprintf (fp, _(txt_tinrc_local_charset));
@@ -1229,13 +1255,15 @@ write_config_file (
 	fprintf (fp, "default_shell_command=%s\n\n", tinrc.default_shell_command);
 
 	fprintf (fp, _(txt_tinrc_newnews));
-	for (i = 0; i < num_newnews; i++) {
+	{
 		char timestring[LEN];
 
-		timestring[0] = '\0';
-		my_strftime(timestring, LEN-1, "%Y-%m-%d %H:%M:%S UTC", gmtime(&(newnews[i].time)));
-		fprintf (fp, "newnews=%s %lu (%s)\n", newnews[i].host,
-								(unsigned long int) newnews[i].time, timestring);
+		for (i = 0; i < num_newnews; i++) {
+			timestring[0] = '\0';
+			my_strftime(timestring, LEN - 1, "%Y-%m-%d %H:%M:%S UTC", gmtime(&(newnews[i].time)));
+			fprintf (fp, "newnews=%s %lu (%s)\n", newnews[i].host,
+									(unsigned long int) newnews[i].time, timestring);
+		}
 	}
 
 	if (ferror (fp) || fclose (fp))
@@ -1244,18 +1272,16 @@ write_config_file (
 		rename_file (file_tmp, file);
 		chmod (file, (mode_t)(S_IRUSR|S_IWUSR));
 	}
-	/* free memory for tmp-filename */
 	free (file_tmp);
 }
 
-#define option_lines_per_page (cLINES - INDEX_TOP - 3)
 
 static int first_option_on_screen;
 static int actual_top_option = 0;
 
+#define option_lines_per_page (cLINES - INDEX_TOP - 3)
 #define TopOfPage(option) option_lines_per_page \
 			* ((option) / option_lines_per_page)
-
 #define OptionInPage(option)	((option) - first_option_on_screen)
 #define OptionIndex(option)	(OptionInPage(option) % option_lines_per_page)
 
@@ -1268,28 +1294,85 @@ option_row (
 }
 
 
+static int
+get_option_num(
+	int act_option)
+{
+	int result = 0;
+
+	if (option_table[act_option].var_type != OPT_TITLE) {
+		while (act_option >= 0) {
+			if (option_table[act_option--].var_type != OPT_TITLE)
+				++result;
+		}
+	}
+	return result;
+}
+
+
+static int
+set_option_num(
+	int option)
+{
+	int result = 0;
+
+	while (option >= 0 && result < LAST_OPT) {
+		while (result < LAST_OPT && option_table[result].var_type == OPT_TITLE)
+			result++;
+		if (result < LAST_OPT) {
+			if (--option < 0)
+				break;
+			++result;
+		}
+	}
+	return result;
+}
+
+
+char *
+fmt_option_prompt (
+	char *dst,
+	int len,
+	t_bool editing,
+	int option)
+{
+	int num = get_option_num(option);
+
+	if (num != 0) {
+		snprintf(dst, len, "%s %3d. %s ",
+			editing ? "->" : "  ", num,
+			_(option_table[option].txt->opt));
+	} else
+		snprintf(dst, len, "  %s", _(option_table[option].txt->opt));
+
+	return dst;
+}
+
+
 static void
 print_any_option (
 	int act_option)
 {
 	constext **list;
-	char temp[LEN], *ptr;
+	char temp[LEN], *ptr, *ptr2;
 	int row = option_row(act_option);
 	int len = sizeof(temp) - 1;
 
 	MoveCursor (row, 0);
 
-	snprintf(temp, len, "   %3d. %s ", act_option+1, option_table[act_option].txt->opt);
-	ptr = temp + strlen(temp);
+	ptr = fmt_option_prompt(temp, len, FALSE, act_option);
+	ptr += strlen(temp);
 	len -= strlen(temp);
 
 	switch (option_table[act_option].var_type) {
 		case OPT_ON_OFF:
-			snprintf(ptr, len, "%s ", print_boolean(*OPT_ON_OFF_list[option_table[act_option].var_index]));
+			snprintf(ptr, len, "%s", print_boolean(*OPT_ON_OFF_list[option_table[act_option].var_index]));
 			break;
 		case OPT_LIST:
 			list = option_table[act_option].opt_list;
-			snprintf(ptr, len, "%s", list[*(option_table[act_option].variable) + ((strcasecmp(list[0], _(txt_default)) == 0) ? 1 : 0)]);
+			ptr2 = my_strdup (list[*(option_table[act_option].variable) + ((strcasecmp(_(list[0]), _(txt_default)) == 0) ? 1 : 0)]);
+			snprintf(ptr, len, "%s", _(ptr2));
+			free(ptr2);
 			break;
 		case OPT_STRING:
 			snprintf(ptr, len, "%s", OPT_STRING_list[option_table[act_option].var_index]);
@@ -1305,7 +1388,13 @@ print_any_option (
 	}
 #ifdef USE_CURSES
 	my_printf("%.*s", cCOLS, temp);
-	clrtoeol();
+	{
+		int y, x;
+
+		getyx(stdscr, y, x);
+		if (x < cCOLS)
+			clrtoeol();
+	}
 #else
 	my_printf("%.*s", cCOLS - 1, temp);
 	/* draw_arrow_mark() will read this back for repainting */
@@ -1336,24 +1425,20 @@ static void
 RepaintOption (
 	int option)
 {
-	if (OptionOnPage(option)) {
+	if (OptionOnPage(option))
 		print_any_option (option);
-	}
 }
 
 
 #ifdef USE_CURSES
-static void DoScroll (
+static void
+DoScroll (
 	int jump)
 {
-#	if 0
-	int y, x;
-	getyx(stdscr, y, x);
-#	endif /* 0 */
 	MoveCursor(INDEX_TOP, 0);
 	SetScrollRegion(INDEX_TOP, INDEX_TOP + option_lines_per_page - 1);
 	ScrollScreen(jump);
-	SetScrollRegion(0, LINES-1);
+	SetScrollRegion(0, LINES - 1);
 }
 #endif /* USE_CURSES */
 
@@ -1364,10 +1449,10 @@ highlight_option (
 {
 	if (!OptionOnPage(option)) {
 #ifdef USE_CURSES
-		if (option > 0 && OptionOnPage(option-1)) {
+		if (option > 0 && OptionOnPage(option - 1)) {
 			DoScroll(1);
 			first_option_on_screen++;
-		} else if (option < LAST_OPT && OptionOnPage(option+1)) {
+		} else if (option < LAST_OPT && OptionOnPage(option + 1)) {
 			DoScroll(-1);
 			first_option_on_screen--;
 		} else
@@ -1424,8 +1509,7 @@ refresh_config_page (
 		ClearScreen ();
 	}
 
-	if ((first_option_on_screen != actual_top_option) || force_redraw)
-	{
+	if ((first_option_on_screen != actual_top_option) || force_redraw) {
 		show_config_page ();
 		actual_top_option = first_option_on_screen;
 	}
@@ -1459,7 +1543,9 @@ change_config_file (
 	int mime_encoding = MIME_ENCODING_7BIT;
 	t_bool change_option = FALSE;
 	t_bool original_on_off_value;
-
+#ifdef CHARSET_CONVERSION
+	int mime_charset = 0;
+#endif /* CHARSET_CONVERSION */
 
 	actual_top_option = -1;
 	option = 0;
@@ -1467,9 +1553,7 @@ change_config_file (
 	ClearScreen ();
 	set_xclick_off ();
 	forever {
-
 		highlight_option (option);
-
 		stow_cursor();
 		ch = ReadCh ();
 
@@ -1478,13 +1562,11 @@ change_config_file (
 		 */
 		switch (ch) {
 
-#ifndef WIN32
 			case ESC:	/* common arrow keys */
 #	ifdef HAVE_KEY_PREFIX
 			case KEY_PREFIX:
 #	endif /* HAVE_KEY_PREFIX */
 				switch (get_arrow_key (ch)) {
-#endif /* !WIN32 */
 					case KEYMAP_UP:
 						ch = map_to_local (iKeyUp, &menukeymap.config_change);
 						break;
@@ -1508,12 +1590,11 @@ change_config_file (
 					case KEYMAP_PAGE_DOWN:
 						ch = map_to_local (iKeyPageDown, &menukeymap.config_change);
 						break;
-#ifndef WIN32
 					default:
 						break;
 				} /* switch (get_arrow_key ()) */
 				break;
-#endif /* !WIN32 */
+
 			default:
 				break;
 		}	/* switch (ch) */
@@ -1597,6 +1678,7 @@ change_config_file (
 				unhighlight_option (option);
 				old_option = option;
 				option = prompt_num (ch, _(txt_enter_option_num)) - 1;
+				option = set_option_num (option);
 				if (option < 0 || option > LAST_OPT) {
 					option = old_option;
 					break;
@@ -1745,11 +1827,9 @@ change_config_file (
 						 * case OPT_STRIP_BLANKS:
 						 * case OPT_TAB_AFTER_X_SELECTION:
 						 * case OPT_TAB_GOTO_NEXT_UNREAD:
+						 * case OPT_TEX2ISO_CONV:
 						 * case OPT_THREAD_CATCHUP_ON_EXIT:
 						 * case OPT_UNLINK_ARTICLE:
-#if defined(NNTP_ABLE) || defined(NNTP_ONLY)
-						 * case OPT_USE_BUILTIN_INEWS:
-#endif
 						 * case OPT_USE_MAILREADER_I:
 						 * case OPT_USE_MOUSE:
 #ifdef HAVE_KEYPAD
@@ -1776,7 +1856,7 @@ change_config_file (
 								OPT_ARG_COLUMN,
 								*(option_table[option].variable), /* post_process */
 								option_table[option].txt->help,
-								option_table[option].txt->opt,
+								_(option_table[option].txt->opt),
 								option_table[option].opt_list,
 								option_table[option].opt_count
 								);
@@ -1798,6 +1878,18 @@ change_config_file (
 							clear_message ();
 							break;
 
+						case OPT_SORT_THREADS_TYPE:
+							/*
+							 * If the sorting strategy of threads has changed, fix things
+							 * so that resorting will occur
+							 */
+							if (tinrc.sort_threads_type != original_list_value && group != (struct t_group *) 0) {
+								group->attribute->sort_threads_type = tinrc.sort_threads_type;
+								make_threads (group, TRUE);
+							}
+							clear_message ();
+							break;
+
 						case OPT_POST_PROCESS:
 							proc_ch_default = POST_PROC_TYPE (tinrc.post_process);
 							break;
@@ -1849,8 +1941,15 @@ change_config_file (
 				case OPT_STRING:
 					switch (option) {
 						case OPT_EDITOR_FORMAT:
+#if defined(NNTP_ABLE) || defined(NNTP_ONLY)
+						case OPT_INEWS_PROG:
+#endif /* NNTP_ABLE || NNTP_ONLY */
 						case OPT_MAILER_FORMAT:
+#ifndef CHARSET_CONVERSION
 						case OPT_MM_CHARSET:
+#else
+						case OPT_MM_LOCAL_CHARSET:
+#endif /* CHARSET_CONVERSION */
 						case OPT_MAIL_QUOTE_FORMAT:
 						case OPT_NEWS_QUOTE_FORMAT:
 						case OPT_QUOTE_CHARS:
@@ -1895,6 +1994,21 @@ change_config_file (
 							joinpath (posted_msgs_file, tinrc.maildir, *tinrc.keep_posted_articles_file ? tinrc.keep_posted_articles_file : POSTED_FILE);
 							break;
 
+#ifdef CHARSET_CONVERSION
+						case OPT_MM_NETWORK_CHARSET:
+							mime_charset = *(option_table[option].variable);
+							mime_charset = prompt_list (option_row(option),
+										OPT_ARG_COLUMN,
+										mime_charset,
+										option_table[option].txt->help,
+										_(option_table[option].txt->opt),
+										option_table[option].opt_list,
+										option_table[option].opt_count
+										);
+							*(option_table[option].variable) = mime_charset;
+							break;
+#endif /* CHARSET_CONVERSION */
+
 						case OPT_MAIL_MIME_ENCODING:
 						case OPT_POST_MIME_ENCODING:
 							mime_encoding = *(option_table[option].variable);
@@ -1902,7 +2016,7 @@ change_config_file (
 										OPT_ARG_COLUMN,
 										mime_encoding,
 										option_table[option].txt->help,
-										option_table[option].txt->opt,
+										_(option_table[option].txt->opt),
 										option_table[option].opt_list,
 										option_table[option].opt_count
 										);
@@ -2015,7 +2129,7 @@ change_config_file (
 					break;
 			} /* switch (option_table[option].var_type) */
 			change_option = FALSE;
-			show_menu_help (_(txt_select_config_file_option));
+			show_menu_help (txt_select_config_file_option);
 			RepaintOption (option);
 		} /* if (change_option) */
 	} /* forever */
@@ -2025,7 +2139,7 @@ change_config_file (
 
 
 /*
- *  expand ~/News to /usr/username/News and print to screen
+ * expand ~/News to /usr/username/News and print to screen
  */
 static void
 expand_rel_abs_pathname (
@@ -2039,7 +2153,7 @@ expand_rel_abs_pathname (
 		if (strlen (str) == 1)
 			strcpy (str, homedir);
 		else {
-			joinpath (buf, homedir, str+2);
+			joinpath (buf, homedir, str + 2);
 			strcpy (str, buf);
 		}
 	}
@@ -2052,15 +2166,15 @@ expand_rel_abs_pathname (
 
 
 /*
- *  show_menu_help
+ * show_menu_help
  */
 void
 show_menu_help (
 	const char *help_message)
 {
-	MoveCursor (cLINES-2, 0);
+	MoveCursor (cLINES - 2, 0);
 	CleartoEOLN ();
-	center_line (cLINES-2, FALSE, help_message);
+	center_line (cLINES - 2, FALSE, _(help_message));
 }
 
 
@@ -2093,7 +2207,7 @@ match_color (
 
 	if (STRNCMPEQ(line, pat, patlen)) {
 		t_bool found = FALSE;
-		for (n = 0; n < MAX_COLOR+1; n++) {
+		for (n = 0; n < MAX_COLOR + 1; n++) {
 			if (!strcasecmp(&line[patlen], txt_colors[n])) {
 				found = TRUE;
 				*dst = n;
@@ -2163,7 +2277,9 @@ match_long (
 }
 
 
-/* If the 'pat' keyword matches, lookup & return an index into the table */
+/*
+ * If the 'pat' keyword matches, lookup & return an index into the table
+ */
 t_bool
 match_list (
 	char *line,
@@ -2212,7 +2328,7 @@ match_string (
 }
 
 
-/* same as match_string() but without safeguard */
+/* like mach_string() but looks for 100% exact matches */
 static t_bool
 match_item (
 	char *line,
@@ -2221,15 +2337,20 @@ match_item (
 	size_t dstlen)
 {
 	char *ptr;
+	char *nline = my_strdup(line);
 	size_t patlen = strlen (pat);
 
-	if (STRNCMPEQ(line, pat, patlen)) {
-		strncpy (dst, &line[patlen], dstlen);
+	nline[strlen(nline) -1] = '\0'; /* remove tailing \n */
+
+	if (STRCMPEQ(nline, pat)) {
+		strncpy (dst, &nline[patlen], dstlen);
 		if ((ptr = strrchr (dst, '\n')) != (char *) 0)
 			*ptr = '\0';
 
+		free (nline);
 		return TRUE;
 	}
+	free (nline);
 	return FALSE;
 }
 
@@ -2243,7 +2364,7 @@ print_boolean (
 
 
 /*
- *  convert underlines to spaces in a string
+ * convert underlines to spaces in a string
  */
 void
 quote_dash_to_space (
@@ -2259,7 +2380,7 @@ quote_dash_to_space (
 
 
 /*
- *  convert spaces to underlines in a string
+ * convert spaces to underlines in a string
  */
 char *
 quote_space_to_dash (
@@ -2299,12 +2420,11 @@ show_config_page (
 	if (first_option_on_screen + option_lines_per_page > LAST_OPT)
 		lines_to_print = LAST_OPT + 1 - first_option_on_screen;
 
-	for (i = 0; i < lines_to_print;i++) {
+	for (i = 0; i < lines_to_print; i++)
 		print_any_option (first_option_on_screen + i);
-	}
 	CleartoEOS ();
 
-	show_menu_help (_(txt_select_config_file_option));
+	show_menu_help (txt_select_config_file_option);
 	my_flush();
 	stow_cursor();
 }
@@ -2315,8 +2435,8 @@ show_config_page (
  *
  * Takes a 1d string and turns it into a 2d array of strings.
  *
- * Watch out for the frees! You must free(*argv) and then free(argv)! NOTHING
- *   ELSE!! Do _NOT_ free the individual args of argv.
+ * Watch out for the frees! You must free(*argv) and then free(argv)!
+ * NOTHING ELSE! Do _NOT_ free the individual args of argv.
  */
 char **
 ulBuildArgv (
@@ -2361,7 +2481,7 @@ ulBuildArgv (
 				tmp++;
 			}
 			i++;
-			new_argv = (char **) realloc (new_argv, ((i+1) * sizeof (char *)));
+			new_argv = (char **) realloc (new_argv, ((i + 1) * sizeof (char *)));
 			new_argv[i] = NULL;
 		} else
 			tmp++;
