@@ -6,7 +6,7 @@
  *  Updated   :
  *  Notes     : Split from page.c
  *
- * Copyright (c) 2000-2001 Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 2000-2002 Jason Faultless <jason@radar.tele2.co.uk>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,11 +43,8 @@
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
-#ifndef MENUKEYS_H
-#	include  "menukeys.h"
-#endif /* !MENUKEYS_H */
 #ifndef RFC2046_H
-#	include  "rfc2046.h"
+#	include "rfc2046.h"
 #endif /* !RFC2046_H */
 
 /*
@@ -62,7 +59,6 @@
 
 #define MATCH_REGEX(x,y,z)	(pcre_exec (x.re, x.extra, y, z, 0, 0, NULL, 0) >= 0)
 
-static t_bool expand_ctrl_chars (char *to, const char *from, int length);
 static void put_cooked (t_bool wrap_lines, int flags, const char *fmt, ...);
 static void set_rest (char **rest, const char *ptr);
 static int put_rest (char **rest, char *dest, int max_line_len);
@@ -76,9 +72,9 @@ static t_bool header_wanted (const char *line);
  * These are used globally within this module for access to the context
  * currently being built. They must not leak outside.
  */
-int cook_width;
-t_bool hide_uue;
-t_openartinfo *art;
+static int cook_width;
+static t_bool hide_uue;
+static t_openartinfo *art;
 
 /*
  * Rewrite frombuf into tobuf to a maximum length
@@ -86,25 +82,24 @@ t_openartinfo *art;
  * Allows \n through
  * Return TRUE if line contains a ^L (form-feed)
  */
-static t_bool
+t_bool
 expand_ctrl_chars (
 	char *to,
 	const char *from,
-	int length)
+	int length,
+	size_t lcook_width)
 {
 	const char *p;
 	char *q;
 	t_bool ctrl_L = FALSE;
 
 	for (p = from, q = to; *p && q < &to[length]; p++) {
-		if (*p == '\b' && q > to) {			/* Backspace */
-			q--;
-		} else if (*p == '\t') {			/* Expand tabs */
+		if (*p == '\t') {			/* Expand tabs */
 			int i, j;
 
 			i = q - to;
-			j = ((i+cook_width)/cook_width) * cook_width;
-/*			j = (i|(tabwidth-1)) + 1; TODO more efficient ? */
+			j = ((i + lcook_width) / lcook_width) * lcook_width;
+/*			j = (i|(tabwidth - 1)) + 1; TODO more efficient ? */
 
 			while (i++ < j)
 				*q++ = ' ';
@@ -176,7 +171,7 @@ put_cooked (
 			 * Grow the array of lines if needed - we resize it properly at the end
 			 */
 			if (art->cooked_lines % CHUNK == 0)
-				art->cookl = my_realloc ((char *)art->cookl, sizeof(t_lineinfo) * CHUNK * ((art->cooked_lines/CHUNK)+1));
+				art->cookl = my_realloc ((char *)art->cookl, sizeof(t_lineinfo) * CHUNK * ((art->cooked_lines / CHUNK) + 1));
 
 			art->cookl[art->cooked_lines].offset = ftell(art->cooked);
 		}
@@ -521,8 +516,7 @@ read_decoded_qp_line (
 	if (count >= 0) {
 		buf2[count] = '\0';
 		ptr = buf2;
-	} else
-		/* error in encoding or no memory, copy raw line */
+	} else	/* error in encoding or no memory, copy raw line */
 		ptr = buf;
 
 	strncpy(dest, ptr, max_line_len - put_chars - 1);
@@ -564,7 +558,7 @@ new_uue(
 	 * If an extension is present, try and add a Content-Type
 	 */
 	if ((name = strrchr(name, '.')) != NULL)
-		lookup_mimetype (name+1, ptr);
+		lookup_mimetype (name + 1, ptr);
 
 	return ptr;
 }
@@ -589,7 +583,7 @@ get_filename(
 
 	/* Use basename() ? */
 	if (((p = strrchr(name, '/'))) || ((p = strrchr(name, '\\'))))
-		return p+1;
+		return p + 1;
 
 	return name;
 }
@@ -616,16 +610,6 @@ process_text_body_part(
 
 	fseek(in, part->offset, SEEK_SET);
 
-#ifdef LOCAL_CHARSET
-	/*
-	 * if we have a different local charset, we also convert articles
-	 * that do not have MIME headers, since e.g. quoted text may contain
-	 * accented chars on non-MIME newsreaders.
-	 */
-	if (IS_PLAINTEXT(part))
-		decode = FALSE;
-#endif /* LOCAL_CHARSET */
-
 	if ((charset = get_param(part->params, "charset")) == NULL)
 		decode = FALSE;				/* Impossible in practice, since it defaults */
 
@@ -653,17 +637,22 @@ process_text_body_part(
 		if (!(line && strlen(line)))
 			break;	/* premature end of file, file error etc. */
 
-#ifdef LOCAL_CHARSET
+#if defined(LOCAL_CHARSET) || defined(MAC_OS_X)
 		if (decode)
 			buffer_to_local (line);
-#endif /* LOCAL_CHARSET */
+#else
+#	ifdef CHARSET_CONVERSION
+		if (decode)
+			 buffer_to_local (line, charset, tinrc.mm_local_charset);
+#	endif /* CHARSET_CONVERSION */
+#endif /* LOCAL_CHARSET || MAC_OS_X */
 
 		/*
 		 * Detect and skip signatures if necessary
 		 * Detect and process uuencoded sections
 		 * Check quoting depth
 		 */
-		len = (int)strlen(line);
+		len = (int) strlen(line);
 
 		if (!in_sig) {
 			if (strcmp (line, SIGDASHES) == 0) {
@@ -677,7 +666,7 @@ process_text_body_part(
 
 		if (hide_uue) {
 			int offsets[6];
-			int size_offsets = sizeof(offsets)/sizeof(int);
+			int size_offsets = (int) (sizeof(offsets)/sizeof(int));
 			t_bool is_uubody = FALSE;		/* Set if this line looks like a uuencoded line */
 
 			/*
@@ -685,7 +674,7 @@ process_text_body_part(
 			 */
 			if (pcre_exec (uubegin_regex.re, uubegin_regex.extra, line, len, 0, 0, offsets, size_offsets) != PCRE_ERROR_NOMATCH) {
 				in_uue = TRUE;
-				curruue = new_uue(&part, line+offsets[1]);
+				curruue = new_uue(&part, line + offsets[1]);
 				continue;
 			} else if (strncmp (line, "end\n", 4) == 0) {
 				if (in_uue) {
@@ -767,7 +756,7 @@ process_text_body_part(
 		if (MATCH_REGEX (news_regex, line, len))
 			flags |= C_NEWS;
 
-		if (tex2iso_supported && art->tex2iso) {
+		if ((CURR_GROUP.attribute->tex2iso_conv) && art->tex2iso) {
 			char texbuf[LEN];
 			strcpy (texbuf, line);
 			convert_tex2iso (texbuf, line);
@@ -786,7 +775,7 @@ process_text_body_part(
 #endif /* 1 */
 /* TODO integrate above into expand_ctrl_chars */
 
-		if (expand_ctrl_chars (to, line, sizeof(to)))
+		if (expand_ctrl_chars (to, line, sizeof(to), cook_width))
 			flags |= C_CTRLL;				/* Line contains form-feed */
 
 		put_cooked (wrap_lines, flags, "%s", to);
@@ -971,7 +960,8 @@ cook_article(
 	dump_cooked();
 #endif /* DEBUG_ART */
 
-	art->cookl = my_realloc ((char *)art->cookl, sizeof(t_lineinfo) * art->cooked_lines);
+	if (art->cooked_lines > 0)
+		art->cookl = my_realloc ((char *)art->cookl, sizeof(t_lineinfo) * art->cooked_lines);
 
 	rewind(art->cooked);
 
