@@ -3,7 +3,7 @@
  *  Module    : curses.c
  *  Author    : D. Taylor & I. Lea
  *  Created   : 1986-01-01
- *  Updated   : 2000-07-19
+ *  Updated   : 2001-07-22
  *  Notes     : This is a screen management library borrowed with permission
  *              from the Elm mail system. This library was hacked to provide
  *              what tin needs.
@@ -17,6 +17,10 @@
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
+#ifndef TNNTP_H
+#	include "tnntp.h"
+#endif /* !TNNTP_H */
+
 
 #ifdef USE_CURSES
 
@@ -33,7 +37,7 @@ t_bool have_linescroll = TRUE;	/* USE_CURSES always allows line scrolling */
 
 #ifndef ns32000
 #	undef	sinix
-#endif /* !ns32000-2001 */
+#endif /* !ns32000 */
 
 #ifdef VMS
 #	include <descrip.h>
@@ -131,7 +135,7 @@ static TTY _raw_tty, _original_tty;
 #			define USE_SGTTY 1
 struct sgttyb _raw_tty, _original_tty;
 #		else
-#			if !defined(M_AMIGA) && !defined(M_OS2)
+#			if !defined(M_AMIGA)
 #				if defined(HAVE_TERMIOS_H) || defined(sinix)
 #					define USE_POSIX_TERMIOS 1
 struct termios _raw_tty, _original_tty;
@@ -139,7 +143,7 @@ struct termios _raw_tty, _original_tty;
 #					define USE_TERMIO 1
 struct termio _raw_tty, _original_tty;
 #				endif /* HAVE_TERMIOS_H || sinix */
-#			endif /* !M_AMIGA && ! M_OS2 */
+#			endif /* !M_AMIGA */
 #		endif /* (M_AMIGA && !__SASC) || BSD || MINIX */
 #	endif /* !VMS */
 #endif /* HAVE_CONFIG_H */
@@ -235,6 +239,7 @@ setup_screen (void)
 
 #	ifdef USE_TERMINFO
 #		define CAPNAME(a,b)       b
+#		define dCAPNAME(a,b)      strcpy(_terminal, b)
 #		define TGETSTR(b,bufp)    tigetstr(b)
 #		define TGETNUM(b)         tigetnum(b) /* may be tigetint() */
 #		define TGETFLAG(b)        tigetflag(b)
@@ -246,6 +251,7 @@ setup_screen (void)
 #		undef USE_TERMCAP
 #		define USE_TERMCAP 1
 #		define CAPNAME(a,b)       a
+#		define dCAPNAME(a,b)      a
 #		define TGETSTR(a, bufp)   tgetstr(a, bufp)
 #		define TGETNUM(a)         tgetnum(a)
 #		define TGETFLAG(a)        tgetflag(a)
@@ -289,12 +295,14 @@ get_termcaps (void)
 	};
 
 	static char _terminal[1024];		/* Storage for terminal entry */
+
+#	if defined(USE_TERMCAP)
 	static char _capabilities[1024];	/* String for cursor motion */
 	static char *ptr = _capabilities;	/* for buffering */
-
-#	if defined(USE_TERMCAP) && defined(HAVE_EXTERN_TCAP_PC)
+#		if defined(HAVE_EXTERN_TCAP_PC)
 	char *t;
-#	endif /* USE_TERMCAP && HAVE_EXTERN_TCAP_PC */
+#		endif /* HAVE_EXTERN_TCAP_PC */
+#	endif /* USE_TERMCAP */
 
 	char the_termname[40], *p;
 	unsigned n;
@@ -306,24 +314,22 @@ get_termcaps (void)
 
 	my_strncpy(the_termname, p, sizeof(the_termname) - 1);
 
+#ifdef USE_TERMINFO
+	setupterm(the_termname, fileno(stdout), (int *)0);
+#else
 	if (tgetent (_terminal, the_termname) != 1) {
 		my_fprintf (stderr, _(txt_cannot_get_term_entry), tin_progname);
 		return (FALSE);
 	}
+#endif /* USE_TERMINFO */
 
 	/* load in all those pesky values */
 	for (n = 0; n < SIZEOF(table); n++) {
 		*(table[n].value) = TGETSTR(table[n].capname, &ptr);
 	}
-	_lines          = TGETNUM (CAPNAME("li", "lines"));
-	_columns        = TGETNUM (CAPNAME("co", "cols"));
-	_hp_glitch      = TGETFLAG(CAPNAME("xs", "xhp"));
-#	ifdef HAVE_BROKEN_TGETSTR
-	_terminalinit   = "";
-	_terminalend    = "";
-	_keypadlocal    = "";
-	_keypadxmit     = "";
-#	endif /* HAVE_BROKEN_TGETSTR */
+	_lines          = TGETNUM (dCAPNAME("li", "lines"));
+	_columns        = TGETNUM (dCAPNAME("co", "cols"));
+	_hp_glitch      = TGETFLAG(dCAPNAME("xs", "xhp"));
 
 #	if defined(USE_TERMCAP) && defined(HAVE_EXTERN_TCAP_PC)
 	t = TGETSTR(CAPNAME("pc", "pad"), &p);
@@ -418,12 +424,6 @@ InitScreen (void)
 	_cleartoeos	= "\033[J";
 	_getwinsize	= "\2330 q";
 #	endif /* M_AMIGA */
-#	ifdef M_OS2
-	_cleartoeos	= NULL;
-	_terminalinit	= NULL;
-	_terminalend	= "";
-	initscr ();
-#	endif /* M_OS2 */
 #	ifdef VMS
 	_cleartoeos	= "\033[J";
 	_terminalinit	= NULL;
@@ -459,12 +459,6 @@ InitScreen (void)
 		_getwinsize = NULL;
 	}
 #	endif /* M_AMIGA */
-#	ifdef M_OS2
-	if (_lines == -1 || _columns == -1) {
-		_lines = LINES;
-		_columns = COLS;
-	}
-#	endif /* M_OS2 */
 #	ifdef VMS /* moved from below InitWin () M.St. 22.01.98 */
 	{
 		int input_chan, status;
@@ -820,40 +814,36 @@ Raw (
 		_inraw = 1;
 	}
 #else
-#	ifndef M_OS2
-
-#		if defined(M_AMIGA) && defined(__SASC)
+#	if defined(M_AMIGA) && defined(__SASC)
 	_inraw = state;
 	rawcon (state);
-#		else
+#	else
 	if (!state && _inraw) {
 		SET_TTY (&_original_tty);
 		_inraw = 0;
 	} else if (state && !_inraw) {
 		GET_TTY (&_original_tty);
 		GET_TTY (&_raw_tty);
-#			if USE_SGTTY
+#		if USE_SGTTY
 		_raw_tty.sg_flags &= ~(ECHO | CRMOD);	/* echo off */
 		_raw_tty.sg_flags |= CBREAK;		/* raw on */
-#				ifdef M_AMIGA
+#			ifdef M_AMIGA
 		_raw_tty.sg_flags |= RAW; /* Manx-C 5.2 does not support CBREAK */
-#				endif /* M_AMIGA */
-#			else
-#				ifdef __FreeBSD__
+#			endif /* M_AMIGA */
+#		else
+#			ifdef __FreeBSD__
 		cfmakeraw(&_raw_tty);
 		_raw_tty.c_lflag |= ISIG;		/* for ^Z */
-#				else
+#			else
 		_raw_tty.c_lflag &= ~(ICANON | ECHO);	/* noecho raw mode */
 		_raw_tty.c_cc[VMIN] = '\01';	/* minimum # of chars to queue */
 		_raw_tty.c_cc[VTIME] = '\0';	/* minimum time to wait for input */
-#				endif /* __FreeBSD__ */
-#			endif /* USE_SGTTY */
+#			endif /* __FreeBSD__ */
+#		endif /* USE_SGTTY */
 		SET_TTY (&_raw_tty);
 		_inraw = 1;
 	}
-#		endif /* M_AMIGA && __SASC */
-
-#	endif /* !M_OS2 */
+#	endif /* M_AMIGA && __SASC */
 #endif /* !VMS */
 }
 
@@ -862,63 +852,6 @@ Raw (
  */
 
 #ifndef VMS
-#	ifdef M_OS2
-
-int
-ReadCh (void)
-{
-	char ch;
-	KBDKEYINFO os2key;
-	int rc;
-	register int result = 0;
-	static secondkey = 0;
-
-	if (secondkey) {
-		result = secondkey;
-		secondkey = 0;
-	} else {
-		rc = KbdCharIn((PKBDKEYINFO)&os2key, IO_WAIT, 0);
-		result = os2key.chChar;
-		if (result == 0xe0) {
-			result = 0x1b;
-			switch (os2key.chScan) {
-				case 'H':
-					secondkey = 'A';
-					break;
-				case 'P':
-					secondkey = 'B';
-					break;
-				case 'K':
-					secondkey = 'D';
-					break;
-				case 'M':
-					secondkey = 'C';
-					break;
-				case 'I':
-					secondkey = 'I';
-					break;
-				case 'Q':
-					secondkey = 'G';
-					break;
-				case 'G':
-					secondkey = 'H';
-					break;
-				case 'O':
-					secondkey = 'F';
-					break;
-				default:
-					secondkey = '?';
-					break;
-			}
-		} else if (result == 0x0d) {
-			result = 0x0a;
-		}
-	}
-	return ((result == EOF) ? EOF : result & 0xFF);
-}
-
-#	endif /* M_OS2 */
-
 #	ifdef M_AMIGA
 #		include <sprof.h>
 
@@ -1145,9 +1078,6 @@ input_pending (
 
 #else
 
-#	ifdef WIN32
-	return kbhit() ? TRUE : FALSE;
-#	endif /* WIN32 */
 #	ifdef M_AMIGA
 	return (WaitForChar(Input(), 1000 * delay) == DOSTRUE) ? TRUE : FALSE;
 #	endif /* M_AMIGA */
