@@ -6,7 +6,7 @@
  *  Updated   : 2000-02-08
  *  Notes     : provides same interface to mail,pipe,print,save & repost commands
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -62,16 +62,16 @@
  * Set to 0 if user aborted save process
  * We do this hackery to honour the "don't postprocess mailboxes" rule
  */
-signed char proc_ch;
+static signed char proc_ch;
 
 #ifndef DONT_HAVE_PIPING
-	FILE *pipe_fp = (FILE *) 0;
+	static FILE *pipe_fp = (FILE *) 0;
 #endif /* !DONT_HAVE_PIPING */
 
-t_bool confirm;
 t_bool is_mailbox = FALSE;
-t_bool redraw_screen = FALSE;
-t_bool supersede = FALSE;			/* for reposting only */
+static t_bool confirm;
+static t_bool redraw_screen = FALSE;
+static t_bool supersede = FALSE;			/* for reposting only */
 
 /*
  * Local prototypes
@@ -147,7 +147,8 @@ get_save_filename(
  * Return a post_proc char or 0 if aborting the save process
  */
 static int
-get_post_proc_type(void)
+get_post_proc_type(
+	void)
 {
 	char ch;
 	char keynone[MAXKEYLEN], keyquit[MAXKEYLEN], keyshar[MAXKEYLEN];
@@ -229,9 +230,9 @@ feed_article(
 		case FEED_PIPE:
 			/* TODO - looks odd because screen mode is raw */
 			if (max)
-				show_progress (_(txt_piping), (*curr)+1, max);
+				show_progress (_(txt_piping), (*curr) + 1, max);
 			else
-				wait_message (0, "%s %d", _(txt_piping), (*curr)+1);
+				wait_message (0, "%s %d", _(txt_piping), (*curr) + 1);
 
 			rewind (openartptr->raw);
 			ok = copy_fp (openartptr->raw, pipe_fp);	/* Check for SIGPIPE on return */
@@ -241,9 +242,9 @@ feed_article(
 #ifndef DISABLE_PRINTING
 		case FEED_PRINT:
 			if (max)
-				show_progress (_(txt_printing), (*curr)+1, max);
+				show_progress (_(txt_printing), (*curr) + 1, max);
 			else
-				wait_message (0, "%s %d", _(txt_printing), (*curr)+1);
+				wait_message (0, "%s %d", _(txt_printing), (*curr) + 1);
 
 			ok = print_file (data /*print_command*/, art, openartptr);
 			break;
@@ -326,7 +327,7 @@ feed_articles (
 	/*
 	 * try and work out what default the user wants
 	 * This is dumb. If you have _any_ selected arts, then it picks 'h'
-	 * No it's not, that's exactly whyt you want (in binarie groups)
+	 * No it's not, that's exactly what you want (in binary groups)
 	 */
 	ch_default = (num_of_tagged_arts ? iKeyFeedTag :
 					(level == GROUP_LEVEL && HAS_FOLLOWUPS (thread_base) ? iKeyFeedThd :
@@ -435,7 +436,7 @@ feed_articles (
 #ifndef DISABLE_PRINTING
 		case FEED_PRINT:
 			/* Setup print - get print command line */
-			sprintf (output, "%s %s", group->attribute->printer, REDIRECT_OUTPUT);
+			sprintf (output, "%s %s", tinrc.printer, REDIRECT_OUTPUT);
 			break;
 #endif /* !DISABLE_PRINTING */
 
@@ -467,7 +468,7 @@ feed_articles (
 							printascii (keysupersede, map_to_local(iKeyFeedSupersede, &menukeymap.feed_supersede_article)),
 							printascii (keyquit, map_to_local(iKeyQuit, &menukeymap.feed_supersede_article)));
 				option = (char) prompt_slk_response (iKeyFeedSupersede,
-										&menukeymap.feed_supersede_article,
+										&menukeymap.feed_supersede_article, "%s",
 										sized_message(buf, arts[respnum].subject));
 
 				switch (option) {
@@ -633,8 +634,13 @@ got_sig_pipe_while_piping:
 
 		case FEED_SAVE:
 		case FEED_AUTOSAVE_TAGGED:
-			if (proc_ch != 'n' && !is_mailbox && processed_ok)
-				ret2 = post_process_files (proc_ch, (function == FEED_SAVE ? FALSE : TRUE));
+			if (proc_ch != 'n' && !is_mailbox && processed_ok) {
+				if (proc_ch < 0) {
+					if (proc_ch_default != 'n')
+					  ret2 = post_process_files (proc_ch_default, (function == FEED_SAVE ? FALSE : TRUE));
+				} else
+					ret2 = post_process_files (proc_ch, (function == FEED_SAVE ? FALSE : TRUE));
+			}
 			free_save_array ();		/* NB: This is where num_save etc.. gets purged */
 			break;
 
@@ -706,7 +712,7 @@ print_file (
 
 #	ifdef DONT_HAVE_PIPING
 	snprintf(file, sizeof(file) - 1, TIN_PRINTFILE, respnum);
-	if ((fp = fopen(file,"w")) == (FILE *) 0)
+	if ((fp = fopen(file, "w")) == (FILE *) 0)
 #	else
 	if ((fp = popen (command, "w")) == (FILE *) 0)
 #	endif /* DONT_HAVE_PIPING */
