@@ -6,7 +6,7 @@
  *  Updated   : 1998-02-04
  *  Notes     : Filter articles. Kill & auto selection are supported.
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 #ifndef MENUKEYS_H
-#	include  "menukeys.h"
+#	include "menukeys.h"
 #endif /* !MENUKEYS_H */
 
 #define IS_READ(i)	(arts[i].status == ART_READ)
@@ -289,7 +289,7 @@ read_filter_file (
 
 		switch(tolower((unsigned char)buf[0])) {
 		case 'c':
-			if (match_integer (buf+1, "ase=", &icase, 1)) {
+			if (match_integer (buf + 1, "ase=", &icase, 1)) {
 				if (arr_ptr && !expired_time)
 					arr_ptr[i].icase = (unsigned) icase;
 
@@ -297,7 +297,7 @@ read_filter_file (
 			}
 			break;
 		case 'f':
-			if (match_string (buf+1, "rom=", from, sizeof (from))) {
+			if (match_string (buf + 1, "rom=", from, sizeof (from))) {
 				if (arr_ptr && !expired_time)
 					arr_ptr[i].from = my_strdup (from);
 
@@ -309,12 +309,12 @@ read_filter_file (
  *       of the "scope" code.
  */
 		case 'g':
-			if (match_string (buf+1, "roup=", scope, sizeof (scope))) {
+			if (match_string (buf + 1, "roup=", scope, sizeof (scope))) {
 #ifdef DEBUG
-if (debug) {
-	my_printf ("scope=[%s] num=[%d]\n", scope, glob_filter.num);
-	my_flush ();
-}
+				if (debug) {
+					my_printf ("scope=[%s] num=[%d]\n", scope, glob_filter.num);
+					my_flush ();
+				}
 #endif /* DEBUG */
 				arr_num = &glob_filter.num;
 				arr_max = &glob_filter.max;
@@ -336,7 +336,7 @@ if (debug) {
 				psGrp = (struct t_group *) 0;		/* fudge for out of order rules */
 				break;
 			}
-			if (match_string (buf+1, "nksa=", gnksa, sizeof (gnksa))) {
+			if (match_string (buf + 1, "nksa=", gnksa, sizeof (gnksa))) {
 				if (arr_ptr && !expired_time) {
 					if (gnksa[0] == '<') {
 						arr_ptr[i].gnksa_cmp = FILTER_LINES_LT;
@@ -354,7 +354,7 @@ if (debug) {
 		break;
 
 		case 'l':
-			if (match_string (buf+1, "ines=", buffer, sizeof (buffer))) {
+			if (match_string (buf + 1, "ines=", buffer, sizeof (buffer))) {
 				if (arr_ptr && !expired_time) {
 					if (buffer[0] == '<') {
 						arr_ptr[i].lines_cmp = FILTER_LINES_LT;
@@ -372,21 +372,21 @@ if (debug) {
 			break;
 
 		case 'm':
-			if (match_string (buf+1, "sgid=", msgid, sizeof (msgid))) {
+			if (match_string (buf + 1, "sgid=", msgid, sizeof (msgid))) {
 				if (arr_ptr) {
 					arr_ptr[i].msgid = my_strdup (msgid);
 					arr_ptr[i].fullref = FILTER_MSGID;
 				}
 				break;
 			}
-			if (match_string (buf+1, "sgid_last=", msgid, sizeof (msgid))) {
+			if (match_string (buf + 1, "sgid_last=", msgid, sizeof (msgid))) {
 				if (arr_ptr) {
 					arr_ptr[i].msgid = my_strdup (msgid);
 					arr_ptr[i].fullref = FILTER_MSGID_LAST;
 				}
 				break;
 			}
-			if (match_string (buf+1, "sgid_only=", msgid, sizeof (msgid))) {
+			if (match_string (buf + 1, "sgid_only=", msgid, sizeof (msgid))) {
 				if (arr_ptr) {
 					arr_ptr[i].msgid = my_strdup (msgid);
 					arr_ptr[i].fullref = FILTER_MSGID_ONLY;
@@ -396,7 +396,7 @@ if (debug) {
 			break;
 
 		case 'r':
-			if (match_string (buf+1, "efs_only=", msgid, sizeof (msgid))) {
+			if (match_string (buf + 1, "efs_only=", msgid, sizeof (msgid))) {
 				if (arr_ptr) {
 					arr_ptr[i].msgid = my_strdup (msgid);
 					arr_ptr[i].fullref = FILTER_REFS_ONLY;
@@ -406,12 +406,12 @@ if (debug) {
 			break;
 
 		case 's':
-			if (match_string (buf+1, "cope=", scope, sizeof (scope))) {
+			if (match_string (buf + 1, "cope=", scope, sizeof (scope))) {
 #ifdef DEBUG
-if (debug) {
-	my_printf ("scope=[%s] num=[%d]\n", scope, glob_filter.num);
-	my_flush ();
-}
+				if (debug) {
+					my_printf ("scope=[%s] num=[%d]\n", scope, glob_filter.num);
+					my_flush ();
+				}
 #endif /* DEBUG */
 				arr_num = &glob_filter.num;
 				arr_max = &glob_filter.max;
@@ -433,15 +433,15 @@ if (debug) {
 				psGrp = (struct t_group *) 0;	/* fudge for out of order rules */
 				break;
 			}
-			if (match_string (buf+1, "ubj=", subj, sizeof (subj))) {
+			if (match_string (buf + 1, "ubj=", subj, sizeof (subj))) {
 				if (arr_ptr && !expired_time)
 					arr_ptr[i].subj = my_strdup (subj);
 
 #ifdef DEBUG
-if (debug) {
-	my_printf ("6. buf=[%s]  Gsubj=[%s]\n", arr_ptr[i].subj, glob_filter.filter[i].subj);
-	my_flush ();
-}
+				if (debug) {
+					my_printf ("6. buf=[%s]  Gsubj=[%s]\n", arr_ptr[i].subj, glob_filter.filter[i].subj);
+					my_flush ();
+				}
 #endif /* DEBUG */
 				break;
 			}
@@ -449,13 +449,13 @@ if (debug) {
 			/*
 			 * read score for rule
 			 */
-			if (match_string (buf+1, "core=", scbuf, PATH_LEN)) {
+			if (match_string (buf + 1, "core=", scbuf, PATH_LEN)) {
 				score = atoi(scbuf);
 #ifdef DEBUG
-if (debug) {
-	my_printf ("score=[%d]\n", score);
-	my_flush();
-}
+				if (debug) {
+					my_printf ("score=[%d]\n", score);
+					my_flush();
+				}
 #endif /* DEBUG */
 				if (arr_ptr && !expired_time) {
 					if (score > SCORE_MAX)
@@ -477,12 +477,12 @@ if (debug) {
 			break;
 
 		case 't':
-			if (match_integer (buf+1, "ype=", &type, 1)) {
+			if (match_integer (buf + 1, "ype=", &type, 1)) {
 #ifdef DEBUG
-if (debug) {
-	my_printf ("type=[%d][%s]\n", type, (!type ? "KILL" : "SELECT"));
-	my_flush ();
-}
+				if (debug) {
+					my_printf ("type=[%d][%s]\n", type, (!type ? "KILL" : "SELECT"));
+					my_flush ();
+				}
 #endif /* DEBUG */
 				if (arr_ptr) {
 					arr_ptr[i].type = (unsigned) type;
@@ -490,15 +490,15 @@ if (debug) {
 				}
 				break;
 			}
-			if (match_long (buf+1, "ime=", &secs)) {
+			if (match_long (buf + 1, "ime=", &secs)) {
 				if (arr_ptr && !expired_time) {
 					arr_ptr[i].time = (time_t) secs;
 					if (secs && current_secs > (time_t) secs) {
 #ifdef DEBUG
-if (debug) {
-	my_printf ("EXPIRED  secs=[%lu]  current_secs=[%lu]\n", (unsigned long int) secs, (unsigned long int) current_secs);
-	my_flush ();
-}
+						if (debug) {
+							my_printf ("EXPIRED  secs=[%lu]  current_secs=[%lu]\n", (unsigned long int) secs, (unsigned long int) current_secs);
+							my_flush ();
+						}
 #endif /* DEBUG */
 						(*arr_num)--;
 						expired_time = TRUE;
@@ -510,19 +510,19 @@ if (debug) {
 			break;
 
 		case 'x':
-			if (match_string (buf+1, "ref=", xref, sizeof (xref))) {
+			if (match_string (buf + 1, "ref=", xref, sizeof (xref))) {
 				if (arr_ptr && ! expired_time)
 					arr_ptr[i].xref = my_strdup (xref);
 
 				break;
 			}
-			if (match_integer (buf+1, "ref_max=", &xref_max, 1000)) {
+			if (match_integer (buf + 1, "ref_max=", &xref_max, 1000)) {
 				if (arr_ptr && ! expired_time)
 					arr_ptr[i].xref_max = xref_max;
 
 				break;
 			}
-			if (match_string (buf+1, "ref_score=", xref_score, sizeof(xref_score))) {
+			if (match_string (buf + 1, "ref_score=", xref_score, sizeof(xref_score))) {
 				if (arr_ptr && !expired_time) {
 					if (xref_score_cnt < 10) {
 						if (isdigit((int)xref_score[0])) {
@@ -558,10 +558,10 @@ if (debug) {
 	return TRUE;
 }
 
+
 /*
  * write filter strings to ~/.tin/filter
  */
-
 static void
 write_filter_file (
 	const char *filename)
@@ -575,8 +575,7 @@ write_filter_file (
 	/* generate tmp-filename */
 	file_tmp = get_tmpfilename(filename);
 
-	if (!backup_file (filename, file_tmp))
-	{
+	if (!backup_file (filename, file_tmp)) {
 		error_message (_(txt_filesystem_full_backup), filename);
 		free (file_tmp);
 		return;
@@ -602,6 +601,7 @@ write_filter_file (
 	free (file_tmp);
 }
 
+
 static void
 write_filter_array (
 	FILE *fp,
@@ -715,7 +715,7 @@ my_flush ();
 			char timestring[LEN];
 
 			timestring[0] = '\0';
-			my_strftime(timestring, LEN-1, "%Y-%m-%d %H:%M:%S UTC", gmtime(&(ptr->filter[i].time)));
+			my_strftime(timestring, LEN - 1, "%Y-%m-%d %H:%M:%S UTC", gmtime(&(ptr->filter[i].time)));
 			fprintf (fp, "time=%lu (%s)\n", (unsigned long int) ptr->filter[i].time, timestring);
 		}
 		fprintf (fp, "#####\n"); /* makes filter file more readable */
@@ -725,6 +725,9 @@ my_flush ();
 }
 
 
+/*
+ * Interactive filter menu
+ */
 static int
 get_choice (
 	int x,
@@ -766,9 +769,63 @@ get_choice (
 	} while (ch != '\n' && ch != '\r' && ch != ESC);
 
 	if (ch == ESC)
-		return (-1);
+		return -1;
 
-	return (i);
+	return i;
+}
+
+static const char *ptr_filter_lines;
+static const char *ptr_filter_menu;
+static const char *ptr_filter_scope;
+static const char *ptr_filter_score;
+static const char *ptr_filter_text;
+static const char *ptr_filter_time;
+static const char *ptr_filter_groupname;
+static char text_subj[PATH_LEN];
+static char text_from[PATH_LEN];
+static char text_msgid[PATH_LEN];
+
+
+static void
+print_filter_menu (
+	void)
+{
+	ClearScreen ();
+
+	center_line (0, TRUE, ptr_filter_menu);
+
+	MoveCursor (INDEX_TOP, 0);
+	my_printf ("%s" cCRLF, ptr_filter_text);
+	my_printf ("%s" cCRLF cCRLF, _(txt_filter_text_type));
+	my_printf ("%s" cCRLF, text_subj);
+	my_printf ("%s" cCRLF, text_from);
+	my_printf ("%s" cCRLF cCRLF, text_msgid);
+	my_printf ("%s" cCRLF, ptr_filter_lines);
+	my_printf ("%s" cCRLF, ptr_filter_score);
+	my_printf ("%s" cCRLF cCRLF, ptr_filter_time);
+	my_printf ("%s%s", ptr_filter_scope, ptr_filter_groupname);
+	my_flush ();
+
+	show_menu_help (txt_help_filter_text);
+}
+
+
+void
+refresh_filter_menu (
+	void)
+{
+	print_filter_menu ();
+
+	/*
+	 * TODO:
+	 * - refresh already entered and accepted information (follow control
+	 *   flow in filter_menu below)
+	 * - refresh help line
+	 * - set cursor into current input field
+	 * - refresh already entered data or selected item in current input field
+	 *   (not everywhere possible yet -- must change getline.c for refreshing
+	 *    string input)
+	 */
 }
 
 
@@ -783,23 +840,14 @@ filter_menu (
 	struct t_article *art)
 {
 	const char *ptr_filter_from;
-	const char *ptr_filter_lines;
-	const char *ptr_filter_menu;
 	const char *ptr_filter_msgid;
-	const char *ptr_filter_scope;
 	const char *ptr_filter_subj;
-	const char *ptr_filter_text;
-	const char *ptr_filter_time;
 	const char *ptr_filter_help_scope;
 	const char *ptr_filter_quit_edit_save;
-	const char *ptr_filter_score;
 	char *ptr;
 	char argv[4][PATH_LEN];
 	char buf[LEN];
 	char keyedit[MAXKEYLEN], keyquit[MAXKEYLEN], keysave[MAXKEYLEN];
-	char text_from[PATH_LEN];
-	char text_msgid[PATH_LEN];
-	char text_subj[PATH_LEN];
 	char text_time[PATH_LEN];
 	char double_time[PATH_LEN];
 	char quat_time[PATH_LEN];
@@ -807,6 +855,10 @@ filter_menu (
 	int ch, i, len;
 	struct t_filter_rule rule;
 
+	signal_context = cFilter;
+
+	signal_context = cFilter;
+
 	rule.text[0] = '\0';
 	rule.scope[0] = '\0';
 	rule.counter = 0;
@@ -826,7 +878,6 @@ filter_menu (
 	/*
 	 * setup correct text for user selected menu
 	 */
-
 	(void) printascii (keyedit, map_to_local (iKeyFilterEdit, &menukeymap.filter_quit_edit_save));
 	(void) printascii (keyquit, map_to_local (iKeyQuit, &menukeymap.filter_quit_edit_save));
 	(void) printascii (keysave, map_to_local (iKeyFilterSave, &menukeymap.filter_quit_edit_save));
@@ -856,40 +907,29 @@ filter_menu (
 	}
 
 	ptr_filter_score = _(txt_filter_score);
+	ptr_filter_groupname = group->name;
 
 	len = cCOLS - 30;
 
-	sprintf (text_time, _(txt_time_default_days), tinrc.filter_days);
-	sprintf (text_subj, ptr_filter_subj, len, len, art->subject);
+	snprintf (text_time, sizeof(text_time), _(txt_time_default_days), tinrc.filter_days);
+	text_time[sizeof(text_time) - 1] = '\0';
+	snprintf (text_subj, sizeof(text_subj), ptr_filter_subj, len, len, art->subject);
+	text_subj[sizeof(text_subj) - 1] = '\0';
 
-	strcpy (buf, art->from);
+	STRCPY (buf, art->from);
 
-	sprintf (text_from, ptr_filter_from, len, len, buf);
-	sprintf (text_msgid, ptr_filter_msgid, len-4, len-4, MSGID(art));
+	snprintf (text_from, sizeof(text_from), ptr_filter_from, len, len, buf);
+	text_from[sizeof(text_from) - 1] = '\0';
+	snprintf (text_msgid, sizeof(text_msgid), ptr_filter_msgid, len - 4, len - 4, MSGID(art));
+	text_msgid[sizeof(text_msgid) - 1] = '\0';
 
-	ClearScreen ();
+	print_filter_menu ();
 
-	center_line (0, TRUE, ptr_filter_menu);
-
-	MoveCursor (INDEX_TOP, 0);
-	my_printf ("%s" cCRLF, ptr_filter_text);
-	my_printf ("%s" cCRLF cCRLF, txt_filter_text_type);
-	my_printf ("%s" cCRLF , text_subj);
-	my_printf ("%s" cCRLF , text_from);
-	my_printf ("%s" cCRLF cCRLF, text_msgid);
-	my_printf ("%s" cCRLF , ptr_filter_lines);
-	my_printf ("%s" cCRLF , ptr_filter_score);
-	my_printf ("%s" cCRLF cCRLF, ptr_filter_time);
-	my_printf ("%s%s", ptr_filter_scope, group->name);
-	my_flush ();
-
-	show_menu_help (_(txt_help_filter_text));
-
-	if (!prompt_menu_string (INDEX_TOP, (int) strlen (ptr_filter_text), rule.text))
+	if (!prompt_menu_string (INDEX_TOP, ptr_filter_text, rule.text))
 		return FALSE;
 
 	if (*rule.text) {
-		i = get_choice (INDEX_TOP+1, _(txt_help_filter_text_type),
+		i = get_choice (INDEX_TOP + 1, _(txt_help_filter_text_type),
 			       _(txt_filter_text_type),
 			       _(txt_subj_line_only_case),
 			       _(txt_subj_line_only),
@@ -926,7 +966,7 @@ filter_menu (
 		/*
 		 * Subject:
 		 */
-		i = get_choice (INDEX_TOP+3, _(txt_help_filter_subj), text_subj, _(txt_yes), _(txt_no), (char *)0, (char *)0, (char *)0);
+		i = get_choice (INDEX_TOP + 3, _(txt_help_filter_subj), text_subj, _(txt_yes), _(txt_no), (char *)0, (char *)0, (char *)0);
 
 		if (i == -1)
 			return FALSE;
@@ -936,7 +976,7 @@ filter_menu (
 		/*
 		 * From:
 		 */
-		i = get_choice (INDEX_TOP+4, _(txt_help_filter_from), text_from, (rule.subj_ok ? _(txt_no) : _(txt_yes)), (rule.subj_ok ? _(txt_yes) : _(txt_no)), (char *)0, (char *)0, (char *)0);
+		i = get_choice (INDEX_TOP + 4, _(txt_help_filter_from), text_from, (rule.subj_ok ? _(txt_no) : _(txt_yes)), (rule.subj_ok ? _(txt_yes) : _(txt_no)), (char *)0, (char *)0, (char *)0);
 
 		if (i == -1)
 			return FALSE;
@@ -947,14 +987,14 @@ filter_menu (
 		 * Message-Id:
 		 */
 		if (rule.subj_ok || rule.from_ok)
-			i = get_choice (INDEX_TOP+5, _(txt_help_filter_msgid), text_msgid, _(txt_no), _(txt_full), _(txt_last), _(txt_only), (char *)0);
+			i = get_choice (INDEX_TOP + 5, _(txt_help_filter_msgid), text_msgid, _(txt_no), _(txt_full), _(txt_last), _(txt_only), (char *)0);
 		else
-			i = get_choice (INDEX_TOP+5, _(txt_help_filter_msgid), text_msgid, _(txt_full), _(txt_last), _(txt_only), _(txt_no), (char *)0);
+			i = get_choice (INDEX_TOP + 5, _(txt_help_filter_msgid), text_msgid, _(txt_full), _(txt_last), _(txt_only), _(txt_no), (char *)0);
 
 		if (i == -1)
 			return FALSE;
 		else {
-			switch ((rule.subj_ok || rule.from_ok) ? i : i+1) {
+			switch ((rule.subj_ok || rule.from_ok) ? i : i + 1) {
 				case 0:
 				case 4:
 					rule.msgid_ok = FALSE;
@@ -988,11 +1028,11 @@ filter_menu (
 	/*
 	 * Lines:
 	 */
-	show_menu_help (_(txt_help_filter_lines));
+	show_menu_help (txt_help_filter_lines);
 
 	buf[0] = '\0';
 
-	if (!prompt_menu_string (INDEX_TOP+7, (int) strlen (ptr_filter_lines), buf))
+	if (!prompt_menu_string (INDEX_TOP + 7, ptr_filter_lines, buf))
 		return FALSE;
 
 	/*
@@ -1021,9 +1061,9 @@ filter_menu (
 	 * Scoring value
 	 */
 	buf[0] = '\0';
-	show_menu_help(_(txt_filter_score_help)); /* FIXME: a sprintf() is necessary here */
+	show_menu_help(txt_filter_score_help); /* FIXME: a sprintf() is necessary here */
 
-	if (!prompt_menu_string(INDEX_TOP+8, (int)strlen(ptr_filter_score), buf))
+	if (!prompt_menu_string(INDEX_TOP + 8, ptr_filter_score, buf))
 		return FALSE;
 
 	rule.score = atoi(buf);
@@ -1043,7 +1083,7 @@ filter_menu (
 	 */
 	sprintf (double_time, "2x %s", text_time);
 	sprintf (quat_time, "4x %s", text_time);
-	i = get_choice (INDEX_TOP+9, _(txt_help_filter_time), ptr_filter_time,
+	i = get_choice (INDEX_TOP + 9, _(txt_help_filter_time), ptr_filter_time,
 			_(txt_unlimited_time), text_time, double_time, quat_time, (char *)0);
 
 	if (i == -1)
@@ -1064,7 +1104,7 @@ filter_menu (
 			*(ptr++) = '*';
 			*ptr = '\0';
 			strcpy (argv[3], argv[2]);
-			argv[3][strlen(argv[3])-2] = '\0';
+			argv[3][strlen(argv[3]) - 2] = '\0';
 			ptr = strrchr (argv[3], '.');
 			if (ptr != (char *) 0) {
 				ptr++;
@@ -1076,13 +1116,13 @@ filter_menu (
 		} else
 			argv[2][0] = '\0';
 
-		i = get_choice (INDEX_TOP+11, ptr_filter_help_scope,
+		i = get_choice (INDEX_TOP + 11, ptr_filter_help_scope,
 			       ptr_filter_scope,
-			       (argv[0][0] ? argv[0] : (char *)0),
-			       (argv[1][0] ? argv[1] : (char *)0),
-			       (argv[2][0] ? argv[2] : (char *)0),
-			       (argv[3][0] ? argv[3] : (char *)0),
-			       (char *)0);
+			       (argv[0][0] ? argv[0] : (char *) 0),
+			       (argv[1][0] ? argv[1] : (char *) 0),
+			       (argv[2][0] ? argv[2] : (char *) 0),
+			       (argv[3][0] ? argv[3] : (char *) 0),
+			       (char *) 0);
 
 		if (i == -1)
 			return FALSE;
@@ -1189,6 +1229,7 @@ quick_filter (
 	return (bAddFilterRule (group, art, &rule));
 }
 
+
 /*
  * Quick command to add an auto-select filter to the article that user
  * has just posted. Selects on Subject: line with limited expire time.
@@ -1197,7 +1238,8 @@ quick_filter (
 t_bool
 quick_filter_select_posted_art (
 	struct t_group *group,
-	const char *subj)	/* return value is always ignored */
+	const char *subj,
+	const char *a_message_id)	/* return value is always ignored */
 {
 	t_bool filtered = FALSE;
 
@@ -1212,12 +1254,6 @@ quick_filter_select_posted_art (
 		if (strlen(group->name) > (sizeof(rule.scope) -1)) /* groupname to long? */
 			return FALSE;
 
-		/*
-		 * Setup dummy article with posted articles subject
-		 */
-		set_article (&art);
-		art.subject = my_strdup (subj);
-
 		/*
 		 * Setup rules
 		 */
@@ -1238,15 +1274,46 @@ quick_filter_select_posted_art (
 
 		strcpy(rule.scope, group->name);
 
-		filtered = bAddFilterRule (group, &art, &rule);
+		/*
+		 * Setup dummy article with posted articles subject
+		 * xor Message-ID
+		 */
+		set_article (&art);
+		if (*a_message_id) {
+			/* initialize art->refptr */
+			struct {
+				struct t_msgid *next;
+				struct t_msgid *parent;
+				struct t_msgid *sibling;
+				struct t_msgid *child;
+				int article;
+				char txt[HEADER_LEN];
+			} refptr_dummyart;
 
-		FreeIfNeeded(art.subject);
+			rule.subj_ok = FALSE;
+			rule.msgid_ok = TRUE;
+			refptr_dummyart.next = (struct t_msgid *) 0;
+			refptr_dummyart.parent = (struct t_msgid *) 0;
+			refptr_dummyart.sibling = (struct t_msgid *) 0;
+			refptr_dummyart.child = (struct t_msgid *) 0;
+			refptr_dummyart.article = ART_NORMAL;
+			my_strncpy(refptr_dummyart.txt, a_message_id, HEADER_LEN);
+			/* Hack */
+			art.refptr=(struct t_msgid *) &refptr_dummyart;
+
+			filtered = bAddFilterRule (group, &art, &rule);
+		} else {
+			art.subject = my_strdup (subj);
+			filtered = bAddFilterRule (group, &art, &rule);
+			FreeIfNeeded(art.subject);
+		}
 
 	}
 
 	return filtered;
 }
 
+
 /*
  * API to add filter rule to the local or global filter array
  */
@@ -1295,8 +1362,7 @@ bAddFilterRule (
 	}
 
 	(void) time (&lCurTime);
-	switch(psRule->expire_time)
-	{
+	switch(psRule->expire_time) {
 		case 1:
 			psPtr[*plNum].time = lCurTime + (time_t) (tinrc.filter_days * DAY);
 			break;
@@ -1375,14 +1441,14 @@ bAddFilterRule (
 #ifdef DEBUG
 		if (debug)
 			wait_message (2, "inscope=[%s] scope=[%s] typ=[%d] case=[%d] subj=[%s] from=[%s] msgid=[%s] fullref=[%d] line=[%d %d] time=[%lu]",
-				bool_unparse(psPtr[*plNum-1].inscope),
+				bool_unparse(psPtr[*plNum - 1].inscope),
 				BlankIfNull(psRule->scope),
-				psPtr[*plNum-1].type, psPtr[*plNum-1].icase,
-				BlankIfNull(psPtr[*plNum-1].subj),
-				BlankIfNull(psPtr[*plNum-1].from),
-				BlankIfNull(psPtr[*plNum-1].msgid),
-				psPtr[*plNum-1].fullref, psPtr[*plNum-1].lines_cmp,
-				psPtr[*plNum-1].lines_num, (unsigned long int) psPtr[*plNum-1].time);
+				psPtr[*plNum - 1].type, psPtr[*plNum - 1].icase,
+				BlankIfNull(psPtr[*plNum - 1].subj),
+				BlankIfNull(psPtr[*plNum - 1].from),
+				BlankIfNull(psPtr[*plNum - 1].msgid),
+				psPtr[*plNum - 1].fullref, psPtr[*plNum - 1].lines_cmp,
+				psPtr[*plNum - 1].lines_num, (unsigned long int) psPtr[*plNum - 1].time);
 #endif /* DEBUG */
 
 		write_filter_file (filter_file);
@@ -1484,13 +1550,17 @@ filter_articles (
 			regex_cache_xref[j].extra = NULL;
 		}
 	}
+	mesg[0] = '\0';				/* Clear system message field */
 
 	/*
 	 * loop thru all arts applying global & local filtering rules
 	 */
-	for (i = 0; i < top_art; i++) {
+	for (i = 0; (i < top_art) && (mesg[0] == '\0'); i++) {
 		arts[i].score = 0;
 
+		/*
+		 * do we really need to 'reset' mesg for every article?
+		 */
 		mesg[0] = '\0';				/* Clear system message field */
 
 		if (tinrc.kill_level == KILL_READ && IS_READ(i)) /* skip only when the article is read */
@@ -1716,21 +1786,23 @@ wait_message (1, "FILTERED Lines arts[%d] > [%d]", arts[i].line_count, ptr[j].li
 	 * now entering the main filter loop:
 	 * all articles have scored, so do kill & select
 	 */
-
-	for (i = 0; i < top_art; i++) {
-		if (arts[i].score <= SCORE_LIM_KILL) {
-			arts[i].killed = TRUE;
-			num_of_killed_arts++;
-			filtered = TRUE;
-			art_mark_read (group, &arts[i]);
-		} else if (arts[i].score >= SCORE_LIM_SEL) {
-			arts[i].selected = TRUE;
-			num_of_selected_arts++;
+	if (mesg[0] == '\0') {
+		for (i = 0; i < top_art; i++) {
+			if (arts[i].score <= SCORE_LIM_KILL) {
+				arts[i].killed = TRUE;
+				num_of_killed_arts++;
+				filtered = TRUE;
+				art_mark_read (group, &arts[i]);
+			} else if (arts[i].score >= SCORE_LIM_SEL) {
+				arts[i].selected = TRUE;
+				num_of_selected_arts++;
+			}
 		}
 	}
 	return filtered;
 }
 
+
 static int
 set_filter_scope (
 	struct t_group *group)
