@@ -6,7 +6,7 @@
  *  Updated   : 1997-12-31
  *  Notes     :
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 #ifndef MENUKEYS_H
-#	include  "menukeys.h"
+#	include "menukeys.h"
 #endif /* !MENUKEYS_H */
 
 #define INDEX2SNUM(i)	((i) % NOTESLINES)
@@ -80,6 +80,7 @@ static void show_group_title (t_bool clear_title);
 static void show_tagged_lines (void);
 static void toggle_read_unread (t_bool force);
 static void update_group_page (void);
+static void build_multipart_header(char* dest, int maxlen, const char* src, int cmplen, int have, int total);
 
 /*
  * grpmenu.curr is an index into base[] and so equates to the cursor location
@@ -113,14 +114,22 @@ group_left (
 		return iKeyQuit;
 }
 
+
 static int
 group_right (
 	void)
 {
-	if (tinrc.auto_list_thread && grpmenu.curr >= 0 && HAS_FOLLOWUPS (grpmenu.curr))
-		return iKeyGroupListThd;
-	else
-		return iKeyGroupReadBasenote;
+	if (grpmenu.curr >= 0 && HAS_FOLLOWUPS (grpmenu.curr)) {
+		if (tinrc.auto_list_thread)
+			return iKeyGroupListThd;
+		else {
+			int n = next_unread ((int) base[grpmenu.curr]);
+
+			if (grpmenu.curr == which_thread(n) && n >= 0)
+				return enter_pager (n, TRUE);
+		}
+	}
+	return iKeyGroupReadBasenote;
 }
 
 
@@ -196,10 +205,8 @@ group_page (
 		set_xclick_on ();
 		switch (ch = handle_keypad (group_left, group_right, &menukeymap.group_nav)) {
 
-#ifndef WIN32
 			case iKeyAbort:		/* Abort */
 				break;
-#endif /* !WIN32 */
 
 			case '1': case '2': case '3': case '4': case '5':
 			case '6': case '7': case '8': case '9':
@@ -242,7 +249,7 @@ group_page (
 				break;
 
 #ifndef DISABLE_PRINTING
-			case iKeyGroupPrint:	/* output art/thread/tagged arts to printer */
+			case iKeyPrint:	/* output art/thread/tagged arts to printer */
 				if (grpmenu.curr >= 0)
 					feed_articles (FEED_PRINT, GROUP_LEVEL, &CURR_GROUP, (int) base[grpmenu.curr]);
 				break;
@@ -268,7 +275,7 @@ group_page (
 				break;
 
 			case iKeySetRange:	/* set range */
-				if (bSetRange (GROUP_LEVEL, 1, grpmenu.max, grpmenu.curr+1)) {
+				if (bSetRange (GROUP_LEVEL, 1, grpmenu.max, grpmenu.curr + 1)) {
 					range_active = TRUE;
 					show_group_page ();
 				}
@@ -297,7 +304,7 @@ group_page (
 			case iKeyGroupReadBasenote:
 			case iKeyGroupReadBasenote2:	/* read current basenote */
 				if (grpmenu.curr >= 0)
-					ret_code = enter_pager ((int) base[grpmenu.curr], FALSE/*TRUE*/);
+					ret_code = enter_pager ((int) base[grpmenu.curr], FALSE /*TRUE*/);
 				else
 					info_message(_(txt_no_arts));
 				break;
@@ -412,7 +419,7 @@ group_page (
 				break;
 
 			case iKeyHelp:					/* help */
-				show_info_page (HELP_INFO, help_group, _(txt_index_page_com));
+				show_help_page (GROUP_LEVEL, _(txt_index_page_com));
 				show_group_page ();
 				break;
 
@@ -910,7 +917,7 @@ group_page (
 
 	art_close (&pgart);				/* Close any open art */
 
-	return (ret_code);
+	return ret_code;
 }
 
 
@@ -1109,7 +1116,7 @@ mark_screen (
 #else
 		int i;
 		for (i = 0; value[i] != '\0'; i++)
-			screen[screen_row].col[screen_col+i] = value[i];
+			screen[screen_row].col[screen_col + i] = value[i];
 #endif /* USE_CURSES */
 		if (level == SELECT_LEVEL)
 			draw_group_arrow();
@@ -1170,6 +1177,32 @@ toggle_subject_from (
 }
 
 
+/*
+ *	Builds the correct header for multipart messages when sorting via
+ *	THREAD_MULTI.
+ */
+static void
+build_multipart_header(
+	char* dest,
+	int maxlen,
+	const char* src,
+	int cmplen,
+	int have,
+	int total)
+{
+	const char* mark = (have == total) ? "*" : "-";
+	char* ss = NULL;
+
+	if (cmplen > maxlen)
+		strncpy(dest, src, maxlen);
+	else {
+		strncpy(dest, src, cmplen);
+		ss = dest + cmplen;
+		snprintf(ss, maxlen - cmplen, "(%s/%d)", mark, total);
+	}
+}
+
+
 /*
  * Build subject line given an index into base[].
  *
@@ -1248,8 +1281,11 @@ bld_sline (
 	if (CURR_GROUP.attribute->show_author != SHOW_FROM_NONE)
 		get_author (FALSE, &arts[j], from, len_from);
 
-	strncpy(arts_sub, arts[j].subject, len_subj+12);
-	arts_sub[len_subj-12+1] = '\0';
+	if (sbuf.multipart_have > 1) /* We have a multipart msg so lets built our new header info. */
+		build_multipart_header(arts_sub, len_subj, arts[j].subject, sbuf.multipart_compare_len, sbuf.multipart_have, sbuf.multipart_total);
+	else
+		strncpy(arts_sub, arts[j].subject, len_subj);
+	arts_sub[len_subj - 12 + 1] = '\0';
 
 #ifndef USE_CURSES
 	buffer = screen[INDEX2SNUM(i)].col;
@@ -1257,13 +1293,13 @@ bld_sline (
 
 	if (tinrc.show_score)
 		sprintf (buffer, "  %s %s %s%6d %-*.*s%s%-*.*s",
-			 tin_ltoa(i+1, 4), new_resps, art_cnt, sbuf.score,
-			 len_subj-12, len_subj-12, arts_sub,
+			 tin_ltoa(i + 1, 4), new_resps, art_cnt, sbuf.score,
+			 len_subj - 12, len_subj - 12, arts_sub,
 			 spaces, len_from, len_from, from);
 	else
 		sprintf (buffer, "  %s %s %s%-*.*s%s%-*.*s",
-			 tin_ltoa(i+1, 4), new_resps, art_cnt,
-			 len_subj-12, len_subj-12, arts_sub,
+			 tin_ltoa(i + 1, 4), new_resps, art_cnt,
+			 len_subj - 12, len_subj - 12, arts_sub,
 			 spaces, len_from, len_from, from);
 
 	/* protect display from non-displayable characters (e.g., form-feed) */
@@ -1334,7 +1370,7 @@ do_search(
 	/*
 	 * Not intuitive to search current thread in fwd search
 	 */
-	start = (forward && grpmenu.curr < grpmenu.max-1) ? prev_response((int)base[grpmenu.curr+1]) : (int)base[grpmenu.curr];
+	start = (forward && grpmenu.curr < grpmenu.max - 1) ? prev_response((int)base[grpmenu.curr + 1]) : (int)base[grpmenu.curr];
 
 	if ((n = search (type, start, forward)) != -1) {
 		grpmenu.curr = which_thread(n);
@@ -1465,7 +1501,7 @@ group_catchup(
 	int ch)
 {
 	char buf[LEN];
-	int yn = 1;
+	int pyn = 1;
 
 	if (num_of_tagged_arts && prompt_yn (cLINES, _(txt_catchup_despite_tags), TRUE) != 1)
 		return 0;
@@ -1473,28 +1509,30 @@ group_catchup(
 	/* FIXME: -> lang.c */
 	snprintf(buf, sizeof(buf) - 1, _(txt_mark_arts_read), (ch == iKeyGroupCatchupNextUnread) ? _(" and enter next unread group") : "");
 
-	if (!CURR_GROUP.newsrc.num_unread || !tinrc.confirm_action || (yn = prompt_yn (cLINES, buf, TRUE)) == 1)
+	if (!CURR_GROUP.newsrc.num_unread || !tinrc.confirm_action || (pyn = prompt_yn (cLINES, buf, TRUE)) == 1)
 		grp_mark_read (&CURR_GROUP, arts);
 
 	switch (ch) {
 		case iKeyGroupCatchup:				/* 'c' */
-			if (yn == 1)
+			if (pyn == 1)
 				return GRP_NEXT;
 			break;
 
 		case iKeyGroupCatchupNextUnread:	/* 'C' */
-			if (yn == 1)
+			if (pyn == 1)
 				return GRP_NEXTUNREAD;
 			break;
 
 		case iKeyCatchupLeft:				/* <- group catchup on exit */
-			switch (yn) {
+			switch (pyn) {
 				case -1:					/* ESCAPE - do nothing */
 					break;
+
 				case 1:						/* We caught up - advance group */
 					return GRP_NEXT;
 					/* NOTREACHED */
 					break;
+
 				default:					/* Just leave the group */
 					return GRP_EXIT;
 					/* NOTREACHED */
