@@ -5,7 +5,7 @@
  *  Created   : 1997-03-10
  *  Updated   : 1997-03-19
  *
- * Copyright (c) 1997-2001 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2002 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,22 +50,17 @@ get_host_name (
 	void)
 {
 	char *ptr;
-	static char hostname[MAXHOSTNAMELEN+1]; /* need space for '\0' */
+	static char hostname[MAXHOSTNAMELEN + 1]; /* need space for '\0' */
 
 	hostname[0] = '\0';
 
 #ifdef HAVE_GETHOSTBYNAME
 	gethostname(hostname, sizeof(hostname));
 #else
-#	if defined(M_AMIGA) || defined(M_OS2)
+#	if defined(M_AMIGA)
 	if ((ptr = getenv("NodeName")) != (char *) 0)
 		strncpy(hostname, ptr, MAXHOSTNAMELEN);
-#	else
-#		ifdef WIN32
-	if ((ptr = getenv("COMPUTERNAME")) != (char *) 0)
-		strncpy(hostname, ptr, MAXHOSTNAMELEN);
-#		endif /* WIN32 */
-#	endif /* M_AMIGA || M_OS2 */
+#	endif /* M_AMIGA */
 #endif /* HAVE_GETHOSTBYNAME */
 #ifdef HAVE_SYS_UTSNAME_H
 	if (! *hostname)
@@ -82,7 +77,7 @@ get_host_name (
 		}
 	}
 	hostname[MAXHOSTNAMELEN] = '\0';
-	return (hostname);
+	return hostname;
 }
 
 
@@ -104,7 +99,7 @@ get_domain_name (
  * which is illegal on a 68000
  */
 static const char *domain_name_hack = DOMAIN_NAME;
-#		undef	DOMAIN_NAME
+#		undef DOMAIN_NAME
 #		define DOMAIN_NAME domain_name_hack
 #	endif /* M_AMIGA */
 
@@ -138,22 +133,22 @@ static const char *domain_name_hack = DOMAIN_NAME;
 			domain[0] = '\0';
 	}
 	domain[MAXHOSTNAMELEN] = '\0';
-	return (domain);
+	return domain;
 }
 #endif /* DOMAIN_NAME */
 
 
 #ifdef HAVE_GETHOSTBYNAME
 #	define MAXLINELEN   1024
-#	define WS	" \f\n\r\t\v"
+#	define WS	" \f\t\v"
 /* find FQDN - gethostbyaddr() */
 const char *
 get_fqdn (
 	const char *host)
 {
 	char *domain;
-	char line[MAXLINELEN+1];
-	char name[MAXHOSTNAMELEN+2];
+	char line[MAXLINELEN + 1];
+	char name[MAXHOSTNAMELEN + 2];
 	static char fqdn[1024];
 	struct hostent *hp;
 	struct in_addr in;
@@ -168,11 +163,11 @@ get_fqdn (
 		(void) strncpy(name, host, MAXHOSTNAMELEN);
 	} else
 		if (gethostname(name, MAXHOSTNAMELEN))
-			return(NULL);
+			return NULL;
 
 	if ('0' <= *name && *name <= '9') {
 		in.s_addr = inet_addr(name);
-		if ((hp = gethostbyaddr((char *)&in.s_addr, 4, AF_INET)))
+		if ((hp = gethostbyaddr((char *) &in.s_addr, 4, AF_INET)))
 			in.s_addr = (*hp->h_addr);
 		return(hp && strchr(hp->h_name, '.') ? hp->h_name : inet_ntoa(in));
 	}
@@ -180,33 +175,44 @@ get_fqdn (
 		if ((hp = gethostbyaddr(hp->h_addr, hp->h_length, hp->h_addrtype)))
 			in.s_addr = (*hp->h_addr);
 
-	sprintf(fqdn,"%s", hp
+	sprintf(fqdn, "%s", hp
 		? strchr(hp->h_name, '.')
 			? hp->h_name
 			: inet_ntoa(in)
 		: "");
-	if (!*fqdn || (fqdn[strlen(fqdn)-1] <= '9')) {
+	if (!*fqdn || (fqdn[strlen(fqdn) - 1] <= '9')) {
 		FILE *inf;
 
 		*fqdn = '\0';
+
 		if ((inf = fopen("/etc/resolv.conf", "r")) != NULL) {
+			char *eos;
+
 			while(fgets(line, MAXLINELEN, inf)) {
-				char *ptr;
+
+				if (line[0] =='#' || line[0] == '\n')
+					continue;
 
 				line[MAXLINELEN] = '\0';
-				ptr = strtok(line, WS);
-				if ((strncmp(ptr, "domain", 6) == 0) || (strncmp(ptr, "search", 6) == 0)) {
-					domain = strtok(NULL, WS);
-					break;
+
+				if ((eos = strpbrk(line, WS)) != NULL) {
+					int j = eos - line;
+
+					if (j) {
+						if ((strncmp(line, "domain", j) == 0) || (strncmp(line, "search", j) == 0)) {
+							domain = strtok(eos, WS);
+							break;
+						}
+					}
 				}
 			}
 			if (domain)
 				sprintf(fqdn, "%s.%s", name, domain);
+
 			fclose(inf);
 		}
 	}
-
-	return(fqdn);
+	return fqdn;
 }
 #endif /* HAVE_GETHOSTBYNAME */
 
@@ -251,9 +257,15 @@ get_user_name (
 	}
 #	else
 	pw = getpwuid (getuid ());
-	strcpy (username, pw->pw_name);
+
+	if (pw != (struct passwd *) 0)
+		strcpy (username, pw->pw_name);
+	else {
+		error_message (_(txt_error_passwd_missing));
+		tin_done (EXIT_FAILURE);
+	}
 #	endif /* M_AMIGA || VMS */
-	return(username);
+	return username;
 }
 
 
@@ -273,11 +285,11 @@ get_full_name (
 
 	if ((p = getenv ("NAME")) != (char *) 0) {
 		strncpy (fullname, p, sizeof (fullname));
-		return (fullname);
+		return fullname;
 	}
-	if ((p =  getenv ("REALNAME")) != (char *) 0) {
+	if ((p = getenv ("REALNAME")) != (char *) 0) {
 		strncpy (fullname, p, sizeof (fullname));
-		return (fullname);
+		return fullname;
 	}
 
 #	ifdef VMS
@@ -291,14 +303,14 @@ get_full_name (
 	if ((p = strchr (buf, '&'))) {
 		*p++ = '\0';
 		STRCPY(tmp, pw->pw_name);
-		if (*tmp && isalpha((int)*tmp) && islower((int)*tmp))
-			*tmp = toupper((int)*tmp);
+		if (*tmp && isalpha((int) *tmp) && islower((int) *tmp))
+			*tmp = toupper((int) *tmp);
 		snprintf (fullname, sizeof(fullname) -1, "%s%s%s", buf, tmp, p);
 	} else
 		STRCPY(fullname, buf);
 #		endif /* !DONT_HAVE_PW_GECOS */
 #	endif /* VMS */
-	return (fullname);
+	return fullname;
 }
 
 
