@@ -6,7 +6,7 @@
  *  Updated   : 1997-12-31
  *  Notes     : NNTP builtin version of inews
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -124,7 +124,7 @@ submit_inews (
 					*ptr = '\0';
 			}
 			if (ptr - line == 10 && !strncasecmp (line, "Message-ID", 10)) {
-				strcpy(message_id, ptr+2);
+				strcpy(message_id, ptr + 2);
 				id_in_article = TRUE;
 				if ((ptr = strchr(message_id, '\n')))
 					*ptr = '\0';
@@ -180,6 +180,9 @@ submit_inews (
 
 				case 1:	/* insert Sender */
 					snprintf (sender_hdr, sizeof(sender_hdr), "Sender: %s", ptr);
+#		if defined(LOCAL_CHARSET) || defined(MAC_OS_X) || defined(CHARSET_CONVERSION)
+					buffer_to_network(sender_hdr);
+#		endif /* LOCAL_CHARSET || MAC_OS_X || CHARSET_CONVERSION */
 					STRCPY (sender_hdr, rfc1522_encode (sender_hdr, ismail));
 					break;
 
@@ -331,8 +334,9 @@ submit_inews (
 
 #if 0
 		if (*message_id && *a_message_id) { /* check if returned ID matches purposed ID */
-			if (strcmp(message_id, a_message_id))
+			if (strcmp(message_id, a_message_id)) {
 				; /* shouldn't happen - warn user? */
+			}
 		}
 #endif /* 0 */
 
@@ -364,35 +368,46 @@ submit_news_file (
 	checknadd_headers (name);
 
 	/* 7bit ISO-2022-KR is NEVER to be used in Korean news posting. */
-	if (!(strcasecmp(tinrc.mm_charset, "euc-kr") || strcasecmp(txt_mime_encodings[tinrc.post_mime_encoding], txt_7bit)))
+#ifdef CHARSET_CONVERSION
+	if (!(strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-KR") || strcasecmp(txt_mime_encodings[tinrc.post_mime_encoding], txt_7bit)))
+#else
+	if (!(strcasecmp(tinrc.mm_charset, "EUC-KR") || strcasecmp(txt_mime_encodings[tinrc.post_mime_encoding], txt_7bit)))
+#endif /* CHARSET_CONVERSION */
 		tinrc.post_mime_encoding = 0;	/* FIXME: txt_8bit */
 
 	rfc15211522_encode(name, txt_mime_encodings[tinrc.post_mime_encoding], tinrc.post_8bit_header, ismail);
 
 #ifdef NNTP_INEWS
-	if (read_news_via_nntp && !read_saved_news && tinrc.use_builtin_inews) {
+	if (read_news_via_nntp && !read_saved_news && 0 == strcasecmp(tinrc.inews_prog, "--internal")) {
 		ret_code = submit_inews (name, a_message_id);
 	} else
 #endif /* NNTP_INEWS */
 		{
 #ifdef M_UNIX
+		/* use 'inewsdir/inews -h' or tinrc.inews_prog or 'inews -h' */
 		if (*inewsdir) {
 			strcpy (buf, inewsdir);
-			strcat (buf, "/");
-			cp += strlen (cp);
-		}
-		sh_format (cp, sizeof(buf) - (cp - buf), "inews -h < %s", name);
+			strcat (buf, "/inews -h");
+		} else if (0 != strcasecmp(tinrc.inews_prog, "--internal"))
+			strcpy (buf, tinrc.inews_prog);
+		else
+			strcpy (buf, "inews -h");
+
+		cp += strlen (cp);
+
+		sh_format (cp, sizeof(buf) - (cp - buf), " < %s", name);
 #else
 		make_post_cmd (cp, name);
 #endif /* M_UNIX */
 
 		ret_code = invoke_cmd (buf);
 #ifdef NNTP_INEWS
-		if (!ret_code && read_news_via_nntp && !read_saved_news && !tinrc.use_builtin_inews) {
+		if (!ret_code && read_news_via_nntp && !read_saved_news && 0 != strcasecmp(tinrc.inews_prog, "--internal")) {
 			if (prompt_yn(cLINES, _(txt_post_via_builtin_inews), TRUE)) {
 				ret_code = submit_inews (name, a_message_id);
 				if (ret_code) {
-					tinrc.use_builtin_inews = prompt_yn(cLINES, _(txt_post_via_builtin_inews_only), TRUE);
+					if (prompt_yn(cLINES, _(txt_post_via_builtin_inews_only), TRUE)==1)
+						strcpy(tinrc.inews_prog,"--internal");
 				}
 			}
 		}
