@@ -3,10 +3,10 @@
  *  Module    : init.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 1997-12-28
+ *  Updated   : 2001-11-10
  *  Notes     :
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,10 +48,10 @@
 #	include "trace.h"
 #endif /* !included_trace_h */
 #ifndef MENUKEYS_H
-#	include  "menukeys.h"
+#	include "menukeys.h"
 #endif /* !MENUKEYS_H */
 #ifndef VERSION_H
-#	include  "version.h"
+#	include "version.h"
 #endif /* !VERSION_H */
 #ifndef BUGREP_H
 #	include "bugrep.h"
@@ -68,7 +68,7 @@ static int read_site_config(void);
 
 
 char active_times_file[PATH_LEN];
-char article[PATH_LEN];			/* ~/.article file */
+char article[PATH_LEN];			/* ~/TIN_ARTICLE_NAME file */
 char bug_nntpserver1[PATH_LEN];		/* welcome message of NNTP server used */
 char bug_nntpserver2[PATH_LEN];		/* welcome message of NNTP server used */
 char cmdline_nntpserver[PATH_LEN];
@@ -119,19 +119,15 @@ char subscriptions_file[PATH_LEN];
 char txt_help_bug_report[LEN];		/* address to send bug reports to */
 char userid[PATH_LEN];
 
-char proc_ch_default;				/* set in change_config_file () */
+char proc_ch_default;			/* set in change_config_file () */
 
 #ifdef VMS
 	char rcdir_asfile[PATH_LEN];	/* rcdir expressed as dev:[dir]tin.dir, for stat() */
 #endif /* VMS */
 
-#ifdef M_OS2
-	char TMPDIR[PATH_LEN];
-#endif /* M_OS2 */
-
 int groupname_len = 0;			/* 'runtime' copy of groupname_max_len */
-int hist_last[HIST_MAXNUM+1];
-int hist_pos[HIST_MAXNUM+1];
+int hist_last[HIST_MAXNUM + 1];
+int hist_pos[HIST_MAXNUM + 1];
 int iso2asc_supported;			/* Convert ISO-Latin1 to Ascii */
 int num_headers_to_display;		/* num headers to display -- swp */
 int num_headers_to_not_display;		/* num headers to not display -- swp */
@@ -166,7 +162,6 @@ t_bool read_local_newsgroups_file;	/* read newsgroups file locally or via NNTP *
 t_bool read_news_via_nntp = FALSE;	/* read news locally or via NNTP */
 t_bool read_saved_news = FALSE;		/* tin -R read saved news from tin -S */
 t_bool show_description = TRUE;		/* current copy of tinrc flag */
-t_bool tex2iso_supported;		/* Convert german style TeX to ISO-Latin1 */
 t_bool batch_mode;			/* update index files only mode */
 t_bool verbose = FALSE;			/* update index files only mode */
 t_bool (*wildcard_func) (const char *str, char *patt, t_bool icase);		/* Wildcard matching function */
@@ -183,7 +178,7 @@ t_bool xref_supported = TRUE;
 t_menu *currmenu;
 
 /* History entries */
-char *input_history[HIST_MAXNUM+1][HIST_SIZE+1];
+char *input_history[HIST_MAXNUM + 1][HIST_SIZE + 1];
 
 #ifdef HAVE_SYS_UTSNAME_H
 	struct utsname system_info;
@@ -245,8 +240,12 @@ struct t_config tinrc = {
 	"In article %M you wrote:",		/* mail_quote_format */
 	"",		/* maildir */
 	"",		/* mail_address */
-	"",		/* mm_charset */
-	"Newsgroups Followup-To Summary Keywords",		/* news_headers_to_display */
+	"",		/* mm_charset, defaults to $MM_CHARSET */
+#ifdef CHARSET_CONVERSION
+	"",		/* mm_local_charset, defaults to mm_charset */
+	0,			/* mm_network_charset, defaults to US-ASCII */
+#endif /* CHARSET_CONVERSION */
+	"Newsgroups Followup-To Summary Keywords X-Comment-To",		/* news_headers_to_display */
 	"",		/* news_headers_to_not_display */
 	"%F wrote:",		/* news_quote_format */
 	DEFAULT_COMMENT,	/* quote_chars */
@@ -275,7 +274,8 @@ struct t_config tinrc = {
 	POST_PROC_NONE,			/* post_process */
 	REREAD_ACTIVE_FILE_SECS,	/* reread_active_file_secs */
 	SHOW_FROM_NAME,				/* show_author */
-	SORT_BY_DATE_ASCEND,		/* sort_article_type */
+	SORT_ARTICLES_BY_DATE_ASCEND,		/* sort_article_type */
+	SORT_THREADS_BY_SCORE_DESCEND,		/* sort_threads_type */
 	BOGUS_ASK,		/* strip_bogus */
 	THREAD_MAX,		/* thread_articles */
 	0,		/* Default to wildmat, not regex */
@@ -360,7 +360,6 @@ struct t_config tinrc = {
 	FALSE,		/* show_only_unread_groups */
 	FALSE,		/* show_score */
 	TRUE,		/* show_signatures */
-	FALSE,		/* show_xcommentto */
 	FALSE,		/* hide_uue */
 	TRUE,		/* sigdashes */
 	TRUE,		/* signature_repost */
@@ -374,9 +373,10 @@ struct t_config tinrc = {
 	FALSE,		/* strip_newsrc */
 	FALSE,		/* tab_after_X_selection */
 	TRUE,		/* tab_goto_next_unread */
+	FALSE,		/* tex2iso_conv */
 	TRUE,		/* thread_catchup_on_exit */
 	TRUE,		/* unlink_article */
-	TRUE,		/* use_builtin_inews */
+	"",		/* inews_prog */
 	FALSE,		/* use_getart_limit */
 	FALSE,		/* use_mailreader_i */
 	FALSE,		/* use_mouse */
@@ -517,56 +517,26 @@ init_selfinfo (
 	setlocale (LC_ALL, "");
 #endif /* HAVE_SETLOCALE && LC_ALL && !NO_LOCALE */
 
+
 /* FIXME: move to get_user_name() [header.c] */
-#ifdef M_AMIGA
-	if ((ptr = getenv ("USERNAME")) != (char *) 0) {
-		my_strncpy (userid, ptr, sizeof (userid));
-	} else {
-		error_message (txt_env_var_not_found, "USERNAME");
-		tin_done (EXIT_FAILURE);
-	}
-	if ((ptr = getenv ("HOME")) != (char *) 0) {
-		my_strncpy (homedir, ptr, sizeof (homedir));
-	} else {
-		error_message (txt_env_var_not_found, "HOME");
-		tin_done (EXIT_FAILURE);
-	}
-#else
-	myentry = (struct passwd *) 0;
-
-	if (myentry == (struct passwd *) 0)
-		myentry = getpwuid (getuid ());
-
-	if (myentry != (struct passwd *) 0) {
+#ifndef M_AMIGA
+	if ((myentry = getpwuid (getuid ())) != (struct passwd *) 0) {
 		memcpy (&pwdentry, myentry, sizeof (struct passwd));
 		myentry = &pwdentry;
 	}
-
-/* we might add a check for $LOGNAME here */
-
-#	if defined(M_OS2) || defined(WIN32)
-	if (myentry == (struct passwd *) 0) {
-		fprintf (stderr, "Environment variable USER not set.\n");
-		giveup();
-	}
-#	else
-#		ifdef VMS
+#	ifdef VMS
+	/* TODO: get_user_name() entirely reles on $USER */
 	if (((ptr = getlogin ()) != (char *) 0) && strlen (ptr))
 		myentry = getpwnam (ptr);
-#		endif /* VMS */
-#	endif /* M_OS2 || WIN32 */
+#	endif /* VMS */
+
+	if (myentry != (struct passwd *) 0)
+		strcpy (userid, myentry->pw_name);
+	else {
+		error_message (_(txt_error_passwd_missing));
+		tin_done (EXIT_FAILURE);
+	}
 
-#	ifdef M_OS2
-	strcpy (TMPDIR, get_val ("TMP", "/tmp/"));
-	if ((TMPDIR[strlen(TMPDIR)-1] != '/') && (TMPDIR[strlen(TMPDIR)-1] != '\\'))
-		strcat(TMPDIR,"/");
-#	endif /* M_OS2 */
-#	ifdef WIN32
-	strcpy (TMPDIR, get_val ("TMP", "\\tmp\\"));
-	if (TMPDIR[strlen(TMPDIR)-1] != '\\')
-		strcat(TMPDIR,"\\");
-#	endif /* WIN32 */
-	strcpy (userid, myentry->pw_name);
 #	ifdef VMS
 	lower (userid);
 #	endif /* VMS */
@@ -579,13 +549,33 @@ init_selfinfo (
 		strcpy (homedir, "/tmp");
 	} else
 		my_strncpy (homedir, myentry->pw_dir, sizeof (homedir));
-#endif /* M_AMIGA */
+
+#else
+	/* TODO: get_user_name() uses $USER, not $USERNAME */
+	if ((ptr = getenv ("USERNAME")) != (char *) 0) {
+		my_strncpy (userid, ptr, sizeof (userid));
+	} else {
+		error_message (_(txt_env_var_not_found), "USERNAME");
+		tin_done (EXIT_FAILURE);
+	}
+	/* TODO: why not also check for TIN_HOME? */
+	if ((ptr = getenv ("HOME")) != (char *) 0) {
+		my_strncpy (homedir, ptr, sizeof (homedir));
+	} else {
+		error_message (_(txt_env_var_not_found), "HOME");
+		tin_done (EXIT_FAILURE);
+	}
+#endif /* !M_AMIGA */
 
 	cmdline_nntpserver[0] = '\0';
 	created_rcdir = FALSE;
 	dangerous_signal_exit = FALSE;
 	disable_gnksa_domain_check = FALSE;
+#ifdef MAC_OS_X	/* usualy they don't have a valid FQDN */
+	disable_sender = TRUE;
+#else
 	disable_sender = FALSE;
+#endif /* MAC_OS_X */
 	filtered_articles = FALSE;
 	iso2asc_supported = atoi (get_val ("ISO2ASC", DEFAULT_ISO2ASC));
 	if (iso2asc_supported > NUM_ISO_TABLES)
@@ -601,7 +591,6 @@ init_selfinfo (
 	read_local_newsgroups_file = FALSE;
 	force_reread_active_file = TRUE;
 	reread_active_for_posted_arts = TRUE;
-	tex2iso_supported = (atoi (get_val ("TEX2ISO", "0")) != 0) ? TRUE : FALSE;
 	batch_mode = FALSE;
 	check_for_new_newsgroups = !batch_mode;
 	wildcard_func = wildmat;
@@ -623,9 +612,8 @@ init_selfinfo (
 	index_savedir[0] = '\0';
 	newsrc[0] = '\0';
 
-	sprintf (page_header, "%s %s release %s (\"%s\") [%s%s%s]",
+	sprintf (page_header, "%s %s release %s (\"%s\") [%s%s]",
 		tin_progname, VERSION, RELEASEDATE, RELEASENAME, OSNAME,
-		(tex2iso_supported ? " TeX2ISO" : ""),
 		(iso2asc_supported >= 0 ? " ISO2ASC" : ""));
 	sprintf (cvers, txt_copyright_notice, page_header);
 
@@ -760,14 +748,13 @@ init_selfinfo (
 		strcpy(tinrc.printer, DEFAULT_BBS_PRINTER);
 #	endif /* M_AMIGA */
 #endif /* !DISABLE_PRINTING */
+#if defined(NNTP_ABLE) || defined(NNTP_ONLY)
+	strcpy (tinrc.inews_prog, DEFAULT_INEWS_PROG);
+#endif /* NNTP_ABLE) || NNTP_ONLY */
 	strcpy (mailer, get_val (ENV_VAR_MAILER, DEFAULT_MAILER));
-#ifdef VMS
-	joinpath (article, homedir, "article.");
-#else
-	joinpath (article, homedir, ".article");
-#endif /* VMS */
+	joinpath (article, homedir, TIN_ARTICLE_NAME);
 #ifdef APPEND_PID
-	sprintf (article+strlen(article), ".%d", (int) process_id);
+	sprintf (article + strlen(article), ".%d", (int) process_id);
 #endif /* APPEND_PID */
 	joinpath (dead_article, homedir, "dead.article");
 	joinpath (dead_articles, homedir, "dead.articles");
@@ -805,16 +792,11 @@ init_selfinfo (
 	joinpath (mailbox, DEFAULT_MAILBOX, userid);
 #endif /* VMS */
 	joinpath (mailgroups_file, rcdir, MAILGROUPS_FILE);
-#ifdef WIN32
-	joinpath (newsrc, rcdir, NEWSRC_FILE);
-	joinpath (newnewsrc, rcdir, NEWNEWSRC_FILE);
-#else
 	joinpath (newsrc, homedir, NEWSRC_FILE);
 	joinpath (newnewsrc, homedir, NEWNEWSRC_FILE);
-#	ifdef APPEND_PID
-	sprintf(newnewsrc+strlen(newnewsrc), "%d", (int) process_id);
-#	endif /* APPEND_PID */
-#endif /* WIN32 */
+#ifdef APPEND_PID
+	sprintf(newnewsrc + strlen(newnewsrc), "%d", (int) process_id);
+#endif /* APPEND_PID */
 	joinpath (posted_info_file, rcdir, POSTED_FILE);
 	joinpath (posted_msgs_file, tinrc.maildir, POSTED_FILE);
 	joinpath (postponed_articles_file, rcdir, POSTPONED_FILE);
@@ -912,7 +894,7 @@ create_mail_save_dirs (
 		created = TRUE;
 	}
 
-	return (created);
+	return created;
 }
 
 
