@@ -3,10 +3,10 @@
  *  Module    : keymap.c
  *  Author    : D. Nimmich, J. Faultless
  *  Created   : 2000-05-25
- *  Updated   : 2000-06-10
+ *  Updated   : 2001-11-10
  *  Notes     : This file contains key mapping routines and variables.
  *
- * Copyright (c) 2000-2001 Dirk Nimmich <nimmich@uni-muenster.de>
+ * Copyright (c) 2000-2002 Dirk Nimmich <nimmich@uni-muenster.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,7 +50,7 @@ static t_bool processkey(t_keynode *keyptr, char *kname, char key);
 
 char *ch_post_process;
 
-struct keymap Key = {
+static struct keymap Key = {
 	{	/* Global keys */
 		{ 0, 0, "" },
 		{ iKeyPageUp, iKeyPageUp, "PageUp" },
@@ -100,6 +100,9 @@ struct keymap Key = {
 		{ iKeyToggleInfoLastLine, iKeyToggleInfoLastLine, "ToggleInfoLastLine" },
 		{ iKeyDown2, iKeyDown2, "Down2" },
 		{ iKeyUp2, iKeyUp2, "Up2" },
+#ifndef DISABLE_PRINTING
+		{ iKeyPrint, iKeyPrint, "Print" },
+#endif /* !DISABLE_PRINTING */
 		{ iKeyQuit, iKeyQuit, "Quit" },
 		{ iKeyVersion, iKeyVersion, "Version" },
 		{ iKeyPost, iKeyPost, "Post" },
@@ -171,9 +174,6 @@ struct keymap Key = {
 		{ iKeyGroupListThd, iKeyGroupListThd, "ListThd" },
 		{ iKeyGroupMail, iKeyGroupMail, "Mail" },
 		{ iKeyGroupNextGroup, iKeyGroupNextGroup, "NextGroup" },
-#ifndef DISABLE_PRINTING
-		{ iKeyGroupPrint, iKeyGroupPrint, "Print" },
-#endif /* !DISABLE_PRINTING */
 		{ iKeyGroupPrevGroup, iKeyGroupPrevGroup, "PrevGroup" },
 		{ iKeyGroupToggleReadUnread, iKeyGroupToggleReadUnread, "ToggleReadUnread" },
 		{ iKeyGroupSave, iKeyGroupSave, "Save" },
@@ -245,9 +245,6 @@ struct keymap Key = {
 		{ iKeyPageListThd, iKeyPageListThd, "ListThd" },
 		{ iKeyPageMail, iKeyPageMail, "Mail" },
 		{ iKeyPageNextArt, iKeyPageNextArt, "NextArt" },
-#ifndef DISABLE_PRINTING
-		{ iKeyPagePrint, iKeyPagePrint, "Print" },
-#endif /* !DISABLE_PRINTING */
 		{ iKeyPagePrevArt, iKeyPagePrevArt, "PrevArt" },
 		{ iKeyPageReplyQuote, iKeyPageReplyQuote, "ReplyQuote" },
 		{ iKeyPageSave, iKeyPageSave, "Save" },
@@ -364,7 +361,7 @@ struct keymap Key = {
 };
 
 /* NULL terminated list of pointers to the start of all the keygroups */
-t_keynode *keygroups[] = {
+static t_keynode *keygroups[] = {
 	&Key.Global.tag,		/* It is important that global be 1st for duplicate checking */
 	&Key.Config.tag,
 	&Key.Feed.tag,
@@ -386,7 +383,7 @@ t_keynode *keygroups[] = {
 
 /* Keymaps for various menus and screens */
 
-t_keynode *keys_config_change[] = {
+static t_keynode *keys_config_change[] = {
 	&Key.Global.Quit, &Key.Config.NoSave, &Key.Global.Up, &Key.Global.Up2,
 	&Key.Global.Down, &Key.Global.Down2, &Key.Global.FirstPage,
 	&Key.Config.FirstPage2, &Key.Global.LastPage, &Key.Config.LastPage2,
@@ -398,7 +395,7 @@ t_keynode *keys_config_change[] = {
 	&Key.Global.Six, &Key.Global.Seven, &Key.Global.Eight, &Key.Global.Nine,
 	NULL };
 
-t_keynode *keys_feed_art_thread_regex_tag[] = {
+static t_keynode *keys_feed_art_thread_regex_tag[] = {
 	&Key.Global.Abort, &Key.Global.Quit, &Key.Feed.Art, &Key.Feed.Hot,
 	&Key.Feed.Thd, &Key.Feed.Tag, &Key.Feed.Pat, NULL };
 
@@ -424,20 +421,20 @@ t_keynode *keys_feed_art_thread_regex_tag[] = {
  * mind that this will probably confuse users who are upgrading from an
  * older tin because the new action replaces the old one.
  */
-t_keynode *keys_feed_post_process_type[] = {
+static t_keynode *keys_feed_post_process_type[] = {
 	&Key.Global.Abort, &Key.Global.Quit, &Key.PProc.None, &Key.PProc.Shar,
 	&Key.PProc.UUDecode,
 	NULL };
 
-t_keynode *keys_feed_supersede_article[] = {
+static t_keynode *keys_feed_supersede_article[] = {
 	&Key.Global.Abort, &Key.Global.Quit, &Key.Feed.Repost, &Key.Feed.Supersede,
 	NULL };
 
-t_keynode *keys_filter_quit_edit_save[] = {
+static t_keynode *keys_filter_quit_edit_save[] = {
 	&Key.Global.Abort, &Key.Global.Quit, &Key.Filter.Edit, &Key.Filter.Save,
 	NULL };
 
-t_keynode *keys_group_nav[] = {
+static t_keynode *keys_group_nav[] = {
 	&Key.Global.Abort, &Key.Global.One, &Key.Global.Two, &Key.Global.Three,
 	&Key.Global.Four, &Key.Global.Five, &Key.Global.Six, &Key.Global.Seven,
 	&Key.Global.Eight, &Key.Global.Nine,
@@ -447,7 +444,7 @@ t_keynode *keys_group_nav[] = {
 	&Key.Global.FirstPage, &Key.Global.LastPage, &Key.Global.LastViewed,
 	&Key.Global.Pipe, &Key.Group.Mail,
 #ifndef DISABLE_PRINTING
-	&Key.Group.Print,
+	&Key.Global.Print,
 #endif /* !DISABLE_PRINTING */
 	&Key.Group.Repost, &Key.Group.Save, &Key.Group.AutoSaveTagged,
 	&Key.Global.SetRange, &Key.Global.SearchAuthF, &Key.Global.SearchAuthB,
@@ -479,11 +476,20 @@ t_keynode *keys_group_nav[] = {
 	&Key.Group.MarkUnselArtRead, &Key.Group.DoAutoSel,
 	&Key.Global.ToggleInfoLastLine, NULL };
 
-t_keynode *keys_nrctbl_create[] = {
+static t_keynode *keys_info_nav[] = {
+	&Key.Global.Abort, &Key.Global.MouseToggle, &Key.Global.Up,
+	&Key.Global.Up2, &Key.Global.Down, &Key.Global.Down2, &Key.Global.PageDown,
+	&Key.Global.PageDown2, &Key.Global.PageDown3, &Key.Global.PageUp,
+	&Key.Global.PageUp2, &Key.Global.PageUp3, &Key.Global.FirstPage,
+	&Key.Help.FirstPage2, &Key.Global.LastPage, &Key.Help.LastPage2,
+	&Key.Global.ToggleHelpDisplay, &Key.Global.SearchSubjF,
+	&Key.Global.SearchSubjB, &Key.Global.Quit, NULL };
+
+static t_keynode *keys_nrctbl_create[] = {
    &Key.Global.Abort, &Key.Nrctbl.Quit, &Key.Nrctbl.Alternative,
    &Key.Nrctbl.Create, &Key.Nrctbl.Default, NULL };
 
-t_keynode *keys_page_nav[] = {
+static t_keynode *keys_page_nav[] = {
 	&Key.Global.Abort, &Key.Global.Zero, &Key.Global.One, &Key.Global.Two,
 	&Key.Global.Three, &Key.Global.Four, &Key.Global.Five, &Key.Global.Six,
 	&Key.Global.Seven, &Key.Global.Eight, &Key.Global.Nine,
@@ -498,11 +504,12 @@ t_keynode *keys_page_nav[] = {
 	&Key.Global.LastViewed, &Key.Global.LookupMessage, &Key.Page.GotoParent,
 	&Key.Global.Pipe, &Key.Page.Mail,
 #ifndef DISABLE_PRINTING
-	&Key.Page.Print,
+	&Key.Global.Print,
 #endif /* !DISABLE_PRINTING */
 	&Key.Page.Repost, &Key.Page.Save, &Key.Page.AutoSaveTagged,
-	&Key.Global.SearchSubjF, &Key.Global.SearchSubjB, &Key.Global.SearchBody,
-	&Key.Page.TopThd, &Key.Page.BotThd, &Key.Page.NextThd, &Key.Page.NextThd2,
+	&Key.Global.SearchAuthF, &Key.Global.SearchAuthB, &Key.Global.SearchSubjF,
+	&Key.Global.SearchSubjB, &Key.Global.SearchBody, &Key.Page.TopThd,
+	&Key.Page.BotThd, &Key.Page.NextThd, &Key.Page.NextThd2,
 #ifdef HAVE_PGP_GPG
 	&Key.Page.PGPCheckArticle,
 #endif /* HAVE_PGP_GPG */
@@ -531,42 +538,42 @@ t_keynode *keys_page_nav[] = {
 #endif /* HAVE_COLOR */
 	&Key.Page.ViewAttach, &Key.Page.ViewUrl, NULL };
 
-t_keynode *keys_pgp_mail[] = {
+static t_keynode *keys_pgp_mail[] = {
 	&Key.Global.Abort, &Key.Global.Quit, &Key.Pgp.EncSign, &Key.Pgp.Encrypt,
 	&Key.Pgp.Sign, NULL };
 
-t_keynode *keys_pgp_news[] = {
+static t_keynode *keys_pgp_news[] = {
 	&Key.Global.Abort, &Key.Global.Quit, &Key.Pgp.Includekey, &Key.Pgp.Sign,
 	NULL };
 
-t_keynode *keys_post_cancel[] = {
+static t_keynode *keys_post_cancel[] = {
 	&Key.Global.Abort, &Key.Global.Quit, &Key.Post.Cancel, &Key.Post.Edit,
 	NULL };
 
-t_keynode *keys_post_cont[] = {
+static t_keynode *keys_post_cont[] = {
 	&Key.Global.Abort, &Key.Post.Abort, &Key.Post.Continue, NULL };
 
-t_keynode *keys_post_delete[] = {
+static t_keynode *keys_post_delete[] = {
 	&Key.Global.Abort, &Key.Global.Quit, &Key.Post.Cancel, &Key.Post.Supersede,
 	NULL };
 
-t_keynode *keys_post_edit[] = {
+static t_keynode *keys_post_edit[] = {
 	&Key.Global.Abort, &Key.Global.Quit, &Key.Post.Edit, &Key.Post.Postpone,
 	NULL };
 
-t_keynode *keys_post_edit_ext[] = {
+static t_keynode *keys_post_edit_ext[] = {
 	&Key.Global.Abort, &Key.Global.Quit, &Key.Post.Edit,
 	&Key.Global.OptionMenu, NULL };
 
-t_keynode *keys_post_ignore_fupto[] = {
+static t_keynode *keys_post_ignore_fupto[] = {
 	&Key.Global.Abort, &Key.Global.Quit, &Key.Post.Ignore, &Key.Global.Post,
 	&Key.Post.Post2, &Key.Post.Post3, NULL };
 
-t_keynode *keys_post_mail_fup[] = {
+static t_keynode *keys_post_mail_fup[] = {
 	&Key.Global.Abort, &Key.Global.Quit, &Key.Global.Post, &Key.Post.Post2,
 	&Key.Post.Post3, &Key.Post.Mail, NULL };
 
-t_keynode *keys_post_post[] = {
+static t_keynode *keys_post_post[] = {
 	&Key.Global.Abort, &Key.Global.Quit,
 #ifdef HAVE_PGP_GPG
 	&Key.Post.PGP,
@@ -577,11 +584,11 @@ t_keynode *keys_post_post[] = {
 	&Key.Post.Edit, &Key.Global.Post, &Key.Post.Post2, &Key.Post.Post3,
 	&Key.Post.Postpone, NULL };
 
-t_keynode *keys_post_postpone[] = {
+static t_keynode *keys_post_postpone[] = {
 	&Key.Global.Abort, &Key.Global.Quit, &Key.Prompt.Yes,
 	&Key.Postpone.All, &Key.Postpone.Override, &Key.Prompt.No, NULL };
 
-t_keynode *keys_post_send[] = {
+static t_keynode *keys_post_send[] = {
 	&Key.Global.Abort, &Key.Global.Quit,
 #ifdef HAVE_PGP_GPG
 	&Key.Post.PGP,
@@ -591,14 +598,14 @@ t_keynode *keys_post_send[] = {
 #endif /* HAVE_ISPELL */
 	&Key.Post.Edit, &Key.Post.Send, &Key.Post.Send2, NULL };
 
-t_keynode *keys_prompt_yn[] = {
+static t_keynode *keys_prompt_yn[] = {
 	&Key.Global.Abort, &Key.Prompt.Yes, &Key.Prompt.No, NULL };
 
-t_keynode *keys_save_append_overwrite_quit[] = {
+static t_keynode *keys_save_append_overwrite_quit[] = {
 	&Key.Global.Abort, &Key.Global.Quit, &Key.Save.AppendFile,
 	&Key.Save.OverwriteFile, NULL };
 
-t_keynode *keys_select_nav[] = {
+static t_keynode *keys_select_nav[] = {
 	&Key.Global.Abort, &Key.Global.One, &Key.Global.Two, &Key.Global.Three,
 	&Key.Global.Four, &Key.Global.Five, &Key.Global.Six, &Key.Global.Seven,
 	&Key.Global.Eight, &Key.Global.Nine,
@@ -629,7 +636,7 @@ t_keynode *keys_select_nav[] = {
 	&Key.Select.YankActive, &Key.Select.SyncWithActive,
 	&Key.Select.MarkGrpUnread, &Key.Select.MarkGrpUnread2, NULL };
 
-t_keynode *keys_thread_nav[] = {
+static t_keynode *keys_thread_nav[] = {
 	&Key.Global.Abort, &Key.Global.One, &Key.Global.Two, &Key.Global.Three,
 	&Key.Global.Four, &Key.Global.Five, &Key.Global.Six, &Key.Global.Seven,
 	&Key.Global.Eight, &Key.Global.Nine,
@@ -653,6 +660,7 @@ t_keynode *keys_thread_nav[] = {
 #ifdef HAVE_COLOR
 	&Key.Global.ToggleColor,
 #endif /* HAVE_COLOR */
+	&Key.Global.OptionMenu,
 	&Key.Global.Quit, &Key.Global.QuitTin, &Key.Thread.Tag,
 	&Key.Thread.BugReport, &Key.Thread.Untag, &Key.Global.Version,
 	&Key.Thread.MarkArtUnread, &Key.Thread.MarkThdUnread, &Key.Thread.SelArt,
@@ -667,6 +675,7 @@ t_menukeymap menukeymap = {
 	{ keys_feed_supersede_article, NULL, NULL },
 	{ keys_filter_quit_edit_save, NULL, NULL },
 	{ keys_group_nav, NULL, NULL },
+	{ keys_info_nav, NULL, NULL },
 	{ keys_nrctbl_create, NULL, NULL },
 	{ keys_page_nav, NULL, NULL },
 	{ keys_pgp_mail, NULL, NULL },
@@ -803,11 +812,15 @@ printascii (
 	char *buf,
 	int ch)
 {
-	if (isgraph(ch)) {			/* Regular printables */
+	if ((ch == 0) || isgraph(ch)) {	/* Regular printables */
 		buf[0] = ch;
 		buf[1] = '\0';
 	} else if (ch == '\t') {	/* TAB */
 		strcpy (buf, _(txt_tab));
+	} else if ((ch == '\n') || (ch == '\r')) {	/* LF, CR */
+		strcpy (buf, _(txt_cr));
+	} else if (ch == ESC) {		/* Escape */
+		strcpy (buf, _(txt_esc));
 	} else if (iscntrl(ch)) {	/* Control keys */
 		buf[0] = '^';
 		buf[1] = (ch & 0xFF) + '@';
@@ -815,7 +828,7 @@ printascii (
 	} else if (ch == ' ')		/* SPACE */
 		strcpy (buf, _(txt_space));
 	else
-		strcpy (buf, "???");	/* Never happen ? */
+		strcpy (buf, "???");	/* Never happens? */
 
 	return buf;
 }
@@ -869,7 +882,7 @@ processkey(
 	i = strlen (buf);
 
 	for (; keyptr->t != NULL; ++keyptr) {
-		strcpy (buf+i, keyptr->t);
+		strcpy (buf + i, keyptr->t);
 
 		if (strcasecmp (kname, buf) == 0) {
 #ifdef DEBUG
@@ -891,8 +904,8 @@ read_keymap_file (
 {
 	FILE *fp = (FILE *) 0;
 	char *line, *keydef, *kname;
-	char *ptr;
-	const char *ptr2, *map;
+	char *map, *ptr;
+	const char *ptr2;
 	char buf[LEN], buff[LEN];
 	char key;
 	int i;
@@ -910,32 +923,34 @@ read_keymap_file (
 	 */
 	ptr2 = get_val("TIN_HOMEDIR", get_val("HOME", homedir));
 	/* get locale suffix */
-	map = get_val("LC_ALL", get_val("LC_CTYPE", get_val("LC_MESSAGES", get_val("LANG", ""))));
+	map = my_strdup(get_val("LC_ALL", get_val("LC_CTYPE", get_val("LC_MESSAGES", get_val("LANG", "")))));
 	if (strlen(map)) {
 		if ((ptr = strchr (map, '.')))
 				*ptr = '\0';
-		snprintf(buff, sizeof(buf) - 1, "%s/.tin/keymap.%s", ptr2, map);
+		snprintf(buff, sizeof(buff) - 1, "%s/.tin/keymap.%s", ptr2, map);
 		if (strfpath (buff, buf, sizeof(buf), &CURR_GROUP))
 			fp = fopen (buf, "r");
 	}
 	if (!fp) {
-		snprintf(buff, sizeof(buf) - 1, "%s/.tin/keymap", ptr2);
+		snprintf(buff, sizeof(buff) - 1, "%s/.tin/keymap", ptr2);
 		if (strfpath (buff, buf, sizeof(buf), &CURR_GROUP))
 			fp = fopen (buf, "r");
 	}
 #ifdef TIN_DEFAULTS_DIR
 	if (strlen(map) && !fp) {
-		snprintf(buff, sizeof(buf) - 1, "%s/keymap.%s", TIN_DEFAULTS_DIR, map);
+		snprintf(buff, sizeof(buff) - 1, "%s/keymap.%s", TIN_DEFAULTS_DIR, map);
 		if (strfpath (buff, buf, sizeof(buf), &CURR_GROUP))
 			fp = fopen (buf, "r");
 	}
 	if (!fp) {
-		snprintf(buff, sizeof(buf) - 1, "%s/keymap", TIN_DEFAULTS_DIR);
+		snprintf(buff, sizeof(buff) - 1, "%s/keymap", TIN_DEFAULTS_DIR);
 		if (strfpath (buff, buf, sizeof(buf), &CURR_GROUP))
 			fp = fopen (buf, "r");
 	}
 #endif /* TIN_DEFAULTS_DIR */
 
+	FreeIfNeeded(map);
+
 	if (!fp)
 		return FALSE;
 
