@@ -6,7 +6,7 @@
  *  Updated   : 1997-01-07
  *  Notes     : Low level functions handling the active[] list and its group_hash index
  *
- * Copyright (c) 1993-2001 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1993-2002 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -57,6 +57,7 @@ init_group_hash (
 	}
 }
 
+
 /*
  * hash group name for fast lookup later
  */
@@ -95,10 +96,11 @@ my_printf ("hash=[%s] [%ld]\n", group, val);
 	}
 	hash_value %= TABLE_SIZE;
 
-	return (hash_value);
+	return hash_value;
 #endif /* NEW_HASH_METHOD */
 }
 
+
 /*
  *  Find group name in active[] array and return index otherwise -1
  */
@@ -125,6 +127,7 @@ find_group_index (
 	return -1;
 }
 
+
 /*
  *  Find group name in active[] array and return pointer to element
  */
@@ -140,6 +143,7 @@ group_find (
 	return (struct t_group *) 0;
 }
 
+
 /*
  * Make sure memory available for next active slot
  * Adds group to the group_hash of active groups and name it
@@ -173,5 +177,5 @@ group_add (
 
 	active[num_active].name = my_strdup(group);
 
-	return(&active[num_active++]);
+	return &active[num_active++];
 }
