@@ -3,10 +3,10 @@
  *  Module    : main.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 1997-12-28
+ *  Updated   : 2001-07-22
  *  Notes     :
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,11 +45,8 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 #ifndef VERSION_H
-#	include  "version.h"
+#	include "version.h"
 #endif /* !VERSION_H */
-#ifndef BUGREP_H
-#	include  "bugrep.h"
-#endif /* !BUGREP_H */
 
 #if defined(M_AMIGA) && defined(__SASC_650)
 	extern int _WBArg;
@@ -68,11 +65,11 @@ static char **cmdargs;
 static int num_cmdargs;
 static int max_cmdargs;
 
-t_bool catchup = FALSE;			/* mark all arts read in all subscribed groups */
-t_bool check_any_unread = FALSE;/* print/return status if any unread */
-t_bool mail_news = FALSE;		/* mail all arts to specified user */
-t_bool save_news = FALSE;		/* save all arts to savedir structure */
-t_bool start_any_unread = FALSE;/* only start if unread news */
+static t_bool catchup = FALSE;			/* mark all arts read in all subscribed groups */
+static t_bool check_any_unread = FALSE;/* print/return status if any unread */
+static t_bool mail_news = FALSE;		/* mail all arts to specified user */
+static t_bool save_news = FALSE;		/* save all arts to savedir structure */
+static t_bool start_any_unread = FALSE;/* only start if unread news */
 
 
 /*
@@ -98,9 +95,13 @@ main (
 	t_bool tmp_no_write;
 
 	/* initialize locale support */
+#if defined(HAVE_SETLOCALE) && !defined(NO_LOCALE)
 	setlocale(LC_ALL, "");
+#	ifdef ENABLE_NLS
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
+#	endif /* ENABLE_NLS */
+#endif /* HAVE_SETLOCALE && !NO_LOCALE */
 
 	set_signal_handlers ();
 
@@ -111,8 +112,7 @@ main (
 	/* Call tzset() here!*/
 	_TZ = "GMT0";
 	tzset();
-	if (argc == 0) /* we are running from the Workbench */
-	{
+	if (argc == 0) { /* we are running from the Workbench */
 		argc = _WBArgc;
 		argv = _WBArgv;
 	}
@@ -438,6 +438,7 @@ read_cmd_line_options (
 				tin_bbs_mode = TRUE;
 				break;
 #	endif /* !M_AMIGA */
+
 			case 'c':
 				catchup = TRUE;
 				batch_mode = TRUE;
@@ -613,26 +614,29 @@ read_cmd_line_options (
 #ifdef TIN_CC
 				error_message("Compiler:");
 				error_message("\tCC       = \"%s\"", TIN_CC);
-#	ifdef TIN_CFLAGS
+#		ifdef TIN_CFLAGS
 				error_message("\tCFLAGS   = \"%s\"", TIN_CFLAGS);
-#	endif /* TIN_CFLAGS */
-#	ifdef	TIN_CPP
+#		endif /* TIN_CFLAGS */
+#	ifdef TIN_CPP
 				error_message("\tCPP      = \"%s\"", TIN_CPP);
 #	endif /* TIN_CPP */
-#	ifdef TIN_CPPFLAGS
+#		ifdef TIN_CPPFLAGS
 				error_message("\tCPPFLAGS = \"%s\"", TIN_CPPFLAGS);
-#	endif /* TIN_CPPFLAGS */
+#		endif /* TIN_CPPFLAGS */
 #endif /* TIN_CC */
+
 #ifdef TIN_LD
 				error_message("Linker and Libraries:");
 				error_message("\tLD       = \"%s\"", TIN_LD);
-#	ifdef TIN_LDFLAGS
+#		ifdef TIN_LDFLAGS
 				error_message("\tLDFLAGS  = \"%s\"", TIN_LDFLAGS);
-#	endif /* TIN_LDFLAGS */
+#		endif /* TIN_LDFLAGS */
+
 #	ifdef TIN_LIBS
 				error_message("\tLIBS     = \"%s\"", TIN_LIBS);
 #	endif /* TIN_LIBS */
 #endif /* TIN_LD */
+
 				error_message("Characteristics:");
 				error_message("\t"
 /* TODO: complete list and do some usefull grouping */
@@ -785,6 +789,9 @@ read_cmd_line_options (
 		}
 	}
 
+	/*
+	 * Sort out conflicts of options....
+	 */
 	if (verbose && !batch_mode) {
 		wait_message(1, _("-v only useful for batch mode operations\n"));
 		verbose = FALSE;
@@ -795,9 +802,6 @@ read_cmd_line_options (
 		read_saved_news = FALSE;
 	}
 
-	/*
-	 * Sort out conflicts of options....
-	 */
 #ifdef NNTP_ABLE
 	/*
 	 * If we're reading from an NNTP server and we've been asked not to look
@@ -946,7 +950,7 @@ static void
 show_intro_page (
 	void)
 {
-	char buf[4096]; /* should be enoght */
+	char buf[4096];
 
 	if (!cmd_line) {
 		ClearScreen ();
@@ -955,7 +959,7 @@ show_intro_page (
 		my_printf("\n");
 	}
 
-	sprintf(buf, _(txt_intro_page), bug_addr);
+	snprintf(buf, sizeof(buf) - 1, _(txt_intro_page), PRODUCT, PRODUCT, PRODUCT, bug_addr);
 
 	my_fputs (buf, stdout);
 	my_flush();
@@ -997,6 +1001,7 @@ read_cmd_line_groups (
 	return matched;
 }
 
+
 void
 giveup (
 	void)
