@@ -3,10 +3,10 @@
  *  Module    : misc.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 1997-12-31
+ *  Updated   : 2001-11-10
  *  Notes     :
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,12 +44,6 @@
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
-#ifndef VERSION_H
-#	include  "version.h"
-#endif /* !VERSION_H */
-#ifndef BUGREP_H
-#	include  "bugrep.h"
-#endif /* !BUGREP_H */
 #ifndef included_trace_h
 #	include "trace.h"
 #endif /* !included_trace_h */
@@ -60,6 +54,10 @@
 #	include "rfc2046.h"
 #endif /* !RFC2046_H */
 
+#ifdef MAC_OS_X
+#	include <CoreFoundation/CoreFoundation.h>
+#endif /* MAC_OS_X */
+
 /*
  * defines to control GNKSA-checks behaviour:
  * - ENFORCE_RFC1034
@@ -76,13 +74,18 @@ static int gnksa_check_domain (char *domain);
 static int gnksa_check_domain_literal (char *domain);
 static int gnksa_check_localpart (const char *localpart);
 static int gnksa_dequote_plainphrase (char *realname, char *decoded, int addrtype);
-static int gnksa_split_from (const char *from, char *address, char *realname, int *addrtype);
 static int strfeditor (char *editor, int linenum, const char *filename, char *s, size_t maxsize, char *format);
 static void write_input_history_file (void);
 #ifdef LOCAL_CHARSET
 	static int to_local (int c);
 	static int to_network (int c);
 #endif /* LOCAL_CHARSET */
+#ifdef CHARSET_CONVERSION
+#	ifndef HAVE_WORKING_ICONV /* TODO: write configure check */
+	static char *utf8_valid(char *line);
+#	endif /* HAVE_WORKING_ICONV */
+#endif /* CHARSET_CONVERSION */
+
 
 /*
  * generate tmp-filename
@@ -94,7 +97,7 @@ get_tmpfilename (
 	char *file_tmp;
 
 	/* alloc memory for tmp-filename */
-	file_tmp = (char *) my_malloc (strlen (filename)+5);
+	file_tmp = (char *) my_malloc (strlen(filename) + 5);
 
 	/* generate tmp-filename */
 	sprintf (file_tmp, "%s.tmp", filename);
@@ -188,7 +191,7 @@ copy_fp (
 
 /*
  * backup file
- * Returns FALSE if backup failed or source file does not exists.
+ * Returns FALSE if backup failed
  */
 t_bool
 backup_file (
@@ -198,8 +201,8 @@ backup_file (
 	FILE *fp_in, *fp_out;
 	t_bool ret = FALSE;
 
-	if ((fp_in = fopen (filename, "r")) == (FILE *) 0)
-		return ret;
+	if ((fp_in = fopen (filename, "r")) == (FILE *) 0)	/* a missing sourcefile is not a real bug */
+		return TRUE;
 
 	/* don't follow links when writing backup files */
 	unlink (backupname);
@@ -263,8 +266,8 @@ copy_body (
 	else {
 		/* strip tailing space from quote-char for quoting quoted lines */
 		strcpy(prefixbuf, prefix);
-		if (prefixbuf[strlen(prefixbuf)-1] == ' ')
-			prefixbuf[strlen(prefixbuf)-1] = '\0';
+		if (prefixbuf[strlen(prefixbuf) - 1] == ' ')
+			prefixbuf[strlen(prefixbuf) - 1] = '\0';
 	}
 
 	while (fgets (buf, (int) sizeof(buf), fp_ip) != (char *) 0) {
@@ -326,7 +329,6 @@ get_val (
  * or 'M'enu
  */
 #define BACKUP_FILE_EXT ".b"
-
 t_bool
 invoke_editor (
 	const char *filename,
@@ -406,7 +408,6 @@ invoke_ispell (
 		return FALSE;
 	}
 
-
 	if ((fp_head = fopen (nam_head, "w")) == NULL) {
 		perror_message(_(txt_cannot_open), nam_head);
 		fclose (fp_all);
@@ -685,7 +686,7 @@ strip_double_ngs (
 				strcpy(ngroup2, cmplist);
 				ptr2 = strchr(ngroup2, ',');
 				if (ptr2 != (char *) 0) {
-					strcpy(cmplist, ptr2+1);
+					strcpy(cmplist, ptr2 + 1);
 					*ptr2 = '\0';
 				} else
 					over2 = TRUE;
@@ -717,11 +718,7 @@ my_mkdir (
 	} else
 		return -1;
 #else
-#	if defined(M_OS2) || defined(WIN32)
-		return mkdir (path);
-#	else
 		return mkdir (path, mode);
-#	endif /* M_OS2 || WIN32 */
 #endif /* !HAVE_MKDIR */
 }
 
@@ -734,12 +731,6 @@ my_chdir (
 
 	retcode = chdir (path);
 
-#ifdef M_OS2
-	if (*path && path[1] == ':') {
-		_chdrive (toupper((unsigned char)path[0]) - 'A' + 1);
-	}
-#endif /* M_OS2 */
-
 	return retcode;
 }
 
@@ -911,20 +902,17 @@ base_name (
 
 	strcpy (program, fullpath);
 
-	for (i = strlen (fullpath)-1; i; i--) {
+	for (i = strlen (fullpath) - 1; i; i--) {
 #ifndef VMS
 		if (fullpath[i] == SEPDIR)
 #else
 		if (fullpath[i] == ']')
 #endif /* !VMS */
 		{
-			strcpy (program, fullpath+(i+1));
+			strcpy (program, fullpath + i + 1);
 			break;
 		}
 	}
-#ifdef M_OS2
-	str_lwr (program);
-#endif /* M_OS2 */
 #ifdef VMS
 	if ((cp = strrchr(program, '.')) != 0)
 		*cp = '\0';
@@ -939,16 +927,15 @@ t_bool
 mail_check (
 	void)
 {
-#ifndef WIN32 /* No unified mail transport on WIN32 */
 	const char *mailbox_name;
 	struct stat buf;
-#	ifdef M_AMIGA
+#ifdef M_AMIGA
 	static long mbox_size = 0;
-#	endif /* M_AMIGA */
+#endif /* M_AMIGA */
 
 	mailbox_name = get_val ("MAIL", mailbox);
 
-#	ifdef M_AMIGA
+#ifdef M_AMIGA
 	/*
 	 * Since AmigaDOS does not distinguish between atime and mtime
 	 * we have to find some other way to figure out if the mailbox
@@ -985,11 +972,10 @@ mail_check (
 				mbox_size = 0;
 		}
 	}
-#	else
+#else
 	if (mailbox_name != 0 && stat (mailbox_name, &buf) >= 0 && buf.st_atime < buf.st_mtime && buf.st_size > 0)
 		return TRUE;
-#	endif /* M_AMIGA */
-#endif /* !WIN32 */
+#endif /* M_AMIGA */
 	return FALSE;
 }
 
@@ -1099,7 +1085,8 @@ parse_from (
 				if (*ap == '\"') {
 					switch (*(ap + 1)) {
 						case '@' : case '%' :
-							*(quoted_p++) = '\"'; *quoted_p = '\0';
+							*(quoted_p++) = '\"';
+							*quoted_p = '\0';
 							APPEND_TO (asp, quoted_buf);
 							APPEND_TO (cmtp, atom_buf);
 							atom_type = 1;
@@ -1219,13 +1206,13 @@ my_isprint (
 	/* use locale */
 	return isprint(c);
 #else
-#	ifdef LOCAL_CHARSET
+#	if defined(LOCAL_CHARSET) || defined(MAC_OS_X)
 		/* use some conversation table */
 		return (isprint(c) || (c >= 0x80 && c <= 0xff));
 #	else
 		/* assume iso-8859-1 */
 		return (isprint(c) || (c >= 0xa0 && c <= 0xff));
-#	endif /* LOCAL_CHARSET */
+#	endif /* LOCAL_CHARSET || MAC_OS_X */
 #endif /* !NO_LOCALE */
 }
 
@@ -1244,7 +1231,8 @@ void
 get_author (
 	t_bool thread,
 	struct t_article *art,
-	char *str, size_t len)
+	char *str,
+	size_t len)
 {
 	int author;
 
@@ -1283,9 +1271,8 @@ toggle_inverse_video (
 #ifndef USE_INVERSE_HACK
 		tinrc.draw_arrow = FALSE;
 #endif /* !USE_INVERSE_HACK */
-	} else {
+	} else
 		tinrc.draw_arrow = TRUE;
-	}
 }
 
 
@@ -1324,16 +1311,10 @@ show_color_status (
 #endif /* HAVE_COLOR */
 
 
-/* moved from art.c */
-#ifdef WIN32
-	/* Don't want the overhead of windows.h */
-	int kbhit(void);
-#endif /* WIN32 */
-
-
 /*
- * Check for lock file to stop multiple copies of tind or tin -U running
- * and if it does not exist create it so this is the only copy running
+ * Check for lock file to stop multiple copies of tind (obsolete) or
+ * tin -U running and if it does not exist create it so this is the
+ * only copy running
  */
 void
 create_index_lock_file (
@@ -1348,7 +1329,7 @@ create_index_lock_file (
 		if ((fp = fopen (the_lock_file, "r")) != (FILE *) 0) {
 			fgets (buf, (int) sizeof(buf), fp);
 			fclose (fp);
-			error_message ("\n%s: Already started pid=[%d] on %s", tin_progname, atoi(buf), buf+8);
+			error_message ("\n%s: Already started pid=[%d] on %s", tin_progname, atoi(buf), buf + 8);
 			giveup();
 		}
 	} else {
@@ -1568,11 +1549,6 @@ strfeditor (
 				case 'n':	/* linefeed */
 					strcpy (tbuf, "\n");
 					break;
-#ifdef WIN32
-				case '\"':
-					strcpy (tbuf, "\\\"");
-					break;
-#endif /* WIN32 */
 				default:
 					tbuf[0] = '%';
 					tbuf[1] = *format;
@@ -1784,9 +1760,9 @@ _strfpath (
 				 */
 				envptr = getenv (tbuf);
 				if (envptr == (char *) 0 || (*envptr == '\0'))
-					strncpy (tbuf, defbuf, sizeof(tbuf)-1);
+					strncpy (tbuf, defbuf, sizeof(tbuf) - 1);
 				else
-					strncpy (tbuf, envptr, sizeof(tbuf)-1);
+					strncpy (tbuf, envptr, sizeof(tbuf) - 1);
 				if ((str = strfpath_cp(str, tbuf, endp)) == NULL)
 					return 0;
 				else if (*tbuf == '\0') {
@@ -1808,7 +1784,7 @@ _strfpath (
 					char buf[PATH_LEN];
 
 					if (strfpath (group->attribute->maildir, buf, sizeof(buf), group)) {
-						if (*(format+1) == '\0')				/* Just an = */
+						if (*(format + 1) == '\0')				/* Just an = */
 							joinpath (tbuf, buf, group->name);
 						else
 							joinpath (tbuf, buf, "");
@@ -2153,8 +2129,8 @@ get_initials (
 
 	iflag = FALSE;
 	j = 0;
-	for (i = 0; tbuf[i] && j < maxsize-1; i++) {
-		if (isalpha((int)tbuf[i])) {
+	for (i = 0; tbuf[i] && j < maxsize - 1; i++) {
+		if (isalpha((int) tbuf[i])) {
 			if (!iflag) {
 				s[j++] = tbuf[i];
 				iflag = TRUE;
@@ -2236,9 +2212,6 @@ make_post_process_cmd (
 
 	get_cwd (currentdir);
 	my_chdir (dir);
-#	ifdef M_OS2
-	backslash (file);
-#	endif /* M_OS2 */
 	sh_format (buf, sizeof(buf), cmd, file);
 	invoke_cmd (buf);
 	my_chdir (currentdir);
@@ -2273,16 +2246,6 @@ file_mtime (
 }
 
 
-void
-vPrintBugAddress (
-	void)
-{
-	my_fprintf (stderr, _("%s %s %s (\"%s\") [%s]: send a DETAILED bug report to %s\n"),
-		tin_progname, VERSION, RELEASEDATE, RELEASENAME, OSNAME, bug_addr);
-	my_fflush (stderr);
-}
-
-
 char *
 random_organization (
 	char *in_org)
@@ -2305,6 +2268,11 @@ random_organization (
 	while (fgets(selorg, (int) sizeof(selorg), orgfp))
 		nool++;
 
+	if (!nool) {
+		fclose(orgfp);
+		return selorg;
+	}
+
 	rewind (orgfp);
 	sol = rand () % nool + 1;
 	nool = 0;
@@ -2455,7 +2423,7 @@ quote_wild (
 		}
 	}
 	*target = '\0';
-	return (buff);
+	return buff;
 }
 
 
@@ -2483,13 +2451,13 @@ quote_wild_whitespace (
 			*target++ = *str;
 	}
 	*target = '\0';
-	return (buff);
+	return buff;
 }
 
 
-
+#if 0 /* unused */
 /*
- * strip_address () removes the address part from a given e-mail address
+ * strip_address() removes the address part from a given e-mail address
  */
 void
 strip_address (
@@ -2500,7 +2468,7 @@ strip_address (
 	char *start_pos;
 
 	if (strchr(the_address, '@') != (char *) 0) {
-		if ((end_pos = strchr(the_address,'<')) == (char *) 0) {
+		if ((end_pos = strchr(the_address, '<')) == (char *) 0) {
 			if ((start_pos = strchr(the_address, ' ')) == (char *) 0)
 				strcpy (stripped_address, the_address);
 			else {
@@ -2522,10 +2490,11 @@ strip_address (
 			stripped_address[strlen(stripped_address) - 1] = '\0';
 	}
 }
+#endif /* 0 */
 
 
 /*
- * strip_name () removes the realname part from a given e-mail address
+ * strip_name() removes the realname part from a given e-mail address
  */
 void
 strip_name (
@@ -2541,26 +2510,24 @@ strip_name (
 		strcpy (stripped_address, the_address);
 		start_pos = stripped_address;
 		if ((end_pos = strchr (start_pos, ' ')) == (char *) 0)
-			end_pos = start_pos+strlen(start_pos);
+			end_pos = start_pos + strlen(start_pos);
 	} else {
 		start_pos++; /* skip '<' */
 		strcpy (stripped_address, start_pos);
 		start_pos = stripped_address;
 		if ((end_pos = strchr (start_pos, '>')) == (char *) 0)
-			end_pos = start_pos+strlen(start_pos); /* skip '>' */
+			end_pos = start_pos + strlen(start_pos); /* skip '>' */
 	}
 	*(end_pos) = '\0';
 }
 
 
-#ifdef LOCAL_CHARSET
+#if defined(LOCAL_CHARSET) && !defined(MAC_OS_X)
 /*
  * convert between local and network charset (e.g. NeXT and latin1)
  */
-
 #	define CHARNUM 256
 #	define BAD (-1)
-
 /* use the appropriate conversion tables */
 #	if LOCAL_CHARSET == 437
 #		include "l1_ibm437.tab"
@@ -2574,7 +2541,6 @@ strip_name (
 #			include "next_l1.tab"
 #		endif /* 850 */
 #	endif /* 437 */
-
 static int
 to_local (
 	int c)
@@ -2589,6 +2555,7 @@ to_local (
 		return c;
 }
 
+
 void
 buffer_to_local (
 	char *b)
@@ -2597,6 +2564,7 @@ buffer_to_local (
 		*b = to_local(*b);
 }
 
+
 static int
 to_network (
 	int c)
@@ -2611,6 +2579,7 @@ to_network (
 		return c;
 }
 
+
 void
 buffer_to_network (
 	char *b)
@@ -2618,7 +2587,198 @@ buffer_to_network (
 	for(; *b; b++)
 		*b = to_network(*b);
 }
-#endif /* LOCAL_CHARSET */
+
+
+#else
+#	ifdef MAC_OS_X
+void
+buffer_to_local (
+	char *b)
+{
+	CFStringRef convb;
+	char *oldb = my_strdup(b);
+
+	if (!oldb)
+		return;
+
+	convb = CFStringCreateWithCString(NULL, b, kCFStringEncodingISOLatin9);
+
+	if (convb) {
+		CFStringGetCString(convb, b, strlen(b) + 1, CFStringGetSystemEncoding());
+		CFRelease(convb);
+		if (!strlen(b))
+			strcpy(b, oldb);
+	}
+	free(oldb);
+}
+
+
+void
+buffer_to_network (
+	char *b)
+{
+	CFStringRef convb;
+	char *oldb = my_strdup(b);
+
+	if (!oldb)
+		return;
+
+	convb = CFStringCreateWithCString(NULL, b, CFStringGetSystemEncoding());
+
+	if (convb) {
+		CFStringGetCString(convb, b, strlen(b) + 1, kCFStringEncodingISOLatin9);
+		CFRelease(convb);
+		if (!strlen(b))
+			strcpy(b, oldb);
+	}
+	free(oldb);
+}
+#	endif /* MAC_OS_X */
+#endif /* LOCAL_CHARSET && !MAC_OS_X */
+
+
+#ifdef CHARSET_CONVERSION
+void
+buffer_to_local (
+	char *line,
+	const char *network_charset,
+	const char *local_charset)
+{
+	char *cnetwork_charset;
+
+	/* FIXME: this should default in RFC2046.c to US-ASCII */
+	if (!(network_charset && *network_charset)) {	/* Content-Type: did't had a charset parameter */
+		/*
+		 * defaulting to US-ASCII would be more correct, but $MM_CHARSET
+		 * might be more usefull
+		 */
+		cnetwork_charset = my_strdup(local_charset);
+	} else {
+		cnetwork_charset = my_strdup(network_charset);
+
+		if (strcasecmp(cnetwork_charset, local_charset)) { /* different charsets? */
+			char * obuf;
+			char * outbuf;
+			const char * inbuf;
+			iconv_t cd;
+			size_t result, osize;
+			size_t inbytesleft, outbytesleft;
+
+#	ifndef MIME_STRICT_CHARSET
+			if (!strcasecmp(cnetwork_charset, "US-ASCII")) {
+#		if 0
+				free(cnetwork_charset);
+				cnetwork_charset = my_strdup(local_charset);
+#		else
+				/* no network_charset case already fixed above */
+				;
+#		endif /* 0 */
+			} else
+#	endif /* !MIME_STRICT_CHARSET */
+			{
+				if ((cd = iconv_open(local_charset, cnetwork_charset)) != (iconv_t) (-1)) {
+#	ifndef HAVE_WORKING_ICONV /* TODO: write configure check */
+					/* iconv() might crash on broken multibyte sequences so check them */
+					if (!strcasecmp(cnetwork_charset, "UTF-8"))
+						utf8_valid(line);
+#	endif /* HAVE_WORKING_ICONV */
+					{
+						inbuf = (char *) line;
+						inbytesleft = strlen (line);
+						outbytesleft = 1 + inbytesleft * 4;	/* should be enough */
+						osize = outbytesleft;
+						obuf = (char *) my_malloc (osize + 1);
+						outbuf = (char *) obuf;
+
+						do {
+							errno = 0;
+							result = iconv (cd, &inbuf, &inbytesleft, &outbuf, &outbytesleft);
+							if (result == (size_t) (-1)) {
+								switch (errno) {
+									case EILSEQ:
+										**&outbuf = '?';
+										outbuf++;
+										inbuf++;
+										inbytesleft--;
+										break;
+
+									case E2BIG:
+										obuf = (char *) my_realloc(obuf, osize * 2);
+										outbuf = (char *) (obuf + osize - outbytesleft);
+										outbytesleft += osize;
+										osize <<= 1; /* double size */
+										break;
+
+									default:
+										inbytesleft = 0;
+								}
+							}
+						} while (inbytesleft > 0);
+
+						**&outbuf = '\0';
+						strcpy(line, obuf); /* FIXME: obuf might be bigger than line! */
+						free(obuf);
+						iconv_close(cd);
+					}
+				}
+			}
+		}
+	}
+	free(cnetwork_charset);
+}
+
+
+void
+buffer_to_network (
+	char *line)
+{
+	char * obuf;
+	char * outbuf;
+	const char * inbuf;
+	iconv_t cd;
+	size_t result, osize;
+	size_t inbytesleft, outbytesleft;
+
+	if ((cd = iconv_open(txt_mime_charsets[tinrc.mm_network_charset], tinrc.mm_local_charset)) != (iconv_t) (-1)) {
+		inbytesleft = strlen (line);
+		inbuf = (char *) line;
+		outbytesleft = 1 + inbytesleft * 4;
+		osize = outbytesleft;
+		obuf = (char *) my_malloc (osize + 1);
+		outbuf = (char *) obuf;
+
+		do {
+			errno = 0;
+			result = iconv (cd, &inbuf, &inbytesleft, &outbuf, &outbytesleft);
+			if (result == (size_t) (-1)) {
+				switch (errno) {
+					case EILSEQ:
+						**&outbuf = '?';
+						outbuf++;
+						inbuf++;
+						inbytesleft--;
+						break;
+
+					case E2BIG:
+						obuf = (char *) my_realloc(obuf, osize * 2);
+						outbuf = (char *) (obuf + osize - outbytesleft);
+						outbytesleft += osize;
+						osize <<= 1; /* double size */
+						break;
+
+					default:	/* EINVAL */
+						inbytesleft = 0;
+				}
+			}
+		} while (inbytesleft > 0);
+
+		**&outbuf = '\0';
+		strcpy(line, obuf); /* FIXME: here we assume that line is big enough to hold obuf */
+		free(obuf);
+		iconv_close(cd);
+	}
+}
+#endif /* CHARSET_CONVERSION */
 
 
 /*
@@ -2880,7 +3040,6 @@ gnksa_dequote_plainphrase (
 	int initialstate;	/* initial state */
 	int state;	/* current state */
 
-
 	/* initialize state machine */
 	switch (addrtype) {
 		case GNKSA_ADDRTYPE_ROUTE:
@@ -3144,7 +3303,6 @@ gnksa_check_domain_literal (
 		|| (127 == x1)))			/* localhost */
 		return GNKSA_LOCAL_DOMAIN_LITERAL;
 
-
 	return GNKSA_OK;
 }
 
@@ -3163,7 +3321,7 @@ gnksa_check_domain (
 		return gnksa_check_domain_literal(domain);
 
 	/* check for leading or trailing dot */
-	if (('.' == *domain) || ('.' == *(domain+strlen(domain)-1)))
+	if (('.' == *domain) || ('.' == *(domain + strlen(domain) - 1)))
 		return GNKSA_ZERO_LENGTH_LABEL;
 
 	/* look for TLD start */
@@ -3287,7 +3445,7 @@ gnksa_check_localpart (
 /*
  * split mail address into realname and address parts
  */
-static int
+int
 gnksa_split_from (
 	const char *from,
 	char *address,
@@ -3302,9 +3460,9 @@ gnksa_split_from (
 	*address = *realname = '\0';
 
 	/* copy raw address into work area */
-	strncpy(work, from, 998);
-	work[998] = '\0';
-	work[999] = '\0';
+	strncpy(work, from, HEADER_LEN - 2);
+	work[HEADER_LEN - 2] = '\0';
+	work[HEADER_LEN - 1] = '\0';
 
 	/* skip trailing whitespace */
 	addr_end = work + strlen(work) - 1;
@@ -3393,6 +3551,9 @@ gnksa_split_from (
 		}
 	}
 
+	if (! strchr(address, '@')) /* check for From: without an @ */
+		return GNKSA_ATSIGN_MISSING;
+
 	/* split successful */
 	return GNKSA_OK;
 }
@@ -3448,7 +3609,7 @@ gnksa_do_check_from (
 
 		/* convert FQDN part to lowercase */
 		for (aux = addr_begin; *aux; aux++)
-			*aux = tolower((int)*aux);
+			*aux = tolower((int) *aux);
 
 		if (GNKSA_OK != (result = gnksa_check_domain(addr_begin))
 		    && (GNKSA_OK == code)) /* error detected */
@@ -3531,3 +3692,143 @@ strip_line (
 
 	*++ptr = '\0';
 }
+
+#ifdef CHARSET_CONVERSION
+#	ifndef HAVE_WORKING_ICONV /* TODO: write configure check */
+/*
+ * 'check' a given UTF-8 strig and '?'-out illegal sequences
+ * TODO: is this check complete?
+ *
+ * UTF-8           = ASCII / UTF-8-non-ascii
+ * ASCII           = %x00-%x7F
+ * UTF-8-non-ascii = UTF8-2 / UTF8-3 / UTF8-4
+ * UTF8-1          = %x80-BF
+ * UTF8-2          = %xC2-DF 1*UTF8-1
+ * UTF8-3          = %xE0 %xA0-BF 1*UTF8-1 / %xE1-EC 2*UTF8-1 /
+ *                   %xED %x80-9F 1*UTF8-1 / %xEE-EF 2*UTF8-1
+ * UTF8-4          = %xF0 %x90-BF 2*UTF8-1 / %xF1-F3 3*UTF8-1 /
+ *                   %xF4 %x80-8F 2*UTF8-1
+ */
+static char *
+utf8_valid(
+	char *line)
+{
+	char *c;
+	unsigned char d, e, f, g;
+	int numc;
+	t_bool illegal;
+
+	c = line;
+
+	while (*c != '\0' && *c != '\n') {
+		if (!(*c & 0x80)) { /* plain US-ASCII? */
+			c++;
+			continue;
+		}
+
+		numc = 0;
+		illegal = FALSE;
+		d = (*c & 0x7c);	/* remove bits 7,1,0 */
+
+		do {
+			numc++;
+		} while ((d <<= 1) & 0x80);	/* get sequence length */
+
+		d = *c;
+		e = *(c + 1);
+
+		switch (numc) {
+			case 2:
+				/* out of range or sequences which would also fit into 1 byte */
+				if (d < 0xc2 || d > 0xdf)
+					illegal = TRUE;
+				break;
+
+			case 3:
+				f = *(c + 2);
+				/* out of range or sequences which would also fit into 2 bytes */
+				if (d < 0xe0 || d > 0xef || (d == 0xe0 && e < 0xa0))
+					illegal = TRUE;
+				/* U+D800 ... U+DFFF */
+				if (d == 0xed && e > 0x9f)
+					illegal = TRUE;
+				/* U+FDD0 ... U+FDEF */
+				if (d == 0xef && e == 0xb7 && (f >= 0x90 && f <= 0xaf))
+					illegal = TRUE;
+				/* U+FFFE, U+FFFF */
+				if (d == 0xef && e == 0xbf && (f == 0xbe || f == 0xbf))
+					illegal = TRUE;
+				break;
+
+			case 4:
+				f = *(c + 2);
+				g = *(c + 3);
+				/* out of range or sequences which would also fit into 3 bytes */
+				if (d < 0xf0 || d > 0xf7 || (d == 0xf0 && e < 0x90))
+					illegal = TRUE;
+				/* largest current used sequence */
+				if (d == 0xf4 && e > 0x8f)
+					illegal = TRUE;
+				/* Unicode 3.1 noncharacters */
+				/* U+1FFFE, U+1FFFF, U+2FFFE, U+2FFFF, U+3FFFE, U+3FFFF; (Unicode 3.1) */
+				if (d == 0xf0 && (e == 0x9f || e == 0xaf || e == 0xbf) && f == 0xbf && (g == 0xbe || g ==0xbf))
+					illegal = TRUE;
+				/* Unicode 3.1 noncharacters */
+				/* U+4FFFE, U+4FFFF, U+5FFFE, U+5FFFF, U+6FFFE, U+6FFFF, U+7FFFE, U+7FFFF */
+				/* U+8FFFE, U+8FFFF, U+9FFFE, U+9FFFF, U+AFFFE, U+AFFFF, U+BFFFE, U+BFFFF */
+				/* U+CFFFE, U+CFFFF, U+DFFFE, U+DFFFF, U+EFFFE, U+EFFFF, U+FFFFE, U+FFFFF */
+				if ((d == 0xf1 || d == 0xf2 || d == 0xf3 ) && (e == 0x8f || e == 0x9f || e == 0xaf || e == 0xbf) && f == 0xbf && (g == 0xbe || g == 0xbf))
+					illegal = TRUE;
+				/* Unicode 3.1 noncharacters */
+				/* U+10FFFE, U+10FFFF */
+				if (d == 0xf4 && e == 0x8f && f == 0xbf && (g == 0xbe || g == 0xbf))
+					illegal = TRUE;
+				break;
+
+#	if 0	/* currently not used, see also check above  */
+			case 5:
+				/* out of range or sequences which would also fit into 4 bytes */
+				if (d < 0xf8 || d > 0xfb || (d == 0xf8 && e < 0x88))
+					illegal = TRUE;
+				break;
+
+			case 6:
+				/* out of range or sequences which would also fit into 5 bytes */
+				if (d < 0xfc || d > 0xfd || (d == 0xfc && e < 0x84))
+					illegal = TRUE;
+				break;
+#	endif /* 0 */
+
+			default:
+				/*
+				 * with the check for plain US-ASCII above
+				 * all other sequence length are illegal.
+				 */
+				illegal = TRUE;
+				break;
+		}
+
+		if (!illegal) {
+			for (d = 1; d < numc; d++) {
+				e = *(c + d);
+				if (e < 0x80 || e > 0xbf || *(c + d) == '\0' || *(c + d) == '\n')
+					illegal = TRUE;
+			}
+		}
+
+		if (!illegal)
+			c += numc; /* skip over valid sequence */
+		else {
+			while (numc--) {
+				if (*c == '\0' || *c == '\n')
+					break;
+				if (*c & 0x80)	/* replace 'dangerous' bytes */
+					*c = '?';
+				c++;
+			}
+		}
+	}
+	return line;
+}
+#	endif /* HAVE_WORKING_ICONV */
+#endif /* CHARSET_CONVERSION */
