@@ -6,7 +6,7 @@
  *  Updated   : 1997-12-28
  *  Notes     : ArtCount = (ArtMax - ArtMin) + 1  [could have holes]
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 
-#define BITS_TO_BYTES(n)	(size_t)((n+NBITS-1)/NBITS)
+#define BITS_TO_BYTES(n)	(size_t)((n + NBITS - 1) / NBITS)
 
 static mode_t newsrc_mode = 0;
 
@@ -324,11 +324,7 @@ backup_newsrc (
 {
 	char buf[HEADER_LEN];
 
-#	ifdef WIN32
-	joinpath (buf, rcdir, OLDNEWSRC_FILE);
-#	else
 	joinpath (buf, homedir, OLDNEWSRC_FILE);
-#	endif /* WIN32 */
 
 	if (!backup_file (newsrc, buf))
 		error_message (_(txt_filesystem_full_backup), NEWSRC_FILE);
@@ -622,7 +618,7 @@ parse_bitmap_seq (
 #ifdef DEBUG_NEWSRC
 	{
 		char buf[NEWSRC_LINE];
-		sprintf (buf, "Parsing [%s%c %.*s]", group->name, SUB_CHAR(group->subscribed), (int) (NEWSRC_LINE-strlen(group->name)-20), ptr);
+		sprintf (buf, "Parsing [%s%c %.*s]", group->name, SUB_CHAR(group->subscribed), (int) (NEWSRC_LINE - strlen(group->name) - 20), ptr);
 		debug_print_comment(buf);
 		debug_print_bitmap(group, NULL);
 	}
@@ -846,7 +842,7 @@ parse_unread_arts (
 		group->newsrc.xmax = group->xmax;
 
 	if (group->newsrc.xmax >= bitmin) {
-		newbitmap = (t_bitmap *)my_malloc(BITS_TO_BYTES(group->newsrc.xmax-bitmin+1));
+		newbitmap = (t_bitmap *) my_malloc(BITS_TO_BYTES(group->newsrc.xmax-bitmin + 1));
 		NSETRNG0(newbitmap, 0L, group->newsrc.xmax - bitmin);
 	}
 
@@ -1173,7 +1169,7 @@ pcParseNewsrcLine (
 	*sub = UNSUBSCRIBED;				/* Default to no entry */
 
 	if ((ptr = strpbrk(line, " !:")) == NULL)			/* space|SUBSCRIBED|UNSUBSCRIBED */
-		return(NULL);
+		return NULL;
 
 	*sub = *ptr;						/* Save the subscription status */
 	tmp = ptr;							/* Keep this blank for later */
@@ -1181,11 +1177,11 @@ pcParseNewsrcLine (
 
 #if 0
 	if (ptr == NULL)					/* No seq info, so return a blank */
-		return(tmp);
+		return tmp;
 #endif /* 0 */
 
 	if ((ptr = strpbrk(ptr, " \t")) == NULL)
-		return(tmp);
+		return tmp;
 
 	return (ptr + 1);	/* Return pointer to sequence info. At worst this will be \0 */
 }
@@ -1220,11 +1216,11 @@ expand_bitmap (
 	if (min >= group->newsrc.xmin)
 		first = group->newsrc.xmin;
 	else
-		first = group->newsrc.xmin - ((group->newsrc.xmin - min + (NBITS-1)) & ~(NBITS-1));
+		first = group->newsrc.xmin - ((group->newsrc.xmin - min + (NBITS - 1)) & ~(NBITS - 1));
 
 	/* adjust first */
 	if (first > group->newsrc.xmax + 1)
-		first = first - ((first - (group->newsrc.xmax + 1) + (NBITS-1)) & ~(NBITS-1));
+		first = first - ((first - (group->newsrc.xmax + 1) + (NBITS - 1)) & ~(NBITS - 1));
 
 	/* check first */
 	if (first < 1) {
@@ -1519,10 +1515,10 @@ vNewsrcTestHarness (
 		while (retry) {
 		/* FIXME - this is secure now, but doesn't write any debug output */
 		/* (it didn't before too) */
-			if ((temp_file = my_tempnam ("","NEWSRC")) != (char *) 0) {
+			if ((temp_file = my_tempnam ("", "NEWSRC")) != (char *) 0) {
 				if ((fd = open (temp_file, (O_CREAT|O_EXCL), (S_IRUSR|S_IWUSR))) != -1) {
 					if ((fp = fopen (temp_file, "w")) != (FILE *) 0) {
-						my_printf ("\n%d. PARSE Seq=[%s]\n", i+1, seq);
+						my_printf ("\n%d. PARSE Seq=[%s]\n", i + 1, seq);
 						parse_bitmap_seq (&group, seq);
 						debug_print_newsrc (&group.newsrc, stdout);
 						print_bitmap_seq (fp, &group);
@@ -1546,7 +1542,7 @@ vNewsrcTestHarness (
 		else {
 			fp = fopen (temp_file, "r");
 			fgets (seq, (int) sizeof(seq), fp);
-			seq[strlen(seq)-1] = '\0';
+			seq[strlen(seq) - 1] = '\0';
 			fclose (fp);
 		}
 	}
@@ -1557,7 +1553,6 @@ vNewsrcTestHarness (
 	set_bitmap_range_unread (&group.newsrc, rng_min, rng_max);
 	debug_print_newsrc (&group.newsrc, stdout);
 
-
 	NSETBLK0(group.newsrc.xbitmap, group.newsrc.xbitlen);
 	debug_print_newsrc (&group.newsrc, stdout);
 
@@ -1615,5 +1610,4 @@ my_printf ("\nRNG Min-Max=[%ld-%ld] Beg-End=[%ld-%ld] OFF=[%ld] LEN=[%ld]\n", ne
 	}
 }
 #	endif /* DEBUG_NEWSRC_FIXME */
-
 #endif /* DEBUG_NEWSRC */
