@@ -7,7 +7,7 @@
  *  Notes     : Routines to make reading news locally (ie. /var/spool/news)
  *              or via NNTP transparent
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,7 +65,7 @@ long head_next;
 
 char *nntp_server = (char *)0;
 static char txt_xover_string[] = "XOVER";
-char *txt_xover = txt_xover_string;
+static char *txt_xover = txt_xover_string;
 
 /*
  * Open a connection to the NNTP server. Authenticate if necessary or
@@ -345,10 +345,9 @@ DEBUG_IO((stderr, "nntp_command(MODE READER)\n"));
 	}
 
 #	if 0 /* TODO */
-	/* if we're using -n, check for XGTITLE */
+	/* if we're using -n, check for LIST NEWSGROUPS <wildmat> */
 	if (newsrc_active && !list_active) { /* -n */
-		if (!nntp_command("XGTITLE", ERR_COMMAND, NULL))
-			xgtitle_supported = TRUE;
+		/* code goes here */
 	}
 #	endif /* 0 */
 
@@ -659,11 +658,11 @@ open_newsgroups_fp (
 			read_local_newsgroups_file = FALSE;
 		}
 #	if 0 /* TODO */
-		if (xgtitle_supported && newsrc_active
+		if (list_newsgroups_wildmat_supported && newsrc_active
 		    && !list_active
 		    && num_active < some_useful_limit) {
 			for (i = 0; i < num_active; i++) {
-				sprintf(buff, "XGTITLE %s", active[i].name);
+				sprintf(buff, "LIST NEWSGROUPS %s", active[i].name);
 				nntp_command(buff, OK_LIST, NULL);
 			}
 		} else
@@ -798,7 +797,7 @@ open_art_fp (
 
 #ifdef NNTP_ABLE
 	if (read_news_via_nntp && CURR_GROUP.type == GROUP_TYPE_NEWS) {
-		snprintf (buf, sizeof(buf)-1, "ARTICLE %ld", art);
+		snprintf (buf, sizeof(buf) - 1, "ARTICLE %ld", art);
 		if ((art_fp = nntp_command (buf, OK_ARTICLE, NULL)) == NULL)
 			return (FILE *) 0;
 	} else {
@@ -921,7 +920,7 @@ setup_hard_base (
 			if (nntp_command(buf, OK_GROUP, line) == NULL)
 				return(-1);
 
-			if (sscanf (line,"%ld %ld %ld", &count, &start, &last) != 3)
+			if (sscanf (line, "%ld %ld %ld", &count, &start, &last) != 3)
 				return(-1);
 
 			total = count;
@@ -966,8 +965,8 @@ setup_hard_base (
 	}
 
 	if (grpmenu.max) {
-		if (base[grpmenu.max-1] > group->xmax)
-			group->xmax = base[grpmenu.max-1];
+		if (base[grpmenu.max - 1] > group->xmax)
+			group->xmax = base[grpmenu.max - 1];
 		expand_bitmap (group, base[0]);
 	}
 
