@@ -3,10 +3,10 @@
  *  Module    : page.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 1995-07-26
+ *  Updated   : 2001-07-22
  *  Notes     :
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,33 +45,27 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 #ifndef MENUKEYS_H
-#	include  "menukeys.h"
+#	include "menukeys.h"
 #endif /* !MENUKEYS_H */
 #ifndef RFC2046_H
-#	include  "rfc2046.h"
+#	include "rfc2046.h"
 #endif /* !RFC2046_H */
 
 #define PAGE_HEADER	4
 
-/*
- * size of page header over and above INDEX_TOP
- * [ Used because NOTESLINES only specifies a 2 line header ]
- */
-#define HDR_ADJUST	(PAGE_HEADER-INDEX_TOP)
-
 /*
  * The number of lines available to display actual article text
  */
-#define ARTLINES	(NOTESLINES-HDR_ADJUST)
+#define ARTLINES	(NOTESLINES-(PAGE_HEADER-INDEX_TOP))
 
-FILE *note_fp;			/* active stream (raw or cooked) */
-int artlines;			/* active # of lines in pager */
 int curr_line;			/* current line in art (indexed from 0) */
-t_lineinfo *artline;	/* active 'lineinfo' data */
+static FILE *note_fp;			/* active stream (raw or cooked) */
+static int artlines;			/* active # of lines in pager */
+static t_lineinfo *artline;	/* active 'lineinfo' data */
 
 t_openartinfo pgart =	/* Global context of article open in the pager */
 	{
-		{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, FALSE, 0 },
+		{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, FALSE, 0 },
 		FALSE, 0,
 		NULL, NULL, NULL, NULL,
 	};
@@ -82,12 +76,18 @@ int RIGHT_POS;			/* set in set_win_size () */
 int last_resp;			/* previous & current article # in arts[] for '-' command */
 int this_resp;
 
-int tabwidth = 8;
+static int tabwidth = 8;
 
-struct t_header *note_h = &pgart.hdr;	/* Easy access to article headers */
+static struct t_header *note_h = &pgart.hdr;	/* Easy access to article headers */
 
 static int rotate;				/* 0=normal, 13=rot13 decode */
+static int scroll_region_top;	/* first screen line for displayed message */
 static int search_line;			/* Line to commence next search from */
+static FILE *info_file;
+static const char *info_title;
+static int curr_info_line;
+static int num_info_lines;
+static t_lineinfo *infoline;
 
 static t_bool show_all_headers;	/* CTRL-H with headers specified */
 static t_bool reveal_ctrl_l;	/* set when ^L hiding is off */
@@ -96,10 +96,14 @@ static t_bool hide_uue;			/* set when uuencoded sections are 'hidden' */
 /*
  * Local prototypes
  */
+static int handle_pager_keypad (t_menukeys *menukeys);
 static int load_article (int new_respnum);
 static int prompt_response (int ch, int curr_respnum);
-static void process_search (void);
+static void preprocess_info_message (FILE *info_fh);
+static void print_message_page (FILE *file, t_lineinfo *messageline, size_t messagelines, size_t lcurr_line, size_t begin, size_t end, int help_level);
+static void process_search (int *lcurr_line, size_t message_lines, size_t screen_lines, int help_level);
 static void process_url (void);
+static void scroll_page (int i);
 static void show_first_header (const char *group);
 #ifdef HAVE_METAMAIL
 	static void show_mime_article (FILE *fp);
@@ -115,13 +119,13 @@ scroll_page (
 	int i)
 {
 #ifdef USE_CURSES
-	scrollok(stdscr, TRUE);
+	scrollok (stdscr, TRUE);
 #endif /* USE_CURSES */
-	SetScrollRegion(INDEX_TOP + HDR_ADJUST, ARTLINES + HDR_ADJUST +1);
-	ScrollScreen(i);
-	SetScrollRegion(0, cLINES);
+	SetScrollRegion (scroll_region_top, NOTESLINES + 1);
+	ScrollScreen (i);
+	SetScrollRegion (0, cLINES);
 #ifdef USE_CURSES
-	scrollok(stdscr, FALSE);
+	scrollok (stdscr, FALSE);
 #endif /* USE_CURSES */
 }
 
@@ -131,18 +135,16 @@ scroll_page (
  */
 static int
 handle_pager_keypad(
-	void)
+	t_menukeys *menukeys)
 {
 	int ch = ReadCh ();
 
 	switch (ch) {
-#ifndef WIN32
 		case ESC:
 #	ifdef HAVE_KEY_PREFIX
 		case KEY_PREFIX:
 #	endif /* HAVE_KEY_PREFIX */
 			switch (get_arrow_key (ch)) {
-#endif /* !WIN32 */
 				case KEYMAP_UP:
 					ch = iKeyUp;
 					break;
@@ -167,17 +169,16 @@ handle_pager_keypad(
 				case KEYMAP_END:
 					ch = iKeyLastPage;
 					break;
-#ifndef WIN32
 				case KEYMAP_MOUSE:
 					switch (xmouse) {
 						case MOUSE_BUTTON_1:
-							if (xrow < PAGE_HEADER || xrow >= cLINES-1)
+							if (xrow < PAGE_HEADER || xrow >= cLINES - 1)
 								ch = iKeyPageDown;
 							else
 								ch = iKeyPageNextUnread;
 							break;
 						case MOUSE_BUTTON_2:
-							if (xrow < PAGE_HEADER || xrow >= cLINES-1)
+							if (xrow < PAGE_HEADER || xrow >= cLINES - 1)
 								ch = iKeyPageUp;
 							else
 								ch = iKeyQuit;
@@ -193,9 +194,8 @@ handle_pager_keypad(
 					break;
 			}
 			break;
-#endif /* !WIN32 */
 		default:
-			ch = map_to_default (ch, &menukeymap.page_nav);
+			ch = map_to_default (ch, menukeys);
 			break;
 	}
 	return ch;
@@ -227,11 +227,15 @@ show_page (
 	int ch, i, n = 0;
 	int filter_state = NO_FILTERING;
 	int old_sort_art_type = tinrc.sort_article_type;
+	int art_type = GROUP_TYPE_NEWS;
 	t_bool mouse_click_on = TRUE;
 
 	filtered_articles = FALSE;	/* used in thread level */
 	make_group_path (group->name, group_path);
 
+	if (group->attribute->mailing_list != (char *) 0)
+		art_type = GROUP_TYPE_MAIL;
+
 	/*
 	 * Peek to see if the pager started due to a body search
 	 * Stop load_article() changing context again
@@ -243,16 +247,14 @@ show_page (
 		return GRP_ARTFAIL;
 
 	if (srch_lineno != -1)
-		process_search();
+		process_search(&curr_line, artlines, ARTLINES, PAGE_LEVEL);
 
 	resize_article (TRUE, &pgart);
 
 	forever {
-		switch ((ch = handle_pager_keypad())) {
-#ifndef WIN32
+		switch ((ch = handle_pager_keypad(&menukeymap.page_nav))) {
 			case ESC:       /* Abort */
 				break;
-#endif /* !WIN32 */
 
 			case '0': case '1': case '2': case '3': case '4': case '5':
 			case '6': case '7': case '8': case '9':
@@ -430,7 +432,7 @@ page_goto_next_unread:
 				break;
 
 #ifndef DISABLE_PRINTING
-			case iKeyPagePrint:	/* output art/thread/tagged arts to printer */
+			case iKeyPrint:	/* output art/thread/tagged arts to printer */
 				feed_articles (FEED_PRINT, PAGE_LEVEL, group, this_resp);
 				break;
 #endif /* !DISABLE_PRINTING */
@@ -457,7 +459,7 @@ page_goto_next_unread:
 				if ((i = search_article ((ch == iKeySearchSubjF), search_line, artlines, artline, reveal_ctrl_l, note_fp)) == -1)
 					break;
 
-				process_search();
+				process_search(&curr_line, artlines, ARTLINES, PAGE_LEVEL);
 				break;
 
 			case iKeySearchBody:	/* article body search */
@@ -465,7 +467,7 @@ page_goto_next_unread:
 					this_resp = n;			/* Stop load_article() changing context again */
 					if (load_article(n) < 0)
 						return GRP_ARTFAIL;
-					process_search();
+					process_search(&curr_line, artlines, ARTLINES, PAGE_LEVEL);
 				}
 				break;
 
@@ -509,12 +511,12 @@ page_goto_next_unread:
 				break;
 
 			case iKeyPageToggleTex2iso:		/* toggle german TeX to ISO latin1 style conversion */
-				if ((tex2iso_supported = !tex2iso_supported))
+				if (((group->attribute->tex2iso_conv) = !(group->attribute->tex2iso_conv)))
 					pgart.tex2iso = is_art_tex_encoded (pgart.raw);
 
 				resize_article (TRUE, &pgart);	/* Also recooks it.. */
 				draw_page (group->name, 0);
-				info_message (_(txt_toggled_tex2iso), (tex2iso_supported) ? "on" : "off");
+				info_message (_(txt_toggled_tex2iso), ((group->attribute->tex2iso_conv)) ? "on" : "off");
 				break;
 
 			case iKeyPageToggleTabs:		/* toggle tab stops 8 vs 4 */
@@ -591,7 +593,7 @@ page_goto_next_unread:
 
 			case iKeyPageCatchup:			/* catchup - mark read, goto next */
 			case iKeyPageCatchupNextUnread:	/* goto next unread */
-				snprintf(buf, sizeof(buf)-1, _(txt_mark_thread_read), (ch == iKeyPageCatchupNextUnread) ? _(txt_enter_next_thread) : "");
+				snprintf(buf, sizeof(buf) - 1, _(txt_mark_thread_read), (ch == iKeyPageCatchupNextUnread) ? _(txt_enter_next_thread) : "");
 				if (!tinrc.confirm_action || prompt_yn (cLINES, buf, TRUE) == 1) {
 					thd_mark_read (group, base[which_thread(this_resp)]);
 					return (ch == iKeyPageCatchupNextUnread) ? GRP_NEXTUNREAD : GRP_NEXT;
@@ -605,7 +607,7 @@ page_goto_next_unread:
 				break;
 
 			case iKeyPageCancel:			/* cancel an article */
-				if (can_post) {
+				if (can_post || art_type != GROUP_TYPE_NEWS) {
 					if (cancel_article (group, &arts[this_resp], this_resp))
 						draw_page (group->name, 0);
 				} else
@@ -620,7 +622,7 @@ page_goto_next_unread:
 			case iKeyPageFollowupQuote:		/* post a followup to this article */
 			case iKeyPageFollowupQuoteHeaders:
 			case iKeyPageFollowup:
-				if (!can_post) {
+				if (!can_post && art_type == GROUP_TYPE_NEWS) {
 					info_message (_(txt_cannot_post));
 					break;
 				}
@@ -631,7 +633,7 @@ page_goto_next_unread:
 				break;
 
 			case iKeyHelp:	/* help */
-				show_info_page (HELP_INFO, help_page, _(txt_art_pager_com));
+				show_help_page (PAGE_LEVEL, _(txt_art_pager_com));
 				draw_page (group->name, 0);
 				break;
 
@@ -706,7 +708,7 @@ return_to_index:
 
 			case iKeyPagePrevArt:	/* previous article */
 				if ((n = prev_response (this_resp)) == -1)
-					return (this_resp);
+					return this_resp;
 
 				if (load_article(n) < 0)
 					return GRP_ARTFAIL;
@@ -753,7 +755,7 @@ return_to_index:
 
 			case iKeyPostponed:
 			case iKeyPostponed2:	/* post postponed article */
-				if (can_post) {
+				if (can_post || art_type != GROUP_TYPE_NEWS) {
 					if (pickup_postponed_articles (FALSE, FALSE))
 						draw_page (group->name, 0);
 				} else
@@ -813,7 +815,83 @@ return_to_index:
 		}
 	}
 	/* NOTREACHED */
-	return GRP_ARTFAIL; /* default-value - I don't think we should get here */
+	return GRP_ARTFAIL;
+}
+
+
+static void
+print_message_page (
+	FILE *file,
+	t_lineinfo *messageline,
+	size_t messagelines,
+	size_t lcurr_line,
+	size_t begin,
+	size_t end,
+	int help_level)
+{
+	size_t i = begin;
+
+	for (; i < end; i++) {
+		t_lineinfo *curr;
+		char *line;
+
+		if (lcurr_line + i >= messagelines)		/* ran out of message */
+			break;
+
+		curr = &messageline[lcurr_line + i];
+		fseek (file, curr->offset, SEEK_SET);
+		if ((line = tin_fgets (file, FALSE)) == NULL)
+			break;	/* ran out of message */
+		if ((int) strlen(line) >= cCOLS)
+			line[cCOLS] = '\0';
+
+		/*
+		 * rotN encoding on body and sig data only
+		 */
+		if ((rotate != 0) && (curr->flags & (C_BODY | C_SIG))) {
+			char *p = line;
+
+			for (p = line; *p; p++) {
+				if (*p >= 'A' && *p <= 'Z')
+					*p = (*p - 'A' + rotate) % 26 + 'A';
+				else if (*p >= 'a' && *p <= 'z')
+					*p = (*p - 'a' + rotate) % 26 + 'a';
+			}
+		}
+
+		strip_line(line);
+
+#ifndef USE_CURSES
+		snprintf (screen[i + scroll_region_top].col, cCOLS, "%s" cCRLF, line);
+#endif /* !USE_CURSES */
+
+		MoveCursor (i + scroll_region_top, 0);
+		draw_pager_line (line, curr->flags);
+
+		/*
+		 * Highlight URL's and mail addresses
+		 */
+		if (curr->flags & C_URL)
+			highlight_regexes (i + scroll_region_top, &url_regex);
+
+		if (curr->flags & C_MAIL)
+			highlight_regexes (i + scroll_region_top, &mail_regex);
+
+		if (curr->flags & C_NEWS)
+			highlight_regexes (i + scroll_region_top, &news_regex);
+
+		/* Blank the screen after a ^L (only occurs when showing cooked) */
+		if (!reveal_ctrl_l && (curr->flags & C_CTRLL)) {
+			CleartoEOS();
+			break;
+		}
+	}
+
+#ifdef HAVE_COLOR
+	fcol(tinrc.col_text);
+#endif /* HAVE_COLOR */
+
+	show_mini_help (help_level);
 }
 
 
@@ -828,7 +906,6 @@ draw_page (
 	const char *group,
 	int part)
 {
-	char *buff;
 	int i;
 	int end;	/* last line to draw */
 
@@ -844,80 +921,21 @@ draw_page (
 
 	search_line = curr_line;	/* Reset search to start from top of display */
 
-	buff = my_malloc(cCOLS+1);	/* Need to account for \n */
-
 	if (part == 0) {
 		ClearScreen();
 		show_first_header (group);
 	} else
 		MoveCursor (0, 0);
 
+	scroll_region_top = PAGE_HEADER;
+
 	/* Down-scroll, only redraw bottom 'part' lines of screen */
 	i = (part > 0) ? ARTLINES-part : 0;
 
 	/* Up-scroll, only redraw the top 'part' lines of screen */
 	end = (part < 0) ? -part : ARTLINES;
 
-	for (; i < end; i++) {
-		t_lineinfo *curr;
-
-		if (curr_line + i >= artlines)		/* ran out of article */
-			break;
-
-		curr = &artline[curr_line+i];
-		fseek (note_fp, curr->offset, SEEK_SET);
-
-		fgets (buff, cCOLS+1, note_fp);
-
-		/*
-		 * rotN encoding on body and sig data only
-		 */
-		if ((rotate != 0) && (curr->flags & (C_BODY | C_SIG))) {
-			char *p = buff;
-
-			for (p = buff; *p; p++) {
-				if (*p >= 'A' && *p <= 'Z')
-					*p = (*p - 'A' + rotate) % 26 + 'A';
-				else if (*p >= 'a' && *p <= 'z')
-					*p = (*p - 'a' + rotate) % 26 + 'a';
-			}
-		}
-
-		strip_line(buff);
-
-#ifndef USE_CURSES
-		snprintf (screen[i+PAGE_HEADER].col, cCOLS, "%s" cCRLF, buff);
-#endif /* !USE_CURSES */
-
-		MoveCursor (i+PAGE_HEADER, 0);
-		draw_pager_line (buff, curr->flags);
-
-		/*
-		 * Highlight URL's and mail addresses
-		 */
-		if (curr->flags & C_URL)
-			highlight_regexes (i+PAGE_HEADER, &url_regex);
-
-		if (curr->flags & C_MAIL)
-			highlight_regexes (i+PAGE_HEADER, &mail_regex);
-
-		if (curr->flags & C_NEWS)
-			highlight_regexes (i+PAGE_HEADER, &news_regex);
-
-		/* Blank the screen after a ^L (only occurs when showing cooked) */
-		if (!reveal_ctrl_l && (curr->flags & C_CTRLL)) {
-			CleartoEOS();
-			break;
-		}
-	}
-
-#ifdef HAVE_COLOR
-	fcol(tinrc.col_text);
-#endif /* HAVE_COLOR */
-
-	free(buff);
-
-	show_mini_help (PAGE_LEVEL);
+	print_message_page (note_fp, artline, artlines, curr_line, i, end, PAGE_LEVEL);
 
 	/*
 	 * Print an appropriate footer
@@ -956,9 +974,9 @@ show_mime_article (
 			decode_save_mime (&pgart, FALSE);
 			return;
 		}
-		snprintf (buf, sizeof(buf)-1, mm);
+		snprintf (buf, sizeof(buf) - 1, mm);
 	} else
-		snprintf (buf, sizeof(buf)-1, METAMAIL_CMD, PATH_METAMAIL);
+		snprintf (buf, sizeof(buf) - 1, METAMAIL_CMD, PATH_METAMAIL);
 
 	EndWin();
 	Raw(FALSE);
@@ -978,7 +996,7 @@ show_mime_article (
 	/* This is redundant, but harmless, unless we are viewing the raw art */
 	fseek (fp, offset, SEEK_SET);	/* goto old position */
 
-	MoveCursor (cLINES, MORE_POS-(5+BLANK_PAGE_COLS));
+	MoveCursor (cLINES, MORE_POS - (5 + BLANK_PAGE_COLS));
 	StartInverse ();
 
 	my_flush ();
@@ -1030,7 +1048,7 @@ show_first_header (
 	buf[i] = '\0';
 
 	if (maxlen != grplen) {					/* ie groupname was too long */
-		strncat (buf, group, maxlen-3);
+		strncat (buf, group, maxlen - 3);
 		strcat (buf, "...");
 	} else
 		strncat (buf, group, maxlen);
@@ -1044,18 +1062,7 @@ show_first_header (
 	fcol(tinrc.col_head);
 #endif /* HAVE_COLOR */
 
-	/* Displaying the value of X-Comment-To header in the upper right corner */
-	if (note_h->ftnto && tinrc.show_xcommentto) {
-		char ftbuf[HEADER_LEN]; /* FTN-To aka X-Comment-To */
-		my_fputs (buf, stdout);
-		strip_address(note_h->ftnto, ftbuf);
-		ftbuf[19] = '\0';
-		convert_to_printable (ftbuf);
-		StartInverse ();
-		my_fputs (ftbuf, stdout);
-		EndInverse ();
-		my_fputs (cCRLF, stdout);
-	} else {
+	{
 		char x[5];
 
 		/* Can't eval tin_ltoa() more than once in a statement due to statics */
@@ -1129,16 +1136,16 @@ show_first_header (
 	if (arts[this_resp].name)
 		sprintf (buf, "%s <%s>", arts[this_resp].name, arts[this_resp].from);
 	else
-		strncpy (buf, arts[this_resp].from, cCOLS-1);
-	buf[cCOLS-1] = '\0';
+		strncpy (buf, arts[this_resp].from, cCOLS - 1);
+	buf[cCOLS - 1] = '\0';
 
 	if (note_h->org) {
 		sprintf (tmp, _(txt_at_s), note_h->org);
-		tmp[sizeof(tmp)-1] = '\0';
+		tmp[sizeof(tmp) - 1] = '\0';
 
-		if ((int) strlen (buf) + (int) strlen (tmp) >= cCOLS -1) {
+		if ((int) strlen (buf) + (int) strlen (tmp) >= cCOLS - 1) {
 			strncat (buf, tmp, cCOLS - 1 - strlen(buf));
-			buf[cCOLS-1] = '\0';
+			buf[cCOLS - 1] = '\0';
 		} else {
 			pos = cCOLS - 1 - (int) strlen(tmp);
 			for (i = strlen(buf); i < pos; i++)
@@ -1273,13 +1280,16 @@ prompt_response (
 
 
 /*
- * Reposition within art as needed, highlight searched string
+ * Reposition within message as needed, highlight searched string
  * This is tied quite closely to the information stored by
  * get_search_vectors()
  */
 static void
 process_search(
-	void)
+	int *lcurr_line,
+	size_t message_lines,
+	size_t screen_lines,
+	int help_level)
 {
 	int i, start, end;
 
@@ -1287,26 +1297,35 @@ process_search(
 		return;
 
 	/*
-	 * Is matching line off the current view ?
+	 * Is matching line off the current view?
 	 * Reposition within article if needed, try to get matched line
 	 * in the middle of the screen
 	 */
-	if (i < curr_line || i >= curr_line+ARTLINES) {
-		int ideal_pos = i - (ARTLINES / 2);
+	if (i < (int)*lcurr_line || i >= (int)(*lcurr_line + screen_lines)) {
+		int ideal_pos = i - (screen_lines / 2);
 
-		if (ideal_pos + ARTLINES > artlines)		/* Off the end */
-			curr_line = artlines - ARTLINES;
+		if (ideal_pos + screen_lines > message_lines)		/* Off the end */
+			*lcurr_line = message_lines - screen_lines;
 		else										/* Pos is just fine */
-			curr_line = ideal_pos;
+			*lcurr_line = ideal_pos;
 	}
 
-	draw_page (CURR_GROUP.name, 0);
+	switch (help_level) {
+		case PAGE_LEVEL:
+			draw_page (CURR_GROUP.name, 0);
+			break;
+		case INFO_PAGER:
+			display_info_page (0);
+			break;
+		default:
+			break;
+	}
 	search_line = i;								/* draw_page() resets this to 0 */
 
 	/*
 	 * Highlight found string
 	 */
-	highlight_string (i - curr_line + HDR_ADJUST + 2, start, end - start);
+	highlight_string (i - *lcurr_line + scroll_region_top, start, end - start);
 }
 
 
@@ -1344,12 +1363,12 @@ toggle_raw(
 				j++;
 				if (j >= chunk) {
 					chunk += 50;
-					pgart.rawl = my_realloc((char *)pgart.rawl, sizeof(t_lineinfo) * chunk);
+					pgart.rawl = my_realloc((char *) pgart.rawl, sizeof(t_lineinfo) * chunk);
 				}
-			} while ((fgets(buff, cCOLS+1, pgart.raw)) != NULL);
+			} while ((fgets(buff, cCOLS + 1, pgart.raw)) != NULL);
 
 			j--;
-			pgart.rawl = my_realloc((char *)pgart.rawl, sizeof(t_lineinfo) * j);
+			pgart.rawl = my_realloc((char *) pgart.rawl, sizeof(t_lineinfo) * j);
 		}
 		artline = pgart.rawl;
 		artlines = j;
@@ -1376,7 +1395,7 @@ process_url(
 	 *       really use curr_line or better start at top of article?
 	 */
 	for (i = curr_line; i < artlines; ++i) {
-		if (!artline[i].flags & C_URL)
+		if (!(artline[i].flags & (C_URL|C_NEWS|C_MAIL)))
 			continue;
 
 		/*
@@ -1388,25 +1407,35 @@ process_url(
 		/*
 		 * Step through, finding URL's
 		 *
-		 * add MAIL_REGEX/NEWS_REGEX
+		 * TODO: add multiple URLs per line handling
+		 *       handle mailto: and news: (not NNTP) URLs internal
 		 */
-		while (pcre_exec (url_regex.re, url_regex.extra, ptr, strlen(ptr), 0, 0, offsets, offsets_size) != PCRE_ERROR_NOMATCH) {
+		forever {
 			char url[LEN];
 
+			/* any matches left? */
+			if (pcre_exec (url_regex.re, url_regex.extra, ptr, strlen(ptr), 0, 0, offsets, offsets_size) == PCRE_ERROR_NOMATCH)
+				if (pcre_exec (mail_regex.re, mail_regex.extra, ptr, strlen(ptr), 0, 0, offsets, offsets_size) == PCRE_ERROR_NOMATCH)
+					if (pcre_exec (news_regex.re, news_regex.extra, ptr, strlen(ptr), 0, 0, offsets, offsets_size) == PCRE_ERROR_NOMATCH)
+						break;
+
 			*(ptr + offsets[1]) = '\0';
+
+			/* TODO: -> lang.c */
 			if (prompt_default_string ("URL:", url, sizeof(url), ptr + offsets[0], HIST_NONE)) {
 				char ubuf[LEN];
 
+				/* TODO: -> lang.c */
 				wait_message(2, "Launching %s\n", url);
+				/* TODO make the handler configurable via 'M'enu */
 				strcpy(ubuf, "url_handler.sh ");
 				strncat(ubuf, escape_shell_meta (url, 0), LEN - 16);
 				invoke_cmd (ubuf);
 			}
-
 			ptr += offsets[1];
 		}
 	}
-
+	/* TODO: -> lang.c */
 	info_message ("No more URL's");
 }
 
@@ -1430,3 +1459,196 @@ resize_article(
 	artlines = pgart.cooked_lines;
 	note_fp = pgart.cooked;
 }
+
+
+/*
+ * Infopager: simply page files
+ */
+void
+info_pager (
+	FILE *info_fh,
+	const char *title,
+	t_bool wrap_at_ends)
+{
+	int ch;
+
+	info_file = info_fh;
+	info_title = title;
+	curr_info_line = 0;
+	preprocess_info_message (info_fh);
+	set_xclick_off ();
+	display_info_page (0);
+	forever {
+
+		switch (ch = handle_pager_keypad(&menukeymap.info_nav)) {
+			case ESC:	/* common arrow keys */
+				break;
+
+			case iKeyUp:				/* line up */
+			case iKeyUp2:
+				if (curr_info_line == 0) {
+					if (!wrap_at_ends) {
+						info_message (_(txt_begin_of_art));
+						break;
+					}
+					curr_info_line = num_info_lines - NOTESLINES;
+					display_info_page (0);
+					break;
+				}
+				curr_info_line--;
+				if (have_linescroll) {
+					scroll_page (-1);
+					display_info_page (-1);
+				} else
+					display_info_page (0);
+				break;
+
+			case iKeyDown:				/* line down */
+			case iKeyDown2:
+				if (curr_info_line + NOTESLINES >= num_info_lines) {
+					if (!wrap_at_ends) {
+						info_message (_(txt_end_of_art));
+						break;
+					}
+					curr_info_line = 0;
+					display_info_page (0);
+					break;
+				}
+				curr_info_line++;
+				if (have_linescroll) {
+					scroll_page (1);
+					display_info_page (1);
+				} else
+					display_info_page (0);
+				break;
+
+			case iKeyPageDown:			/* page down */
+			case iKeyPageDown2:
+			case iKeyPageDown3:
+				if (curr_info_line + NOTESLINES >= num_info_lines) {	/* End is already on screen */
+					if (!wrap_at_ends) {
+						info_message (_(txt_end_of_art));
+						break;
+					}
+					curr_info_line = 0;
+					display_info_page (0);
+					break;
+				}
+				curr_info_line += (tinrc.full_page_scroll) ? NOTESLINES : NOTESLINES/2;
+				display_info_page (0);
+				break;
+
+			case iKeyPageUp:			/* page up */
+			case iKeyPageUp2:
+			case iKeyPageUp3:
+				if (curr_info_line == 0) {
+					if (!wrap_at_ends) {
+						info_message (_(txt_begin_of_art));
+						break;
+					}
+					curr_info_line = num_info_lines - NOTESLINES;
+					display_info_page (0);
+					break;
+				}
+				curr_info_line -= (tinrc.full_page_scroll) ? NOTESLINES : NOTESLINES/2;
+				display_info_page (0);
+				break;
+
+			case iKeyFirstPage:			/* Home */
+			case iKeyHelpFirstPage2:
+				if (curr_info_line != 0) {
+					curr_info_line = 0;
+					display_info_page (0);
+				}
+				break;
+
+			case iKeyLastPage:			/* End */
+			case iKeyHelpLastPage2:
+				if (curr_info_line + NOTESLINES != num_info_lines) {
+					/* Display a full last page for neatness */
+					curr_info_line = num_info_lines - NOTESLINES;
+					display_info_page (0);
+				}
+				break;
+
+			case iKeyToggleHelpDisplay:
+				toggle_mini_help (INFO_PAGER);
+				display_info_page (0);
+				break;
+
+			case iKeySearchSubjF:
+			case iKeySearchSubjB:
+				if ((search_article ((ch == iKeySearchSubjF), search_line, num_info_lines, infoline, TRUE, info_file)) == -1)
+					break;
+
+				process_search (&curr_info_line, num_info_lines, NOTESLINES, INFO_PAGER);
+				break;
+
+			default:
+				/* any other key quits pager; useful? */
+				ClearScreen ();
+				return;
+		}
+	}
+}
+
+
+void
+display_info_page (
+	int part)
+{
+	int end, i;
+
+	signal_context = cInfopager;
+
+	if (curr_info_line < 0)
+		curr_info_line = 0;
+	if (curr_info_line >= num_info_lines)
+		curr_info_line = num_info_lines - 1;
+
+	/* Print titel */
+	if (part == 0) {
+		ClearScreen ();
+		center_line (0, TRUE, info_title);
+	}
+
+	scroll_region_top = INDEX_TOP;
+
+	/* Down-scroll, only redraw bottom 'part' lines of screen */
+	i = (part > 0) ? NOTESLINES-part : 0;
+
+	/* Up-scroll, only redraw the top 'part' lines of screen */
+	end = (part < 0) ? -part : NOTESLINES;
+
+	print_message_page (info_file, infoline, num_info_lines, curr_info_line, i, end, INFO_PAGER);
+
+	/* print footer */
+	draw_percent_mark (curr_info_line + (curr_info_line + NOTESLINES < num_info_lines ? NOTESLINES : num_info_lines - curr_info_line), num_info_lines);
+	stow_cursor ();
+}
+
+
+static void
+preprocess_info_message (
+	FILE *info_fh)
+{
+	int chunk = 50;
+
+	rewind (info_fh);
+	FreeIfNeeded ((char *)infoline);
+	infoline = my_malloc (sizeof(t_lineinfo) * chunk);
+	num_info_lines = 0;
+
+	do {
+		infoline[num_info_lines].offset = ftell(info_fh);
+		infoline[num_info_lines].flags  = 0;
+		num_info_lines++;
+		if (num_info_lines >= chunk) {
+			chunk += 50;
+			infoline = my_realloc((char *)infoline, sizeof(t_lineinfo) * chunk);
+		}
+	} while (tin_fgets(info_fh, FALSE) != NULL);
+
+	num_info_lines--;
+	infoline = my_realloc((char *)infoline, sizeof(t_lineinfo) * num_info_lines);
+}
