@@ -6,7 +6,7 @@
  *  Updated   : 1999-12-02
  *  Notes     : PGP support
  *
- * Copyright (c) 1995-2001 Steven J. Madsen <steve@erinet.com>
+ * Copyright (c) 1995-2002 Steven J. Madsen <steve@erinet.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -149,10 +149,17 @@ init_pgp (
 
 	pgpopts = get_val("PGPOPTS", "");
 
-	if ((ptr = getenv("PGPPATH")) != (char *) 0)
+#ifdef HAVE_GPG
+	if ((ptr = getenv("GNUPGHOME")) != (char *) 0)
 		my_strncpy (pgp_data, ptr, sizeof(pgp_data));
 	else
-		joinpath (pgp_data, homedir, PGPDIR);
+#endif /* HAVE_GPG */
+	{
+		if ((ptr = getenv("PGPPATH")) != (char *) 0)
+			my_strncpy (pgp_data, ptr, sizeof(pgp_data));
+		else
+			joinpath (pgp_data, homedir, PGPDIR);
+	}
 }
 
 
@@ -196,9 +203,9 @@ split_file (
 	char buf[LEN];
 	mode_t mask;
 
-	snprintf(hdr, sizeof(hdr)-1, HEADERS, TMPDIR, process_id);
-	snprintf(pt, sizeof(pt)-1, PLAINTEXT, TMPDIR, process_id);
-	snprintf(ct, sizeof(ct)-1, CIPHERTEXT, TMPDIR, process_id);
+	snprintf(hdr, sizeof(hdr) - 1, HEADERS, TMPDIR, process_id);
+	snprintf(pt, sizeof(pt) - 1, PLAINTEXT, TMPDIR, process_id);
+	snprintf(ct, sizeof(ct) - 1, CIPHERTEXT, TMPDIR, process_id);
 
 	if ((art = fopen(file, "r")) == (FILE *) 0)
 		return;
@@ -235,9 +242,11 @@ do_pgp (
 	const char *mail_to)
 {
 	char cmd[LEN];
-	char mailfrom[LEN] = "";
+	char mailfrom[LEN];
 	const char *mailto = (mail_to) ? mail_to : "";
 
+	mailfrom[0] = '\0';
+
 	split_file(file);
 
 	/*
@@ -273,23 +282,18 @@ pgp_append_public_key (
 	FILE *fp, *key;
 	char keyfile[PATH_LEN], cmd[LEN], buf[LEN];
 
-	if ((CURR_GROUP.attribute->from) != (char *) 0)
+	if ((CURR_GROUP.attribute->from) != (char *) 0 && strlen(CURR_GROUP.attribute->from))
 		strip_name (CURR_GROUP.attribute->from, buf);
 	else
-		snprintf(buf, sizeof(buf)-1, "%s@%s", userid, host_name);
+		snprintf(buf, sizeof(buf) - 1, "%s@%s", userid, host_name);
 
-	snprintf(keyfile, sizeof(buf)-1, KEYFILE, TMPDIR, process_id);
+	snprintf(keyfile, sizeof(keyfile) - 1, KEYFILE, TMPDIR, process_id);
 
-/* TODO I'm guessing the pgp append key command creates 'keyfile' and that we should remove it */
-#	ifdef HAVE_GPG
-	/*
-	 * FIXME: reoder !gpg opts (if possible) and make this obsolete
-	 *        we can't reoder the gpg opts as --export must be the last one
-	 */
-		sh_format (cmd, sizeof(cmd), APPEND_KEY, PGPNAME, pgpopts, buf, keyfile);
-#	else
-		sh_format (cmd, sizeof(cmd), APPEND_KEY, PGPNAME, pgpopts, keyfile, buf);
-#	endif /* HAVE_GPG */
+/*
+ * TODO: I'm guessing the pgp append key command creates 'keyfile' and that
+ * we should remove it
+ */
+	sh_format (cmd, sizeof(cmd), APPEND_KEY, PGPNAME, pgpopts, keyfile, buf);
 
 	if (invoke_cmd (cmd)) {
 		if ((fp = fopen(file, "a")) != NULL) {
@@ -433,10 +437,10 @@ pgp_check_article (
 	if (!pgp_available())
 		return FALSE;
 
-	joinpath(artfile, homedir, ".article");
+	joinpath(artfile, homedir, TIN_ARTICLE_NAME);
 
 #	ifdef APPEND_PID
-	snprintf (artfile+strlen(artfile), sizeof(artfile)-1, ".%d", (int) process_id);
+	snprintf (artfile + strlen(artfile), sizeof(artfile) - 1, ".%d", (int)process_id);
 #	endif /* APPEND_PID */
 
 	if ((art = fopen(artfile, "w")) == (FILE *) 0) {
@@ -468,7 +472,7 @@ pgp_check_article (
 		/*
 		 * We don't use sh_format here else the redirection get misquoted
 		 */
-		snprintf (cmd, sizeof(cmd)-1, CHECK_SIGN, PGPNAME, pgpopts, artfile, REDIRECT_PGP_OUTPUT);
+		snprintf (cmd, sizeof(cmd) - 1, CHECK_SIGN, PGPNAME, pgpopts, artfile, REDIRECT_PGP_OUTPUT);
 		invoke_cmd(cmd);
 		my_printf("\n");
 		Raw(TRUE);
