@@ -6,7 +6,7 @@
  *  Updated   : 1997-12-31
  *  Notes     :
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 #ifndef MENUKEYS_H
-#	include  "menukeys.h"
+#	include "menukeys.h"
 #endif /* !MENUKEYS_H */
 
 /*
@@ -73,7 +73,7 @@ prompt_num (
 
 	clear_message ();
 
-	return (num);
+	return num;
 }
 
 
@@ -131,7 +131,7 @@ prompt_default_string (
 t_bool
 prompt_menu_string (
 	int line,
-	int col,
+	const char *prompt,
 	char *var)
 {
 	char *p;
@@ -139,13 +139,13 @@ prompt_menu_string (
 	/*
 	 * clear buffer - this is needed, otherwise a lost
 	 * connection right before a resync_active() call
-	 * would lead to a 'n' answer to the reconect prompt
+	 * would lead to a 'n' answer to the reconnect prompt
 	 */
 	fflush(stdin);
 
-	MoveCursor (line, col);
+	MoveCursor (line, 0);
 
-	if ((p = tin_getline ("", FALSE, var, 0, FALSE, HIST_OTHER)) == (char *) 0)
+	if ((p = tin_getline (prompt, FALSE, var, 0, FALSE, HIST_OTHER)) == (char *) 0)
 		return FALSE;
 
 	strcpy (var, p);
@@ -201,13 +201,12 @@ prompt_yn (
 		yn_loop = FALSE; /* normal case: leave loop */
 
 		switch (ch) {
-#ifndef WIN32
 			case ESC:	/* (ESC) common arrow keys */
 #	ifdef HAVE_KEY_PREFIX
 			case KEY_PREFIX:
 #	endif /* HAVE_KEY_PREFIX */
 				switch (get_arrow_key (ch)) {
-#endif /* !WIN32 */
+
 					case KEYMAP_UP:
 					case KEYMAP_DOWN:
 						default_answer = !default_answer;
@@ -221,12 +220,11 @@ prompt_yn (
 					case KEYMAP_RIGHT:
 						ch = prompt_ch;
 						break;
-#ifndef WIN32
 					default:
 						break;
 				}
 				break;
-#endif /* !WIN32 */
+
 			default:
 				break;
 		}
@@ -267,7 +265,7 @@ prompt_list (
 {
 	int ch, var_orig;
 	int i;
-	int adjust = (strcasecmp(list[0], _(txt_default)) == 0);
+	int adjust = (strcasecmp(_(list[0]), _(txt_default)) == 0);
 	size_t width = 0;
 
 	var += adjust;
@@ -278,13 +276,13 @@ prompt_list (
 	 * Find the length of longest printable text
 	 */
 	for (i = 0; i < size; i++)
-		width = MAX(width, strlen(list[i]));
+		width = MAX(width, strlen(_(list[i])));
 
 	show_menu_help (help_text);
 	cursoron ();
 
 	do {
-		MoveCursor (row, col + (int) strlen (prompt_text));
+		MoveCursor (row, col + (int) strlen (_(prompt_text)));
 		if ((ch = (char) ReadCh ()) == ' ') {
 
 			/*
@@ -293,7 +291,7 @@ prompt_list (
 			++var;
 			var %= size;
 
-			my_printf("%-*s", (int)width, list[var]);
+			my_printf("%-*s", (int)width, _(list[var]));
 			my_flush ();
 		}
 	} while (ch != '\r' && ch != '\n' && ch != ESC);
@@ -301,7 +299,7 @@ prompt_list (
 	if (ch == ESC) {
 		var = var_orig;
 
-		my_printf("%-*s", (int)width, list[var]);
+		my_printf("%-*s", (int)width, _(list[var]));
 		my_flush ();
 	}
 
@@ -324,7 +322,7 @@ prompt_on_off (
 {
 	t_bool ret;
 
-	ret = prompt_list (row, col, (int)*var, help_text, prompt_text, txt_onoff, 2);
+	ret = prompt_list (row, col, (int)*var, help_text, _(prompt_text), txt_onoff, 2);
 	*var = (ret != 0);
 }
 
@@ -333,10 +331,8 @@ prompt_on_off (
  * Displays option text and actual option value for string based options in
  * one line, help text for that option near the bottom of the screen. Allows
  * change of the old value by normal editing; history function of tin_getline()
- * will be used properly so that editing won't leave the actual line. Note
- * that "option" is the number the user will see, which is not the same as
- * the array position for this option in option_table (since the latter
- * starts counting with zero instead of one).
+ * will be used properly so that editing won't leave the actual line.
+ *
  * The function returns TRUE, if the value was changed, FALSE otherwise.
  */
 t_bool
@@ -344,30 +340,20 @@ prompt_option_string (
 	int option) /* return value is always ignored */
 {
 	char prompt[LEN];
-	char *p;
 	char *variable = OPT_STRING_list[option_table[option].var_index];
 
 	show_menu_help (option_table[option].txt->help);
-	MoveCursor (option_row(option), 0);
-	sprintf (&prompt[0], "-> %3d. %s ", option+1, option_table[option].txt->opt);
-
-	if ((p = tin_getline (prompt, FALSE, variable, 0, FALSE, HIST_OTHER)) == (char *) 0)
-		return FALSE;
-
-	strcpy (variable, p);
-
-	return TRUE;
+	fmt_option_prompt (prompt, sizeof(prompt)-1, TRUE, option);
+	return prompt_menu_string (option_row(option), prompt, variable);
 }
 
 
 /*
- * Displays option text and actual option value for number based options in
+ * Displays option text and current option value for number based options in
  * one line, help text for that option near the bottom of the screen. Allows
  * change of the old value by normal editing; history function of tin_getline()
- * will be used properly so that editing won't leave the actual line. Note
- * that "option" is the number the user will see, which is not the same as
- * the array position for this option in option_table (since the latter
- * starts counting with zero instead of one).
+ * will be used properly so that editing won't leave the current line.
+ *
  * The function returns TRUE if the value was changed, FALSE otherwise.
  */
 t_bool
@@ -381,7 +367,7 @@ prompt_option_num (
 
 	show_menu_help (option_table[option].txt->help);
 	MoveCursor (option_row(option), 0);
-	sprintf (&prompt[0], "-> %3d. %s ", option+1, option_table[option].txt->opt);
+	fmt_option_prompt (prompt, sizeof(prompt)-1, TRUE, option);
 	sprintf (&number[0], "%d", *(option_table[option].variable));
 
 	if ((p = tin_getline (prompt, 2, number, 0, FALSE, HIST_OTHER)) == (char *) 0)
@@ -400,10 +386,8 @@ prompt_option_num (
 /*
  * Displays option text and actual option value for character based options
  * in one line, help text for that option near the bottom of the screen.
- * Allows change of the old value by normal editing. Note that "option" is
- * the number the user will see, which is not the same as the array position
- * for this option in option_table (since the latter starts counting with
- * zero instead of one).
+ * Allows change of the old value by normal editing.
+ *
  * The function returns TRUE if the value was changed, FALSE otherwise.
  */
 t_bool
@@ -420,7 +404,7 @@ prompt_option_char (
 
 	show_menu_help (option_table[option].txt->help);
 	MoveCursor (option_row(option), 0);
-	sprintf (&prompt[0], "-> %3d. %s ", option+1, option_table[option].txt->opt);
+	fmt_option_prompt (prompt, sizeof(prompt)-1, TRUE, option);
 
 	if ((p = tin_getline (prompt, FALSE, p, 1, FALSE, HIST_OTHER)) == (char *) 0)
 		return FALSE;
@@ -462,7 +446,7 @@ prompt_string_default (
 		}
 	}
 
-	return(def);					/* use the default */
+	return def;					/* use the default */
 }
 
 
@@ -477,8 +461,8 @@ prompt_msgid (
 {
 	char buf[LEN];
 
-	if (prompt_string(_(txt_enter_message_id), buf+1, HIST_MESSAGE_ID) && buf[1]) {
-		char *ptr = buf+1;
+	if (prompt_string(_(txt_enter_message_id), buf + 1, HIST_MESSAGE_ID) && buf[1]) {
+		char *ptr = buf + 1;
 		struct t_msgid *msgid;
 
 		/*
@@ -533,12 +517,12 @@ sized_message (
 	int have = cCOLS - strlen (format) + 4 - 1;
 	int want = strlen(subject);
 
-	if (want > 0 && subject[want-1] == '\n')
+	if (want > 0 && subject[want - 1] == '\n')
 		want--;
 	if (have > want)
 		have = want;
 	sprintf (mesg, format, have, subject);
-	return(mesg);
+	return mesg;
 }
 
 
@@ -550,7 +534,8 @@ int
 prompt_slk_response (
 	int ch_default,
 	const t_menukeys /* char */ *responses,
-	const char *fmt, ...)
+	const char *fmt,
+	...)
 {
 	va_list ap;
 	char ch;
@@ -595,7 +580,6 @@ prompt_continue (
 	info_message (_(txt_return_key));
 	ch = ReadCh ();
 
-#ifndef WIN32
 	switch (ch) {
 		case ESC:
 #	ifdef HAVE_KEY_PREFIX
@@ -606,7 +590,6 @@ prompt_continue (
 		default:
 			break;
 	}
-#endif /* !WIN32 */
 
 #ifdef USE_CURSES
 	cmd_line = FALSE;
