@@ -5,7 +5,7 @@
  *  Created   : 1997-04-10
  *  Updated   : 1998-07-04
  *
- * Copyright (c) 1997-2001 Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 1997-2002 Jason Faultless <jason@radar.tele2.co.uk>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,6 +40,9 @@
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
+#ifndef TNNTP_H
+#	include "tnntp.h"
+#endif /* !TNNTP_H */
 
 /*
  * The initial and expansion sizes to use for allocating read data
@@ -99,13 +102,13 @@ wait_for_input (
 
 		FD_ZERO(&readfds);
 		FD_SET(STDIN_FILENO, &readfds);
-/*		FD_SET(fileno(NEED_REAL_NNTP_FD_HERE), &readfds);*/
+/*		FD_SET(fileno(NEED_REAL_NNTP_FD_HERE), &readfds); */
 
-		tv.tv_sec = 0;		/*NNTP_READ_TIMEOUT;*/
+		tv.tv_sec = 0;		/* NNTP_READ_TIMEOUT; */
 		tv.tv_usec = 0;
 
 /*DEBUG_IO((stderr, "waiting on %d and %d...", STDIN_FILENO, fileno(fd)));*/
-		if ((nfds = select(STDIN_FILENO+1, &readfds, NULL, NULL, &tv)) == -1) {
+		if ((nfds = select(STDIN_FILENO + 1, &readfds, NULL, NULL, &tv)) == -1) {
 			if (errno != EINTR) {
 				perror_message("select() failed");
 				giveup();
@@ -124,12 +127,26 @@ wait_for_input (
 
 			/*
 			 * User pressed something. If 'q'uit, then handle this. Process
-			 * user input 1st so they get chance to quit on busy (or stalled) reads
+			 * user input 1st so they get chance to quit on busy (or stalled)
+			 * reads
 			 */
 			if (FD_ISSET(STDIN_FILENO, &readfds)) {
 				if ((ch = ReadCh()) == EOF)
 					return FALSE;
 
+				/*
+				 * check if keymap key was pressed. Unfortunately, keymap keys
+				 * are encoded by a sequence of bytes starting with ESC, so we
+				 * must first see if it was really an ESC or a keymap key before
+				 * asking the user if he wants to abort.
+				 */
+				if (ch == ESC) {
+					int keymap_ch = get_arrow_key (ch);
+
+					if (keymap_ch != KEYMAP_UNKNOWN)
+						ch = keymap_ch;
+				}
+
 				if (ch == 'q' || ch == 'z' || ch == ESC) {
 					if (prompt_yn (cLINES, _(txt_read_abort), FALSE) == 1)
 						return TRUE;
@@ -203,7 +220,7 @@ tin_read (
 		clear_message();
 		tin_errno = TIN_ABORT;
 		fflush(stdin);
-		return(NULL);
+		return NULL;
 	}
 
 	errno = 0;		/* To check errno after read, clear it here */
@@ -228,7 +245,7 @@ tin_read (
 #endif /* DEBUG */
 
 	if (ptr == 0)	/* End of data ? */
-		return(NULL);
+		return NULL;
 
 	/*
 	 * Was this only a partial read ?
@@ -247,10 +264,11 @@ tin_read (
 		}
 
 		/*
-		 * If we're looking for continuation headers, check for whitespace.
-		 * If we find some, mark this as a partial read and skip it ready for
-		 * the next read. Push a space to effectively condense leading whitespace
-		 * to a single ' '
+		 * If we're looking for continuation headers, mark this as a partial
+		 * read and put back a newline. Unfolding (removing of this newline
+		 * and whitespace, if necessary) must be done at a higher level --
+		 * there are headers where whitespace is significant even in folded
+		 * lines.
 		 */
 #ifdef NNTP_ABLE
 		if (check_dot_only_line && i == 1 && buffer[0] == '.') { /* EMPTY */
@@ -262,16 +280,16 @@ tin_read (
 				if (!i) { /* EMPTY */
 					/* Find a header separator, don't check next line. */
 				} else {
-					while ((c = fgetc (get_nntp_fp(fp))) == ' ' || c == '\t')
+					if ((c = fgetc (get_nntp_fp(fp))) == ' ' || c == '\t') {
 						partial_read = TRUE;
-
-					/* Push back the 1st char after the now-skipped whitespace */
-					if (c != EOF) {
-						ungetc(c, get_nntp_fp(fp));
-						/* TODO - is this portable ? Push back a single ' ' to compress the white-space */
-						if (partial_read)
-							ungetc(' ', get_nntp_fp(fp));
+						/* This is safe because we removed at least one char above */
+						buffer[offset++] = '\n';
+						buffer[offset] = '\0';
 					}
+
+					/* Push back the 1st char of next line */
+					if (c != EOF)
+						ungetc(c, get_nntp_fp(fp));
 				}
 			}
 		}
@@ -280,7 +298,7 @@ tin_read (
 		offset = i;
 	}
 
-	return(buffer);
+	return buffer;
 }
 
 
@@ -295,11 +313,13 @@ tin_read (
  * required post-processing of the data will be done such that we look like
  * a local read to the calling function.
  *
- * Header lines:
- *   If header is TRUE, then we assume we're reading a news article header.
- *   In some cases, article headers are split over multiple lines. The rule
- *   is that if the next line starts with \t or ' ', then it will be included
- *   as part of the current line. We condense leading whitespace to a single ' '
+ * Header lines: If header is TRUE, then we assume we're reading a news
+ *   article header.  In some cases, article headers are split over multiple
+ *   lines. The rule is that if the next line starts with \t or ' ', then it
+ *   will be included as part of the current line. Line breaks are NOT
+ *   stripped (but replaced by \n) in continuated lines except the trailing
+ *   one; unfolding MUST be done at a higher level because it may be
+ *   significant or not.
  *
  * Dynamic read code based on code by <emcmanus@gr.osf.org>
  *
@@ -341,7 +361,7 @@ tin_fgets (
 
 	if (tin_errno != 0) {
 		DEBUG_IO((stderr, _("Aborted read\n")));
-		return(NULL);
+		return NULL;
 	}
 
 	next = offset;
@@ -355,7 +375,7 @@ tin_fgets (
 		next += offset;
 
 		if (tin_errno != 0)
-			return(NULL);
+			return NULL;
 	}
 
 	/*
@@ -368,17 +388,17 @@ tin_fgets (
 		if (dynbuf[0] == '.') {			/* reduce leading .'s */
 			if (dynbuf[1] == '\0') {
 				DEBUG_IO((stderr, "tin_fgets (NULL)\n"));
-				return (NULL);
+				return NULL;
 			}
-			DEBUG_IO((stderr, "tin_fgets (%s)\n", dynbuf+1));
-			return (dynbuf+1);
+			DEBUG_IO((stderr, "tin_fgets (%s)\n", dynbuf + 1));
+			return (dynbuf + 1);
 		}
 	}
 #endif /* NNTP_ABLE */
 
 DEBUG_IO((stderr, "tin_fgets (%s)\n", (dynbuf) ? dynbuf : "NULL"));
 
-	return (dynbuf);
+	return dynbuf;
 }
 
 
