@@ -8,7 +8,7 @@
  *  Credits   : Richard Hodson <richard@macgyver.tele2.co.uk>
  *              hash_msgid, free_msgid
  *
- * Copyright (c) 1996-2001 Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 1996-2002 Jason Faultless <jason@radar.tele2.co.uk>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -129,7 +129,7 @@ hash_msgid (
 
 	hash %= MSGID_HASH_SIZE;
 
-	return(hash);
+	return hash;
 }
 
 /*
@@ -216,7 +216,7 @@ add_msgid (
 		giveup();
 	}
 
-	h = hash_msgid(msgid+1);				/* Don't hash the initial '<' */
+	h = hash_msgid(msgid + 1);				/* Don't hash the initial '<' */
 
 	DEBUG_PRINT((dbgfd, "---------------- Add %s %s with parent %s\n", (key == MSGID_REF) ? "MSG" : "REF", msgid, (newparent == NULL) ? _("unchanged") : newparent->txt));
 
@@ -234,7 +234,7 @@ add_msgid (
 		 */
 		if (newparent == NULL) {
 			DEBUG_PRINT((dbgfd, "nop: %s No parent specified\n", i->txt));
-			return(i);
+			return i;
 		}
 
 		/*
@@ -242,7 +242,7 @@ add_msgid (
 		 */
 		if (newparent == i->parent) {
 			DEBUG_PRINT((dbgfd, "dup: %s -> %s (no change)\n", i->txt, i->parent ? i->parent->txt : "NULL"));
-			return(i);
+			return i;
 		}
 
 		/*
@@ -258,7 +258,7 @@ add_msgid (
 			for (ptr = newparent; ptr != NULL; ptr = ptr->parent) {
 				if (ptr == i) {
 					DEBUG_PRINT((dbgfd, "Avoiding circular reference! (%s)\n", (key == MSGID_REF) ? "MSG" : "REF"));
-					return(i);
+					return i;
 				}
 			}
 
@@ -266,7 +266,7 @@ add_msgid (
 			add_to_parent(i);
 
 			DEBUG_PRINT((dbgfd, "set: %s -> %s\n", i->txt, newparent ? newparent->txt : _("None")));
-			return(i);
+			return i;
 		}
 
 		/*
@@ -281,11 +281,11 @@ add_msgid (
 				(key == MSGID_REF) ? "MSG" : "REF", i->txt,
 				newparent ? newparent->txt : "None", i->parent->txt));
 
-			return(i);
+			return i;
 		}
 
 		error_message("Error: Impossible combination of conditions !\n");
-		return(i);
+		return i;
 	}
 
 	DEBUG_PRINT((dbgfd, "new: %s -> %s\n", msgid, (newparent)?newparent->txt:"None"));
@@ -308,7 +308,7 @@ add_msgid (
 	ptr->next = msgids[h];
 	msgids[h] = ptr;
 
-	return(ptr);
+	return ptr;
 }
 
 /*
@@ -322,7 +322,7 @@ find_msgid (
 	unsigned int h;
 	struct t_msgid *i;
 
-	h = hash_msgid(msgid+1);				/* Don't hash the initial '<' */
+	h = hash_msgid(msgid + 1);				/* Don't hash the initial '<' */
 
 	/*
 	 * Look for this message id in the cache.
@@ -330,10 +330,10 @@ find_msgid (
 	 */
 	for (i = msgids[h]; i != NULL; i = i->next) {
 		if (strcasecmp(i->txt, msgid) == 0)				/* Found it */
-			return(i);
+			return i;
 	}
 
-	return(NULL);
+	return NULL;
 }
 
 /*
@@ -353,7 +353,7 @@ parse_references (
 	struct t_msgid *parent, *current;
 
 	if (!r)
-		return(NULL);
+		return NULL;
 
 	DEBUG_PRINT((dbgfd, "parse_references: %s\n", r));
 
@@ -362,7 +362,7 @@ parse_references (
 	 * A msgid can't contain a space, right ?
 	 */
 	if ((ptr = strtok(r, REF_SEP)) == NULL)
-		return(NULL);
+		return NULL;
 
 	/*
 	 * By definition, the head of the thread has no parent
@@ -375,7 +375,7 @@ parse_references (
 		current = add_msgid(REF_REF, ptr, parent);
 	}
 
-	return(current);
+	return current;
 }
 
 /*
@@ -401,7 +401,7 @@ _get_references (
 		refs = (char *) my_malloc(HEADER_LEN);
 		len = 0;
 	} else
-		refs = _get_references(refptr->parent, depth+1);
+		refs = _get_references(refptr->parent, depth + 1);
 
 	/*
 	 * Attempt at damage limitation in case of broken Refs fields
@@ -411,7 +411,7 @@ _get_references (
 		len = strlen(refs);
 	}
 
-	return (refs);
+	return refs;
 }
 
 /*
@@ -426,17 +426,17 @@ get_references (
 	size_t len;
 
 	if (refptr == NULL)
-		return(NULL);
+		return NULL;
 
 	refs = _get_references(refptr, 1);
 
 	len = strlen(refs);
 
-	refs[len-1] = '\0';
+	refs[len - 1] = '\0';
 
 	refs = (char *) my_realloc(refs, len);
 
-	return(refs);
+	return refs;
 }
 
 /*
@@ -452,7 +452,7 @@ free_msgids (
 
 	msgptr = msgids;				/* first list */
 
-	for (i = MSGID_HASH_SIZE-1; i >= 0; i--) {	/* count down is faster */
+	for (i = MSGID_HASH_SIZE - 1; i >= 0; i--) {	/* count down is faster */
 		ptr = *msgptr;
 		*msgptr++ = NULL;			/* declare list empty */
 
@@ -542,7 +542,7 @@ clear_art_ptrs (
 	int i;
 	struct t_msgid *ptr;
 
-	for (i = MSGID_HASH_SIZE-1; i >= 0; i--) {
+	for (i = MSGID_HASH_SIZE - 1; i >= 0; i--) {
 		for (ptr = msgids[i]; ptr != NULL; ptr = ptr->next)
 			ptr->article = ART_UNAVAILABLE;
 	}
@@ -572,9 +572,9 @@ dump_thread (
 	i = cCOLS - len - 20;
 
 	if (msgid->article >= 0)
-		sprintf(ptr+len, "%-*.*s   %-17.17s", i, i, arts[msgid->article].subject, (arts[msgid->article].name) ? arts[msgid->article].name : arts[msgid->article].from);
+		sprintf(ptr + len, "%-*.*s   %-17.17s", i, i, arts[msgid->article].subject, (arts[msgid->article].name) ? arts[msgid->article].name : arts[msgid->article].from);
 	else
-		sprintf(ptr+len, "%-*.*s", i, i, _("[- Unavailable -]"));
+		sprintf(ptr + len, "%-*.*s", i, i, _("[- Unavailable -]"));
 
 	fprintf(fp, "%s\n", ptr);
 
@@ -597,10 +597,10 @@ dump_msgid_thread (
 	struct t_msgid *ptr,
 	int level)
 {
-	fprintf(dbgfd, "%*s %s (%d)\n", level*3, "   ", ptr->txt, ptr->article);
+	fprintf(dbgfd, "%*s %s (%d)\n", level * 3, "   ", ptr->txt, ptr->article);
 
 	if (ptr->child != NULL)
-		dump_msgid_thread(ptr->child, level+1);
+		dump_msgid_thread(ptr->child, level + 1);
 
 	if (ptr->sibling != NULL)
 		dump_msgid_thread(ptr->sibling, level);
@@ -711,7 +711,7 @@ find_next (
 		}
 	}
 
-	return(ptr);
+	return ptr;
 }
 
 /*
@@ -881,14 +881,14 @@ build_references (
 	/*
 	 * The articles are currently unsorted, and are as they were put by setup_hard_base()
 	 */
-	if (group->attribute && group->attribute->sort_art_type != SORT_BY_NOTHING)
+	if (group->attribute && group->attribute->sort_art_type != SORT_ARTICLES_BY_NOTHING)
 		sort_arts (group->attribute->sort_art_type);
 
 	if (group->attribute)
-		sort_ascend = (group->attribute->sort_art_type == SORT_BY_SUBJ_ASCEND ||
-		               group->attribute->sort_art_type == SORT_BY_FROM_ASCEND ||
-		               group->attribute->sort_art_type == SORT_BY_DATE_ASCEND ||
-		               group->attribute->sort_art_type == SORT_BY_SCORE_ASCEND);
+		sort_ascend = (group->attribute->sort_art_type == SORT_ARTICLES_BY_SUBJ_ASCEND ||
+		               group->attribute->sort_art_type == SORT_ARTICLES_BY_FROM_ASCEND ||
+		               group->attribute->sort_art_type == SORT_ARTICLES_BY_DATE_ASCEND ||
+		               group->attribute->sort_art_type == SORT_ARTICLES_BY_SCORE_ASCEND);
 
 #ifdef DEBUG_REFS
 	dbgfd = fopen("Refs.dump", "w");
@@ -912,7 +912,7 @@ build_references (
 			 * it again later
 			 * Check for circular references to current article
 			 */
-			while (((s = strrchr(art->refs, ' ')) != NULL) && (!strcmp(art->msgid, s+1))) {
+			while (((s = strrchr(art->refs, ' ')) != NULL) && (!strcmp(art->msgid, s + 1))) {
 				/*
 				 * Remove circular reference to current article
 				 */
@@ -920,7 +920,7 @@ build_references (
 				*s = '\0';
 			}
 			if (s != NULL) {
-				art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, s+1, NULL));
+				art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, s + 1, NULL));
 				*s = '\0';
 			} else {
 				art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, art->refs, NULL));
