@@ -6,7 +6,7 @@
  *  Updated   : 1998-01-03
  *  Notes     : RFC 2045/2047 encoding
  *
- * Copyright (c) 1995-2001 Chris Blum <chris@resolution.de>
+ * Copyright (c) 1995-2002 Chris Blum <chris@resolution.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -107,6 +107,7 @@ rfc1521_encode (
 					xpos++;
 				}
 				pattern = 0;
+				bits = 0;
 			}
 			if (xpos) {
 				*b = 0;
@@ -116,20 +117,20 @@ rfc1521_encode (
 			b = NULL;
 		} else {
 			char *line_crlf = line;
-			int len = strlen(line);
+			size_t len = strlen(line);
 			char tmpbuf[2050]; /* FIXME: this is sizeof(buffer)+2 from rfc15211522_encode() */
 
 			/*
 			 * base64 requires CRLF line endings in text types
 			 * convert LF to CRLF if not CRLF already (Windows?)
 			 */
-			if ((len > 0) && (line[len-1] == '\n') &&
-					((len == 1) || (line[len-2] != '\r'))) {
+			if ((len > 0) && (line[len - 1] == '\n') &&
+					((len == 1) || (line[len - 2] != '\r'))) {
 				STRCPY(tmpbuf, line);
 				line_crlf = tmpbuf;
-				line_crlf[len-1] = '\r';
+				line_crlf[len - 1] = '\r';
 				line_crlf[len] = '\n';
-				line_crlf[len+1] = '\0';
+				line_crlf[len + 1] = '\0';
 			}
 
 			while (*line_crlf) {
@@ -161,6 +162,7 @@ rfc1521_encode (
 			 * \n.
 			 */
 			xpos = 0;
+			b = NULL;
 			return;
 		}
 		b = buffer;
