@@ -6,7 +6,7 @@
  *  Updated   : 28/02/2000
  *  Notes     : RFC 2046 MIME article parsing
  *
- * Copyright (c) 2000-2001 Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 2000-2002 Jason Faultless <jason@radar.tele2.co.uk>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -51,6 +51,7 @@
  */
 static int boundary_cmp (const char *line, const char *boundary);
 static void add_persist (struct t_header *hdr, const char *p_header, char *p_content);
+static int count_lines (char *line);
 
 /*
  * Local variables
@@ -374,6 +375,7 @@ free_and_init_header(
 	 * Initialise the header struct
 	 */
 	FreeAndNull(hdr->from);
+	FreeAndNull(hdr->to);
 	FreeAndNull(hdr->date);
 	FreeAndNull(hdr->subj);
 	FreeAndNull(hdr->org);
@@ -412,12 +414,12 @@ parse_header (
 	t_bool decode)
 {
 	size_t plen = strlen (pat);
-	char *ptr = buf+plen;
+	char *ptr = buf + plen;
 
 	/*
 	 * Does ': ' follow the header text?
 	 */
-	if (! (*ptr && *(ptr+1) && *ptr == ':' && *(ptr+1) == ' '))
+	if (! (*ptr && *(ptr + 1) && *ptr == ':' && *(ptr + 1) == ' '))
 		return NULL;
 
 	/*
@@ -459,14 +461,17 @@ parse_rfc822_headers(
 	hdr->ext = new_part(NULL);		/* Initialise MIME data */
 
 	while ((line = tin_fgets (from, TRUE)) != (char *) 0) {
-#ifdef LOCAL_CHARSET
+#if defined(LOCAL_CHARSET) || defined(MAC_OS_X)
 		buffer_to_local(line);
-#endif /* LOCAL_CHARSET */
+#else
+#	ifdef CHARSET_CONVERSION
+	/* headers should be in US-ASCII */
+	buffer_to_local(line, "US-ASCII", tinrc.mm_local_charset);
+#	endif /* CHARSET_CONVERSION */
+#endif /* LOCAL_CHARSET || MAC_OS_X */
 
-		if (to) {
+		if (to)
 			fprintf(to, "%s\n", line);		/* Put raw data */
-			progress(++hdr->ext->line_count);	/* Should we count header lines ? */
-		}
 
 		/*
 		 * End of headers ?
@@ -477,10 +482,27 @@ parse_rfc822_headers(
 			return 0;
 		}
 
+		/*
+		 * FIXME: multiple headers of the same name could lead to memory leak
+		 * and loss of information (multiple Cc: lines are allowed, for example)
+		 */
+		unfold_header (line);
 		if ((ptr = parse_header (line, "From", TRUE))) {
 			hdr->from = my_strdup(ptr);
 			continue;
 		}
+		if ((ptr = parse_header (line, "To", TRUE))) {
+			hdr->to = my_strdup(ptr);
+			continue;
+		}
+		if ((ptr = parse_header (line, "Cc", TRUE))) {
+			hdr->cc = my_strdup (ptr);
+			continue;
+		}
+		if ((ptr = parse_header (line, "Bcc", TRUE))) {
+			hdr->bcc = my_strdup (ptr);
+			continue;
+		}
 		if ((ptr = parse_header (line, "Date", FALSE))) {
 			hdr->date = my_strdup(ptr);
 			continue;
@@ -529,9 +551,9 @@ parse_rfc822_headers(
 			hdr->ftnto = my_strdup(ptr);
 			continue;
 		}
-		if (((ptr = parse_header (line, "Author-IDs", TRUE))) ||
-			((ptr = parse_header (line, "P-Author-IDs", TRUE))) ||
-			((ptr = parse_header (line, "X-P-Author-IDs", TRUE)))) {
+		if ((ptr = parse_header (line, "Author-IDs", TRUE)) ||
+			(ptr = parse_header (line, "P-Author-IDs", TRUE)) ||
+			(ptr = parse_header (line, "X-P-Author-IDs", TRUE))) {
 			hdr->authorids = my_strdup(ptr);
 			continue;
 		}
@@ -556,7 +578,7 @@ parse_rfc822_headers(
 		 */
 		if ((strncmp (line, "P-", 2) == 0 || strncmp(line, "X-P-", 4) == 0) && (ptr = strstr(line, ": "))) {
 			*ptr = '\0';
-			add_persist (hdr, line, ptr+2);
+			add_persist (hdr, line, ptr + 2);
 			continue;
 		}
 	}
@@ -565,6 +587,43 @@ parse_rfc822_headers(
 }
 
 
+/*
+ * Count lines in a continuated header.
+ * line MUST NOT end in a newline.
+ */
+static int
+count_lines(
+	char *line)
+{
+	char *src = line;
+	char c;
+	int lines = 1;
+
+	while ((c = *src++))
+		if (c == '\n')
+			lines++;
+	return lines;
+}
+
+
+/*
+ * Unfold header, i.e. strip any newline off it. Don't strip other
+ * whitespace, it depends on the header if this is legal (structured
+ * headers) or not (unstructured headers, e.g. Subject)
+ */
+void
+unfold_header(
+	char *line)
+{
+	char *src = line, *dst = line;
+	char c;
+
+	while ((c = *src++))
+		if (c != '\n') *dst++ = c;
+	*dst = c;
+}
+
+
 #define M_SEARCHING	1	/* Looking for boundary */
 #define M_HDR		2	/* In MIME headers */
 #define M_BODY		3	/* In MIME body */
@@ -600,7 +659,9 @@ parse_multipart_article(
 /*fprintf(stderr, "---:%s\n", line);*/
 
 		fprintf(artinfo->raw, "%s\n", line);
-		progress(++artinfo->hdr.ext->line_count);		/* Overall line count */
+
+		artinfo->hdr.ext->line_count += count_lines (line);
+		progress(artinfo->hdr.ext->line_count);		/* Overall line count */
 
 		if (bnd == BOUND_END) {							/* End of this part detected */
 #ifdef NNTP_ABLE
@@ -652,7 +713,7 @@ parse_multipart_article(
 					if (curr_part->type == TYPE_MULTIPART) {	/* Complex mutlipart article */
 						int ret;
 
-						if ((ret = parse_multipart_article(infile, artinfo, curr_part, depth+1)) != 0)
+						if ((ret = parse_multipart_article(infile, artinfo, curr_part, depth + 1)) != 0)
 							return ret;
 						else
 							break;
@@ -831,7 +892,7 @@ art_open (
 	if (parse_rfc2045_article (fp, art->line_count, artinfo) != 0)
 		return ART_ABORT;
 
-	if ((pgart.tex2iso = (tex2iso_supported ? is_art_tex_encoded (artinfo->raw) : FALSE)))
+	if ((pgart.tex2iso = ((CURR_GROUP.attribute->tex2iso_conv) ? is_art_tex_encoded (artinfo->raw) : FALSE)))
 		wait_message (0, _(txt_is_tex_encoded));
 
 	/* Maybe fix it so if this fails, we default to raw ? */
