@@ -6,7 +6,7 @@
  *  Updated   : 1998-04-05
  *  Notes     : MIME header encoding/decoding stuff
  *
- * Copyright (c) 1995-2001 Chris Blum <chris@resolution.de>
+ * Copyright (c) 1995-2002 Chris Blum <chris@resolution.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -130,13 +130,17 @@ mmdecode (
 	t_bool decode_gt128 = FALSE;
 
 #ifdef MIME_STRICT_CHARSET
+#	ifdef CHARSET_CONVERSION
+	if (charset && !strcasecmp(charset, tinrc.mm_local_charset))
+#	else
 	if (charset && !strcasecmp(charset, tinrc.mm_charset))
+#	endif /* CHARSET_CONVERSION */
 #endif /* MIME_STRICT_CHARSET */
 		decode_gt128 = TRUE;
 
 	t = where;
 	encoding = tolower((unsigned char)encoding);
-	if (encoding == 'q') {		  /* quoted-printable */
+	if (encoding == 'q') {		/* quoted-printable */
 		int x;
 		unsigned hi, lo;
 
@@ -175,8 +179,9 @@ mmdecode (
 		unsigned char x;
 
 		build_base64_rank_table();
-		if (!what || !where) {	  /* flush */
-			pattern = bits = 0;
+		if (!what || !where) {		/* flush */
+			pattern = 0;
+			bits = 0;
 			return 0;
 		}
 		while (*what != delimiter) {
@@ -217,8 +222,10 @@ rfc1522_decode (
 	char encoding;
 	t_bool adjacentflag = FALSE;
 
+	charset[0] = '\0';
 	c = s;
 	t = buffer;
+
 	while (*c && t - buffer < 2048) {
 		if (*c != '=') {
 			if (adjacentflag && isspace((unsigned char) *c)) {
@@ -227,7 +234,7 @@ rfc1522_decode (
 				dd = c + 1;
 				while (isspace((unsigned char) *dd))
 					dd++;
-				if (*dd == '=') {	  /* brute hack, makes mistakes under certain circumstances comp. 6.2 */
+				if (*dd == '=') {		/* brute hack, makes mistakes under certain circumstances comp. 6.2 */
 					c++;
 					continue;
 				}
@@ -274,9 +281,13 @@ rfc1522_decode (
 	}
 	*t = '\0';
 
-#ifdef LOCAL_CHARSET
+#if defined(LOCAL_CHARSET) || defined(MAC_OS_X)
 	buffer_to_local(buffer);
-#endif /* LOCAL_CHARSET */
+#else
+#	ifdef CHARSET_CONVERSION
+	buffer_to_local(buffer, charset, tinrc.mm_local_charset);
+#	endif /* CHARSET_CONVERSION */
+#endif /* LOCAL_CHARSET || MAC_OS_X */
 
 	return buffer;
 }
@@ -355,7 +366,7 @@ do_b_encode (
  * find out whether encoding is necessary and which encoding
  * to use if necessary by scanning the whole header field
  * instead of each fragment of it.
- * This will ensure that  either Q or B encoding will be used in a single
+ * This will ensure that either Q or B encoding will be used in a single
  * header (i.e. two encoding won't be mixed in a single header line).
  * Mixing two encodings is not a violation of RFC 2047 but may break
  * some news/mail clients.
@@ -386,8 +397,13 @@ which_encoding (
 		 * EUC-KR for backward compatibility with old Korean mail program
 		 */
 		if (chars + 2 * (nonprint + schars) /* QP size */ >
-			 (chars * 4 + 3) / 3	  /* B64 size */
-			 || !strcasecmp(tinrc.mm_charset, "EUC-KR"))
+			 (chars * 4 + 3) / 3		/* B64 size */
+#ifdef CHARSET_CONVERSION
+			 || !strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-KR")
+#else
+			 || !strcasecmp(tinrc.mm_charset, "EUC-KR")
+#endif /* CHARSET_CONVERSION */
+			)
 			return 'B';
 		return 'Q';
 	}
@@ -428,11 +444,9 @@ contains_nonprintables (
 
 
 /*
- * Uncommented this conditional compilation statement
- * in order to implement mandatory break-up of long lines
- * in mail messages in accordance with rfc 2047 (rfc 1522)
+ * implement mandatory break-up of long lines in mail messages in accordance
+ * with rfc 2047 (rfc 1522)
  */
-/* #ifdef MIME_BREAK_LONG_LINES */
 static int
 sizeofnextword (
 	char *w)
@@ -446,11 +460,10 @@ sizeofnextword (
 		x++;
 	return x - w;
 }
-/* #endif */
 
 
 static int
-rfc1522_do_encode (
+rfc1522_do_encode(
 	char *what,
 	char **where,
 	t_bool break_long_line)
@@ -485,23 +498,8 @@ rfc1522_do_encode (
 	t_bool any_quoting_done = FALSE;
 	t_bool isbroken_within = FALSE;	/* is word broken due to length restriction on encoded of word? */
 	t_bool rightafter_ew = FALSE;
-
-/*
- * Uncommented this and other conditional compilation statement
- * depending on MIME_BREAK_LONG_LINES in this function
- * in order to implement mandatory break-up of long lines
- * in mail messages in accordance with RFC 2047 (RFC 1522).
- * Whether or not long lines are broken up depends on
- * boolean variable break_long_line, instead.
- * break_long_line is  FALSE for news posting unless MIME_BREAK_LONG_LINES
- * is defined, but it's TRUE for mail messages regardless of whether or not
- * MIME_BREAK_LONG_LINES is defined
- */
-/* #ifdef MIME_BREAK_LONG_LINES */
 	int column = 0;				/* current column */
 	int word_cnt = 0;
-
-/* #endif */
 	size_t ewsize = 0;			/* size of current encoded-word */
 	char buf[2048];				/* buffer for encoded stuff */
 	char buf2[80];				/* buffer for this and that */
@@ -514,11 +512,10 @@ rfc1522_do_encode (
  * the list of structured header fields where '(' and ')' are
  * treated specially in rfc 1522 encoding
  */
-	static const char *struct_header[] =
-	{
+	static const char *struct_header[] = {
 		"Approved: ", "From: ", "Originator: ",
 		"Reply-To: ", "Sender: ", "X-Cancelled-By: ", "X-Comment-To: ",
-		"X-Submissions-To: ", "To: ", "Cc: ", "Bcc: ", "X-Originator: ", 0};
+		"X-Submissions-To: ", "To: ", "Cc: ", "Bcc: ", "X-Originator: ", 0 };
 	const char **strptr = struct_header;
 
 	do {
@@ -530,8 +527,11 @@ rfc1522_do_encode (
 
 	t = buf;
 	encoding = which_encoding(what);
+#ifdef CHARSET_CONVERSION
+	ew_taken_len = strlen(txt_mime_charsets[tinrc.mm_network_charset]) + 7 /* =?c?E?d?= */;
+#else
 	ew_taken_len = strlen(tinrc.mm_charset) + 7;		/* the minimum encoded word length without any encoded text */
-
+#endif /* CHARSET_CONVERSION */
 	while (*what) {
 		if (break_long_line)
 			word_cnt++;
@@ -543,7 +543,11 @@ rfc1522_do_encode (
 		if (contains_nonprintables(what, isstruct_head) || isbroken_within) {
 			if (encoding == 'Q') {
 				if (!quoting) {
+#ifdef CHARSET_CONVERSION
+					sprintf(buf2, "=?%s?%c?", txt_mime_charsets[tinrc.mm_network_charset], encoding);
+#else
 					sprintf(buf2, "=?%s?%c?", tinrc.mm_charset, encoding);
+#endif /* CHARSET_CONVERSION */
 					ewsize = mystrcat(&t, buf2);
 					if (break_long_line) {
 						if (word_cnt == 2) {
@@ -579,7 +583,7 @@ rfc1522_do_encode (
 					/* Be sure to encode at least one char, even if
 					 * that overflows the line limit, otherwise, we
 					 * will be stuck in a loop (if this were in the
-					 * while condition above).  (Can only happen in
+					 * while condition above). (Can only happen in
 					 * the first line, if we have a very long
 					 * header keyword, I think).
 					 */
@@ -588,7 +592,12 @@ rfc1522_do_encode (
 						break;
 					}
 				}
-				if (!contains_nonprintables(what, isstruct_head) || ewsize >= 70 - strlen(tinrc.mm_charset)) {
+#ifdef CHARSET_CONVERSION
+				if (!contains_nonprintables(what, isstruct_head) || ewsize >= 70 - strlen(txt_mime_charsets[tinrc.mm_network_charset]))
+#else
+				if (!contains_nonprintables(what, isstruct_head) || ewsize >= 70 - strlen(tinrc.mm_charset))
+#endif /* CHARSET_CONVERSION */
+				{
 					/* next word is 'clean', close encoding */
 					*t++ = '?';
 					*t++ = '=';
@@ -598,7 +607,12 @@ rfc1522_do_encode (
  * after the point where it's split should be encoded (i.e. even if
  * they are made of only 7bit chars)
  */
-					if (ewsize >= 70 - strlen(tinrc.mm_charset) && (contains_nonprintables(what, isstruct_head) || isbroken_within)) {
+#ifdef CHARSET_CONVERSION
+					if (ewsize >= 70 - strlen(txt_mime_charsets[tinrc.mm_network_charset]) && (contains_nonprintables(what, isstruct_head) || isbroken_within))
+#else
+					if (ewsize >= 70 - strlen(tinrc.mm_charset) && (contains_nonprintables(what, isstruct_head) || isbroken_within))
+#endif /* CHARSET_CONVERSION */
+					{
 						*t++ = ' ';
 						ewsize++;
 					}
@@ -627,8 +641,11 @@ rfc1522_do_encode (
 				 * as they're already excluded with contain_nonprintables used in outer if-clause
 				 */
 				while (*what && (!isbetween(*what, isstruct_head) || rightafter_ew)) {
-
+#ifdef CHARSET_CONVERSION
+					sprintf(buf2, "=?%s?%c?", txt_mime_charsets[tinrc.mm_network_charset], encoding);
+#else
 					sprintf(buf2, "=?%s?%c?", tinrc.mm_charset, encoding);
+#endif /* CHARSET_CONVERSION */
 					ewsize = mystrcat(&t, buf2);
 
 					if (word_cnt == 2)
@@ -684,7 +701,7 @@ rfc1522_do_encode (
 					 * (due to a long header keyword), we cannot
 					 * stick to that, since we would break the line
 					 * directly after the keyword's colon, which is
-					 * not allowed.  The same is necessary for a
+					 * not allowed. The same is necessary for a
 					 * continuation line with an unencoded word
 					 * that is too long.
 					 */
@@ -726,7 +743,7 @@ rfc1522_encode (
 	static char buf[2048];
 
 /*
- * break_long_line is  FALSE for news posting unless MIME_BREAK_LONG_LINES
+ * break_long_line is FALSE for news posting unless MIME_BREAK_LONG_LINES
  * is defined, but it's TRUE for mail messages regardless of whether or not
  * MIME_BREAK_LONG_LINES is defined
  */
@@ -762,51 +779,42 @@ rfc15211522_encode (
 {
 	FILE *f;
 	FILE *g;
-	char *c, *d;
+	char *c;
+	char *header;
 	char encoding;
 	char buffer[2048];
-	char header[4096];
 	t_bool body_encoding_needed = FALSE;
 	t_bool umlauts = FALSE;
 	BodyPtr body_encode;
 
-	g = tmpfile();
-	if (!g)
+	if ((g = tmpfile()) == NULL)
 		return;
-	f = fopen(filename, "r");
-	if (!f) {
+	if ((f = fopen(filename, "r")) == NULL) {
 		fclose(g);
 		return;
 	}
-	header[0] = 0;
-	d = header;
+
 	quoteflag = 0;
-	while (fgets(buffer, 2048, f)) {
-#ifdef LOCAL_CHARSET
-		buffer_to_network(buffer);
-#endif /* LOCAL_CHARSET */
-		if (header[0]
-			 && (!isspace((unsigned char) buffer[0]) || isreturn(buffer[0]))) {
-			if (allow_8bit_header)
-				fputs(header, g);
-			else
-				fputs(rfc1522_encode(header, ismail), g);
-			fputc('\n', g);
-			header[0] = '\0';
-			d = header;
-		}
-		if (isreturn(buffer[0]))
+
+	while ((header = tin_fgets(f, TRUE))) {
+#if defined(LOCAL_CHARSET) || defined(MAC_OS_X) || defined(CHARSET_CONVERSION)
+		buffer_to_network(header);
+#endif /* LOCAL_CHARSET || MAC_OS_X || defined(CHARSET_CONVERSION) */
+		if (*header == '\0')
 			break;
-		c = buffer;
-		while (*c && !isreturn(*c))
-			*d++ = *c++;
-		*d = 0;
+		if (allow_8bit_header)
+			fputs(header, g);
+		else
+			fputs(rfc1522_encode(header, ismail), g);
+		fputc('\n', g);
 	}
+
 	fputc('\n', g);
+
 	while (fgets(buffer, 2048, f)) {
-#ifdef LOCAL_CHARSET
+#if defined(LOCAL_CHARSET) || defined(MAC_OS_X) || defined(CHARSET_CONVERSION)
 		buffer_to_network(buffer);
-#endif /* LOCAL_CHARSET */
+#endif /* LOCAL_CHARSET || MAC_OS_X || defined(CHARSET_CONVERSION) */
 		fputs(buffer, g);
 		/* see if there are any umlauts in the body... */
 		for (c = buffer; *c && !isreturn(*c); c++)
@@ -818,11 +826,12 @@ rfc15211522_encode (
 	}
 	fclose(f);
 	rewind(g);
-	f = fopen(filename, "w");
-	if (!f) {
+
+	if ((f = fopen(filename, "w")) == NULL) {
 		fclose(g);
 		return;
 	}
+
 	while (fgets(buffer, 2048, g) && !isreturn(buffer[0]))
 		fputs(buffer, f);
 
@@ -836,14 +845,20 @@ rfc15211522_encode (
 	{
 		fputs("MIME-Version: 1.0\n", f);
 		if (body_encoding_needed) {
-
 		/* added for CJK charsets like EUC-KR/JP/CN and others */
-
-			if (!strncasecmp(tinrc.mm_charset, "euc-", 4) &&
+#ifdef CHARSET_CONVERSION
+			if (!strncasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-", 4) &&
+				 !strcasecmp(mime_encoding, txt_7bit))
+				fprintf(f, "Content-Type: text/plain; charset=ISO-2022-%s\n", &txt_mime_charsets[tinrc.mm_network_charset][4]);
+			else
+				fprintf(f, "Content-Type: text/plain; charset=%s\n", txt_mime_charsets[tinrc.mm_network_charset]);
+#else
+			if (!strncasecmp(tinrc.mm_charset, "EUC-", 4) &&
 				 !strcasecmp(mime_encoding, txt_7bit))
 				fprintf(f, "Content-Type: text/plain; charset=ISO-2022-%s\n", &tinrc.mm_charset[4]);
 			else
 				fprintf(f, "Content-Type: text/plain; charset=%s\n", tinrc.mm_charset);
+#endif /* CHARSET_CONVERSION */
 			fprintf(f, "Content-Transfer-Encoding: %s\n", mime_encoding);
 		} else {
 			fputs("Content-Type: text/plain; charset=US-ASCII\n", f);
@@ -869,24 +884,39 @@ rfc15211522_encode (
 
 /* For EUC-KR, 7bit means conversion to ISO-2022-KR specified in RFC 1557 */
 
-		if (!strcasecmp(tinrc.mm_charset, "euc-kr"))
+#ifdef CHARSET_CONVERSION
+		if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-KR"))
+#else
+		if (!strcasecmp(tinrc.mm_charset, "EUC-KR"))
+#endif /* CHARSET_CONVERSION */
 			body_encode = rfc1557_encode;
 
 #if 0
 /*
  * Not only EUC-JP but also other Japanese charsets such as
- * SJIS  might need RFC 1468 encoding. To be confirmed.
+ * SJIS might need RFC 1468 encoding. To be confirmed.
  */
-		else if (!strcasecmp(tinrc.mm_charset, "euc-jp"))
+#	ifdef CHARSET_CONVERSION
+		else if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-JP"))
 			body_encode = rfc1468_encode;
+#	else
+		else if (!strcasecmp(tinrc.mm_charset, "EUC-JP"))
+			body_encode = rfc1468_encode;
+#	endif /* CHARSET_CONVERSION */
 
 /*
- * Not only  EUC-CN but also other Chinese charsets such as
+ * Not only EUC-CN but also other Chinese charsets such as
  * BIG5 and EUC-TW might need RFC 1922 encoding. To be confirmed.
  */
-		else if (!strcasecmp(tinrc.mm_charset, "euc-cn"))
+#	ifdef CHARSET_CONVERSION
+		else if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-CN"))
 			body_encode = rfc1922_encode;
+#	else
+		else if (!strcasecmp(tinrc.mm_charset, "EUC-CN"))
+			body_encode = rfc1922_encode;
+#	endif /* CHARSET_CONVERSION */
 #endif /* 0 */
+
 		else {
 			body_encode = rfc1521_encode;
 			encoding = '8';
