@@ -3,10 +3,10 @@
  *  Module    : save.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2000-07-08
+ *  Updated   : 2001-11-10
  *  Notes     :
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,10 +45,10 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 #ifndef MENUKEYS_H
-#	include  "menukeys.h"
+#	include "menukeys.h"
 #endif /* !MENUKEYS_H */
 #ifndef RFC2046_H
-#	include  "rfc2046.h"
+#	include "rfc2046.h"
 #endif /* !RFC2046_H */
 
 #ifdef HAVE_UUDEVIEW_H
@@ -62,9 +62,9 @@
 enum state { INITIAL, MIDDLE, OFF, END };
 
 #ifdef VMS
-	const char DIRSEP = ']';		/* Seperator between dir + filename */
+	static const char DIRSEP = ']';		/* Seperator between dir + filename */
 #else
-	const char DIRSEP = '/';
+	static const char DIRSEP = '/';
 #endif /* VMS */
 
 /*
@@ -252,6 +252,7 @@ fprintf(stderr, "start_save: create_path(%s)\n", tmp);
 					fclose (fp);
 					saved_arts++;
 
+					/* TODO: if article allready contains To: Cc: Bcc: it get's 'relayed' */
 					if (function == MAIL_ANY_NEWS) {
 						strfmailer (mailer, arts[j].subject, mail_news_user, savefile, buf, sizeof (buf), tinrc.mailer_format);
 						invoke_cmd (buf);		/* Keep trying after errors */
@@ -293,7 +294,8 @@ fprintf(stderr, "start_save: create_path(%s)\n", tmp);
 		case MAIL_ANY_NEWS:
 		case SAVE_ANY_NEWS:
 			snprintf (buf, sizeof(buf) - 1, _(txt_saved_summary), (function == MAIL_ANY_NEWS ? _(txt_mailed) : _(txt_saved)),
-					PLURAL(saved_arts, txt_article), PLURAL(group_count, txt_group));
+					saved_arts, PLURAL(saved_arts, txt_article),
+					group_count, PLURAL(group_count, txt_group));
 			fprintf (fp_log, "%s", buf);
 			if (verbose)
 				wait_message (0, buf);
@@ -464,7 +466,7 @@ fprintf(stderr, "save_arts, create_path(%s)\n", save[0].path);
 
 	for (i = 0 ; i < num_save ; i++) {
 		/* the tailing spaces are needed for the progress-meter */
-		wait_message (0, "%s%d  ", _(txt_saving), i+1);
+		wait_message (0, "%s%d  ", _(txt_saving), i + 1);
 
 		memset (&artinfo, 0, sizeof(t_openartinfo));
 		switch (art_open (FALSE, save[i].artptr, group_path, &artinfo)) {
@@ -515,22 +517,25 @@ save_batch (
 		return FALSE;
 	}
 
-	if (count != num_save)
+	if (count != num_save)	/* FIMME: -> lang.c */
 		wait_message (2, _("Warning: Only %d out of %d articles were saved"), count, num_save);
 
 	first = get_first_savefile ();
 
-	switch (type) {
-		case iKeyFeedArt:
-			STRCPY (what, _("Article"));
-			break;
-		case iKeyFeedThd:
-			STRCPY (what, _("Thread"));
+	switch (type) {	/* FIMME: -> lang.c */
+		case iKeyFeedHot:
+			snprintf (what, sizeof(what), _("Hot %s"), PLURAL(count, txt_article));
 			break;
 		case iKeyFeedTag:
 			snprintf (what, sizeof(what), _("Tagged %s"), PLURAL(count, txt_article));
 			break;
+		case iKeyFeedThd:
+			STRCPY (what, _("Thread"));
+			break;
+		case iKeyFeedArt:
+		case iKeyFeedPat:
 		default:
+			snprintf (what, sizeof(what), "%s", PLURAL(count, txt_article));
 			break;
 	}
 
@@ -570,8 +575,8 @@ create_path (
 
 	for (i = 0, j = 0; i < len; i++, j++) {
 		buf[j] = path[i];
-		if (i+1 < len && path[i+1] == '/') {
-			buf[j+1] = '\0';
+		if (i + 1 < len && path[i + 1] == '/') {
+			buf[j + 1] = '\0';
 			if (stat (buf, &st) == -1) {
 				if (my_mkdir (buf, (mode_t)(S_IRWXU|S_IRUGO|S_IXUGO)) == -1) {
 					if (errno != EEXIST) {
@@ -643,7 +648,7 @@ add_to_save_list (
 {
 	char tmp[PATH_LEN];
 
-	if (num_save == max_save-1)
+	if (num_save == max_save - 1)
 		expand_save ();
 
 	save[num_save].is_mailbox = get_save_filename (tmp, path);
@@ -676,7 +681,7 @@ add_to_save_list (
 		 * Strip off any default filename
 		 */
 		if ((ptr = strrchr(tmp, DIRSEP)) != NULL)
-			*(ptr+1) = '\0';
+			*(ptr + 1) = '\0';
 
 		/* Add on the archive name as a directory */
 		joinpath(archpath, tmp, artptr->archive);
@@ -703,12 +708,12 @@ add_to_save_list (
 			 * clean
 			 */
 			if (num_save == 1) {
-				save[0].path = my_realloc(save[0].path, strlen(save[0].path)+4);
+				save[0].path = my_realloc(save[0].path, strlen(save[0].path) + 5);
 				strcat(save[0].path, ".001");
 				save[0].file = strrchr (save[0].path, DIRSEP) + 1;	/* ptr to filename portion */
 			}
 
-			sprintf (&tmp[strlen(tmp)], "%c%03d", pathsep, num_save+1);
+			sprintf (&tmp[strlen(tmp)], "%c%03d", pathsep, num_save + 1);
 		}
 	}
 
@@ -719,7 +724,7 @@ add_to_save_list (
 #if 0
 fprintf(stderr, "ATSL (%s) (%s)\n", save[num_save].path, save[num_save].file);
 #endif /* 0 */
-	return (save[num_save++].is_mailbox);		/* NB: num_save is bumped here */
+	return save[num_save++].is_mailbox;		/* NB: num_save is bumped here */
 }
 
 
@@ -811,7 +816,7 @@ get_last_savefile (
 	int i;
 	static const char *dummy = "";
 
-	for (i = num_save-1; i >= 0; i--) {
+	for (i = num_save - 1; i >= 0; i--) {
 		if (save[i].saved)
 			return (save[i].is_mailbox ? save[i].path : save[i].file);
 	}
@@ -964,7 +969,7 @@ post_process_uud (
 						/* don't use PATH_LEN - we use an absolute value (128) below  */
 						char name[130];
 
-						if (sscanf (s+6, "%*o %128c\n", name) != 1)     /* Get the real filename */
+						if (sscanf (s + 6, "%*o %128c\n", name) != 1)     /* Get the real filename */
 							name[0] = '\0';
 						else
 							strtok (name, "\n");
@@ -1108,7 +1113,7 @@ sum_and_view (
 	part = new_part(NULL);
 
 	if ((ext = strrchr(file, '.')) != NULL)
-		lookup_mimetype(ext+1, part);				/* Get MIME type/subtype */
+		lookup_mimetype(ext + 1, part);				/* Get MIME type/subtype */
 
 	/*
 	 * Needed for the mime-type processor
@@ -1205,9 +1210,7 @@ delete_processed_files (
 	t_bool delete_it = FALSE;
 
 	if (any_saved_files ()) {
-		if (CURR_GROUP.attribute->delete_tmp_files || auto_delete)
-			delete_it = TRUE;
-		else if (prompt_yn (cLINES, _(txt_delete_processed_files), TRUE) == 1)
+		if (CURR_GROUP.attribute->delete_tmp_files && (auto_delete || prompt_yn(cLINES, _(txt_delete_processed_files), TRUE) == 1))
 			delete_it = TRUE;
 
 		my_printf (cCRLF);
@@ -1387,9 +1390,11 @@ decode_save_one(
 	/*
 	 * View the attachment
 	 */
-	if (postproc && tinrc.post_process_view) {
-		start_viewer (part, savepath);
-		my_printf (cCRLF);
+	if (postproc) {
+		if (tinrc.post_process_view) {
+			start_viewer (part, savepath);
+			my_printf (cCRLF);
+		}
 	} else {
 		snprintf (buf, sizeof(buf) - 1, _(txt_view_attachment), savepath, content_types[part->type], part->subtype);
 		if (prompt_yn (cLINES, buf, TRUE) == 1)
@@ -1399,10 +1404,11 @@ decode_save_one(
 	/*
 	 * Save the attachment
 	 */
-	if (postproc) {
+	if (postproc && tinrc.post_process_view) {
 		my_printf (_(txt_uu_success), savepath);
 		my_printf (cCRLF);
-	} else {
+	}
+	if (!postproc) {
 		snprintf (buf, sizeof(buf) - 1, _(txt_save_attachment), savepath, content_types[part->type], part->subtype);
 		if (prompt_yn (cLINES, buf, FALSE) != 1)
 			unlink (savepath);
