@@ -6,7 +6,7 @@
  *  Updated   : 1997-12-27
  *  Notes     :
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -69,7 +69,7 @@ int srch_lineno = -1;
  */
 static int srch_offsets[6];
 static int srch_offsets_size = sizeof(srch_offsets)/sizeof(int);
-struct regex_cache srch_regex;
+static struct regex_cache srch_regex;
 
 
 /*
@@ -87,7 +87,7 @@ get_search_pattern (
 	sprintf (tmpbuf, (forward ? fwd_msg : bwd_msg), def);
 
 	if (!prompt_string_default(tmpbuf, def, _(txt_no_search_string), which_hist))
-		return (NULL);
+		return NULL;
 
 	wait_message (0, _(txt_searching));
 
@@ -95,7 +95,7 @@ get_search_pattern (
 
 	if (tinrc.wildcard) {			/* ie, not wildmat() */
 		strcpy(def, quote_wild_whitespace (def));
-		return(def);
+		return def;
 	}
 
 	/*
@@ -103,7 +103,7 @@ get_search_pattern (
 	 */
 /* TODO somehow use REGEX_FMT here ? */
 	sprintf(tmpbuf, "*%s*", def);
-	return(tmpbuf);
+	return tmpbuf;
 }
 
 
@@ -201,46 +201,6 @@ search_active (
 }
 
 
-/*
- * Called by help.c
- */
-int
-search_help (
-	t_bool forward,
-	int current,
-	int last)
-{
-	char *buf;
-	int incr = forward ? 1 : -1;
-	int result = current;
-	int n;
-
-	if (!(buf = get_search_pattern(
-				forward,
-				_(txt_search_forwards),
-				_(txt_search_backwards),
-				tinrc.default_search_group,
-				HIST_HELP_SEARCH
-	))) return result;
-
-	current += incr;
-	n = current;
-	do {
-		if (n < 0)
-			n = last;
-		else if (n > last)
-			n = 0;
-		if (REGEX_MATCH (info_help[n], buf, TRUE)) {
-			result = n;
-			break;
-		}
-		n += incr;
-	} while (n != current);
-	clear_message ();
-	return result;
-}
-
-
 /*
  * Scan the body of an arts[i] for searchbuf
  * used only by search_body()
@@ -269,7 +229,7 @@ body_search (
 			code = -1;
 			goto exit_search;
 		case ART_UNAVAILABLE:
-			code =  0;					/* Treat as string not present */
+			code = 0;					/* Treat as string not present */
 			goto exit_search;
 	}
 
@@ -334,7 +294,7 @@ author_search (
 	if (arts[i].name == (char *) 0)
 		ptr = arts[i].from;
 	else
-		snprintf (buf, sizeof(buf)-1, "%s <%s>", arts[i].name, arts[i].from);
+		snprintf (buf, sizeof(buf) - 1, "%s <%s>", arts[i].name, arts[i].from);
 
 	return (REGEX_MATCH (ptr, searchbuf, TRUE)) ? 1 : 0;
 }
