@@ -6,7 +6,7 @@
  *  Updated   : 1994-12-21
  *  Notes     :
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 #ifndef MENUKEYS_H
-#	include  "menukeys.h"
+#	include "menukeys.h"
 #endif /* !MENUKEYS_H */
 
 /*
@@ -55,6 +55,7 @@ static int reposition_group (struct t_group *group, int default_num);
 static int select_left (void);
 static int select_right (void);
 static t_bool pos_next_unread_group (t_bool redraw);
+static t_bool yank_in_active_file = TRUE;
 static void catchup_group (struct t_group *group, t_bool goto_next_unread_group);
 static void read_groups (void);
 static void select_quit (void);
@@ -117,23 +118,28 @@ selection_page (
 	show_selection_page ();	/* display group selection page */
 
 	forever {
-		if (!resync_active_file () && reread_active_after_posting ())	/* reread active file if necessary */
-			show_selection_page ();
+		if (!resync_active_file ()) {
+			if (reread_active_after_posting ()) /* reread active file if necessary */
+				show_selection_page ();
+		} else {
+			if (!yank_in_active_file)
+				yank_in_active_file = !yank_in_active_file; /* yank out if yanked in */
+		}
 
 		set_xclick_on ();
 		ch = handle_keypad (select_left, select_right, &menukeymap.select_nav);
 
 		switch (ch) {
 
-#ifndef WIN32
 			case ESC:		/* Abort */
 				break;
-#endif /* !WIN32 */
 
 			case '1': case '2': case '3': case '4': case '5':
 			case '6': case '7': case '8': case '9':
 				if (selmenu.max)
 					prompt_item_num (ch, _(txt_select_group));
+				else
+					info_message(_(txt_no_groups));
 				break;
 
 #ifndef NO_SHELL_ESCAPE
@@ -173,8 +179,11 @@ selection_page (
 				break;
 
 			case iKeySetRange:	/* set range */
-				if (bSetRange (SELECT_LEVEL, 1, selmenu.max, selmenu.curr+1))
-					show_selection_page ();
+				if (selmenu.max) {
+					if (bSetRange (SELECT_LEVEL, 1, selmenu.max, selmenu.curr + 1))
+						show_selection_page ();
+				} else
+					info_message(_(txt_no_groups));
 				break;
 
 			case iKeySearchSubjF:	/* search forward */
@@ -204,7 +213,7 @@ selection_page (
 
 			case iKeySelectResetNewsrc:	/* reset .newsrc */
 				if (no_write) {
-					wait_message(0, _(txt_info_no_write));
+					info_message(_(txt_info_no_write));
 					break;
 				}
 				if (prompt_yn (cLINES, _(txt_reset_newsrc), FALSE) == 1) {
@@ -217,9 +226,10 @@ selection_page (
 
 			case iKeySelectCatchup:	/* catchup - mark all articles as read */
 			case iKeySelectCatchupNextUnread:	/* and goto next unread group */
-				if (!selmenu.max)
-					break;
-				catchup_group (&CURR_GROUP, (ch == iKeySelectCatchupNextUnread));
+				if (selmenu.max)
+					catchup_group (&CURR_GROUP, (ch == iKeySelectCatchupNextUnread));
+				else
+					info_message(_(txt_no_groups));
 				break;
 
 			case iKeySelectToggleDescriptions:	/* toggle newsgroup descriptions */
@@ -238,7 +248,7 @@ selection_page (
 				break;
 
 			case iKeyHelp:					/* help */
-				show_info_page (HELP_INFO, help_select, _(txt_group_select_com));
+				show_help_page (SELECT_LEVEL, _(txt_group_select_com));
 				show_selection_page ();
 				break;
 
@@ -268,12 +278,18 @@ selection_page (
 				break;
 
 			case iKeySelectMoveGrp:	/* reposition group within group list */
-				if (no_write) {
-					wait_message(0, _(txt_info_no_write));
+				if (!selmenu.max) {
+					info_message(_(txt_no_groups));
 					break;
 				}
+
+				if (no_write) {
+					info_message(_(txt_info_no_write));
+					break;
+				}
+
 				if (!CURR_GROUP.subscribed) {
-					wait_message(0, _(txt_info_not_subscribed));
+					info_message (_(txt_info_not_subscribed));
 					break;
 				}
 
@@ -337,10 +353,12 @@ selection_page (
 				break;
 
 			case iKeySelectSubscribe:	/* subscribe to current group */
-				if (!selmenu.max)
+				if (!selmenu.max) {
+					info_message(_(txt_no_groups));
 					break;
+				}
 				if (no_write) {
-					wait_message(0, _(txt_info_no_write));
+					info_message(_(txt_info_no_write));
 					break;
 				}
 				if (!CURR_GROUP.subscribed && !CURR_GROUP.bogus) {
@@ -353,17 +371,19 @@ selection_page (
 
 			case iKeySelectSubscribePat:	/* subscribe to groups matching pattern */
 				if (no_write) {
-					wait_message(0, _(txt_info_no_write));
+					info_message(_(txt_info_no_write));
 					break;
 				}
 				subscribe_pattern (_(txt_subscribe_pattern), _(txt_subscribing), _(txt_subscribed_num_groups), TRUE);
 				break;
 
 			case iKeySelectUnsubscribe:	/* unsubscribe to current group */
-				if (!selmenu.max)
+				if (!selmenu.max) {
+					info_message(_(txt_no_groups));
 					break;
+				}
 				if (no_write) {
-					wait_message(0, _(txt_info_no_write));
+					info_message(_(txt_info_no_write));
 					break;
 				}
 				if (CURR_GROUP.subscribed) {
@@ -386,7 +406,7 @@ selection_page (
 
 			case iKeySelectUnsubscribePat:	/* unsubscribe to groups matching pattern */
 				if (no_write) {
-					wait_message(0, _(txt_info_no_write));
+					info_message(_(txt_info_no_write));
 					break;
 				}
 				subscribe_pattern (_(txt_unsubscribe_pattern),
@@ -435,14 +455,18 @@ selection_page (
 				yank_active_file();
 				break;
 
-			case iKeySelectSyncWithActive:  /* Re-read active file to see if any new news */
+			case iKeySelectSyncWithActive:	/* Re-read active file to see if any new news */
 				sync_active_file ();
+				if (!yank_in_active_file)
+					yank_active_file();			/* yank out if yanked in */
 				break;
 
 			case iKeySelectMarkGrpUnread:
 			case iKeySelectMarkGrpUnread2:	/* mark group unread */
-				if (!selmenu.max)
+				if (!selmenu.max) {
+					info_message(_(txt_no_groups));
 					break;
+				}
 				grp_mark_unread (&CURR_GROUP);
 				if (CURR_GROUP.newsrc.num_unread)
 					strcpy (mesg, tin_ltoa(CURR_GROUP.newsrc.num_unread, 5));
@@ -534,19 +558,19 @@ show_selection_page (
 		if (show_description) {
 			if (active[n].description)
 				sprintf (sptr, "  %c %s %s  %-*.*s  %-*.*s" cCRLF,
-				         subs, tin_ltoa(i+1, 4), tmp,
+				         subs, tin_ltoa(i + 1, 4), tmp,
 				         groupname_len, groupname_len, active_name,
 				         blank_len, blank_len, group_descript);
 			else
 				sprintf (sptr, "  %c %s %s  %-*.*s  " cCRLF,
-				         subs, tin_ltoa(i+1, 4), tmp,
-				         (groupname_len+blank_len),
-				         (groupname_len+blank_len), active[n].name);
+				         subs, tin_ltoa(i + 1, 4), tmp,
+				         (groupname_len + blank_len),
+				         (groupname_len + blank_len), active[n].name);
 		} else {
 			if (tinrc.draw_arrow)
-				sprintf (sptr, "  %c %s %s  %-*.*s" cCRLF, subs, tin_ltoa(i+1, 4), tmp, groupname_len, groupname_len, active_name);
+				sprintf (sptr, "  %c %s %s  %-*.*s" cCRLF, subs, tin_ltoa(i + 1, 4), tmp, groupname_len, groupname_len, active_name);
 			else
-				sprintf (sptr, "  %c %s %s  %-*.*s%*s" cCRLF, subs, tin_ltoa(i+1, 4), tmp, groupname_len, groupname_len, active_name, blank_len, " ");
+				sprintf (sptr, "  %c %s %s  %-*.*s%*s" cCRLF, subs, tin_ltoa(i + 1, 4), tmp, groupname_len, groupname_len, active_name, blank_len, " ");
 		}
 		if (tinrc.strip_blanks) {
 			strip_line (sptr);
@@ -593,7 +617,6 @@ sync_active_file (
 	resync_active_file ();
 }
 
-
 static void
 yank_active_file (
 	void)
@@ -601,7 +624,6 @@ yank_active_file (
 	char *oldgroup = (char *) 0;
 	int i;
 	int oldmax = selmenu.max;
-	static t_bool yank_in_active_file = TRUE;
 
 	if (oldmax)
 		oldgroup = my_strdup(CURR_GROUP.name);
@@ -695,7 +717,7 @@ skip_newgroups (
 			i++;
 	}
 
-	return(i);
+	return i;
 }
 
 
@@ -772,7 +794,7 @@ reposition_group (
 	 */
 	if (pos_num <= newgroups) {
 		error_message(_(txt_skipping_newgroups));
-		return(default_num);
+		return default_num;
 	}
 
 	wait_message (0, _(txt_moving), group->name);
@@ -790,10 +812,10 @@ reposition_group (
 	if (pos_group_in_newsrc (group, pos_num - newgroups)) {
 		read_newsrc (newsrc, TRUE);
 		tinrc.default_move_group = pos_num;
-		return (pos_num-1);
+		return (pos_num - 1);
 	} else {
 		tinrc.default_move_group = default_num + 1;
-		return (default_num);
+		return default_num;
 	}
 }
 
@@ -1118,13 +1140,13 @@ select_read_group (
 {
 	struct t_group currgrp;
 
-	currgrp = CURR_GROUP;
-
-	if (!selmenu.max) {
+	if (!selmenu.max || selmenu.curr == -1) {
 		info_message (_(txt_no_groups));
 		return;
 	}
 
+	currgrp = CURR_GROUP;
+
 	if (currgrp.bogus) {
 		info_message (_(txt_not_exist));
 		return;
