@@ -3,10 +3,10 @@
  *  Module    : signal.c
  *  Author    : I.Lea
  *  Created   : 1991-04-01
- *  Updated   : 1994-12-21
+ *  Updated   : 2001-07-22
  *  Notes     : signal handlers for different modes and window resizing
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,6 +47,9 @@
 #ifndef included_trace_h
 #	include "trace.h"
 #endif /* !included_trace_h */
+#ifndef VERSION_H
+#	include "version.h"
+#endif /* !VERSION_H */
 
 
 #if defined (VMS) && defined (SIGBUS)
@@ -117,6 +120,7 @@ int need_resize = cNo;
 
 int NOTESLINES;			/* # lines of useable area */
 
+#ifndef __LCLINT__ /* lclint doesn't like it */
 static const struct {
 	int code;
 	const char *name;
@@ -127,20 +131,18 @@ static const struct {
 #ifdef SIGQUIT
 	{ SIGQUIT,	"SIGQUIT " },	/* ctrl-\ */
 #endif /* SIGQUIT */
-#ifndef WIN32
-#	ifdef SIGILL
+#ifdef SIGILL
 	{ SIGILL,	"SIGILL" },	/* illegal instruction */
-#	endif /* SIGILL */
-#	ifdef SIGFPE
+#endif /* SIGILL */
+#ifdef SIGFPE
 	{ SIGFPE,	"SIGFPE" },	/* floating point exception */
-#	endif /* SIGFPE */
-#	ifdef SIGBUS
+#endif /* SIGFPE */
+#ifdef SIGBUS
 	{ SIGBUS,	"SIGBUS" },	/* bus error */
-#	endif /* SIGBUS */
-#	ifdef SIGSEGV
+#endif /* SIGBUS */
+#ifdef SIGSEGV
 	{ SIGSEGV,	"SIGSEGV" },	/* segmentation violation */
-#	endif /* SIGSEGV */
-#endif /* !WIN32 */
+#endif /* SIGSEGV */
 #ifdef SIGPIPE
 	{ SIGPIPE,	"SIGPIPE" },	/* broken pipe */
 #endif /* SIGPIPE */
@@ -166,14 +168,16 @@ static const struct {
 	{ SIGWINCH,	"SIGWINCH" },	/* window-size change */
 #endif /* SIGWINCH */
 };
+#endif /* !__LCLINT__ */
 
-#	ifdef HAVE_NESTED_PARAMS
+
+#ifdef HAVE_NESTED_PARAMS
 RETSIGTYPE (*sigdisp(int signum, RETSIGTYPE (_CDECL *func)(SIG_ARGS)))(SIG_ARGS)
-#	else
+#else
 RETSIGTYPE (*sigdisp(signum, func))(SIG_ARGS)
 	int signum;
 	RETSIGTYPE (_CDECL *func)(SIG_ARGS);
-#	endif /* HAVE_NESTED_PARAMS */
+#endif /* HAVE_NESTED_PARAMS */
 {
 #ifdef HAVE_POSIX_JC
 #	define RESTORE_HANDLER(x, y)
@@ -202,23 +206,23 @@ void
 allow_resize (
 	t_bool allow)
 {
-#	ifdef HAVE_POSIX_JC
+#ifdef HAVE_POSIX_JC
 	struct sigaction sa, osa;
 
 	sa.sa_handler = signal_handler;
 	sigemptyset (&sa.sa_mask);
 	sa.sa_flags = 0;
-#		ifdef SA_RESTART
+#	ifdef SA_RESTART
 	if (!allow)
 		sa.sa_flags |= SA_RESTART;
-#		endif /* SA_RESTART */
-#		ifdef SIGWINCH
+#	endif /* SA_RESTART */
+#	ifdef SIGWINCH
 	sigaction(SIGWINCH, &sa, &osa);
-#		endif /* SIGWINCH */
-#		ifdef SIGTSTP
+#	endif /* SIGWINCH */
+#	ifdef SIGTSTP
 	sigaction(SIGTSTP, &sa, &osa);
-#		endif /* SIGTSTP */
-#	endif /* HAVE_POSIX_JC */
+#	endif /* SIGTSTP */
+#endif /* HAVE_POSIX_JC */
 }
 
 static const char *
@@ -263,7 +267,7 @@ handle_resize (
 
 #	ifdef USE_CURSES
 #		ifdef HAVE_RESIZETERM
-	resizeterm(cLINES+1, cCOLS);
+	resizeterm(cLINES + 1, cCOLS);
 	my_retouch();					/* seems necessary if win size unchanged */
 #		else
 	my_retouch();
@@ -278,12 +282,11 @@ handle_resize (
 		case cConfig:
 			refresh_config_page (-1);
 			break;
-		case cHelp:
-#	ifdef USE_CURSES
-			display_info_page (TRUE);
-#	else
-			display_info_page();
-#	endif /* USE_CURSES*/
+		case cFilter:
+			refresh_filter_menu ();
+			break;
+		case cInfopager:
+			display_info_page (0);
 			break;
 		case cGroup:
 		case cSelect:
@@ -369,39 +372,46 @@ signal_handler (
 			RESTORE_HANDLER(sig, signal_handler);
 			return;
 #endif /* SIGPIPE */
+
 #ifdef SIGTSTP
 		case SIGTSTP:
 			handle_suspend();
 			return;
 #endif /* SIGTSTP */
+
 #ifdef SIGWINCH
 		case SIGWINCH:
 			need_resize = cYes;
 			RESTORE_HANDLER (SIGWINCH, signal_handler);
 			return;
 #endif /* SIGWINCH */
+
 		default:
 			break;
 	}
 	fprintf (stderr, "\n%s: signal handler caught %s signal (%d).\n", tin_progname, signal_name(sig), sig);
+
 #if defined(SIGHUP)
 	if (sig == SIGHUP) {
 		dangerous_signal_exit = TRUE;
 		tin_done (- SIGHUP);
 	}
 #endif /* SIGHUP */
+
 #if defined(SIGUSR1)
 	if (sig == SIGUSR1) {
 		dangerous_signal_exit = TRUE;
 		tin_done (- SIGUSR1);
 	}
 #endif /* SIGUSR1 */
+
 #if defined(SIGTERM)
 	if (sig == SIGTERM) {
 		dangerous_signal_exit = TRUE;
 		tin_done (- SIGTERM);
 	}
 #endif /* SIGTERM */
+
 #if defined(SIGBUS) || defined(SIGSEGV)
 	if (
 #	ifdef SIGBUS
@@ -414,7 +424,10 @@ signal_handler (
 		sig == SIGSEGV
 #	endif /* SIGSEGV */
 		) {
-		vPrintBugAddress ();
+		/* FIXME: -> lang.c */
+		my_fprintf (stderr, _("%s %s %s (\"%s\") [%s]: send a DETAILED bug report to %s\n"),
+		tin_progname, VERSION, RELEASEDATE, RELEASENAME, OSNAME, bug_addr);
+		my_fflush (stderr);
 	}
 #endif /* SIGBUS || SIGSEGV */
 	cleanup_tmp_files ();
@@ -428,6 +441,7 @@ signal_handler (
 #endif /* 1 */ /* apollo || HAVE_COREFILE */
 }
 
+
 /*
  * Turn on (flag != FALSE) our signal handler for TSTP and WINCH
  * Otherwise revert to the default handler
@@ -446,6 +460,7 @@ set_signal_catcher (
 #endif /* SIGWINCH */
 }
 
+
 void
 set_signal_handlers (
 	void)
@@ -496,6 +511,11 @@ set_win_size (
 	old_lines = *num_lines;
 	old_cols = *num_cols;
 
+#ifdef HAVE_XCURSES
+	*num_lines = LINES - 1;		/* FIXME */
+	*num_cols = COLS;
+#else	/* curses/ncurses */
+
 #ifndef USE_CURSES
 	init_screen_array (FALSE);		/* deallocate screen array */
 #endif /* !USE_CURSES */
@@ -527,6 +547,8 @@ set_win_size (
 	init_screen_array (TRUE);		/* allocate screen array for resize */
 #endif /* !USE_CURSES */
 
+#endif /* HAVE_XCURSES */
+
 	set_subj_from_size (*num_cols);
 
 	RIGHT_POS = *num_cols - 20;
