@@ -3,10 +3,10 @@
  *  Module    : thread.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 1997-12-26
+ *  Updated   : 2001-11-10
  *  Notes     :
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 #ifndef MENUKEYS_H
-#	include  "menukeys.h"
+#	include "menukeys.h"
 #endif /* !MENUKEYS_H */
 
 #define INDEX2TNUM(i)	((i) % NOTESLINES)
@@ -106,7 +106,7 @@ bld_tline (
 	 * Start with 2 spaces for ->
 	 * then index number of the message and whitespace (2+4+1 chars)
 	 */
-	sprintf (buff, "  %s ", tin_ltoa(l+1, 4));
+	sprintf (buff, "  %s ", tin_ltoa(l + 1, 4));
 	rest_of_line -= 7;
 
 	/*
@@ -152,15 +152,11 @@ bld_tline (
 	}
 
 	if (tinrc.show_score) {
-		char tmp_score[15];
-
 		if (tinrc.show_lines) { /* insert a separator if show lines and score */
 			strcat (buff, ",");
 			rest_of_line--;
 		}
-
-		sprintf (tmp_score, "%6d", art->score);
-		strcat (buff, tmp_score); /* add score */
+		strcat (buff, tin_ltoa(art->score, 6));
 		rest_of_line -= 6;
 	}
 
@@ -199,7 +195,7 @@ bld_tline (
 		 * Insert tree-structure strings "`->", "+->", ...
 		 */
 
-		make_prefix(art->refptr, buff+strlen(buff), len_subj);
+		make_prefix(art->refptr, buff + strlen(buff), len_subj);
 
 		/*
 		 * Copy in the subject up to where the author (if any) starts
@@ -220,7 +216,7 @@ bld_tline (
 			if (!(ptr && arts[ptr->article].subject == art->subject))
 				strncat(buff, art->subject, gap);
 
-			buff[len+gap] = '\0';	/* Just in case */
+			buff[len + gap] = '\0';	/* Just in case */
 		}
 
 		/*
@@ -237,7 +233,7 @@ bld_tline (
 		}
 
 	} else /* Add the author info. This is always shown if subject is not */
-		get_author (TRUE, art, buff+strlen(buff), cCOLS-strlen(buff));
+		get_author (TRUE, art, buff + strlen(buff), cCOLS - strlen(buff));
 
 	/* protect display from non-displayable characters (e.g., form-feed) */
 	convert_to_printable (buff);
@@ -269,7 +265,7 @@ draw_line (
 	int i,
 	int magic)
 {
-	int startpos = (!magic) ? 0 : (MARK_OFFSET-2);
+	int startpos = (!magic) ? 0 : (MARK_OFFSET - 2);
 	int tlen;
 #ifdef USE_CURSES
 	char buffer[BUFSIZ];
@@ -312,7 +308,7 @@ draw_line (
 		EndInverse();
 #endif /* 0 */
 	}
-	MoveCursor(INDEX2LNUM(i)+1, 0);
+	MoveCursor(INDEX2LNUM(i) + 1, 0);
 	return;
 }
 
@@ -376,7 +372,7 @@ thread_page (
 	/*
 	 * If threading by Refs, it helps to see the subject line
 	 */
-	show_subject = ((arts[thread_respnum].archive != (char *)0) || (group->attribute->thread_arts >= THREAD_REFS));
+	show_subject = ((arts[thread_respnum].archive != (char *)0) || (group->attribute->thread_arts == THREAD_REFS) || (group->attribute->thread_arts == THREAD_BOTH));
 
 	/*
 	 * Set the cursor to the last response unless pos_first_unread is on
@@ -421,10 +417,8 @@ thread_page (
 		set_xclick_on ();
 		switch (ch = handle_keypad (thread_left, thread_right, &menukeymap.thread_nav)) {
 
-#ifndef WIN32
 			case iKeyAbort:			/* Abort */
 				break;
-#endif /* !WIN32 */
 
 			case '1': case '2': case '3': case '4': case '5':
 			case '6': case '7': case '8': case '9':
@@ -540,7 +534,7 @@ thread_page (
 					bld_tline (thdmenu.curr, &arts[n]);
 					draw_line (thdmenu.curr, MAGIC);
 				}
-				if ((n = next_unread (n)) == -1)  {	/* no more unread articles */
+				if ((n = next_unread (n)) == -1) {	/* no more unread articles */
 					ret_code = GRP_EXIT;
 					break;
 				}
@@ -556,8 +550,13 @@ thread_page (
 				}
 				break;
 
+			case iKeyOptionMenu:
+				(void) change_config_file(group);
+				show_thread_page ();
+				break;
+
 			case iKeyHelp:					/* help */
-				show_info_page (HELP_INFO, help_thread, _(txt_thread_com));
+				show_help_page (THREAD_LEVEL, _(txt_thread_com));
 				show_thread_page ();
 				break;
 
@@ -651,7 +650,7 @@ thread_page (
 
 			case iKeyThreadMarkArtUnread:		/* mark article as unread */
 				n = find_response (thread_basenote, thdmenu.curr);
-				art_mark_will_return (group, &arts[n]); /*art_mark_unread (group, &arts[n]);*/
+				art_mark_will_return (group, &arts[n]); /* art_mark_unread (group, &arts[n]); */
 				bld_tline (thdmenu.curr, &arts[n]);
 				draw_line (thdmenu.curr, MAGIC);
 				info_message (_(txt_marked_as_unread), _("Article"));
@@ -741,9 +740,9 @@ show_thread_page (
 	assert(thdmenu.first != 0 || the_index == thread_respnum);
 
 	if (show_subject)
-		sprintf (mesg, _("List Thread (%d of %d)"), grpmenu.curr+1, grpmenu.max);
+		sprintf (mesg, _("List Thread (%d of %d)"), grpmenu.curr + 1, grpmenu.max);
 	else
-		sprintf (mesg, _("Thread (%.*s)"), cCOLS-23, arts[thread_respnum].subject);
+		sprintf (mesg, _("Thread (%.*s)"), cCOLS - 23, arts[thread_respnum].subject);
 
 	/*
 	 * Slight misuse of the 'mesg' buffer here. We need to clear it so that progress messages
@@ -829,7 +828,7 @@ fixup_thread (
 
 
 /*
- *  Return the number of unread articles there are within a thread
+ * Return the number of unread articles there are within a thread
  */
 int
 new_responses (
@@ -848,16 +847,16 @@ new_responses (
 
 
 /*
- *  Which base note (an index into base[]) does a respnum
- *  (an index into arts[]) correspond to?
+ * Which base note (an index into base[]) does a respnum (an index into
+ * arts[]) correspond to?
  *
- *  In other words, base[] points to an entry in arts[] which is
- *  the head of a thread, linked with arts[].thread.  For any q: arts[q],
- *  find i such that base[i]->arts[n]->arts[o]->...->arts[q]
+ * In other words, base[] points to an entry in arts[] which is the head of
+ * a thread, linked with arts[].thread. For any q: arts[q], find i such that
+ * base[i]->arts[n]->arts[o]->...->arts[q]
  *
- *  Note that which_thread() can return -1 if in show_read_only mode and
- *  the article of interest has been read as well as all other articles in
- *  the thread,  thus resulting in no base[] entry for it.
+ * Note that which_thread() can return -1 if in show_read_only mode and the
+ * article of interest has been read as well as all other articles in the
+ * thread, thus resulting in no base[] entry for it.
  */
 int
 which_thread (
@@ -870,9 +869,10 @@ which_thread (
 			if (j == n)
 #ifdef JUST_TESTING
 			{
-				/* We can much more rapidly locate the topmost available parent than
-				 * the brute force method above. This is definitely the Right Way to
-				 * go in 1.5
+				/*
+				 * We can much more rapidly locate the topmost available parent
+				 * than the brute force method above. This is definitely the
+				 * Right Way to go in 1.5
 				 */
 				int val = n;
 				struct t_msgid *ptr;
@@ -898,7 +898,7 @@ which_thread (
 
 
 /*
- *  Find how deep in its' thread arts[n] is.  Start counting at zero
+ * Find how deep in its' thread arts[n] is. Start counting at zero
  */
 int
 which_response (
@@ -922,8 +922,8 @@ which_response (
 
 
 /*
- *  Given an index into base[], find the number of responses for
- *  that basenote
+ * Given an index into base[], find the number of responses for
+ * that basenote
  */
 int
 num_of_responses (
@@ -949,6 +949,49 @@ num_of_responses (
 }
 
 
+/*
+ * Calculating the score of a thread has been extracted from stat_thread()
+ * because we need it also in art.c to sort base[].
+ * get_score_of_thread expects the number of the first article of a thread.
+ */
+int
+get_score_of_thread (
+	int n)
+{
+	int i;
+	int score = 0;
+#ifdef THREAD_WEIGHT
+	int j = 0;
+#endif /* THREAD_WEIGHT */
+
+	for (i = n; i >= 0; i = arts[i].thread)
+		if (arts[i].status == ART_UNREAD) {
+#ifndef THREAD_SUM
+			/* we use the maximum article score for the complete thread */
+			if ((arts[i].score > score) && (arts[i].score > 0))
+				score = arts[i].score;
+			else {
+				if ((arts[i].score < score) && (score <= 0))
+					score = arts[i].score;
+			}
+#else
+			/* sum scores of unread arts and count num. arts */
+			score += arts[i].score;
+#	ifdef THREAD_WEIGHT
+			j++;
+#	endif /* THREAD_WEIGHT */
+#endif /* !THREAD_SUM */
+		}
+#ifdef THREAD_SUM
+#  ifdef THREAD_WEIGHT /* 'weight' thread-score or just sum? */
+	if (j)
+		score /= j;
+#  endif /* THREAD_WEIGHT */
+#endif /* THREAD_SUM */
+	return score;
+}
+
+
 /*
  * Given an index into base[], return relevant statistics
  */
@@ -958,21 +1001,22 @@ stat_thread (
 	struct t_art_stat *sbuf) /* return value is always ignored */
 {
 	int i;
-#ifdef THREAD_SUM
-	int j = 0;
-#endif /* THREAD_SUM */
+	MultiPartInfo minfo;
 
-	sbuf->total  = 0;
+	sbuf->total = 0;
 	sbuf->unread = 0;
-	sbuf->seen   = 0;
+	sbuf->seen = 0;
 	sbuf->deleted = 0;
 	sbuf->inrange = 0;
 	sbuf->selected_total = 0;
 	sbuf->selected_unread= 0;
-	sbuf->selected_seen  = 0;
+	sbuf->selected_seen = 0;
 	sbuf->art_mark = tinrc.art_marked_read;
 	sbuf->score = 0 /*-(SCORE_MAX) */;
 	sbuf->time = 0;
+	sbuf->multipart_compare_len = 0;
+	sbuf->multipart_total = 0;
+	sbuf->multipart_have = 0;
 
 	for (i = (int) base[n]; i >= 0; i = arts[i].thread) {
 		++sbuf->total;
@@ -987,19 +1031,6 @@ stat_thread (
 
 			if (arts[i].date > sbuf->time)
 				sbuf->time = arts[i].date;
-#ifndef THREAD_SUM
-			/* we use the maximum article score for the complete thread */
-			if ((arts[i].score > sbuf->score) && (arts[i].score > 0))
-				sbuf->score = arts[i].score;
-			else {
-				if ((arts[i].score < sbuf->score) && (sbuf->score <= 0))
-					sbuf->score = arts[i].score;
-			}
-#else
-			/* sum scores of unread arts and count num. arts */
-			sbuf->score += arts[i].score;
-			j++;
-#endif /* !THREAD_SUM */
 		} else if (arts[i].status == ART_WILL_RETURN)
 			++sbuf->seen;
 
@@ -1015,19 +1046,23 @@ stat_thread (
 		if (arts[i].killed)
 			++sbuf->killed;
 #endif /* 0 */
+
+		if ((CURR_GROUP.attribute && CURR_GROUP.attribute->thread_arts == THREAD_MULTI) && global_get_multipart_info(i, &minfo) && (minfo.total >= 1)) {
+			sbuf->multipart_compare_len = minfo.subject_compare_len;
+			sbuf->multipart_total = minfo.total;
+			sbuf->multipart_have++;
+		}
 	}
-#ifdef THREAD_SUM
-	if (j)
-		sbuf->score /= j;
-#endif /* THREAD_SUM */
+
+	sbuf->score = get_score_of_thread((int) base[n]);
 	sbuf->art_mark = (sbuf->inrange ? tinrc.art_marked_inrange : (sbuf->deleted ? tinrc.art_marked_deleted : (sbuf->selected_unread ? tinrc.art_marked_selected : (sbuf->unread ? (tinrc.recent_time && (time((time_t) 0) - sbuf->time) < (tinrc.recent_time * DAY)) ? tinrc.art_marked_recent : tinrc.art_marked_unread : (sbuf->seen ? tinrc.art_marked_return : tinrc.art_marked_read)))));
-	return(sbuf->total);
+	return sbuf->total;
 }
 
 
 /*
- *  Find the next response to arts[n].  Go to the next basenote if there
- *  are no more responses in this thread
+ * Find the next response to arts[n]. Go to the next basenote if there
+ * are no more responses in this thread
  */
 int
 next_response (
@@ -1048,8 +1083,8 @@ next_response (
 
 
 /*
- *  Given a respnum (index into arts[]), find the respnum of the
- *  next basenote
+ * Given a respnum (index into arts[]), find the respnum of the
+ * next basenote
  */
 int
 next_thread (
@@ -1066,9 +1101,9 @@ next_thread (
 
 
 /*
- *  Find the previous response.  Go to the last response in the previous
- *  thread if we go past the beginning of this thread.
- *  Return -1 if we are are the start of the group
+ * Find the previous response. Go to the last response in the previous
+ * thread if we go past the beginning of this thread.
+ * Return -1 if we are are the start of the group
  */
 int
 prev_response (
@@ -1080,7 +1115,7 @@ prev_response (
 	resp = which_response (n);
 
 	if (resp > 0)
-		return find_response (which_thread (n), resp-1);
+		return find_response (which_thread (n), resp - 1);
 
 	i = which_thread (n) - 1;
 
@@ -1111,9 +1146,9 @@ find_response (
 
 
 /*
- *  Find the next unread response to art[n] in this group. If no response is found
- *  from current point to the end restart from beginning of articles. If no more
- *  responses can be found, return -1
+ * Find the next unread response to art[n] in this group. If no response is
+ * found from current point to the end restart from beginning of articles.
+ * If no more responses can be found, return -1
  */
 int
 next_unread (
@@ -1141,7 +1176,7 @@ next_unread (
 
 
 /*
- *  Find the previous unread response in this thread
+ * Find the previous unread response in this thread
  */
 int
 prev_unread (
@@ -1181,8 +1216,8 @@ has_sibling (
 
 /*
  * mutt-like subject according. by sjpark@sparcs.kaist.ac.kr
- * string in prefix will be overwritten up to length len
- * prefix will always be terminated with \0
+ * string in prefix will be overwritten up to length len prefix will always
+ * be terminated with \0
  * make sure prefix is at least len+1 bytes long (to hold the terminating
  * null byte)
  */
@@ -1195,6 +1230,7 @@ make_prefix (
 	char *buf;
 	int prefix_ptr;
 	int depth = 0;
+	int depth_level = 0;
 	struct t_msgid *ptr;
 
 	for (ptr = art->parent; ptr; ptr = ptr->parent)
@@ -1207,18 +1243,35 @@ make_prefix (
 
 	prefix_ptr = depth * 2 - 1;
 
+	if (prefix_ptr > maxlen - 1 - !(maxlen % 2)) {
+		int odd = !(maxlen % 2);
+
+		prefix_ptr -= maxlen - ++depth_level - 2 - odd;
+
+		while (prefix_ptr > maxlen - 2 - odd) {
+			if (depth_level < maxlen / 5)
+				depth_level++;
+			prefix_ptr -= maxlen - depth_level - 2 - odd;
+			odd = !odd;
+		}
+	}
+
 	if (!(buf = my_malloc (prefix_ptr + 3)))
 		return;	/* out of memory */
 
 	strcpy (&buf[prefix_ptr], "->");
 	buf[--prefix_ptr] = (has_sibling (art) ? '+' : '`');
 
-	for (ptr = art->parent; prefix_ptr != 0; ptr = ptr->parent) {
+	for (ptr = art->parent; prefix_ptr > 1; ptr = ptr->parent) {
 		if (EXPIRED (ptr))
 			continue;
 		buf[--prefix_ptr] = ' ';
 		buf[--prefix_ptr] = (has_sibling (ptr) ? '|' : ' ');
 	}
+
+	while (depth_level)
+		buf[--depth_level] = '>';
+
 	strncpy (prefix, buf, maxlen);
 	prefix[maxlen] = '\0'; /* just in case strlen(buf) > maxlen */
 	free (buf);
@@ -1239,7 +1292,7 @@ thread_catchup(
 {
 	char buf[LEN];
 	int i, n;
-	int yn = 1;
+	int pyn = 1;
 
 	/* Find first unread art in this thread */
 	n = ((thdmenu.curr == 0) ? thread_respnum : find_response (thread_basenote, 0));
@@ -1251,23 +1304,23 @@ thread_catchup(
 /* FIXME do some NLS/snprintf work here - see equivalent code in group_catchup() */
 	if (i != -1) {				/* still unread arts in this thread */
 		sprintf(buf, _(txt_mark_thread_read), (ch == iKeyThreadCatchupNextUnread) ? _(txt_enter_next_thread) : "");
-		if (!tinrc.confirm_action || (tinrc.confirm_action && (yn = prompt_yn (cLINES, buf, TRUE)) == 1))
+		if (!tinrc.confirm_action || (tinrc.confirm_action && (pyn = prompt_yn (cLINES, buf, TRUE)) == 1))
 			thd_mark_read (&CURR_GROUP, base[thread_basenote]);
 	}
 
 	switch (ch) {
 		case iKeyThreadCatchup:				/* 'c' */
-			if (yn == 1)
+			if (pyn == 1)
 				return GRP_NEXT;
 			break;
 
 		case iKeyThreadCatchupNextUnread:	/* 'C' */
-			if (yn == 1)
+			if (pyn == 1)
 				return GRP_NEXTUNREAD;
 			break;
 
 		case iKeyCatchupLeft:			/* <- thread catchup on exit */
-			switch (yn) {
+			switch (pyn) {
 				case -1:				/* ESC from prompt, stay in group */
 					break;
 				case 1:					/* We caught up - advance group */
@@ -1291,7 +1344,7 @@ thread_catchup(
  * Return:
  *	<0 to quit to group menu
  *	 0 to stay in thread menu
- *  >0 after normal exit from pager to return to previous menu level
+ * >0 after normal exit from pager to return to previous menu level
  */
 static int
 enter_pager(
